Package: rstatix
Type: Package
Title: Pipe-Friendly Framework for Basic Statistical Tests
Version: 1.0.0
Date: 2026-07-03
Authors@R: c(
    person("Alboukadel", "Kassambara", role = c("aut", "cre"), email = "alboukadel.kassambara@gmail.com"))
Description: Provides a simple and intuitive pipe-friendly framework, coherent with the 'tidyverse' design philosophy, 
    for performing basic statistical tests, including t-test, Wilcoxon test, ANOVA, Kruskal-Wallis and correlation analyses. 
    The output of each test is automatically transformed into a tidy data frame to facilitate visualization. 
    Additional functions are available for reshaping, reordering, manipulating and visualizing correlation matrix.  
    Functions are also included to facilitate the analysis of factorial experiments, including purely 'within-Ss' designs 
    (repeated measures), purely 'between-Ss' designs, and mixed 'within-and-between-Ss' designs. 
    It's also possible to compute several effect size metrics, including "eta squared" for ANOVA, "Cohen's d" for t-test and 
    'Cramer V' for the association between categorical variables. 
    The package contains helper functions for identifying univariate and multivariate outliers, assessing normality and homogeneity of variances.
License: GPL-2
Encoding: UTF-8
Depends: R (>= 3.3.0)
Imports: stats, utils, tidyr (>= 1.0.0), purrr, broom (>= 0.7.4), rlang
        (>= 0.3.1), tibble (>= 2.1.3), dplyr (>= 0.7.1), magrittr,
        corrplot, tidyselect (>= 1.2.0), car, generics (>= 0.0.2)
Suggests: knitr, rmarkdown, ggpubr, graphics, emmeans, coin, boot,
        testthat, spelling
URL: https://rpkgs.datanovia.com/rstatix/
BugReports: https://github.com/kassambara/rstatix/issues
RoxygenNote: 7.3.3
Collate: 'utilities.R' 'add_cld.R' 'add_significance.R'
        'adjust_pvalue.R' 'factorial_design.R'
        'utilities_two_sample_test.R' 'anova_summary.R' 'anova_test.R'
        'as_cor_mat.R' 'binom_test.R' 'box_m.R' 'chisq_test.R'
        'cochran_qtest.R' 'cohens_d.R' 't_test.R' 'dunn_test.R'
        'conover_test.R' 'cor_as_symbols.R' 'replace_triangle.R'
        'pull_triangle.R' 'cor_mark_significant.R' 'cor_mat.R'
        'cor_plot.R' 'cor_reorder.R' 'cor_reshape.R' 'cor_select.R'
        'cor_test.R' 'counts_to_cases.R' 'cramer_v.R' 'df.R' 'doo.R'
        'emmeans_test.R' 'dunnett_test.R' 'eta_squared.R' 'factors.R'
        'fisher_test.R' 'fligner_test.R' 'freq_table.R'
        'friedman_test.R' 'friedman_conover_test.R'
        'friedman_effsize.R' 'friedman_nemenyi_test.R'
        'games_howell_test.R' 'get_comparisons.R' 'get_manova_table.R'
        'get_mode.R' 'get_pvalue_position.R' 'get_summary_stats.R'
        'get_test_label.R' 'kruskal_effesize.R' 'kruskal_test.R'
        'ks_test.R' 'levene_test.R' 'mahalanobis_distance.R'
        'make_clean_names.R' 'mcnemar_test.R' 'multinom_test.R'
        'outliers.R' 'p_value.R' 'prop_test.R' 'prop_trend_test.R'
        'reexports.R' 'remove_ns.R' 'rstatix-programming.R'
        'sample_n_by.R' 'shapiro_test.R' 'sign_test.R' 'tukey_hsd.R'
        'utils-manova.R' 'utils-pipe.R' 'welch_anova_test.R'
        'wilcox_effsize.R' 'wilcox_test.R'
Language: en-US
NeedsCompilation: no
Packaged: 2026-07-03 15:05:16 UTC; kassambara
Author: Alboukadel Kassambara [aut, cre]
Maintainer: Alboukadel Kassambara <alboukadel.kassambara@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-03 16:10:02 UTC
Built: R 4.4.3; ; 2026-07-03 18:34:44 UTC; unix

Package: rstatix
Type: Package
Title: Pipe-Friendly Framework for Basic Statistical Tests
Version: 0.7.3
Authors@R: c(
    person("Alboukadel", "Kassambara", role = c("aut", "cre"), email = "alboukadel.kassambara@gmail.com"))
Description: Provides a simple and intuitive pipe-friendly framework, coherent with the 'tidyverse' design philosophy, 
    for performing basic statistical tests, including t-test, Wilcoxon test, ANOVA, Kruskal-Wallis and correlation analyses. 
    The output of each test is automatically transformed into a tidy data frame to facilitate visualization. 
    Additional functions are available for reshaping, reordering, manipulating and visualizing correlation matrix.  
    Functions are also included to facilitate the analysis of factorial experiments, including purely 'within-Ss' designs 
    (repeated measures), purely 'between-Ss' designs, and mixed 'within-and-between-Ss' designs. 
    It's also possible to compute several effect size metrics, including "eta squared" for ANOVA, "Cohen's d" for t-test and 
    'Cramer V' for the association between categorical variables. 
    The package contains helper functions for identifying univariate and multivariate outliers, assessing normality and homogeneity of variances.
License: GPL-2
Encoding: UTF-8
Depends: R (>= 3.3.0)
Imports: stats, utils, tidyr (>= 1.0.0), purrr, broom (>= 0.7.4), rlang
        (>= 0.3.1), tibble (>= 2.1.3), dplyr (>= 0.7.1), magrittr,
        corrplot, tidyselect (>= 1.2.0), car, generics (>= 0.0.2)
Suggests: knitr, rmarkdown, ggpubr, graphics, emmeans, coin, boot,
        testthat, spelling
URL: https://rpkgs.datanovia.com/rstatix/
BugReports: https://github.com/kassambara/rstatix/issues
RoxygenNote: 7.3.3
Collate: 'utilities.R' 'add_significance.R' 'adjust_pvalue.R'
        'factorial_design.R' 'utilities_two_sample_test.R'
        'anova_summary.R' 'anova_test.R' 'as_cor_mat.R' 'binom_test.R'
        'box_m.R' 'chisq_test.R' 'cochran_qtest.R' 'cohens_d.R'
        'cor_as_symbols.R' 'replace_triangle.R' 'pull_triangle.R'
        'cor_mark_significant.R' 'cor_mat.R' 'cor_plot.R'
        'cor_reorder.R' 'cor_reshape.R' 'cor_select.R' 'cor_test.R'
        'counts_to_cases.R' 'cramer_v.R' 'df.R' 'doo.R' 't_test.R'
        'dunn_test.R' 'emmeans_test.R' 'eta_squared.R' 'factors.R'
        'fisher_test.R' 'freq_table.R' 'friedman_test.R'
        'friedman_effsize.R' 'games_howell_test.R' 'get_comparisons.R'
        'get_manova_table.R' 'get_mode.R' 'get_pvalue_position.R'
        'get_summary_stats.R' 'get_test_label.R' 'kruskal_effesize.R'
        'kruskal_test.R' 'levene_test.R' 'mahalanobis_distance.R'
        'make_clean_names.R' 'mcnemar_test.R' 'multinom_test.R'
        'outliers.R' 'p_value.R' 'prop_test.R' 'prop_trend_test.R'
        'reexports.R' 'remove_ns.R' 'sample_n_by.R' 'shapiro_test.R'
        'sign_test.R' 'tukey_hsd.R' 'utils-manova.R' 'utils-pipe.R'
        'welch_anova_test.R' 'wilcox_effsize.R' 'wilcox_test.R'
Language: en-US
NeedsCompilation: no
Packaged: 2025-10-18 13:21:05 UTC; kassambara
Author: Alboukadel Kassambara [aut, cre]
Maintainer: Alboukadel Kassambara <alboukadel.kassambara@gmail.com>
Repository: CRAN
Date/Publication: 2025-10-18 13:50:02 UTC
Built: R 4.6.0; ; 2026-04-25 14:06:31 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,anova_test)
S3method(print,anova_test)
S3method(tukey_hsd,data.frame)
S3method(tukey_hsd,default)
S3method(tukey_hsd,lm)
export("%>%")
export(Anova)
export(Manova)
export(add_cld)
export(add_significance)
export(add_x_position)
export(add_xy_position)
export(add_y_position)
export(adjust_pvalue)
export(anova_summary)
export(anova_test)
export(as_cor_mat)
export(augment)
export(binom_test)
export(box_m)
export(chisq_descriptives)
export(chisq_test)
export(cochran_qtest)
export(cohens_d)
export(conover_test)
export(convert_as_factor)
export(cor_as_symbols)
export(cor_gather)
export(cor_get_pval)
export(cor_mark_significant)
export(cor_mat)
export(cor_plot)
export(cor_pmat)
export(cor_reorder)
export(cor_select)
export(cor_spread)
export(cor_test)
export(counts_to_cases)
export(cramer_v)
export(create_test_label)
export(desc)
export(df_arrange)
export(df_get_var_names)
export(df_group_by)
export(df_label_both)
export(df_label_value)
export(df_nest_by)
export(df_select)
export(df_split_by)
export(df_unite)
export(df_unite_factors)
export(doo)
export(drop_na)
export(dunn_test)
export(dunnett_test)
export(emmeans_test)
export(eta_squared)
export(expected_freq)
export(factorial_design)
export(filter)
export(fisher_test)
export(fligner_test)
export(freq_table)
export(friedman_conover_test)
export(friedman_effsize)
export(friedman_nemenyi_test)
export(friedman_test)
export(games_howell_test)
export(gather)
export(get_anova_table)
export(get_comparisons)
export(get_description)
export(get_emmeans)
export(get_mode)
export(get_n)
export(get_pwc_label)
export(get_summary_stats)
export(get_test_label)
export(get_y_position)
export(group_by)
export(identify_outliers)
export(is_extreme)
export(is_outlier)
export(kruskal_effsize)
export(kruskal_test)
export(ks_test)
export(levene_test)
export(mahalanobis_distance)
export(make_clean_names)
export(mcnemar_test)
export(mshapiro_test)
export(multinom_test)
export(mutate)
export(observed_freq)
export(p_adj_names)
export(p_detect)
export(p_format)
export(p_mark_significant)
export(p_names)
export(p_round)
export(pairwise_binom_test)
export(pairwise_binom_test_against_p)
export(pairwise_chisq_gof_test)
export(pairwise_chisq_test_against_p)
export(pairwise_fisher_test)
export(pairwise_mcnemar_test)
export(pairwise_prop_test)
export(pairwise_sign_test)
export(pairwise_t_test)
export(pairwise_wilcox_test)
export(partial_eta_squared)
export(pearson_residuals)
export(prop_test)
export(prop_trend_test)
export(pull_lower_triangle)
export(pull_triangle)
export(pull_upper_triangle)
export(remove_ns)
export(reorder_levels)
export(replace_lower_triangle)
export(replace_triangle)
export(replace_upper_triangle)
export(row_wise_fisher_test)
export(row_wise_prop_test)
export(sample_n_by)
export(select)
export(set_ref_level)
export(shapiro_test)
export(sign_test)
export(spread)
export(std_residuals)
export(t_test)
export(tibble)
export(tidy)
export(tukey_hsd)
export(welch_anova_test)
export(wilcox_effsize)
export(wilcox_test)
importFrom(broom,tidy)
importFrom(car,Anova)
importFrom(car,Manova)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,as_data_frame)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,do)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,is_grouped_df)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,mutate_at)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,tibble)
importFrom(dplyr,ungroup)
importFrom(generics,augment)
importFrom(generics,tidy)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(magrittr,extract)
importFrom(magrittr,set_colnames)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(rlang,"!!!")
importFrom(rlang,"!!")
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,quo_name)
importFrom(rlang,quos)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(stats,IQR)
importFrom(stats,TukeyHSD)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,cor.test)
importFrom(stats,cov)
importFrom(stats,ks.test)
importFrom(stats,median)
importFrom(stats,pbinom)
importFrom(stats,pchisq)
importFrom(stats,qbinom)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,shapiro.test)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(tibble,add_column)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,gather)
importFrom(tidyr,nest)
importFrom(tidyr,separate)
importFrom(tidyr,spread)

<!-- README.md is generated from README.Rmd. Please edit that file -->

[![R build
status](https://github.com/kassambara/rstatix/workflows/R-CMD-check/badge.svg)](https://github.com/kassambara/rstatix/actions)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/rstatix)](https://cran.r-project.org/package=rstatix)
[![CRAN
Checks](https://cranchecks.info/badges/summary/rstatix)](https://cran.r-project.org/web/checks/check_results_rstatix.html)
[![Downloads](https://cranlogs.r-pkg.org/badges/rstatix)](https://cran.r-project.org/package=rstatix)
[![Total
Downloads](https://cranlogs.r-pkg.org/badges/grand-total/rstatix?color=orange)](https://cran.r-project.org/package=rstatix)

# rstatix

Provides a simple and intuitive pipe-friendly framework, coherent with
the ‘tidyverse’ design philosophy, for performing basic statistical
tests, including t-test, Wilcoxon test, ANOVA, Kruskal-Wallis and
correlation analyses.

The output of each test is automatically transformed into a tidy data
frame to facilitate visualization.

Additional functions are available for reshaping, reordering,
manipulating and visualizing correlation matrix. Functions are also
included to facilitate the analysis of factorial experiments, including
purely ‘within-Ss’ designs (repeated measures), purely ‘between-Ss’
designs, and mixed ‘within-and-between-Ss’ designs.

It’s also possible to compute several effect size metrics, including
“eta squared” for ANOVA, “Cohen’s d” for t-test and “Cramer’s V” for the
association between categorical variables. The package contains helper
functions for identifying univariate and multivariate outliers,
assessing normality and homogeneity of variances.

## Key functions

### Descriptive statistics

-   `get_summary_stats()`: Compute summary statistics for one or
    multiple numeric variables. Can handle grouped data.
-   `freq_table()`: Compute frequency table of categorical variables.
-   `get_mode()`: Compute the mode of a vector, that is the most
    frequent values.
-   `identify_outliers()`: Detect univariate outliers using boxplot
    methods.
-   `mahalanobis_distance()`: Compute Mahalanobis Distance and Flag
    Multivariate Outliers.
-   `shapiro_test()` and `mshapiro_test()`: Univariate and multivariate
    Shapiro-Wilk normality test.

### Comparing means

-   `t_test()`: perform one-sample, two-sample and pairwise t-tests
-   `wilcox_test()`: perform one-sample, two-sample and pairwise
    Wilcoxon tests
-   `sign_test()`: perform sign test to determine whether there is a
    median difference between paired or matched observations.
-   `anova_test()`: an easy-to-use wrapper around `car::Anova()` to
    perform different types of ANOVA tests, including **independent
    measures ANOVA**, **repeated measures ANOVA** and **mixed ANOVA**.
-   `get_anova_test_table()`: extract ANOVA table from `anova_test()`
    results. Can apply sphericity correction automatically in the case
    of within-subject (repeated measures) designs.
-   `welch_anova_test()`: Welch one-Way ANOVA test. A pipe-friendly
    wrapper around the base function `stats::oneway.test()`. This is is
    an alternative to the standard one-way ANOVA in the situation where
    the homogeneity of variance assumption is violated.
-   `kruskal_test()`: perform kruskal-wallis rank sum test
-   `friedman_test()`: Provides a pipe-friendly framework to perform a
    Friedman rank sum test, which is the non-parametric alternative to
    the one-way repeated measures ANOVA test.
-   `get_comparisons()`: Create a list of possible pairwise comparisons
    between groups.
-   `get_pvalue_position()`: autocompute p-value positions for plotting
    significance using ggplot2.

### Facilitating ANOVA computation in R

-   `factorial_design()`: build factorial design for easily computing
    ANOVA using the `car::Anova()` function. This might be very useful
    for repeated measures ANOVA, which is hard to set up with the `car`
    package.
-   `anova_summary()`: Create beautiful summary tables of ANOVA test
    results obtained from either `car::Anova()` or `stats::aov()`. The
    results include ANOVA table, generalized effect size and some
    assumption checks, such as Mauchly’s test for sphericity in the case
    of repeated measures ANOVA.

### Post-hoc analyses

-   `tukey_hsd()`: performs tukey post-hoc tests. Can handle different
    inputs formats: aov, lm, formula.
-   `dunn_test()`: compute multiple pairwise comparisons following
    Kruskal-Wallis test.
-   `games_howell_test()`: Performs Games-Howell test, which is used to
    compare all possible combinations of group differences when the
    assumption of homogeneity of variances is violated.
-   `emmeans_test()`: pipe-friendly wrapper arround `emmeans` function
    to perform pairwise comparisons of estimated marginal means. Useful
    for post-hoc analyses following up ANOVA/ANCOVA tests.

### Comparing proportions

-   `prop_test()`, `pairwise_prop_test()` and `row_wise_prop_test()`.
    Performs one-sample and two-samples z-test of proportions. Wrappers
    around the R base function `prop.test()` but have the advantage of
    performing pairwise and row-wise z-test of two proportions, the
    post-hoc tests following a significant chi-square test of
    homogeneity for 2xc and rx2 contingency tables.
-   `fisher_test()`, `pairwise_fisher_test()` and
    `row_wise_fisher_test()`: Fisher’s exact test for count data.
    Wrappers around the R base function `fisher.test()` but have the
    advantage of performing pairwise and row-wise fisher tests, the
    post-hoc tests following a significant chi-square test of
    homogeneity for 2xc and rx2 contingency tables.
-   `chisq_test()`, `pairwise_chisq_gof_test()`,
    `pairwise_chisq_test_against_p()`: Performs chi-squared tests,
    including goodness-of-fit, homogeneity and independence tests.
-   `binom_test()`, `pairwise_binom_test()`,
    `pairwise_binom_test_against_p()`: Performs exact binomial test and
    pairwise comparisons following a significant exact multinomial test.
    Alternative to the chi-square test of goodness-of-fit-test when the
    sample.
-   `multinom_test()`: performs an exact multinomial test. Alternative
    to the chi-square test of goodness-of-fit-test when the sample size
    is small.
-   `mcnemar_test()`: performs McNemar chi-squared test to compare
    paired proportions. Provides pairwise comparisons between multiple
    groups.
-   `cochran_qtest()`: extension of the McNemar Chi-squared test for
    comparing more than two paired proportions.
-   `prop_trend_test()`: Performs chi-squared test for trend in
    proportion. This test is also known as Cochran-Armitage trend test.

### Comparing variances

-   `levene_test()`: Pipe-friendly framework to easily compute Levene’s
    test for homogeneity of variance across groups. Handles grouped
    data.
-   `box_m()`: Box’s M-test for homogeneity of covariance matrices

### Effect Size

-   `cohens_d()`: Compute cohen’s d measure of effect size for t-tests.
-   `wilcox_effsize()`: Compute Wilcoxon effect size (r).
-   `eta_squared()` and `partial_eta_squared()`: Compute effect size for
    ANOVA.
-   `kruskal_effsize()`: Compute the effect size for Kruskal-Wallis test
    as the eta squared based on the H-statistic.
-   `friedman_effsize()`: Compute the effect size of Friedman test using
    the Kendall’s W value.
-   `cramer_v()`: Compute Cramer’s V, which measures the strength of the
    association between categorical variables.

### Correlation analysis

**Computing correlation**:

-   `cor_test()`: correlation test between two or more variables using
    Pearson, Spearman or Kendall methods.
-   `cor_mat()`: compute correlation matrix with p-values. Returns a
    data frame containing the matrix of the correlation coefficients.
    The output has an attribute named “pvalue”, which contains the
    matrix of the correlation test p-values.
-   `cor_get_pval()`: extract a correlation matrix p-values from an
    object of class `cor_mat()`.
-   `cor_pmat()`: compute the correlation matrix, but returns only the
    p-values of the correlation tests.
-   `as_cor_mat()`: convert a `cor_test` object into a correlation
    matrix format.

**Reshaping correlation matrix**:

-   `cor_reorder()`: reorder correlation matrix, according to the
    coefficients, using the hierarchical clustering method.
-   `cor_gather()`: takes a correlation matrix and collapses (or melt)
    it into long format data frame (paired list)
-   `cor_spread()`: spread a long correlation data frame into wide
    format (correlation matrix).

**Subsetting correlation matrix**:

-   `cor_select()`: subset a correlation matrix by selecting variables
    of interest.
-   `pull_triangle()`, `pull_upper_triangle()`, `pull_lower_triangle()`:
    pull upper and lower triangular parts of a (correlation) matrix.
-   `replace_triangle()`, `replace_upper_triangle()`,
    `replace_lower_triangle()`: replace upper and lower triangular parts
    of a (correlation) matrix.

**Visualizing correlation matrix**:

-   `cor_as_symbols()`: replaces the correlation coefficients, in a
    matrix, by symbols according to the value.
-   `cor_plot()`: visualize correlation matrix using base plot.
-   `cor_mark_significant()`: add significance levels to a correlation
    matrix.

### Adjusting p-values, formatting and adding significance symbols

-   `adjust_pvalue()`: add an adjusted p-values column to a data frame
    containing statistical test p-values
-   `add_significance()`: add a column containing the p-value
    significance level
-   `p_round(), p_format(), p_mark_significant()`: rounding and
    formatting p-values

### Extract information from statistical tests

Extract information from statistical test results. Useful for labelling
plots with test outputs.

-   `get_pwc_label()`: Extract label from pairwise comparisons.
-   `get_test_label()`: Extract label from statistical tests.
-   `create_test_label()`: Create labels from user specified test
    results.

### Data manipulation helper functions

These functions are internally used in the `rstatix` and in the `ggpubr`
R package to make it easy to program with tidyverse packages using non
standard evaluation.

-   `df_select()`, `df_arrange()`, `df_group_by()`: wrappers arround
    dplyr functions for supporting standard and non standard
    evaluations.
-   `df_nest_by()`: Nest a tibble data frame using grouping
    specification. Supports standard and non standard evaluations.
-   `df_split_by()`: Split a data frame by groups into subsets or data
    panel. Very similar to the function `df_nest_by()`. The only
    difference is that, it adds labels to each data subset. Labels are
    the combination of the grouping variable levels.
-   `df_unite()`: Unite multiple columns into one.
-   `df_unite_factors()`: Unite factor columns. First, order factors
    levels then merge them into one column. The output column is a
    factor.
-   `df_label_both()`, `df_label_value()`: functions to label data
    frames rows by by one or multiple grouping variables.
-   `df_get_var_names()`: Returns user specified variable names.
    Supports standard and non standard evaluation.

### Others

-   `doo()`: alternative to dplyr::do for doing anything. Technically it
    uses `nest(...) %>% mutate(...) %>% map(...)` to apply arbitrary
    computation to a grouped data frame.
-   `sample_n_by()`: sample n rows by group from a table
-   `convert_as_factor(), set_ref_level(), reorder_levels()`: Provides
    pipe-friendly functions to convert simultaneously multiple variables
    into a factor variable.
-   `make_clean_names()`: Pipe-friendly function to make syntactically
    valid column names (for input data frame) or names (for input
    vector).
-   `counts_to_cases()`: converts a contingency table or a data frame of
    counts into a data frame of individual observations.

## Installation and loading

-   Install the latest developmental version from
    [GitHub](https://github.com/kassambara/rstatix) as follow:

``` r
if(!require(devtools)) install.packages("devtools")
devtools::install_github("kassambara/rstatix")
```

-   Or install from [CRAN](https://cran.r-project.org/package=ggpubr) as
    follow:

``` r
install.packages("rstatix")
```

-   Loading packages

``` r
library(rstatix)  
library(ggpubr)  # For easy data-visualization
```

## Descriptive statistics

``` r
# Summary statistics of some selected variables
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
iris %>% 
  get_summary_stats(Sepal.Length, Sepal.Width, type = "common")
#> # A tibble: 2 × 10
#>   variable         n   min   max median   iqr  mean    sd    se    ci
#>   <fct>        <dbl> <dbl> <dbl>  <dbl> <dbl> <dbl> <dbl> <dbl> <dbl>
#> 1 Sepal.Length   150   4.3   7.9    5.8   1.3  5.84 0.828 0.068 0.134
#> 2 Sepal.Width    150   2     4.4    3     0.5  3.06 0.436 0.036 0.07

# Whole data frame
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
iris %>% get_summary_stats(type = "common")
#> # A tibble: 4 × 10
#>   variable         n   min   max median   iqr  mean    sd    se    ci
#>   <fct>        <dbl> <dbl> <dbl>  <dbl> <dbl> <dbl> <dbl> <dbl> <dbl>
#> 1 Sepal.Length   150   4.3   7.9   5.8    1.3  5.84 0.828 0.068 0.134
#> 2 Sepal.Width    150   2     4.4   3      0.5  3.06 0.436 0.036 0.07 
#> 3 Petal.Length   150   1     6.9   4.35   3.5  3.76 1.76  0.144 0.285
#> 4 Petal.Width    150   0.1   2.5   1.3    1.5  1.20 0.762 0.062 0.123


# Grouped data
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
iris %>%
  group_by(Species) %>% 
  get_summary_stats(Sepal.Length, type = "mean_sd")
#> # A tibble: 3 × 5
#>   Species    variable         n  mean    sd
#>   <fct>      <fct>        <dbl> <dbl> <dbl>
#> 1 setosa     Sepal.Length    50  5.01 0.352
#> 2 versicolor Sepal.Length    50  5.94 0.516
#> 3 virginica  Sepal.Length    50  6.59 0.636
```

## Comparing two means

To compare the means of two groups, you can use either the function
`t_test()` (parametric) or `wilcox_test()` (non-parametric). In the
following example the t-test will be illustrated.

### Data

Preparing the demo data set:

``` r
df <- ToothGrowth
df$dose <- as.factor(df$dose)
head(df)
#>    len supp dose
#> 1  4.2   VC  0.5
#> 2 11.5   VC  0.5
#> 3  7.3   VC  0.5
#> 4  5.8   VC  0.5
#> 5  6.4   VC  0.5
#> 6 10.0   VC  0.5
```

### One-sample test

The one-sample test is used to compare the mean of one sample to a known
standard (or theoretical / hypothetical) mean (`mu`).

``` r
df %>% t_test(len ~ 1, mu = 0)
#> # A tibble: 1 × 7
#>   .y.   group1 group2         n statistic    df        p
#> * <chr> <chr>  <chr>      <int>     <dbl> <dbl>    <dbl>
#> 1 len   1      null model    60      19.1    59 6.94e-27
# One-sample test of each dose level
df %>% 
  group_by(dose) %>%
  t_test(len ~ 1, mu = 0)
#> # A tibble: 3 × 8
#>   dose  .y.   group1 group2         n statistic    df        p
#> * <fct> <chr> <chr>  <chr>      <int>     <dbl> <dbl>    <dbl>
#> 1 0.5   len   1      null model    20      10.5    19 2.24e- 9
#> 2 1     len   1      null model    20      20.0    19 3.22e-14
#> 3 2     len   1      null model    20      30.9    19 1.03e-17
```

### Compare two independent groups

-   Create a simple box plot with p-values:

``` r
# T-test
stat.test <- df %>% 
  t_test(len ~ supp, paired = FALSE) 
stat.test
#> # A tibble: 1 × 8
#>   .y.   group1 group2    n1    n2 statistic    df      p
#> * <chr> <chr>  <chr>  <int> <int>     <dbl> <dbl>  <dbl>
#> 1 len   OJ     VC        30    30      1.92  55.3 0.0606

# Create a box plot
p <- ggboxplot(
  df, x = "supp", y = "len", 
  color = "supp", palette = "jco", ylim = c(0,40)
  )
# Add the p-value manually
p + stat_pvalue_manual(stat.test, label = "p", y.position = 35)
```

![](tools/README-unpaired-two-sample-t-test-1.png)

-   Customize labels using [glue
    expression](https://github.com/tidyverse/glue):

``` r
p +stat_pvalue_manual(stat.test, label = "T-test, p = {p}", 
                      y.position = 36)
```

![](tools/README-custoize-p-value-labels-1.png)

-   Grouped data: compare supp levels after grouping the data by “dose”

``` r
# Statistical test
stat.test <- df %>%
  group_by(dose) %>%
  t_test(len ~ supp) %>%
  adjust_pvalue() %>%
  add_significance("p.adj")
stat.test
#> # A tibble: 3 × 11
#>   dose  .y.   group1 group2    n1    n2 statistic    df       p   p.adj
#>   <fct> <chr> <chr>  <chr>  <int> <int>     <dbl> <dbl>   <dbl>   <dbl>
#> 1 0.5   len   OJ     VC        10    10    3.17    15.0 0.00636 0.0127 
#> 2 1     len   OJ     VC        10    10    4.03    15.4 0.00104 0.00312
#> 3 2     len   OJ     VC        10    10   -0.0461  14.0 0.964   0.964  
#> # ℹ 1 more variable: p.adj.signif <chr>

# Visualization
ggboxplot(
  df, x = "supp", y = "len",
  color = "supp", palette = "jco", facet.by = "dose",
  ylim = c(0, 40)
  ) +
  stat_pvalue_manual(stat.test, label = "p.adj", y.position = 35)
```

![](tools/README-grouped-two-sample-t-test-1.png)

### Compare paired samples

``` r
# T-test
stat.test <- df %>% 
  t_test(len ~ supp, paired = TRUE) 
stat.test
#> # A tibble: 1 × 8
#>   .y.   group1 group2    n1    n2 statistic    df       p
#> * <chr> <chr>  <chr>  <int> <int>     <dbl> <dbl>   <dbl>
#> 1 len   OJ     VC        30    30      3.30    29 0.00255

# Box plot
p <- ggpaired(
  df, x = "supp", y = "len", color = "supp", palette = "jco", 
  line.color = "gray", line.size = 0.4, ylim = c(0, 40)
  )
p + stat_pvalue_manual(stat.test, label = "p", y.position = 36)
```

![](tools/README-paired-t-test-1.png)

### Multiple pairwise comparisons

-   Pairwise comparisons: if the grouping variable contains more than
    two categories, a pairwise comparison is automatically performed.

``` r
# Pairwise t-test
pairwise.test <- df %>% t_test(len ~ dose)
pairwise.test
#> # A tibble: 3 × 10
#>   .y.   group1 group2    n1    n2 statistic    df        p    p.adj p.adj.signif
#> * <chr> <chr>  <chr>  <int> <int>     <dbl> <dbl>    <dbl>    <dbl> <chr>       
#> 1 len   0.5    1         20    20     -6.48  38.0 1.27e- 7 2.54e- 7 ****        
#> 2 len   0.5    2         20    20    -11.8   36.9 4.40e-14 1.32e-13 ****        
#> 3 len   1      2         20    20     -4.90  37.1 1.91e- 5 1.91e- 5 ****
# Box plot
ggboxplot(df, x = "dose", y = "len")+
  stat_pvalue_manual(
    pairwise.test, label = "p.adj", 
    y.position = c(29, 35, 39)
    )
```

![](tools/README-pairwise-comparisons-1.png)

-   Multiple pairwise comparisons against reference group: each level is
    compared to the ref group

``` r
# Comparison against reference group
#::::::::::::::::::::::::::::::::::::::::
# T-test: each level is compared to the ref group
stat.test <- df %>% t_test(len ~ dose, ref.group = "0.5")
stat.test
#> # A tibble: 2 × 10
#>   .y.   group1 group2    n1    n2 statistic    df        p    p.adj p.adj.signif
#> * <chr> <chr>  <chr>  <int> <int>     <dbl> <dbl>    <dbl>    <dbl> <chr>       
#> 1 len   0.5    1         20    20     -6.48  38.0 1.27e- 7 1.27e- 7 ****        
#> 2 len   0.5    2         20    20    -11.8   36.9 4.40e-14 8.80e-14 ****
# Box plot
ggboxplot(df, x = "dose", y = "len", ylim = c(0, 40)) +
  stat_pvalue_manual(
    stat.test, label = "p.adj.signif", 
    y.position = c(29, 35)
    )
```

![](tools/README-comaprison-against-reference-group-1.png)

``` r
# Remove bracket
ggboxplot(df, x = "dose", y = "len", ylim = c(0, 40)) +
  stat_pvalue_manual(
    stat.test, label = "p.adj.signif", 
    y.position = c(29, 35),
    remove.bracket = TRUE
    )
```

![](tools/README-comaprison-against-reference-group-2.png)

-   Multiple pairwise comparisons against all (base-mean): Comparison of
    each group against base-mean.

``` r
# T-test
stat.test <- df %>% t_test(len ~ dose, ref.group = "all")
stat.test
#> # A tibble: 3 × 10
#>   .y.   group1 group2    n1    n2 statistic    df         p   p.adj p.adj.signif
#> * <chr> <chr>  <chr>  <int> <int>     <dbl> <dbl>     <dbl>   <dbl> <chr>       
#> 1 len   all    0.5       60    20     5.82   56.4   2.90e-7 8.70e-7 ****        
#> 2 len   all    1         60    20    -0.660  57.5   5.12e-1 5.12e-1 ns          
#> 3 len   all    2         60    20    -5.61   66.5   4.25e-7 8.70e-7 ****
# Box plot with horizontal mean line
ggboxplot(df, x = "dose", y = "len") +
  stat_pvalue_manual(
    stat.test, label = "p.adj.signif", 
    y.position = 35,
    remove.bracket = TRUE
    ) +
  geom_hline(yintercept = mean(df$len), linetype = 2)
```

![](tools/README-comparison-against-base-mean-1.png)

## ANOVA test

``` r
# One-way ANOVA test
#:::::::::::::::::::::::::::::::::::::::::
df %>% anova_test(len ~ dose)
#> ANOVA Table (type II tests)
#> 
#>   Effect DFn DFd      F        p p<.05   ges
#> 1   dose   2  57 67.416 9.53e-16     * 0.703

# Two-way ANOVA test
#:::::::::::::::::::::::::::::::::::::::::
df %>% anova_test(len ~ supp*dose)
#> ANOVA Table (type II tests)
#> 
#>      Effect DFn DFd      F        p p<.05   ges
#> 1      supp   1  54 15.572 2.31e-04     * 0.224
#> 2      dose   2  54 92.000 4.05e-18     * 0.773
#> 3 supp:dose   2  54  4.107 2.20e-02     * 0.132

# Two-way repeated measures ANOVA
#:::::::::::::::::::::::::::::::::::::::::
df$id <- rep(1:10, 6) # Add individuals id
# Use formula
# df %>% anova_test(len ~ supp*dose + Error(id/(supp*dose)))
# or use character vector
df %>% anova_test(dv = len, wid = id, within = c(supp, dose))
#> ANOVA Table (type III tests)
#> 
#> $ANOVA
#>      Effect DFn DFd       F        p p<.05   ges
#> 1      supp   1   9  34.866 2.28e-04     * 0.224
#> 2      dose   2  18 106.470 1.06e-10     * 0.773
#> 3 supp:dose   2  18   2.534 1.07e-01       0.132
#> 
#> $`Mauchly's Test for Sphericity`
#>      Effect     W     p p<.05
#> 1      dose 0.807 0.425      
#> 2 supp:dose 0.934 0.761      
#> 
#> $`Sphericity Corrections`
#>      Effect   GGe      DF[GG]    p[GG] p[GG]<.05   HFe      DF[HF]    p[HF]
#> 1      dose 0.838 1.68, 15.09 2.79e-09         * 1.008 2.02, 18.15 1.06e-10
#> 2 supp:dose 0.938 1.88, 16.88 1.12e-01           1.176 2.35, 21.17 1.07e-01
#>   p[HF]<.05
#> 1         *
#> 2

# Use model as arguments
#:::::::::::::::::::::::::::::::::::::::::
.my.model <- lm(yield ~ block + N*P*K, npk)
anova_test(.my.model)
#> ANOVA Table (type II tests)
#> 
#>   Effect DFn DFd      F     p p<.05   ges
#> 1  block   4  12  4.959 0.014     * 0.623
#> 2      N   1  12 12.259 0.004     * 0.505
#> 3      P   1  12  0.544 0.475       0.043
#> 4      K   1  12  6.166 0.029     * 0.339
#> 5    N:P   1  12  1.378 0.263       0.103
#> 6    N:K   1  12  2.146 0.169       0.152
#> 7    P:K   1  12  0.031 0.863       0.003
#> 8  N:P:K   0  12     NA    NA  <NA>    NA
```

## Correlation tests

``` r
# Data preparation
mydata <- mtcars %>% 
  select(mpg, disp, hp, drat, wt, qsec)
head(mydata, 3)
#>                mpg disp  hp drat    wt  qsec
#> Mazda RX4     21.0  160 110 3.90 2.620 16.46
#> Mazda RX4 Wag 21.0  160 110 3.90 2.875 17.02
#> Datsun 710    22.8  108  93 3.85 2.320 18.61

# Correlation test between two variables
mydata %>% cor_test(wt, mpg, method = "pearson")
#> # A tibble: 1 × 8
#>   var1  var2    cor statistic        p conf.low conf.high method 
#>   <chr> <chr> <dbl>     <dbl>    <dbl>    <dbl>     <dbl> <chr>  
#> 1 wt    mpg   -0.87     -9.56 1.29e-10   -0.934    -0.744 Pearson

# Correlation of one variable against all
mydata %>% cor_test(mpg, method = "pearson")
#> # A tibble: 5 × 8
#>   var1  var2    cor statistic        p conf.low conf.high method 
#>   <chr> <chr> <dbl>     <dbl>    <dbl>    <dbl>     <dbl> <chr>  
#> 1 mpg   disp  -0.85     -8.75 9.38e-10  -0.923     -0.708 Pearson
#> 2 mpg   hp    -0.78     -6.74 1.79e- 7  -0.885     -0.586 Pearson
#> 3 mpg   drat   0.68      5.10 1.78e- 5   0.436      0.832 Pearson
#> 4 mpg   wt    -0.87     -9.56 1.29e-10  -0.934     -0.744 Pearson
#> 5 mpg   qsec   0.42      2.53 1.71e- 2   0.0820     0.670 Pearson

# Pairwise correlation test between all variables
mydata %>% cor_test(method = "pearson")
#> # A tibble: 36 × 8
#>    var1  var2    cor statistic        p conf.low conf.high method 
#>    <chr> <chr> <dbl>     <dbl>    <dbl>    <dbl>     <dbl> <chr>  
#>  1 mpg   mpg    1       Inf    0          1          1     Pearson
#>  2 mpg   disp  -0.85     -8.75 9.38e-10  -0.923     -0.708 Pearson
#>  3 mpg   hp    -0.78     -6.74 1.79e- 7  -0.885     -0.586 Pearson
#>  4 mpg   drat   0.68      5.10 1.78e- 5   0.436      0.832 Pearson
#>  5 mpg   wt    -0.87     -9.56 1.29e-10  -0.934     -0.744 Pearson
#>  6 mpg   qsec   0.42      2.53 1.71e- 2   0.0820     0.670 Pearson
#>  7 disp  mpg   -0.85     -8.75 9.38e-10  -0.923     -0.708 Pearson
#>  8 disp  disp   1       Inf    0          1          1     Pearson
#>  9 disp  hp     0.79      7.08 7.14e- 8   0.611      0.893 Pearson
#> 10 disp  drat  -0.71     -5.53 5.28e- 6  -0.849     -0.481 Pearson
#> # ℹ 26 more rows
```

## Correlation matrix

``` r
# Compute correlation matrix
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
cor.mat <- mydata %>% cor_mat()
cor.mat
#> # A tibble: 6 × 7
#>   rowname   mpg  disp    hp   drat    wt   qsec
#> * <chr>   <dbl> <dbl> <dbl>  <dbl> <dbl>  <dbl>
#> 1 mpg      1    -0.85 -0.78  0.68  -0.87  0.42 
#> 2 disp    -0.85  1     0.79 -0.71   0.89 -0.43 
#> 3 hp      -0.78  0.79  1    -0.45   0.66 -0.71 
#> 4 drat     0.68 -0.71 -0.45  1     -0.71  0.091
#> 5 wt      -0.87  0.89  0.66 -0.71   1    -0.17 
#> 6 qsec     0.42 -0.43 -0.71  0.091 -0.17  1

# Show the significance levels
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
cor.mat %>% cor_get_pval()
#> # A tibble: 6 × 7
#>   rowname      mpg     disp           hp      drat       wt       qsec
#>   <chr>      <dbl>    <dbl>        <dbl>     <dbl>    <dbl>      <dbl>
#> 1 mpg     0        9.38e-10 0.000000179  1.78e-  5 1.29e-10 0.0171    
#> 2 disp    9.38e-10 0        0.0000000714 5.28e-  6 1.22e-11 0.0131    
#> 3 hp      1.79e- 7 7.14e- 8 0            9.99e-  3 4.15e- 5 0.00000577
#> 4 drat    1.78e- 5 5.28e- 6 0.00999      7.44e-232 4.78e- 6 0.62      
#> 5 wt      1.29e-10 1.22e-11 0.0000415    4.78e-  6 0        0.339     
#> 6 qsec    1.71e- 2 1.31e- 2 0.00000577   6.2 e-  1 3.39e- 1 0

# Replacing correlation coefficients by symbols
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
cor.mat %>%
  cor_as_symbols() %>%
  pull_lower_triangle()
#>   rowname mpg disp hp drat wt qsec
#> 1     mpg                         
#> 2    disp   *                     
#> 3      hp   *    *                
#> 4    drat   +    +  .             
#> 5      wt   *    *  +    +        
#> 6    qsec   .    .  +

# Mark significant correlations
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
cor.mat %>%
  cor_mark_significant()
#>   rowname       mpg      disp        hp      drat    wt qsec
#> 1     mpg                                                   
#> 2    disp -0.85****                                         
#> 3      hp -0.78****  0.79****                               
#> 4    drat  0.68**** -0.71****   -0.45**                     
#> 5      wt -0.87****  0.89****  0.66**** -0.71****           
#> 6    qsec     0.42*    -0.43* -0.71****     0.091 -0.17


# Draw correlogram using R base plot
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
cor.mat %>%
  cor_reorder() %>%
  pull_lower_triangle() %>% 
  cor_plot()
```

![](tools/README-unnamed-chunk-10-1.png)

## Related articles

-   [How to Add P-Values onto Basic
    GGPLOTS](https://www.datanovia.com/en/blog/how-to-add-p-values-onto-basic-ggplots/)
-   [How to Add Adjusted P-values to a Multi-Panel
    GGPlot](https://www.datanovia.com/en/blog/ggpubr-how-to-add-adjusted-p-values-to-a-multi-panel-ggplot/)
-   [How to Add P-values to GGPLOT
    Facets](https://www.datanovia.com/en/blog/how-to-add-p-values-to-ggplot-facets/)
-   [How to Add P-Values Generated Elsewhere to a
    GGPLOT](https://www.datanovia.com/en/blog/ggpubr-how-to-add-p-values-generated-elsewhere-to-a-ggplot/)
-   [How to Add P-Values onto a Grouped GGPLOT using the GGPUBR R
    Package](https://www.datanovia.com/en/blog/how-to-add-p-values-onto-a-grouped-ggplot-using-the-ggpubr-r-package/)
-   [How to Create Stacked Bar Plots with Error Bars and
    P-values](https://www.datanovia.com/en/blog/how-to-create-stacked-bar-plots-with-error-bars-and-p-values/)
-   [How to Add P-Values onto Horizontal
    GGPLOTS](https://www.datanovia.com/en/blog/how-to-add-p-values-onto-horizontal-ggplots/)
-   [Add P-values and Significance Levels to
    ggplots](https://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/76-add-p-values-and-significance-levels-to-ggplots/)
-   [Comparing Means of Two Groups in
    R](https://www.datanovia.com/en/courses/comparing-means-of-two-groups-in-r/)
    -   [T-test in R](https://www.datanovia.com/en/lessons/t-test-in-r/)
    -   [Wilcoxon Test in
        R](https://www.datanovia.com/en/lessons/wilcoxon-test-in-r/)
    -   [Sign Test in
        R](https://www.datanovia.com/en/lessons/sign-test-in-r/)
-   [Comparing Multiple Means in
    R](https://www.datanovia.com/en/courses/comparing-multiple-means-in-r/)
    -   [ANOVA in R](https://www.datanovia.com/en/lessons/anova-in-r/)
    -   [Repeated Measures ANOVA in
        R](https://www.datanovia.com/en/lessons/repeated-measures-anova-in-r/)
    -   [Mixed ANOVA in
        R](https://www.datanovia.com/en/lessons/mixed-anova-in-r/)
    -   [ANCOVA in R](https://www.datanovia.com/en/lessons/ancova-in-r/)
    -   [One-Way MANOVA in
        R](https://www.datanovia.com/en/lessons/one-way-manova-in-r/)
    -   [Kruskal-Wallis Test in
        R](https://www.datanovia.com/en/lessons/kruskal-wallis-test-in-r/)
    -   [Friedman Test in
        R](https://www.datanovia.com/en/lessons/friedman-test-in-r/)

#' @include utilities.R
NULL
#'Alternative to dplyr::do for Doing Anything
#'
#'
#'@description Provides a flexible alternative to the \code{dplyr:do()} function.
#'  Technically it uses \code{nest() + mutate() + map()} to apply arbitrary
#'  computation to a grouped data frame.
#'
#'  The output is a data frame. If the applied function returns a data frame,
#'  then the output will be automatically unnested. Otherwise, the output includes the grouping
#'  variables and a column named ".results." (by default), which is a "list-columns"
#'  containing the results for group combinations.
#'
#'@param data a (grouped) data frame
#'@param .f A function, formula, or atomic vector. For example
#'  \code{~t.test(len ~ supp, data = .)}.
#' @param ... Additional arguments passed on to .f
#' @param result the column name to hold the results. Default is ".results.".
#' @return a data frame
#' @examples
#' # Custom function
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' stat_test <- function(data, formula){
#'   t.test(formula, data) %>%
#'     tidy()
#' }
#' # Example 1: pipe-friendly stat_test().
#' # Two possibilities of usage are available
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Use this
#' ToothGrowth %>%
#'   group_by(dose) %>%
#'   doo(~stat_test(data =., len ~ supp))
#'
#' # Or this
#' ToothGrowth %>%
#'   group_by(dose) %>%
#'   doo(stat_test, len ~ supp)
#'
#' # Example 2: R base function t.test() (not pipe friendly)
#' # One possibility of usage
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' comparisons <- ToothGrowth %>%
#'    group_by(dose) %>%
#'    doo(~t.test(len ~ supp, data =.))
#' comparisons
#' comparisons$.results.
#'
#' # Example 3: R base function combined with tidy()
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' ToothGrowth %>%
#'    group_by(dose) %>%
#'    doo(~t.test(len ~ supp, data =.) %>% tidy())
#'@export
doo <- function(data, .f, ..., result = ".results."){
  if(is_grouped_df(data)){
    .results <- data %>% nest()
  }
  else{
    .results <- data %>% nest(data = everything())
  }
  .results <- .results %>%
    dplyr::ungroup() %>%
    mutate(data = map(.data$data, droplevels)) %>%
    mutate(data = map(.data$data, .f, ...))
  if(inherits(.results$data[[1]], c("data.frame", "tbl_df"))){
    # Suppress warning such as:
    #  Binding character and factor vector, coercing into character vector
    .results <- suppressWarnings(unnest(.results))
  }
  else{
    colnames(.results)[ncol(.results)] <- result
  }
  if(is_grouped_df(data)){
    .groups <- dplyr::group_vars(data)
    .results <- dplyr::arrange(.results, !!!syms(.groups))
  }
  .results
}





# To be removed
doo_old_version <- function(data, .f, ..., result = ".results."){
  .nested <- data %>%
    nest() %>%
    dplyr::ungroup() %>%
    mutate(data = map(data, droplevels))
  .computed <- .nested$data %>%
    map(.f, ...)
  .results <- .nested %>%
    select(-data) %>%
    mutate(!!result := .computed)
  if(inherits(.computed[[1]], c("data.frame", "tbl_df"))){
    # Suppress warning such as:
    #  Binding character and factor vector, coercing into character vector
    .results <- suppressWarnings(unnest(.results))
  }
  if(is_grouped_df(data)){
    .groups <- dplyr::group_vars(data)
    .results <- dplyr::arrange(.results, !!!syms(.groups))
  }
  .results
}

#' @include utilities.R
NULL

#' Select Columns in a Data Frame
#'
#' @description A wrapper around the \code{\link[dplyr]{select}()} function for
#'   selection data frame columns. Supports standard and non standard
#'   evaluations. Usefull to easily program with \code{dplyr}
#' @param data a data frame
#' @param vars a character vector containing the variable names of interest.
#' @param ... One or more unquoted expressions (or variable names) separated by
#'   commas. Used to select a variable of interest.
#'
#' @return a data frame
#' @examples
#' df <- head(ToothGrowth)
#' df
#'
#' # Select column using standard evaluation
#' df %>% df_select(vars = c("dose", "len"))
#'
#' # Select column using non-standard evaluation
#' df %>% df_select(dose, len)
#' @rdname df_select
#' @export
df_select <- function(data, ...,  vars = NULL){
  if(is.null(vars)){
    results <- data %>% select(...)
  }
  else{
    results <- data %>% select(!!!syms(vars))
  }
  results
}

#' Arrange Rows by Column Values
#'
#' @description Order the rows of a data frame by values of specified columns.
#'   Wrapper arround the \code{\link[dplyr]{arrange}()} function. Supports
#'   standard and non standard evaluation.
#' @param data a data frame
#' @param vars a character vector containing the variable names of interest.
#' @param ... One or more unquoted expressions (or variable names) separated by
#'   commas. Used to select a variable of interest. Use
#'   \code{\link[dplyr]{desc}()} to sort a variable in descending order.
#' @param .by_group If TRUE, will sort first by grouping variable. Applies to
#'   grouped data frames only.
#'
#' @return a data frame
#' @examples
#' df <- head(ToothGrowth)
#' df
#'
#' # Select column using standard evaluation
#' df %>% df_arrange(vars = c("dose", "len"))
#'
#' # Select column using non-standard evaluation
#' df %>% df_arrange(dose, desc(len))
#' @rdname df_arrange
#' @export
df_arrange <- function(data, ..., vars = NULL, .by_group = FALSE ){
  if(is.null(vars)){
    results <- data %>%
      dplyr::arrange(..., .by_group = .by_group)
  }
  else{
    results <- data %>%
      dplyr::arrange(!!!syms(vars), .by_group = .by_group)
  }
  results
}


#' Group a Data Frame by One or more Variables
#'
#' @description Group a data frame by one or more variables. Supports standard
#'   and non standard evaluation.
#' @inheritParams df_select
#' @examples
#'
#' # Non standard evaluation
#' by_dose <- head(ToothGrowth) %>%
#'    df_group_by(dose)
#' by_dose
#'
#' # Standard evaluation
#' head(ToothGrowth) %>%
#'    df_group_by(vars = c("dose", "supp"))
#' @rdname df_group_by
#' @export
df_group_by <- function(data, ..., vars = NULL){
  if(is.null(vars)){
    results <- data %>% group_by(...)
  }
  else{
    results <- data %>% group_by(!!!syms(vars))
  }
  results
}


#' Nest a Tibble By Groups
#'
#' @description Nest a tibble data frame using grouping specification. Supports standard and non standard evaluation.
#' @param data a data frame
#' @param ... One or more unquoted expressions (or variable names) separated by
#'   commas. Used as grouping variables.
#' @param vars a character vector containing the grouping variables of interest.
#'
#' @return A tbl with one row per unique combination of the grouping variables.
#'   The first columns are the grouping variables, followed by a list column of
#'   tibbles with matching rows of the remaining columns.
#' @examples
#'
#' # Non standard evaluation
#' ToothGrowth %>%
#'  df_nest_by(dose, supp)
#'
#' # Standard evaluation
#' ToothGrowth %>%
#'  df_nest_by(vars = c("dose", "supp"))
#'
#' @rdname df_nest_by
#' @export
df_nest_by <- function(data, ..., vars = NULL){
  data %>%
    df_group_by(..., vars = vars) %>%
    nest() %>%
    ungroup()
}


#' Split a Data Frame into Subset
#'
#' @description Split a data frame by groups into subsets or data panel. Very
#'   similar to the function \code{\link{df_nest_by}()}. The only difference is
#'   that, it adds label to each data subset. Labels are the combination of the
#'   grouping variable levels. The column holding labels are named "label".
#' @inheritParams df_nest_by
#' @inheritParams df_label_both
#' @param labeller A function that takes a data frame, the grouping variables,
#'   label_col and label_sep arguments, and add labels into the data frame.
#'   Example of possible values are: \code{\link{df_label_both}()} and
#'   \code{\link{df_label_value}()}.
#'
#' @return A tbl with one row per unique combination of the grouping variables.
#'   The first columns are the grouping variables, followed by a list column of
#'   tibbles with matching rows of the remaining columns, and a column named
#'   label, containing labels.
#' @examples
#'
#' # Split a data frame
#' # :::::::::::::::::::::::::::::::::::::::::::::::::
#' # Create a grouped data
#' res <- ToothGrowth %>%
#'   df_split_by(dose, supp)
#' res
#'
#' # Show subsets
#' res$data
#'
#' # Add panel/subset labels
#' res <- ToothGrowth %>%
#'   df_split_by(dose, supp)
#' res
#' @rdname df_split_by
#' @export
df_split_by <- function(data, ..., vars = NULL, label_col = "label",
                        labeller = df_label_both, sep = c(", ", ":")){
  groups <- df_get_var_names(data, ..., vars = vars)
  data %>%
    df_nest_by(vars = groups) %>%
    labeller(vars = groups, label_col = label_col, sep = sep) %>%
    mutate(data = map2(.data$data, .data[[label_col]], add_panel_name, col = label_col))
}

#' Functions to Label Data Frames by Grouping Variables
#'
#' @description Functions to label data frame rows by one or multiple grouping
#'   variables.
#'
#' @inheritParams df_nest_by
#' @param label_col column to hold the label of the data subsets. Default column
#'   name is "label".
#' @param sep String separating labelling variables and values. Should be of
#'   length 2 in the function \code{df_label_both()}. 1) One sep is used to
#'   separate groups, for example ','; 2) The other sep between group name and
#'   levels; for example ':'.
#' @return a modified data frame with a column containing row labels.
#' @examples
#' # Data preparation
#' df <- head(ToothGrowth)
#'
#' # Labelling: Non standard evaluation
#' df %>%
#'   df_label_both(dose, supp)
#'
#' # Standard evaluation
#' df %>%
#'   df_label_both(dose, supp)
#'
#' # Nesting the data then label each subset by groups
#' ToothGrowth %>%
#'   df_nest_by(dose, supp) %>%
#'   df_label_both(supp, dose)
#'
#' @describeIn df_label_value Displays both the variable name and the factor value.
#' @export
df_label_both <- function(data, ..., vars = NULL, label_col = "label", sep = c(", ", ":")){
  vars <- df_get_var_names(data, ..., vars = vars)
  if(length(sep) < 2){
    warning(
      "Argument sep sould be of length 2, otherwise it will be ignored; example: sep = c(', ', ':', )\n",
      "  2. One sep is used to separate groups, for example ','\n",
      "  1. The other sep between group name and levels; for example ':'",
      call. = FALSE
    )
    sep <- c(":", ", ")
  }
  label <- data %>%
    df_select(vars = vars) %>%
    concat_groupname_to_levels(vars, sep = sep[2]) %>%
    df_unite_factors(col = label_col, vars = vars, sep = sep[1]) %>%
    pull(!!label_col)
  data %>% mutate(!!label_col := label)
}


#' @describeIn df_label_value Displays only the value of a factor.
#' @export
df_label_value <- function(data, ..., vars = NULL, label_col = "label", sep = ", "){
  vars <- df_get_var_names(data, ..., vars = vars)
  label <- data %>%
    df_select(vars = vars) %>%
    df_unite_factors(col = label_col, vars = vars, sep = sep[1]) %>%
    pull(!!label_col)
  data %>% mutate(!!label_col := label)
}


# Add panel label to a data
# Labels are the combination of the grouping variable labels
add_panel_label <- function(data, groups, col = "label") {
  label <- data %>%
    df_select(vars = groups) %>%
    concat_groupname_to_levels(groups, sep = ":") %>%
    df_unite_factors(col = col, vars = groups, sep = ", ") %>%
    pull(!!col)
  data %>% mutate(!!col := label)
}
# Add a column containing panel name
add_panel_name <- function(data, panel, col = "label") {
  data %>% mutate(!!col := !!panel)
}
concat_groupname_to_levels <- function(group.data, groups, sep = ":"){
  purrr::map2(
    group.data, groups,
    function(x, name) {paste(name, x, sep = sep)}
  ) %>%
    as_tibble()
}



#' Unite Multiple Columns into One
#'
#' @description Paste together multiple columns into one. Wrapper arround
#'   \code{\link[tidyr]{unite}()} that supports standard and non standard
#'   evaluation.
#' @inheritParams tidyr::unite
#' @param data a data frame
#' @param col the name of the new column as a string or a symbol.
#' @param ... a selection of columns. One or more unquoted expressions (or variable names) separated by
#'   commas.
#' @param vars a character vector containing the column names of interest.
#' @examples
#' # Non standard evaluation
#' head(ToothGrowth) %>%
#'  df_unite(col = "dose_supp", dose, supp)
#'
#' # Standard evaluation
#' head(ToothGrowth) %>%
#'  df_unite(col = "dose_supp", vars = c("dose", "supp"))
#' @describeIn df_unite Unite multiple columns into one.
#' @export
df_unite <- function(data, col, ..., vars = NULL, sep = "_", remove = TRUE, na.rm = FALSE){
  if(is.null(vars)){
    results <- data %>%
      tidyr::unite(
        col = !!col, ..., sep = sep,
        remove = remove, na.rm = na.rm
      )
  }
  else{
    results <- data %>%
      tidyr::unite(
        col = !!col, !!!syms(vars), sep = sep,
        remove = remove, na.rm = na.rm
      )
  }
  results
}

#' @export
#' @describeIn df_unite Unite factor columns. First, order factors levels then
#'   merge them into one column. The output column is a factor.
df_unite_factors <- function(data, col, ..., vars = NULL, sep = "_", remove = TRUE, na.rm = FALSE){
  vars <- df_get_var_names(data, ..., vars = vars)
  data %>%
    dplyr::arrange(!!!syms(vars)) %>%
    df_unite(col = col, vars = vars, sep = sep, remove = remove, na.rm = na.rm) %>%
    dplyr::mutate_at(col, function(x){factor(x, levels = unique(x))})
}



#' Get User Specified Variable Names
#'
#' @description Returns user specified variable names. Supports standard and non standard evaluation.
#' @inheritParams df_select
#' @return a character vector
#' @examples
#'
#' # Non standard evaluation
#' ToothGrowth %>%
#'  df_get_var_names(dose, len)
#'
#' # Standard evaluation
#' ToothGrowth %>%
#'  df_get_var_names(vars = c("len", "dose"))
#' @rdname df_get_var_names
#' @export
df_get_var_names <- function(data, ..., vars = NULL){
  dot_vars <- tidyselect::vars_select(colnames(data), !!!rlang::quos(...))
  unique(c(vars, dot_vars))
}

#' @include utilities.R
NULL
#'Compute Mahalanobis Distance and Flag Multivariate Outliers
#'
#'@description Pipe-friendly wrapper around to the function
#'  \code{\link[stats]{mahalanobis}()}, which returns the squared
#'  Mahalanobis distance of all rows in x. Compared to the base function, it
#'  automatically flags multivariate outliers.
#'
#'  Mahalanobis distance is a common metric used to identify multivariate
#'  outliers. The larger the value of Mahalanobis distance, the more unusual the
#'  data point (i.e., the more likely it is to be a multivariate outlier).
#'
#'  The distance tells us how far an observation is from the center of the cloud, taking into
#'  account the shape (covariance) of the cloud as well.
#'
#'  To detect outliers, the calculated Mahalanobis distance is compared against
#'  a chi-square (X^2) distribution with degrees of freedom equal to the number
#'  of dependent (outcome) variables and an alpha level of 0.001.
#'
#'  The threshold to declare a multivariate outlier is determined using the
#'  function \code{qchisq(0.999, df) }, where df is the degree of freedom (i.e.,
#'  the number of dependent variable used in the computation).
#'
#'@param data a data frame. Columns are variables.
#'@param ... One unquoted expressions (or variable name). Used to select a
#'  variable of interest. Can be also used to ignore a variable that are not
#'  needed for the computation. For example specify \code{-id} to ignore the id
#'  column.
#'
#'@return  Returns the input data frame with two additional columns: 1)
#'  "mahal.dist": Mahalanobis distance values; and 2) "is.outlier": logical
#'  values specifying whether a given observation is a multivariate outlier
#'
#' @examples
#'
#' # Compute mahalonobis distance and flag outliers if any
#' iris %>%
#'   doo(~mahalanobis_distance(.))
#'
#'# Compute distance by groups and filter outliers
#' iris %>%
#'  group_by(Species) %>%
#'  doo(~mahalanobis_distance(.)) %>%
#'  filter(is.outlier == TRUE)
#'
#'@export
mahalanobis_distance <- function(data, ...){
  if(is_grouped_df(data)){
    results <- data %>%
      doo(~mahalanobis_distance(.))
  }
  data <- data %>% select_numeric_columns()
  vars <- data %>% get_selected_vars(...)
  n.vars <- length(vars)
  threshold <- stats::qchisq(0.999, n.vars)
  .data <- data %>%
    select(!!!syms(vars)) %>%
    as.matrix()
  distance <- stats::mahalanobis(
    .data,
    center = colMeans(.data),
    cov = cov(.data)
  )
  results <- data %>%
    mutate(
      mahal.dist = round(distance, 3),
      is.outlier = .data$mahal.dist > threshold
      )
  results
}

#' @include utilities.R
#' @importFrom stats shapiro.test
#' @importFrom stats complete.cases
NULL
#' Shapiro-Wilk Normality Test
#'
#' @description Provides a pipe-friendly framework to performs Shapiro-Wilk test
#'   of normality. Support grouped data and multiple variables for multivariate
#'   normality tests. Wrapper around the R base function
#'   \code{\link[stats]{shapiro.test}()}. Can handle grouped data. Read more:
#'   \href{https://www.datanovia.com/en/lessons/normality-test-in-r/}{Normality
#'   Test in R}.
#' @param data a data frame. Columns are variables.
#' @param ... One or more unquoted expressions (or variable names) separated by
#'   commas. Used to select a variable of interest.
#' @param vars optional character vector containing variable names. Ignored when
#'   dot vars are specified.
#' @return  a data frame containing the value of the Shapiro-Wilk statistic and
#'   the corresponding p.value.
#' @examples
#'
#' # Shapiro Wilk normality test for one variable
#' iris %>% shapiro_test(Sepal.Length)
#'
#' # Shapiro Wilk normality test for two variables
#' iris %>% shapiro_test(Sepal.Length, Petal.Width)
#'
#' # Multivariate normality test
#' mshapiro_test(iris[, 1:3])
#'

#' @describeIn shapiro_test univariate Shapiro-Wilk normality test
#' @export
shapiro_test <- function(data, ..., vars = NULL){
  if(is_grouped_df(data)){
    results <- data %>%
      doo(shapiro_test, ..., vars = vars)
    return(results)
  }
  else if(is.numeric(data)){
    results <- shapiro.test(data)
    data.name <- deparse(substitute(data))
    results <-  tidy(results) %>%
      add_column(variable = data.name, .before = 1) %>%
      select(-.data$method)
    return(results)
  }
  vars <- c(get_dot_vars(...), vars) %>%
    unique()
  n.vars <- length(vars)
  if(.is_empty(vars)){
    stop("Specify one or more variables...")
  }
  data <- data %>%
    select(!!!syms(vars))
  variable <- value <- method <-  NULL
  data <- data %>%
    gather(key = "variable", value = "value") %>%
    filter(!is.na(value))
  data %>%
    group_by(variable) %>%
    doo(~tidy(shapiro.test(.$value))) %>%
    select(-.data$method) %>%
    rename(p = .data$p.value)
}


#' @describeIn shapiro_test multivariate Shapiro-Wilk normality test. This is a
#'   modified copy of the \code{mshapiro.test()} function of the package
#'   mvnormtest, for internal convenience.
#' @export
mshapiro_test <- function(data)
{
  if(is_grouped_df(data)){
    results <- data %>%
      doo(~shapiro.test(.))
  }
  x <- data
  if (!is.matrix(x)) {
    x <- as.matrix(x)
  }
  x <- x[complete.cases(x), ]
  x <- t(x)
  n <- ncol(x)
  if (n < 3 || n > 5000) {
    stop("sample size must be between 3 and 5000")
  }
  rng <- range(x)
  rng <- rng[2] - rng[1]
  if (rng == 0) {
    stop("all `x[]' are identical")
  }
  Us <- apply(x, 1, mean)
  R <- x - Us
  M.1 <- solve(R %*% t(R), tol = 1e-18)
  Rmax <- diag(t(R) %*% M.1 %*% R)
  C <- M.1 %*% R[, which.max(Rmax)]
  Z <- t(C) %*% x
  result <- shapiro.test(Z)
  result$method <- "Multivariate Shapiro-Wilk normality test"
  result$data.name <- paste("(", paste(rownames(x), collapse = ","),
                            ")", sep = "")

  tidy(result) %>%
    select(-.data$method)
}

context("test-shapiro_test")

test_that("Shapiro test works when input data contains column names 'value' or 'variable'", {
  df <- iris
  # Column names contain value
  colnames(df)[2] <- "value"
  res <- df %>% shapiro_test(value)
  res_value <- round(res$p, 3)
  # Column names contain variable
  colnames(df)[2] <- "variable"
  res <- df %>% shapiro_test(variable)
  res_variable <- round(res$p, 3)
  expect_equal(res_value, 0.101)
  expect_equal(res_variable, 0.101)
})

#' @include utilities.R
NULL
#'Compute Summary Statistics
#'@description Compute summary statistics for one or multiple numeric variables.
#'@param data a data frame
#'@param ... (optional) One or more unquoted expressions (or variable names)
#'  separated by commas. Used to select a variable of interest. If no variable
#'  is specified, then the summary statistics of all numeric variables in the
#'  data frame is computed.
#'@param type type of summary statistics. Possible values include: \code{"full",
#'  "common", "robust",  "five_number", "mean_sd", "mean_se", "mean_ci",
#'  "median_iqr", "median_mad", "quantile", "mean", "median",  "min", "max"}
#'@param show a character vector specifying the summary statistics you want to
#'  show. Example: \code{show = c("n", "mean", "sd")}. This is used to filter
#'  the output after computation.
#' @param probs numeric vector of probabilities with values in [0,1]. Used only when type = "quantile".
#'@return A data frame containing descriptive statistics, such as: \itemize{
#'  \item \strong{n}: the number of individuals \item \strong{min}: minimum
#'  \item \strong{max}: maximum \item \strong{median}: median \item
#'  \strong{mean}: mean \item \strong{q1, q3}: the first and the third quartile,
#'  respectively. \item \strong{iqr}: interquartile range \item \strong{mad}:
#'  median absolute deviation (see ?MAD) \item \strong{sd}: standard deviation
#'  of the mean \item \strong{se}: standard error of the mean \item \strong{ci}: 95 percent confidence interval of the mean }
#' @examples
#' # Full summary statistics
#' data("ToothGrowth")
#' ToothGrowth %>% get_summary_stats(len)
#'
#' # Summary statistics of grouped data
#' # Show only common summary
#' ToothGrowth %>%
#'   group_by(dose, supp) %>%
#'   get_summary_stats(len, type = "common")
#'
#' # Robust summary statistics
#' ToothGrowth %>% get_summary_stats(len, type = "robust")
#'
#' # Five number summary statistics
#' ToothGrowth %>% get_summary_stats(len, type = "five_number")
#'
#' # Compute only mean and sd
#' ToothGrowth %>% get_summary_stats(len, type = "mean_sd")
#'
#' # Compute full summary statistics but show only mean, sd, median, iqr
#' ToothGrowth %>%
#'     get_summary_stats(len, show = c("mean", "sd", "median", "iqr"))
#'
#'@export
get_summary_stats <- function(
  data, ..., type = c("full", "common", "robust",  "five_number",
                      "mean_sd", "mean_se", "mean_ci", "median_iqr", "median_mad", "quantile",
                      "mean", "median",  "min", "max" ),
  show = NULL, probs = seq(0, 1, 0.25)
  ){
  type = match.arg(type)
  if(is_grouped_df(data)){
    results <- data %>%
      doo(get_summary_stats, ..., type = type, show = show)
    return(results)
  }
  data <- data %>% select_numeric_columns()
  vars <- data %>% get_selected_vars(...)
  n.vars <- length(vars)
  if(n.vars >= 1){
    data <- data %>% select(!!!syms(vars))
  }
  variable <- .value. <- NULL
  data <- data %>%
    gather(key = "variable", value = ".value.") %>%
    filter(!is.na(.value.)) %>%
    group_by(variable)
  results <- switch(
    type,
    common = common_summary(data),
    robust = robust_summary(data),
    five_number = five_number_summary(data),
    mean_sd = mean_sd(data),
    mean_se = mean_se(data),
    mean_ci = mean_ci(data),
    median_iqr = median_iqr(data),
    median_mad = median_mad(data),
    quantile = quantile_summary(data, probs),
    mean = mean_(data),
    median = median_(data),
    min = min_(data),
    max = max_(data),
    full_summary(data)
  ) %>%
    dplyr::mutate_if(is.numeric, round, digits = 3)

  if(!is.null(show)){
    show <- unique(c("variable", "n", show))
    results <- results %>% select(!!!syms(show))
  }

  results
}


full_summary <- function(data){
  confidence <- 0.95
  alpha <- 1 - confidence
  .value. <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      min = min(.value., na.rm=TRUE),
      max = max(.value., na.rm=TRUE),
      median = stats::median(.value., na.rm=TRUE),
      q1 = stats::quantile(.value., 0.25, na.rm = TRUE),
      q3 = stats::quantile(.value., 0.75, na.rm = TRUE),
      iqr = stats::IQR(.value., na.rm=TRUE),
      mad = stats::mad(.value., na.rm=TRUE),
      mean = mean(.value., na.rm = TRUE),
      sd = stats::sd(.value., na.rm = TRUE)
    ) %>%
    mutate(
      se = .data$sd / sqrt(.data$n),
      ci = abs(stats::qt(alpha/2, .data$n-1)*.data$se)
    )
}

common_summary <- function(data){
  confidence <- 0.95
  alpha <- 1 - confidence
  .value. <- ci <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      min = min(.value., na.rm=TRUE),
      max = max(.value., na.rm=TRUE),
      median = stats::median(.value., na.rm=TRUE),
      iqr = stats::IQR(.value., na.rm=TRUE),
      mean = mean(.value., na.rm = TRUE),
      sd = stats::sd(.value., na.rm = TRUE)
    ) %>%
    mutate(
      se = .data$sd / sqrt(.data$n),
      ci = abs(stats::qt(alpha/2, .data$n-1)*.data$se)
    )
}

robust_summary <- function(data){
  .value. <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      median = stats::median(.value., na.rm=TRUE),
      iqr = stats::IQR(.value., na.rm=TRUE)
    )
}

quantile_summary <- function(data, probs = seq(0, 1, 0.25)){
  core_func <- function(data, probs){
    .value. <- NULL
    n <- sum(!is.na(data$.value.))
    names(n) <- "n"
    q <- stats::quantile(data$.value., probs, na.rm = TRUE)
    results <- t(matrix(c(n, q)))
    colnames(results) <- c("n", names(q))
    tibble::as_tibble(results)
  }
  results  <- data %>%
    nest() %>%
    mutate(.results. = map(data, core_func, probs)) %>%
    select(.data$variable, .data$.results.) %>%
    unnest(cols = ".results.")
  results
}



five_number_summary <- function(data){
  .value.  <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      min = min(.value., na.rm=TRUE),
      max = max(.value., na.rm=TRUE),
      q1 = stats::quantile(.value., 0.25, na.rm = TRUE),
      median = stats::median(.value., na.rm=TRUE),
      q3 = stats::quantile(.value., 0.75, na.rm = TRUE)
    )
}

mean_ <- function(data){
  .value.  <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      mean = mean(.value., na.rm = TRUE)
    )
}
median_ <- function(data){
  .value.  <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      median = stats::median(.value., na.rm=TRUE)
    )
}
max_ <- function(data){
  .value.  <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      max = max(.value., na.rm = TRUE)
    )
}
min_ <- function(data){
  .value.  <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      min = min(.value., na.rm = TRUE)
    )
}

mean_sd <- function(data){
  .value.  <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      mean = mean(.value., na.rm = TRUE),
      sd = stats::sd(.value., na.rm = TRUE)
    )
}

mean_se <- function(data){
  .value.  <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      mean = mean(.value., na.rm = TRUE),
      sd = stats::sd(.value., na.rm = TRUE)
    ) %>%
    mutate(se = .data$sd / sqrt(.data$n))%>%
    select(-.data$sd)
}

mean_ci <- function(data){
  confidence <- 0.95
  alpha <- 1 - confidence
  .value. <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      mean = mean(.value., na.rm = TRUE),
      sd = stats::sd(.value., na.rm = TRUE)
    ) %>%
    mutate(
      se = .data$sd / sqrt(.data$n),
      ci = abs(stats::qt(alpha/2, .data$n-1)*.data$se)
    )%>%
    select(-.data$se, -.data$sd)
}

median_iqr <- function(data){
  .value.  <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      median = stats::median(.value., na.rm=TRUE),
      iqr = stats::IQR(.value., na.rm=TRUE)
    )
}

median_mad <- function(data){
  .value.  <- NULL
  data %>%
    dplyr::summarise(
      n = sum(!is.na(.value.)),
      median = stats::median(.value., na.rm=TRUE),
      mad = stats::mad(.value., na.rm=TRUE)
    )
}

context("test-get_summary_stats")

test_that("Checking that get_summary_stats keeps the order of columns specified by the user", {
  res <- data.frame(
    c = rnorm(50,10,5),
    b = rnorm(50,100,20),
    a = rnorm(50,0,1)
    ) %>%
    get_summary_stats(a, c, b, type = "mean_sd")
  obtained_var_order <- as.character(res$variable)
  expected_var_order <- c("a", "c", "b")
  expect_equal(obtained_var_order, expected_var_order)
})

test_that("get_summary_stats `digits` controls the rounding precision (#145, #186, #218)", {
  small <- data.frame(x = c(0.0001234, 0.0002, 0.00033, 0.00041))
  # default (digits = 3) rounds very small values to 0
  expect_equal(get_summary_stats(small, x, type = "mean")$mean, 0)
  # a higher `digits` keeps the precision
  expect_gt(get_summary_stats(small, x, type = "mean", digits = 8)$mean, 0)
})

test_that("get_summary_stats default (digits = 3) output is unchanged (no regression, #145)", {
  for (ty in c("full", "common", "mean_sd", "quantile", "robust")) {
    expect_equal(
      ToothGrowth %>% get_summary_stats(len, type = ty),
      ToothGrowth %>% get_summary_stats(len, type = ty, digits = 3)
    )
  }
  # grouped: default unchanged, and `digits` is honoured per group
  expect_equal(
    ToothGrowth %>% group_by(supp) %>% get_summary_stats(len),
    ToothGrowth %>% group_by(supp) %>% get_summary_stats(len, digits = 3)
  )
  g8 <- ToothGrowth %>% group_by(supp) %>% get_summary_stats(len, type = "mean", digits = 8)
  expect_equal(nrow(g8), 2L)
})

test_that("get_summary_stats: skewness/kurtosis values are correct (#99)", {
  x <- ToothGrowth$len
  n <- length(x); m <- mean(x)
  m2 <- mean((x - m)^2); m3 <- mean((x - m)^3); m4 <- mean((x - m)^4)
  exp.skew <- (m3 / m2^1.5) * sqrt(n * (n - 1)) / (n - 2)
  exp.kurt <- ((n + 1) * (m4 / m2^2 - 3) + 6) * (n - 1) / ((n - 2) * (n - 3))
  res <- ToothGrowth %>% get_summary_stats(len, show = c("skewness", "kurtosis"), digits = 8)
  # values match the type-2 (bias-corrected) estimator (validated against e1071 type 2)
  expect_equal(res$skewness, round(exp.skew, 8))
  expect_equal(res$kurtosis, round(exp.kurt, 8))
})

test_that("get_summary_stats: skewness/kurtosis are opt-in via show, default unchanged (#99)", {
  default.cols <- colnames(ToothGrowth %>% get_summary_stats(len))
  expect_false(any(c("skewness", "kurtosis") %in% default.cols))
  # show controls inclusion + order
  res <- ToothGrowth %>% get_summary_stats(len, show = c("mean", "sd", "skewness", "kurtosis"))
  expect_equal(colnames(res), c("variable", "n", "mean", "sd", "skewness", "kurtosis"))
  # works on a non-full type too
  res2 <- ToothGrowth %>% get_summary_stats(len, type = "mean_sd", show = c("mean", "skewness"))
  expect_equal(colnames(res2), c("variable", "n", "mean", "skewness"))
})

test_that("get_summary_stats: skewness/kurtosis per group + NA edge cases (#99)", {
  g <- ToothGrowth %>% group_by(supp) %>%
    get_summary_stats(len, show = c("skewness", "kurtosis"), digits = 8)
  expect_equal(nrow(g), 2L)
  oj <- ToothGrowth$len[ToothGrowth$supp == "OJ"]
  no <- length(oj); mo <- mean(oj); mo2 <- mean((oj - mo)^2); mo3 <- mean((oj - mo)^3)
  exp.oj <- (mo3 / mo2^1.5) * sqrt(no * (no - 1)) / (no - 2)
  expect_equal(g$skewness[g$supp == "OJ"], round(exp.oj, 8))
  # too-few-observations -> NA, no error
  small <- data.frame(x = c(1, 2, 3))  # n=3: skewness defined, kurtosis NA
  res <- small %>% get_summary_stats(x, show = c("skewness", "kurtosis"))
  expect_false(is.na(res$skewness))
  expect_true(is.na(res$kurtosis))
})

#' @include utilities.R add_significance.R pull_triangle.R
NULL
#' Add Significance Levels To a Correlation Matrix
#' @description Combines correlation coefficients and significance levels in a
#'   correlation matrix data.
#' @inheritParams add_significance
#' @param x an object of class \code{\link{cor_mat}()}.
#' @return a data frame containing the lower triangular part of the correlation
#'   matrix marked by significance symbols.
#' @examples
#' mtcars %>%
#'   select(mpg, disp, hp, drat, wt, qsec) %>%
#'   cor_mat() %>%
#'   cor_mark_significant()
#' @export
cor_mark_significant <- function(x, cutpoints = c(0, 0.0001, 0.001, 0.01, 0.05,  1),
                                 symbols = c("****", "***", "**", "*",  ""))
  {
  if(!inherits(x, c("cor_mat", "cor_mat_tri")))
    stop("x should be an object of class cor_mat or cor_mat_tri.")

  cor <- p.signif <- var1 <- var2 <- NULL
  res <- x %>%
    cor_gather (drop.na = FALSE) %>%
    add_significance(cutpoints = cutpoints, symbols = symbols) %>%
    mutate(cor = paste0(cor, p.signif)) %>%
    select(var1, var2, cor) %>%
    cor_spread()

  if(inherits(x, "upper_tri"))
    res %>% pull_upper_triangle()
  else
    res %>% pull_lower_triangle()
}

#' @include utilities.R
NULL
#' Replace Lower and Upper Triangular Part of a Matrix
#' @description Replace the lower or the upper triangular part of a
#'   (correlation) matrix.
#' @param x a (correlation) matrix
#' @param diagonal logical. Default is FALSE. If TRUE, the matrix diagonal is
#'   included.
#' @param triangle the triangle to replace. Allowed values are one of
#'   "upper" and "lower".
#' @param by a replacement argument. Appropriate values are either "" or NA. Used to replace
#'   the upper, lower or the diagonal part of the matrix.
#' @return an object of class \code{cor_mat_tri}, which is a data frame
#' @seealso \code{\link{pull_triangle}()}
#' @examples

#' # Compute correlation matrix and pull triangles
#' #::::::::::::::::::::::::::::::::::::::::::
#' # Correlation matrix
#'  cor.mat <- mtcars %>%
#'    select(mpg, disp, hp, drat, wt, qsec) %>%
#'    cor_mat()
#'  cor.mat
#'
#'  # Replace upper triangle by NA
#' #::::::::::::::::::::::::::::::::::::::::::
#' cor.mat %>% replace_upper_triangle(by = NA)
#'
#'
#' # Replace upper triangle by NA and reshape the
#' # correlation matrix to have unique combinations of variables
#' #::::::::::::::::::::::::::::::::::::::::::
#' cor.mat %>%
#'   replace_upper_triangle(by = NA) %>%
#'   cor_gather()

#' @describeIn replace_triangle replaces the specified triangle by empty or NA.
#' @export
replace_triangle <- function(x, triangle = c("lower", "upper"), by = "", diagonal = FALSE){

  triangle <- match.arg(triangle)
  remaining.triangle <- ifelse(
    triangle == "lower", "upper", "lower"
  )
  remaining.triangle.class <- paste0(remaining.triangle, "_tri")

  replacement <- by
  get_tri <- switch(
    triangle, upper = upper.tri, lower = lower.tri
  )

  res <- as_matrix(x)
  res[get_tri(res)] <- replacement
  if (!diagonal) diag(res) <- replacement
  res <- res %>% matrix_to_dataframe()

  if(.is_cor_mat(x)){
    pvalue <- x %>% attr("pvalue") %>% as_matrix()
    pvalue[get_tri(pvalue)] <- replacement
    if (!diagonal) diag(pvalue) <- replacement
    pvalue <- pvalue %>% matrix_to_dataframe()

    res <- res %>%
      set_attrs(pvalue = pvalue)  %>%
      add_class("cor_mat_tri")
  }

  res %>% add_class(remaining.triangle.class)
}

#' @describeIn replace_triangle replaces the upper triangular part of a matrix.
#'   Returns an object of class \code{lower_tri}.
#' @export
replace_upper_triangle <- function(x,  by = "", diagonal = FALSE){
  x %>% replace_triangle("upper", by = by, diagonal = diagonal)
}

#' @describeIn replace_triangle replaces the lower triangular part of a matrix.
#'   Returns an object of class \code{lower_tri}
#' @export
replace_lower_triangle <- function(x,  by = "", diagonal = FALSE){
  x %>% replace_triangle("lower", by = by, diagonal = diagonal)
}

#' @include utilities.R
NULL
#'Build Factorial Designs for ANOVA
#'
#'
#'@description Provides helper functions to build factorial design for easily
#'  computing ANOVA using the \code{\link[car]{Anova}()} function. This might be
#'  very useful for repeated measures ANOVA, which is hard to set up with the
#'  \code{car} package.
#'@inheritParams anova_test
#'@param data a data frame containing the variables
#'@return a list with the following components: \itemize{ \item \strong{the
#'  specified arguments}: \code{dv, wid, between, within} \item \strong{data}:
#'  the original data (long format) or independent ANOVA. The wide format is
#'  returned for repeated measures ANOVA. \item \strong{idata}: an optional data
#'  frame giving the levels of factors defining the intra-subject model for
#'  multivariate repeated-measures data. \item \strong{idesign}: a one-sided
#'  model formula using the “data” in idata and specifying the intra-subject
#'  design. \item \strong{repeated}: logical. Value is TRUE when the data is a
#'  repeated design. \item \strong{lm_formula}: the formula used to build the
#'  \code{lm} model. \item \strong{lm_data}: the data used to build the \code{lm}
#'  model. Can be either in a long format (i.e., the original data for
#'  independent measures ANOVA) or in a wide format (case of repeated measures ANOVA). \item \strong{model}: the \code{lm} model }
#'@author Alboukadel Kassambara, \email{alboukadel.kassambara@@gmail.com}
#'@seealso \code{\link{anova_test}()}, \code{\link{anova_summary}()}
#'@examples
#'# Load data
#'#:::::::::::::::::::::::::::::::::::::::
#'data("ToothGrowth")
#'df <- ToothGrowth
#' head(df)
#'
#'# Repeated measures designs
#'#:::::::::::::::::::::::::::::::::::::::::
#'# Prepare the data
#'df$id <- rep(1:10, 6) # Add individuals id
#'head(df)
#'# Build factorial designs
#'design <- factorial_design(df, dv = len, wid = id, within = c(supp, dose))
#'design
#'# Easily perform repeated measures ANOVA using the car package
#' res.anova <- Anova(design$model, idata = design$idata, idesign = design$idesign, type = 3)
#' summary(res.anova, multivariate = FALSE)
#'
#'# Independent measures designs
#'#:::::::::::::::::::::::::::::::::::::::::
#'# Build factorial designs
#' df$id <- 1:nrow(df)
#' design <- factorial_design(df, dv = len, wid = id, between = c(supp, dose))
#' design
#' # Perform ANOVA
#' Anova(design$model, type = 3)
#'
#'@rdname factorial_design
#'@export
factorial_design <- function(data, dv, wid, between, within, covariate){
  # Check factorial design %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  . <- NULL
  .args <- rlang::enquos(dv = dv, between = between, wid = wid, within = within, covariate = covariate) %>%
    get_quo_vars_list(data, .) %>%
    remove_null_items() %>%
    add_item(data = data) %>%
    check_factorial_design()
  data <- .args$data
  dv <- .args$dv
  between <- .args$between
  within <- .args$within
  covariate <- .args$covariate
  rhs <- create_formula_right_hand_side(between, covariate)
  # Repeated measures  designs %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if(!is.null(within)){
    not.within.vars <- setdiff(colnames(data), within)
    nested <- data %>%
      group_by(!!!syms(within)) %>%
      nest()
    # Get intra-subject factor levels
    .args$idata <- nested %>%
      select(-data) %>%
      dplyr::arrange(!!!syms(within)) %>%
      as.data.frame()
    .args$idesign <- paste(within, collapse = "*") %>%
      paste0('~',.) %>%
      stats::as.formula()
    # Unite intra-subject factors into one grouping column,
    # then spread the data into wide format
    wide <- nested %>%
      tidyr::unite(!!!syms(within), col = ".group.", sep = "_") %>%
      select(.data$.group., data) %>%
      unnest() %>%
      spread(key = ".group.", value = dv) %>%
      as_tibble()
    .args$lm_data <- wide
    .args$repeated <- TRUE
    # Build model formula: orders of wide dv name and data colnames should match
    # dv are all possible combinations of within-subjects factor levels
    wide.dv.name <- setdiff(colnames(wide), not.within.vars) %>%
      paste(collapse = ", ")
    lm_formula <- paste0("cbind(", wide.dv.name, ") ~ ", rhs)
  }
  # Independent measures designs %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  else if(!is.null(between)){
    .args$lm_data <- .args$data
    .args$repeated <- FALSE
    lm_formula <- paste0(dv, " ~ ", rhs)
  }
  # Fit lm
  lm_formula <- .args$lm_formula <- stats::as.formula(lm_formula)
  data <- .args$lm_data
  opt <- options( "contrasts" = c( "contr.sum", "contr.poly" ) )
  .args$model <- stats::lm(lm_formula, data)
  options(opt)
  .args
}


create_formula_right_hand_side <- function(between, covariate = NULL){
  covariate <- paste(covariate, collapse = "+")
  between <- paste(between, collapse = "*")
  bc.sep <- ifelse(covariate != "" & between != "", " + ", "") # Between and covariate vars separator
  rhs <- paste0(covariate, bc.sep, between)
  if(rhs == "") rhs <- "1"
  rhs
}


# Cheking the design
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
check_factorial_design <- function(.args){
  if(!inherits(.args$data, "data.frame")){
    stop('data should be a data frame.')
  }
  if(is.null(.args$within) & is.null(.args$between)){
    stop("Specify at least one of the arguments: 'within' and 'between'")
  }
  .args$data <- droplevels(.args$data) %>%
    as_tibble()
  .args <- .args %>%
    remove_missing_values_in_data() %>%
    assertthat_dv_is_numeric() %>%
    assertthat_wid_is_specified() %>%
    asserthat_wid_is_unique() %>%
    convert_grouping_vars_to_factor() %>%
    assertthat_iv_has_enough_levels()
  .args
}

# Remove missing values
remove_missing_values_in_data <- function(.args){
  model.variables <- c(
    .args$dv, .args$wid, .args$between,
    .args$within, .args$covariate
    )
  complete.rows <- stats::complete.cases(.args$data[, model.variables])
  na.rows <- which(!complete.rows)
  na.exists <- length(na.rows) > 0
  if(na.exists){
    warning(
      "NA detected in rows: ", paste(na.rows, collapse = ","), ".",
      "\nRemoving this rows before the analysis.",
      call. = FALSE
    )
    .args$data <- .args$data[complete.rows, ]
  }
  .args
}

# Make sure the dependent variable (dv) is numeric
assertthat_dv_is_numeric <- function(.args){
  if(is.null(.args$dv)){
    stop("The dependent variable argument 'dv' is required")
  }
  dv.data <- .args$data %>% select(!!!syms(.args$dv))
  if(!is_all_columns_numeric(dv.data)){
    stop("The dependent variable 'dv' should be numeric")
  }
  invisible(.args)
}

# Make sure that the id is provided, otherwise
# Create it in the case of between-Ss ANOVA
assertthat_wid_is_specified <- function(.args){
  if(is.null(.args$wid)){
    if(!is.null(.args$within)){
      stop("Specify the argument 'wid'",
           ", required for repeated measures ANOVA")
    }
    else{
      .args$wid <- ".id."
      .args$data$.id. <- factor(1:nrow(.args$data))
    }
  }
  .args
}

# Check if individual id is unique in each between groups
# otherwise, create unique id accross between groups
asserthat_wid_is_unique <- function(.args){
  if(is.null(.args$between)) return(.args)
  if(!is_id_unique_by_between_vars(.args)){
    warning("The 'wid' column contains duplicate ids across ",
            "between-subjects variables. ",
            "Automatic unique id will be created",
            immediate. = TRUE, call. = FALSE)
    wid <- .args$wid
    .args$data <- .args$data %>%
      mutate(!!wid := create_uniqueId_by_bteween_vars(.args))
  }
  .args
}
is_id_unique_by_between_vars <- function(.args){
  data <- .args$data
  wid <- .args$wid
  between <- .args$between
  # Split the data by between variables
  nested <- data %>%
    group_by(!!!syms(between)) %>%
    nest()  %>%
    mutate(data = map(.data$data, dplyr::distinct, !!sym(wid), .keep_all = TRUE))
  # Check that id is unique accross between groups
  freq <- nested %>%
    unnest() %>%
    group_by(!!!syms(c(wid))) %>%
    summarise(count = n()) %>%
    pull(.data$count)
  all(freq == 1)
}
create_uniqueId_by_bteween_vars <- function(.args){
  data <- .args$data
  vars <- c(.args$wid, .args$between)
  data %>%
    select(!!!syms(vars)) %>%
    dplyr::mutate_all(as.character) %>%
    purrr::pmap(paste, sep = ".") %>%
    unlist() %>%
    factor()
}

# Make sure that independent variables (iv) has more than one levels
assertthat_iv_has_enough_levels <- function(.args){
  vars <- c(.args$within, .args$between)
  data <- .args$data
  for(.var in vars){
    n.levels <- unique(data[[.var]]) %>% length()
    if(n.levels == 1){
      stop("Variable ", .var, "has only one level. ",
           "Remove it from the model.")
    }
  }
  .args
}
# Convert the grouping variables to factor
convert_grouping_vars_to_factor <- function(.args){
  .args$data <- .args$data %>%
    convert_as_factor(vars = c(.args$wid, .args$between)) %>%
    convert_as_factor(vars = .args$within, make.valid.levels = TRUE)
  .args
}

#' @include utilities.R
NULL

# Helper function to get MANOVA table
# The codes is from: getAnywhere("print.Anova.mlm")
#
# x a manova test result
get_manova_table <- function (x)
{
  if ((!is.null(x$singular)) && x$singular)
    stop(
      "singular error SSP matrix; multivariate tests unavailable\n",
      "try summary(object, multivariate=FALSE)"
      )
  test <- x$test
  repeated <- x$repeated
  ntests <- length(x$terms)
  tests <- matrix(NA, ntests, 4)
  . <- NULL
  if (!repeated)
    SSPE.qr <- qr(x$SSPE)
  for (term in 1:ntests) {
    eigs <- qr.coef(
      if (repeated) qr(x$SSPE[[term]]) else SSPE.qr,
      x$SSP[[term]]
      ) %>%
      eigen(symmetric = FALSE) %>%
      .$values %>%
      Re()
    tests[term, 1:4] <- switch(
      test,
      Pillai = Pillai(eigs, x$df[term], x$error.df),
      Wilks = Wilks(eigs, x$df[term], x$error.df),
      `Hotelling-Lawley` = HL(eigs, x$df[term], x$error.df),
      Roy = Roy(eigs, x$df[term], x$error.df)
      )
  }
  ok <- tests[, 2] >= 0 & tests[, 3] > 0 & tests[, 4] > 0
  ok <- !is.na(ok) & ok
  tests <- cbind(
    x$df, tests,
    stats::pf(tests[ok, 2], tests[ok, 3], tests[ok, 4], lower.tail = FALSE)
    )
  rownames(tests) <- x$terms
  colnames(tests) <- c("Df", "test stat", "approx F", "num Df",
                       "den Df", "Pr(>F)")
  heading <- paste(
    "\nType ", x$type, if (repeated) " Repeated Measures",
    " MANOVA Tests: ", test, " test statistic",sep = ""
    )

  tests <- structure(
    as.data.frame(tests), heading = heading,
    class = c("anova", "data.frame")
    )
  tests
}


Pillai <- function (eig, q, df.res)
{
  test <- sum(eig/(1 + eig))
  p <- length(eig)
  s <- min(p, q)
  n <- 0.5 * (df.res - p - 1)
  m <- 0.5 * (abs(p - q) - 1)
  tmp1 <- 2 * m + s + 1
  tmp2 <- 2 * n + s + 1
  c(test, (tmp2/tmp1 * test)/(s - test), s * tmp1, s * tmp2)
}

Wilks <- function (eig, q, df.res)
{
  test <- prod(1/(1 + eig))
  p <- length(eig)
  tmp1 <- df.res - 0.5 * (p - q + 1)
  tmp2 <- (p * q - 2)/4
  tmp3 <- p^2 + q^2 - 5
  tmp3 <- if (tmp3 > 0)
    sqrt(((p * q)^2 - 4)/tmp3)
  else 1
  c(test, ((test^(-1/tmp3) - 1) * (tmp1 * tmp3 - 2 * tmp2))/p/q,
    p * q, tmp1 * tmp3 - 2 * tmp2)
}

HL <- function (eig, q, df.res)
{
  test <- sum(eig)
  p <- length(eig)
  m <- 0.5 * (abs(p - q) - 1)
  n <- 0.5 * (df.res - p - 1)
  s <- min(p, q)
  tmp1 <- 2 * m + s + 1
  tmp2 <- 2 * (s * n + 1)
  c(test, (tmp2 * test)/s/s/tmp1, s * tmp1, tmp2)
}

Roy <- function (eig, q, df.res)
{
  p <- length(eig)
  test <- max(eig)
  tmp1 <- max(p, q)
  tmp2 <- df.res - tmp1 + q
  c(test, (tmp2 * test)/tmp1, tmp1, tmp2)
}

#' @include utilities.R
NULL

#'Factors
#'
#'@description Provides pipe-friendly functions to convert simultaneously
#'  multiple variables into a factor variable.
#'
#'  Helper functions are also available to set the reference level and the
#'  levels order.
#'
#'@param data a data frame
#'@param ... one unquoted expressions (or variable name) specifying the name of
#'  the variables you want to convert into factor. Alternative to the argument
#'  \code{vars}.
#'@param vars a character vector specifying the variables to convert into
#'  factor.
#'@param name a factor variable name. Can be unquoted. For example, use
#'  \code{group} or \code{"group"}.
#'@param ref the reference level.
#'@param order a character vector specifying the order of the factor levels
#'@param make.valid.levels logical. Default is FALSE. If TRUE, converts the
#'  variable to factor and add a leading character (x) if starting with a digit.

#'@examples
#' # Create a demo data
#' df <- tibble(
#'   group = c("a", "a", "b", "b", "c", "c"),
#'   time = c("t1", "t2", "t1", "t2", "t1", "t2"),
#'   value = c(5, 6, 1, 3, 4, 5)
#' )
#' df
#' # Convert group and time into factor variable
#' result <- df %>% convert_as_factor(group, time)
#' result
#' # Show group levels
#' levels(result$group)
#'
#' # Set c as the reference level (the first one)
#' result <- result %>%
#'   set_ref_level("group", ref = "c")
#' levels(result$group)
#'
#' # Set the order of levels
#' result <- result %>%
#'   reorder_levels("group", order = c("b", "c", "a"))
#' levels(result$group)
#'

#' @describeIn factors Convert one or multiple variables into factor.
#' @export
convert_as_factor <- function(data, ..., vars = NULL, make.valid.levels = FALSE){
  vars <- c(get_dot_vars(...), vars) %>%
    unique()
  if(.is_empty(vars)){
    return(data)
  }
  if(make.valid.levels){
    for(variable in vars) {
      data <- make_valid_levels(data, variable)
    }
  }
  else{
    data <- data %>% dplyr::mutate_at(vars, as.factor)
  }
  data
}

#' @describeIn factors Change a factor reference level or group.
#' @export
set_ref_level <- function(data, name, ref){
  .args <- rlang::enquos(name = name) %>% select_quo_variables(data)
  data[[.args$name]] <- stats::relevel(data[[.args$name]], ref)
  data
}

#' @describeIn factors Change the order of a factor levels
#' @export
reorder_levels <- function(data, name, order){
  .args <- rlang::enquos(name = name) %>% select_quo_variables(data)
  data[[.args$name]] <- factor(data[[.args$name]], levels = order)
  data
}



make_valid_levels <- function(data, name){
  .args <- rlang::enquos(name = name) %>% select_quo_variables(data)
  name <- .args$name
  value <- data %>% pull(!!name)
  if(is.factor(value)){
    levels(value) <- make.names(levels(value), unique = TRUE)
  }
  else{
    value <- as.character(value)
    lab <- make.names(unique(value),unique=TRUE)
    value <- factor(value, levels = unique(value), labels = lab)
  }
  data[[name]] <- value
  data
}

#' @include utilities.R
NULL
#' Reorder Correlation Matrix
#' @description reorder correlation matrix, according to the coefficients,
#'   using the hierarchical clustering method.
#'@param x a correlation matrix. Particularly, an object of class \code{cor_mat}.
#'@return a data frame
#'@seealso \code{\link{cor_mat}()}, \code{\link{cor_gather}()}, \code{\link{cor_spread}()}
#' @examples
#' # Compute correlation matrix
#' #::::::::::::::::::::::::::::::::::::::::::
#' cor.mat <- mtcars %>%
#'   select(mpg, disp, hp, drat, wt, qsec) %>%
#'   cor_mat()
#'
#
#' # Reorder by correlation and get p-values
#' #::::::::::::::::::::::::::::::::::::::::::
#' # Reorder
#' cor.mat %>%
#'   cor_reorder()
#' # Get p-values of the reordered cor_mat
#' cor.mat %>%
#'   cor_reorder() %>%
#'   cor_get_pval()
#'
#' @name cor_reorder
#' @export
cor_reorder <- function(x){

  pvalue <- attr(x, "pvalue")
  x <- as_matrix(x)
  hc <- stats::as.dist(1 - x) %>%
    stats::hclust(method = "complete")

  x <- x %>% subset_matrix(hc$order)

  if(!is.null(pvalue)){
    pvalue <- pvalue %>% subset_matrix(hc$order)
    x <- x %>% set_attrs(pvalue = pvalue) %>%
      add_class("cor_mat")
  }
  x
}

#' @include utilities.R
NULL
#'Rounding and Formatting p-values
#'
#'@description Round and format p-values. Can also mark significant p-values with stars.
#'@param x a numeric vector of p-values or a data frame containing a p value
#'  column. If data frame, the p-value column(s) will be automatically detected.
#'  Known p-value column names can be obtained using the functions
#'  \code{p_names()} and \code{p_adj_names()}
#'@param digits the number of significant digits to be used.
#'@param accuracy number to round to, that is the threshold value above wich the
#'  function will replace the pvalue by "<0.0xxx".
#'@param decimal.mark the character to be used to indicate the numeric decimal
#'  point.
#'@param leading.zero logical. If FALSE, remove the leading zero.
#'@param trailing.zero logical. If FALSE (default), remove the training extra
#'  zero.
#'@param space logical. If TRUE (default) use space as separator between
#'  different elements and symbols.
#'@param cutpoints numeric vector used for intervals
#'@param symbols character vector, one shorter than cutpoints, used as
#'  significance symbols.
#'@param add.p logical value. If TRUE, add "p=" before the value.
#'@param ... column names to manipulate in the case where \code{x} is a data
#'  frame. P value columns are automatically detected if not specified.
#'@param new.col logical, used only when \code{x} is a data frame. If TRUE, add
#'  a new column to hold the results. The new column name is created by adding,
#'  to the p column, the suffix "format" (for \code{p_format()}), "signif" (for
#'  \code{p_mak_significant()}).
#'@return a vector or a data frame containing the rounded/formatted p-values.
#' @examples
#'
#' # Round and format a vector of p-values
#' # ::::::::::::::::::::::::::::::::::::::::::::
#' # Format
#' p <- c(0.5678, 0.127, 0.045, 0.011, 0.009, 0.00002, NA)
#' p_format(p)
#'
#'# Specify the accuracy
#' p_format(p, accuracy = 0.01)
#'
#' # Add p and remove the leading zero
#' p_format(p, add.p = TRUE, leading.zero = FALSE)
#'
#' # Remove space before and after "=" or "<".
#' p_format(p, add.p = TRUE, leading.zero = FALSE, space = FALSE)
#'
#' # Mark significant p-values
#' # ::::::::::::::::::::::::::::::::::::::::::::
#' p_mark_significant(p)
#'
#' # Round, the mark significant
#' p %>% p_round(digits = 2) %>% p_mark_significant()
#'
#' # Format, then mark significant
#' p %>% p_format(digits = 2) %>% p_mark_significant()
#'
#' # Perform stat test, format p and mark significant
#' # ::::::::::::::::::::::::::::::::::::::::::::
#' ToothGrowth %>%
#'   group_by(dose) %>%
#'   t_test(len ~ supp) %>%
#'   p_format(digits = 2, leading.zero = FALSE) %>%
#'   p_mark_significant()
#'
#'@describeIn p_value round p-values
#'@export
p_round <- function(x, ..., digits = 3){
  if(is.numeric(x)){
    round_value(x, digits = digits)
  }
  else if(is.data.frame(x)){
    p_round_at(x, ..., digits = digits)
  }
  else{
    stop("x should be a numeric vector or a data frame")
  }
}

#' @describeIn p_value format p-values. Add a symbol "<" for small p-values.
#' @export
p_format <- function(x, ..., new.col = FALSE, digits = 2, accuracy = 0.0001, decimal.mark = ".",
                     leading.zero = TRUE, trailing.zero = FALSE,
                     add.p = FALSE, space = FALSE){
  if(is.data.frame(x)){
    .attributes <- attributes(x)
    res <- x %>%
      keep_only_tbl_df_classes() %>%
      p_format_at(
      ..., new.col = new.col, digits = digits, accuracy = accuracy,
      decimal.mark = decimal.mark, leading.zero = leading.zero,
      trailing.zero = trailing.zero, add.p = add.p, space = space
      )
    .attributes$names <- colnames(res)
    attributes(res) <- .attributes
    return(res)
  }
  res <- format.pval(
    pv = x,
    digits = digits, eps = accuracy,
    # nsmall = how much tails 0 to keep if digits of
    # original value < to digits defined
    nsmall = 0
  )
  res <- gsub(pattern = " ", replacement = "", res, fixed = TRUE)
  res <- gsub("<1e-04", "<0.0001", res)
  if(!leading.zero)
    res <- remove_leading_zero(res)
  if(!trailing.zero)
    res <- remove_trailing_zero(res)
  if(!missing(decimal.mark))
    res <- gsub("\\.", decimal.mark, res)
  if(add.p){
    contain.inf.symbol <- grepl("<", res)
    res2 <- paste0("p", "=", res)
    if(sum(contain.inf.symbol) > 0){
      # no need to add =
      res2[contain.inf.symbol] <- paste0("p", res[contain.inf.symbol])
    }
    res <- res2
  }
  if(space){
    if(add.p)
      res <- gsub(pattern = "(=|<)", replacement = " \\1 ", x = res)
    else
      res <- gsub(pattern = "(=|<)", replacement = "\\1 ", x = res)
  }
  res
}

#' @describeIn p_value mark p-values with significance levels
#' @export
p_mark_significant <- function(x,  ..., new.col = FALSE, cutpoints = c(0, 1e-04, 0.001, 0.01, 0.05, 1),
                               symbols = c("****", "***", "**", "*", "")){
  if(is.data.frame(x)){
    .attributes <- attributes(x)
    res <- x %>%
      keep_only_tbl_df_classes() %>%
      p_mark_significant_at(
      ..., new.col = new.col, cutpoints = cutpoints,
      symbols = symbols
    )
    attributes(res) <- .attributes
    return(res)
  }
  contains.leading.zero <- TRUE
  is.char.x <- is.character(x)
  if(is.char.x){
    contains.leading.zero <- p_contains_leading_zero(x)
    leading.character <- replace_number(x, "")
    leading.character <- gsub("NA", "", leading.character)
    x <- extract_number(x)
  }
  x <- tibble(p = x) %>%
    add_significance("p", cutpoints = cutpoints, symbols = symbols) %>%
    mutate(.signif = paste0(.data$p, .data$p.signif)) %>%
    pull(".signif")
  if(!contains.leading.zero)
    x <- remove_leading_zero(x)
  if(is.char.x)
    x <- paste(leading.character, x, sep = "")
  x <- gsub("NA?", "NA", x, fixed = TRUE)
  x <- gsub("<1e-04", "<0.0001", x, fixed = TRUE)
  x
}
#' @describeIn p_value detects and returns p-value column names in a data frame.
#' @param data a data frame
#' @param type the type of p-value to detect. Can be one of \code{c("all", "p", "p.adj")}.
#' @export
p_detect <- function(data, type = c("all", "p", "p.adj")){
  type <- match.arg(type)
  p.cols <- switch (type,
                    all = c(p_adj_names(), p_names()),
                    p = p_names(), p.adj = p_adj_names()
  )
  existing.p.cols <- intersect(p.cols, colnames(data))
  if(.is_empty(existing.p.cols)) existing.p.cols <- NULL
  existing.p.cols
}

#' @describeIn p_value returns known p-value column names
#' @export
p_names <- function(){
  c("p", "pval", "pvalue", "p.val", "p.value")
}

#' @describeIn p_value returns known adjust p-value column names
#' @export
p_adj_names <- function(){
  p_names() %>% paste0(".adj")
}


# Rounding specified columns
p_round_at <- function(data, ..., digits = 3){
  p.cols <- p_select(data, ...)
  if(!is.null(p.cols)){
    data %<>% dplyr::mutate_at(rlang::quos(p.cols), round_value, digits = digits)
  }
  data
}
# Formatting (specified) p value columns
p_format_at <- function(data, ..., new.col = FALSE, digits = 2, accuracy = 0.0001, decimal.mark = ".",
                         leading.zero = TRUE, trailing.zero = FALSE,
                         add.p = FALSE, space = TRUE){
  mutate_func <- dplyr::mutate_at
  if(new.col) mutate_func <- dplyr::transmute_at
  results <- data
  p.cols <- p_select(data, ...)
  if(!is.null(p.cols)){
    results <- results %>% mutate_func(
      p.cols, p_format, digits = digits, accuracy = accuracy,
      decimal.mark = decimal.mark, leading.zero = leading.zero,
      trailing.zero = trailing.zero, add.p = add.p, space = space
    )
    if(new.col){
      colnames(results) <- paste0(p.cols, ".format")
      results <- dplyr::bind_cols(data, results)
    }
  }
  results
}
# Mark significant at a specifyied column
p_mark_significant_at <- function(data, ..., new.col = FALSE, cutpoints = c(0, 1e-04, 0.001, 0.01, 0.05, 1),
                               symbols = c("****", "***", "**", "*", "")){
  mutate_func <- dplyr::mutate_at
  if(new.col) mutate_func <- dplyr::transmute_at
  results <- data
  p.cols <- p_select(data, ...)
  if(!is.null(p.cols)){
    results %<>% mutate_func(
      p.cols, p_mark_significant, cutpoints = cutpoints,
      symbols = symbols
    )
    if(new.col){
      colnames(results) <- paste0(p.cols, ".signif")
      results <- dplyr::bind_cols(data, results)
    }
  }
  results
}

# Manipulating leading zero-----------------------------
# Check if formatted p-values contain leading zero
p_contains_leading_zero <- function(p){
  any(grepl(pattern = "0.", p, fixed = TRUE))
}
remove_leading_zero <- function(x){
  sapply(x, function(x){ sub("^([-|<|=|>]?)0[.]", "\\1.", x)}) %>%
    as.character()
}
remove_trailing_zero <- function(x){
  gsub("0+$", "", x)
}

# Select p-value columns: p and p.adj -----------------------
p_select <- function(data, ...){
  p.col <- get_existing_dot_vars(data, ...)
  if(is.null(p.col) | .is_empty(p.col)){
    p.col <- p_detect(data, type = "all")
  }
  p.col
}

context("test-tidyselect-deprecations")

# Guards against reintroducing tidyselect deprecation warnings
# (".data in tidyselect", "external vector in selections", all_of() misuse).
# See the package-wide cleanup referenced in NEWS (#202).
#
# NOTE: tidyselect/lifecycle suppress these deprecation signals for usage that
# originates *inside* a loaded package namespace, so in an ordinary local run
# this test passes whether or not the deprecated patterns are present. Its real
# teeth are under R-devel / `R CMD check` conditions (where #202 surfaced) and
# alongside the suite-wide "WARN 0" expectation. Kept as a CI-oriented guard.

# capture any tidyselect deprecation emitted while evaluating `expr`
tidyselect_deprecations <- function(expr) {
  rlang::local_options(lifecycle_verbosity = "warning")
  deps <- character(0)
  withCallingHandlers(
    force(expr),
    warning = function(w) {
      m <- conditionMessage(w)
      if (grepl("external vector|Use of .data in tidyselect|all_of\\(\\) outside", m)) {
        deps[[length(deps) + 1]] <<- m
      }
      invokeRestart("muffleWarning")
    }
  )
  deps
}

test_that("core test functions emit no tidyselect deprecation (#202)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  deps <- tidyselect_deprecations({
    df %>% anova_test(len ~ dose)
    df %>% anova_test(len ~ supp * dose)
    df %>% cohens_d(len ~ supp)
    df %>% wilcox_effsize(len ~ dose)
    df %>% dunn_test(len ~ dose)
    df %>% games_howell_test(len ~ dose)
    df %>% get_summary_stats(len)
    df %>% get_summary_stats(len, type = "mean_sd")
    df %>% levene_test(len ~ dose)
    df %>% shapiro_test(len)
    aov(len ~ dose, data = df) %>% tukey_hsd()
    df %>% freq_table(supp)
    binom_test(c(10, 90))
    chisq_test(as.table(rbind(c(10, 20, 30), c(15, 25, 20))))
    invisible(NULL)
  })
  expect_equal(deps, character(0))
})

test_that("p-value formatting and label helpers emit no tidyselect deprecation (#202)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  st <- df %>% t_test(len ~ dose)
  deps <- tidyselect_deprecations({
    st %>% p_round(digits = 2)
    st %>% p_format()
    st %>% p_format(new.col = TRUE)
    st %>% p_mark_significant()
    get_test_label(df %>% anova_test(len ~ dose), detailed = TRUE, type = "text")
    rm <- data.frame(id = factor(rep(1:5, 3)),
                     time = factor(rep(c("t1", "t2", "t3"), each = 5)),
                     score = stats::rnorm(15))
    get_test_label(anova_test(rm, dv = score, wid = id, within = time),
                   detailed = TRUE, type = "text")
    st %>% add_xy_position(x = "dose")
    convert_as_factor(df, supp, dose)
    invisible(NULL)
  })
  expect_equal(deps, character(0))
})

context("test-p_format")

test_that("p_format preserves scientific-notation exponents (#112)", {
  # the trailing 0 of "e-10" was wrongly stripped, giving "5.1e-1"
  expect_equal(p_format(5.1e-10, accuracy = 1e-15), "5.1e-10")
  expect_equal(p_format(5.1e-11, accuracy = 1e-15), "5.1e-11")
  expect_equal(p_format(1e-20, accuracy = 1e-30), "1e-20")
})

test_that("p_format is unchanged for ordinary decimal p-values (#112)", {
  expect_equal(p_format(c(0.1, 0.01)), c("0.1", "0.01"))
  # trailing.zero padding (its intended use) still works
  expect_equal(p_format(c(0.1, 0.01), trailing.zero = TRUE), c("0.10", "0.01"))
  expect_equal(p_format(c(0.5, 0.001, 0.049)), c("0.5", "0.001", "0.049"))
  # very small values below the accuracy threshold still collapse to the sentinel
  expect_equal(p_format(1e-8), "<0.0001")
})

test_that("extract_number parses scientific notation without dropping the exponent (#148)", {
  expect_equal(extract_number("1e-04"), 1e-4)
  expect_equal(extract_number("<0.0001"), 1e-4)
  # ordinary strings are unchanged
  expect_equal(extract_number("0.001"), 0.001)
  expect_equal(extract_number(c("0.05", "1e-04")), c(0.05, 1e-4))
})

context("test-p_mark_significance")

test_that("p_mark_significance works when NA only", {
  na_signif <- p_mark_significant(NA)
  expect_equal(na_signif, "NA")
})

test_that("p_mark_significant handles scientific-notation strings (#148)", {
  # The reported edge case: a formatted scientific p-value used to yield "e-NA"
  # with a coercion warning
  res <- expect_warning(
    data.frame(p = 1e-4) %>% p_format() %>% p_mark_significant(),
    regexp = NA
  )
  expect_equal(res$p, "1e-04****")
  expect_false(grepl("NA", res$p))
})

test_that("p_mark_significant is robust to mixed-magnitude scientific vectors (#148)", {
  # a tiny + a large p makes format.pval render the whole vector in scientific
  # notation; this used to yield "e-NA" for every element
  res <- expect_warning(
    data.frame(p = c(1e-4, 0.5)) %>% p_format() %>% p_mark_significant(),
    regexp = NA
  )
  expect_false(any(grepl("NA", res$p)))     # no coercion artefact
  expect_false(any(grepl("e-$|e$", res$p))) # no dangling exponent
  expect_true(grepl("\\*\\*\\*\\*$", res$p[1]))  # 1e-4 is highly significant
  expect_false(grepl("\\*", res$p[2]))            # 0.5 is not significant
})

test_that("p_mark_significant is unchanged for ordinary decimal strings (#148)", {
  expect_equal(p_mark_significant("0.001"), "0.001***")
  expect_equal(p_mark_significant("0.5"), "0.5")
  df <- data.frame(p = c(0.5, 0.001, 0.049)) %>% p_format() %>% p_mark_significant()
  expect_equal(df$p, c("0.5", "0.001***", "0.049*"))
})

#' @include utilities.R emmeans_test.R
NULL
#'Dunnett's Many-to-One Comparisons Test
#'
#'@description Performs Dunnett's test for comparing each of several treatment
#'  groups against a single control (reference) group. Unlike all-pairwise
#'  post-hoc tests, Dunnett's procedure controls the family-wise error rate over
#'  only the \code{k - 1} treatment-vs-control comparisons, using the exact
#'  multivariate-t distribution (which accounts for the correlation between the
#'  comparisons that share the control group).
#'
#'  This is a pipe-friendly wrapper around \code{emmeans::emmeans()} +
#'  \code{emmeans::contrast()} (with \code{adjust = "mvt"}), so the
#'  \code{emmeans} package must be installed. The results match
#'  \code{DescTools::DunnettTest()} and \code{multcomp::glht(..., mcp(... =
#'  "Dunnett"))}.
#'
#'@inheritParams t_test
#'@param ref.group a character string specifying the reference (control) group.
#'  Each remaining group level is compared against this group. If \code{NULL}
#'  (default), the first level of the grouping variable is used as the control.
#'@param conf.level confidence level of the (simultaneous) confidence intervals.
#'@return a data frame with some of the following columns: \itemize{ \item
#'  \code{.y.}: the outcome variable used in the test. \item
#'  \code{group1,group2}: the compared groups; \code{group1} is the control
#'  (reference) and \code{group2} is the treatment, consistent with the
#'  \code{ref.group} convention of \code{t_test()}/\code{wilcox_test()}/
#'  \code{dunn_test()}/\code{emmeans_test()}. \item \code{n1,n2}: sample sizes of
#'  the control and treatment groups. \item \code{estimate}: the estimated mean
#'  difference \code{group1 - group2} (control minus treatment). \item
#'  \code{conf.low,conf.high}: simultaneous confidence interval for the
#'  difference. \item \code{statistic}: the t-statistic. \item \code{df}:
#'  degrees of freedom. \item \code{p.adj}: the Dunnett-adjusted p-value. \item
#'  \code{method}: the statistical test used. \item \code{p.adj.signif}: the
#'  significance level of the adjusted p-value. } The estimate, confidence
#'  interval, se and method columns are returned only when \code{detailed =
#'  TRUE}.
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#'@references Dunnett, C. W. (1955) A multiple comparison procedure for comparing
#'  several treatments with a control. Journal of the American Statistical
#'  Association, 50, 1096-1121.
#'@seealso \code{\link{tukey_hsd}()}, \code{\link{games_howell_test}()},
#'  \code{\link{emmeans_test}()}
#' @examples
#' # Compare each dose to the control dose ("0.5")
#' ToothGrowth %>% dunnett_test(len ~ dose)
#'
#' # Detailed output (estimate + simultaneous confidence interval)
#' ToothGrowth %>% dunnett_test(len ~ dose, detailed = TRUE)
#'
#' # Grouped data
#' ToothGrowth %>%
#'   group_by(supp) %>%
#'   dunnett_test(len ~ dose)
#'@export
dunnett_test <- function(data, formula, ref.group = NULL, conf.level = 0.95,
                         detailed = FALSE){
  if(!is.null(ref.group)) ref.group <- as.character(ref.group)
  args <- as.list(environment()) %>%
    .add_item(method = "dunnett_test")
  if(is_grouped_df(data)){
    results <- data %>%
      doo(.dunnett_test, formula, ref.group = ref.group, conf.level = conf.level)
  }
  else{
    results <- .dunnett_test(data, formula, ref.group = ref.group, conf.level = conf.level)
  }
  if(!detailed){
    results <- results %>%
      select(-any_of(c("estimate", "conf.low", "conf.high", "se", "method", "null.value")))
  }
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "dunnett_test"))
}

.dunnett_test <- function(data, formula, ref.group = NULL, conf.level = 0.95){
  required_package("emmeans")
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  number.of.groups <- guess_number_of_groups(data, group)
  if(number.of.groups == 1){
    stop("all observations are in the same group")
  }

  data <- data %>%
    select(!!!syms(c(outcome, group))) %>%
    get_complete_cases()
  # Validate ref.group BEFORE releveling (.as_factor would otherwise raise a
  # cryptic relevel error for an unknown reference).
  available.levels <- data %>% .as_factor(group) %>% get_levels(group)
  if(!is.null(ref.group) && !(ref.group %in% available.levels)){
    stop(
      "Specified reference group ('", ref.group, "') is not a level of the ",
      "grouping variable. Valid levels are: ",
      paste(available.levels, collapse = ", "), ".",
      call. = FALSE
    )
  }
  data <- data %>% .as_factor(group, ref.group = ref.group)
  group.levels <- get_levels(data, group)
  if(is.null(ref.group)) ref.group <- group.levels[1]
  group.size <- data %>% get_group_size(group)

  # Each treatment vs the control, oriented as group1 = control (reference),
  # group2 = treatment, so estimate = control - treatment (= group1 - group2),
  # consistent with t_test()/wilcox_test()/dunn_test()/emmeans_test() ref.group.
  treatments <- setdiff(group.levels, ref.group)
  comparisons <- treatments %>% map(~c(ref.group, .x))
  contrasts <- get_emmeans_contrasts(data, group, comparisons)

  # Fit the model and apply the exact Dunnett (multivariate-t) adjustment.
  lm.formula <- stats::as.formula(paste(outcome, group, sep = " ~ "))
  model <- stats::lm(lm.formula, data)
  emm <- emmeans::emmeans(model, stats::as.formula(paste0("~", group)))
  res <- emmeans::contrast(emm, method = contrasts, adjust = "mvt") %>%
    broom::tidy(conf.int = TRUE, conf.level = conf.level) %>%
    keep_only_tbl_df_classes()

  # group1/group2 are taken from the (ordered) comparison list rather than parsed
  # from the contrast name, so factor levels containing "-" are handled correctly.
  # The contrast rows are returned in the same order as `contrasts`.
  group1 <- vapply(comparisons, function(x) x[[1]], character(1))
  group2 <- vapply(comparisons, function(x) x[[2]], character(1))
  # With a single contrast (k = 2) there is no multiplicity column: emmeans
  # returns "p.value" (which is the Dunnett p for one comparison) instead of
  # "adj.p.value". Use whichever is present.
  p.col <- intersect(c("adj.p.value", "p.value"), colnames(res))[1]
  colnames(res)[colnames(res) == "std.error"] <- "se"
  colnames(res)[colnames(res) == p.col] <- "p.adj"

  res %>%
    mutate(
      .y. = outcome, method = "Dunnett",
      group1 = group1, group2 = group2,
      n1 = group.size[group1], n2 = group.size[group2]
    ) %>%
    add_significance("p.adj") %>%
    select(all_of(c(
      ".y.", "group1", "group2", "n1", "n2", "estimate",
      "conf.low", "conf.high", "se", "statistic", "df", "p.adj",
      "p.adj.signif", "method"
    )))
}

context("test-dunnett_test")

test_that("dunnett_test compares each group to the control (#129)", {
  skip_if_not_installed("emmeans")
  res <- ToothGrowth %>% dunnett_test(len ~ dose)
  expect_equal(nrow(res), 2L)                       # k - 1 comparisons
  expect_true(all(res$group1 == "0.5"))             # control = first level (group1)
  expect_setequal(res$group2, c("1", "2"))          # treatments
  expect_equal(
    colnames(res),
    c(".y.", "group1", "group2", "n1", "n2", "statistic", "df", "p.adj", "p.adj.signif")
  )
})

test_that("dunnett_test matches emmeans trt.vs.ctrl with mvt adjustment (#129)", {
  skip_if_not_installed("emmeans")
  df <- ToothGrowth; df$dose <- factor(df$dose)
  emm <- emmeans::emmeans(stats::lm(len ~ dose, df), ~dose)
  ref <- as.data.frame(emmeans::contrast(emm, method = "trt.vs.ctrl", ref = 1, adjust = "mvt"))
  res <- ToothGrowth %>% dunnett_test(len ~ dose, detailed = TRUE)
  # rstatix orients control as group1 (estimate = control - treatment), so the
  # estimate/statistic are the negation of emmeans' trt.vs.ctrl; p.adj is identical.
  expect_equal(res$estimate, -ref$estimate, tolerance = 1e-6)
  expect_equal(res$statistic, -ref$t.ratio, tolerance = 1e-6)
  expect_equal(res$p.adj, ref$p.value, tolerance = 1e-6)
})

test_that("dunnett_test detailed returns estimate and simultaneous CI (#129)", {
  skip_if_not_installed("emmeans")
  res <- ToothGrowth %>% dunnett_test(len ~ dose, detailed = TRUE)
  expect_true(all(c("estimate", "conf.low", "conf.high", "se", "method") %in% colnames(res)))
  expect_true(all(res$estimate < 0))                # control - treatment (doses raise length)
  expect_equal(unique(res$method), "Dunnett")
})

test_that("dunnett_test respects ref.group (#129)", {
  skip_if_not_installed("emmeans")
  res <- ToothGrowth %>% dunnett_test(len ~ dose, ref.group = "2")
  expect_equal(nrow(res), 2L)
  expect_true(all(res$group1 == "2"))               # control = "2" (group1)
  expect_setequal(res$group2, c("0.5", "1"))
})

test_that("dunnett_test works on grouped data (#129)", {
  skip_if_not_installed("emmeans")
  res <- ToothGrowth %>% dplyr::group_by(supp) %>% dunnett_test(len ~ dose)
  expect_true("supp" %in% colnames(res))
  expect_equal(nrow(res), 4L)                        # 2 supp x 2 comparisons
})

test_that("get_description gives a friendly dunnett_test label (#129)", {
  skip_if_not_installed("emmeans")
  res <- ToothGrowth %>% dunnett_test(len ~ dose)
  expect_equal(get_description(res), "Dunnett test")   # not the raw "dunnett_test"
})

test_that("dunnett_test gives an informative error for an invalid ref.group (#129)", {
  skip_if_not_installed("emmeans")
  expect_error(
    ToothGrowth %>% dunnett_test(len ~ dose, ref.group = "9"),
    "not a level of the grouping variable"
  )
})

test_that("dunnett_test works with exactly two groups (k = 2) (#129)", {
  skip_if_not_installed("emmeans")
  set.seed(1)
  df <- data.frame(y = c(rnorm(12), rnorm(12) + 1), g = rep(c("ctrl", "trt"), each = 12))
  res <- df %>% dunnett_test(y ~ g, ref.group = "ctrl", detailed = TRUE)
  expect_equal(nrow(res), 1L)                        # single treatment vs control
  expect_equal(res$group1, "ctrl")                   # control = group1
  expect_equal(res$group2, "trt")
  expect_false(is.na(res$p.adj))
  # with one comparison the Dunnett p equals the plain emmeans p
  emm <- emmeans::emmeans(stats::lm(y ~ g, df), ~g)
  ref <- as.data.frame(emmeans::contrast(emm, method = list(c(-1, 1))))
  expect_equal(res$p.adj, ref$p.value, tolerance = 1e-6)
})

test_that("dunnett_test handles factor levels containing '-' (#129)", {
  skip_if_not_installed("emmeans")
  set.seed(1)
  df <- data.frame(
    y = c(rnorm(10), rnorm(10) + 1, rnorm(10) + 2),
    g = rep(c("ctrl", "low-dose", "high-dose"), each = 10)
  )
  res <- df %>% dunnett_test(y ~ g, ref.group = "ctrl")
  expect_equal(nrow(res), 2L)
  expect_true(all(res$group1 == "ctrl"))            # control label intact (group1)
  expect_setequal(res$group2, c("low-dose", "high-dose"))  # dashed labels intact
  expect_false(any(is.na(res$n2)))                  # n lookups succeed
  expect_equal(unique(res$n2), 10L)
})

# Note: correctness is validated above against emmeans' own trt.vs.ctrl + mvt
# contrast (a declared Suggests). DescTools / multcomp would be unstated test
# dependencies, so they are intentionally not used here (cf. R CMD check
# "unstated dependencies in tests").

#' @include utilities.R replace_triangle.R
NULL
#' Pull Lower and Upper Triangular Part of a Matrix
#' @description Returns the lower or the upper triangular part of a
#'   (correlation) matrix.
#' @param x a (correlation) matrix
#' @param diagonal logical. Default is FALSE. If TRUE, the matrix diagonal is
#'   included.
#' @param triangle the triangle to pull. Allowed values are one of
#'   "upper" and "lower".
#' @return an object of class \code{cor_mat_tri}, which is a data frame
#' @seealso \code{\link{replace_triangle}()}
#' @examples
#'
#' # Data preparation
#' #::::::::::::::::::::::::::::::::::::::::::
#' mydata <- mtcars %>%
#'   select(mpg, disp, hp, drat, wt, qsec)
#' head(mydata, 3)
#'
#' # Compute correlation matrix and pull triangles
#' #::::::::::::::::::::::::::::::::::::::::::
#' # Correlation matrix
#' cor.mat <- cor_mat(mydata)
#' cor.mat
#'
#' # Pull lower triangular part
#' cor.mat %>% pull_lower_triangle()
#'
#' # Pull upper triangular part
#' cor.mat %>% pull_upper_triangle()
#'
#'
#' @describeIn pull_triangle returns either the lower or upper triangular part of a matrix.
#' @export
pull_triangle <- function(x, triangle = c("lower", "upper"),
                          diagonal = FALSE){

  triangle.to.pull <- match.arg(triangle)
  triangle.to.replace <- ifelse(
    triangle.to.pull == "lower", "upper", "lower"
  )
  triangle.class <- paste0(triangle.to.pull, "_tri")

  res <- x %>%
    replace_triangle(triangle.to.replace, by = "", diagonal = diagonal) %>%
    add_class(triangle.class)
  res
}

#' @describeIn pull_triangle returns an object of class \code{upper_tri}, which
#'   is a data frame containing the upper triangular part of a matrix.
#' @export
pull_upper_triangle <- function(x,  diagonal = FALSE){
  x %>% pull_triangle("upper", diagonal = diagonal)
}

#' @describeIn pull_triangle returns an object of class \code{lower_tri}, which
#'   is a data frame containing the lower triangular part of a matrix.
#' @export
pull_lower_triangle <- function(x, diagonal = FALSE){
  x %>% pull_triangle("lower", diagonal = diagonal)
}

#' @include utilities.R
NULL
#'Cochran's Q Test
#'@description Performs the Cochran's Q test for unreplicated randomized block
#'  design experiments with a binary response variable and paired data. This
#'  test is analogue to the \code{\link{friedman.test}()} with 0,1 coded
#'  response. It's an extension of the McNemar Chi-squared test for comparing
#'  more than two paired proportions.
#'@param data a data frame containing the variables in the formula.
#'@param formula a formula of the form \code{a ~ b | c}, where \code{a} is the
#'  outcome variable name; b is the within-subjects factor variables; and c
#'  (factor) is the column name containing individuals/subjects identifier.
#'  Should be unique per individual.
#'@examples
#' # Generate a demo data
#' mydata <- data.frame(
#'   outcome = c(0,1,1,0,0,1,0,1,1,1,1,1,0,0,1,1,0,1,0,1,1,0,0,1,0,1,1,0,0,1),
#'   treatment = gl(3,1,30,labels=LETTERS[1:3]),
#'   participant = gl(10,3,labels=letters[1:10])
#' )
#' mydata$outcome <- factor(
#'   mydata$outcome, levels = c(1, 0),
#'   labels = c("success", "failure")
#'   )
#' # Cross-tabulation
#' xtabs(~outcome + treatment, mydata)
#'
#' # Compare the proportion of success between treatments
#' cochran_qtest(mydata, outcome ~ treatment|participant)
#'
#' # pairwise comparisons between groups
#' pairwise_mcnemar_test(mydata, outcome ~ treatment|participant)
#'
#'@export
cochran_qtest <- function(data, formula){
  args <- as.list(environment()) %>%
    add_item(method = "cochran_qtest")
 friedman_test(data, formula) %>%
   keep_only_tbl_df_classes() %>%
   mutate(method = "Cochran's Q test") %>%
   remove_class("friedman_test") %>%
   set_attrs(args = args) %>%
   add_class(c("rstatix_test", "cochran_qtest"))
}

# http://geai.univ-brest.fr/carpentier/2008-2009/Notes-Cochran-Q.pdf
exact_cochran_qtest <- function(data, formula, nboot = 500)
{
  # Data preparation
  data <- data %>% select(!!!syms(all.vars(formula)))
  colnames(data) <- c("outcome", "groups", "participant")
  nb.outcome <- length(unique(data$outcome))
  if(nb.outcome > 2 | nb.outcome == 1){
    stop("Unique possible outcome values should be 2")
  }
  # Convert outcome into 0/1
  if(!is.numeric(data$outcome)){
    data$outcome <- as.numeric(as.factor(data$outcome)) - 1
  }
  if(!all(unique(data$outcome) %in% c(0, 1))){
    stop("Outcome values should be 0 or 1")
  }
  data.wide <- data %>%
    spread(key = "groups", value = "outcome") %>%
    select(-.data$participant)
  nb.row <- nrow(data.wide)
  nb.col <- ncol(data.wide)

  results <- cochran_qtest(data, outcome ~ groups|participant)
  qobs <- results$statistic
  freq <- 0

  perm <- permutations(nb.col)
  # perm.list <- purrr::array_tree(perm)

  for (boot in 1:nboot) {
    data.permutated <- data.wide
    k <- 1+ as.integer(stats::runif(nb.row)*gamma(nb.col+1))
    for (j in 1:nb.row) {
      k <- 1+ as.integer(stats::runif(1)*gamma(nb.col+1))
      data.permutated[j,] <- data.wide[j, perm[k,]]
    }
    qperm <- get_cochran_q(data.permutated)
    if (qperm >= qobs) {freq <- freq + 1}}

  results %>%
    select(-.data$df) %>%
    mutate(
      p = freq/nboot,
      method = "Exact Cochran's Q test"
    )
}


# e1071::permutations
# Returns a matrix containing all permutations of the integers 1:n (one permutation per row).
permutations <- function (n)
{
  if (n == 1)
    return(matrix(1))
  else if (n < 2)
    stop("n must be a positive integer")
  z <- matrix(1)
  for (i in 2:n) {
    x <- cbind(z, i)
    a <- c(1:i, 1:(i - 1))
    z <- matrix(0, ncol = ncol(x), nrow = i * nrow(x))
    z[1:nrow(x), ] <- x
    for (j in 2:i - 1) {
      z[j * nrow(x) + 1:nrow(x), ] <- x[, a[1:i + j]]
    }
  }
  dimnames(z) <- NULL
  z
}


get_cochran_q <- function(data.wide){
  # Compute rows and column totals
  row.total <- apply(data.wide, 1, sum)
  column.total <- apply(data.wide, 2, sum)
  grand.total <- sum(data.wide)
  k <- ncol(data.wide)
  # Cochran's Q test statistic
  numerator <- sum((column.total - (grand.total/k))^2)
  denominator <- sum(row.total * (k - row.total))
  q = k*(k-1)*(numerator/denominator)
  q
}

#' @include utilities.R cor_mat.R
NULL
#' Reshape Correlation Data
#' @description Reshape correlation analysis results. Key functions: \itemize{
#'   \item \code{cor_gather()}: takes a correlation matrix and collapses (i.e. melt) it into a paired list
#'   (long format). \item \code{cor_spread()}: spread a long correlation data format across
#'   multiple columns. Particularly, it takes the results of \code{\link{cor_test}}
#'   and transforms it into a correlation matrix. }
#' @param data a data frame or matrix.
#' @param drop.na logical. If TRUE, drop rows containing missing values after gathering the data.
#' @param value column name containing the value to spread.
#' @seealso \code{\link{cor_mat}()}, \code{\link{cor_reorder}()}
#' @examples
#' # Data preparation
#' #::::::::::::::::::::::::::::::::::::::::::
#' mydata <- mtcars %>%
#'   select(mpg, disp, hp, drat, wt, qsec)
#' head(mydata, 3)
#'
#' # Reshape a correlation matrix
#' #::::::::::::::::::::::::::::::::::::::::::
#' # Compute a correlation matrix
#' cor.mat <- mydata %>% cor_mat()
#' cor.mat
#'
#' # Collapse the correlation matrix into long format
#' # paired list data frame
#' long.format <- cor.mat %>% cor_gather()
#' long.format
#'
#' # Spread a correlation data format
#' #::::::::::::::::::::::::::::::::::::::::::
#' # Spread the correlation coefficient value
#' long.format %>% cor_spread(value = "cor")
#' # Spread the p-value
#' long.format %>% cor_spread(value = "p")


#' @describeIn cor_reshape takes a correlation matrix and collapses (or melt) it into long
#'   format data frame (paired list)
#' @export
cor_gather <- function(data, drop.na = TRUE){

  rowname <- column <- NULL

  if(inherits(data, "cor_mat")){
    cor.value <- data
    p.value <- data %>% cor_get_pval()
  }
  else if(inherits(data, "cor_mat_tri")){
    cor.value <- data %>% as_numeric_triangle()
    p.value <- data %>%
      cor_get_pval() %>%
      as_numeric_triangle()
  }
  else if(inherits(data, "rcorr")){
    cor.value <- data$r %>% as_tibble(rownames = "rowname")
    p.value <- data$P %>% as_tibble(rownames = "rowname")
  }
  else {
    cor.value <- data %>% as_tibble(rownames = "rowname")
    p.value <- NULL
  }

  cor.value <- cor.value %>%
    keep_only_tbl_df_classes() %>%
    gather(key = "column", value = "cor", -rowname)

  if(!is.null(p.value)){
    p.value <- p.value %>%
      keep_only_tbl_df_classes() %>%
      gather(key = "column", value = "p", -rowname)
    cor.value <- cor.value %>%
      left_join(p.value, by = c("rowname", "column"))
    colnames(cor.value) <- c("var1", "var2", "cor", "p")
  }
  else{
    colnames(cor.value) <- c("var1", "var2", "cor")
  }

  if(drop.na)
    cor.value <- cor.value %>% drop_na()

  cor.value
}


#' @describeIn cor_reshape spread a long correlation data frame into wide format
#'   (correlation matrix).
#' @export
cor_spread <- function(data, value = "cor"){

  var1 <- var2 <- cor <- p <- NULL
  row.vars <- data %>% pull(var1) %>% unique()
  col.vars <- data %>% pull(var2) %>% unique()

  res <- data %>%
    keep_only_tbl_df_classes() %>%
    select(var1, var2, !!value) %>%
    spread(key = "var2", value = value) %>%
    rename(rowname = var1) %>%
    respect_variables_order(row.vars = row.vars, col.vars = col.vars)

  colnames(res)[1] <- "rowname"

  res
}


# Helper functions
# :::::::::::::::::::::::::::::::::::::::::::::::::::::

# Reorder a correlation matrix according
# to the order of variables in vars
respect_variables_order <- function(data, vars, row.vars = vars, col.vars = vars){
  data %>% subset_matrix(row.vars = row.vars, col.vars = col.vars)
}

#' @include utilities.R
NULL
#'Compact Letter Display of All-Pairwise Comparisons
#'
#'@description Adds the \strong{compact letter display} (CLD) to a pairwise
#'  comparison result. Groups that do \emph{not} share a letter are
#'  significantly different. This is a convenient way to annotate plots
#'  (e.g. one letter per box/bar) after an all-pairwise post-hoc test such as
#'  \code{\link{tukey_hsd}()}, \code{\link{dunn_test}()},
#'  \code{\link{games_howell_test}()}, \code{\link{conover_test}()},
#'  \code{\link{wilcox_test}()} or \code{\link{t_test}()}.
#'
#'  The letters are computed with the insert-and-absorb algorithm (Piepho, 2004)
#'  using base R only, so no additional package is required (the results match
#'  \code{multcompView::multcompLetters()}).
#'
#'@param test an all-pairwise comparison result returned by an \code{rstatix}
#'  function (e.g. \code{tukey_hsd()}, \code{dunn_test()}, a pairwise
#'  \code{t_test()}/\code{wilcox_test()}, ...). Must contain the \code{group1}
#'  and \code{group2} columns and a p-value column.
#'@param p.col character. The p-value column to threshold. If \code{NULL}
#'  (default), \code{"p.adj"} is used when present, otherwise \code{"p"}.
#'@param threshold the significance threshold (default 0.05). Comparisons with a
#'  p-value below \code{threshold} are treated as significant; comparisons with a
#'  missing (\code{NA}) p-value are treated as non-significant.
#'@param reversed logical. If \code{TRUE}, reverses the order in which the
#'  letters are assigned (so that, with groups ordered by increasing level, the
#'  later groups receive the earlier letters). Default is \code{FALSE}.
#'@param ... not used.
#'@return a tibble with one row per group and the following columns: any grouping
#'  variables (for a grouped test), \code{.y.} (the outcome variable, when
#'  present), \code{group} (the group level) and \code{cld} (the compact letter
#'  display). Groups sharing a letter are not significantly different.
#'@references Piepho, H.-P. (2004) An Algorithm for a Letter-Based Representation
#'  of All-Pairwise Comparisons. Journal of Computational and Graphical
#'  Statistics, 13(2), 456-466.
#'@seealso \code{\link{tukey_hsd}}, \code{\link{dunn_test}},
#'  \code{\link{games_howell_test}}, \code{\link{add_significance}}
#' @examples
#' # Tukey HSD post-hoc, then compact letter display
#' res <- ToothGrowth %>%
#'   mutate(dose = factor(dose)) %>%
#'   tukey_hsd(len ~ dose)
#' res %>% add_cld()
#'
#' # Works on rank-based post-hocs too
#' ToothGrowth %>% dunn_test(len ~ dose) %>% add_cld()
#'
#' # Grouped pairwise test -> one CLD per group
#' ToothGrowth %>%
#'   mutate(dose = factor(dose)) %>%
#'   group_by(supp) %>%
#'   tukey_hsd(len ~ dose) %>%
#'   add_cld()
#'@name add_cld
#'@export
add_cld <- function(test, p.col = NULL, threshold = 0.05, reversed = FALSE, ...){
  if(!all(c("group1", "group2") %in% colnames(test))){
    stop("add_cld() requires an all-pairwise comparison result with 'group1' ",
         "and 'group2' columns.", call. = FALSE)
  }
  if(is.null(p.col)){
    p.col <- intersect(c("p.adj", "p"), colnames(test))[1]
  }
  if(is.na(p.col) || !(p.col %in% colnames(test))){
    stop("add_cld(): no p-value column found. Specify one with `p.col`.",
         call. = FALSE)
  }
  data <- keep_only_tbl_df_classes(test)
  # All columns before 'group1' identify the comparison context: any group_by()
  # grouping variables plus the outcome/term identifier ('.y.' for most tests,
  # 'term' for tukey_hsd()). Splitting by all of them computes one compact letter
  # display per group_by() group and keeps those columns in the output. (Within a
  # single test the outcome/term is constant, so including it is harmless.)
  g1.index <- match("group1", colnames(data))
  split.vars <- if(g1.index > 1) colnames(data)[seq_len(g1.index - 1)] else character(0)
  if(length(split.vars) == 0){
    results <- .add_cld_core(data, p.col, threshold, reversed)
  }
  else{
    results <- data %>%
      group_by(!!!syms(split.vars)) %>%
      doo(.add_cld_core, p.col = p.col, threshold = threshold, reversed = reversed)
  }
  results %>% add_class("rstatix_test")
}

# Compute the compact letter display for a single ungrouped pairwise table.
.add_cld_core <- function(data, p.col, threshold = 0.05, reversed = FALSE){
  raw1 <- data$group1; raw2 <- data$group2
  group1 <- as.character(raw1)
  group2 <- as.character(raw2)
  # Display/letter order is deterministic: follow the factor levels when the
  # group columns are factors, otherwise the order of first appearance (group1
  # then group2). rstatix test outputs are already level-ordered.
  if(is.factor(raw1) || is.factor(raw2)){
    levs <- union(levels(as.factor(raw1)), levels(as.factor(raw2)))
    groups <- levs[levs %in% c(group1, group2)]
  }
  else {
    groups <- unique(c(group1, group2))
  }
  # The compact letter display is only meaningful for an ALL-pairwise input.
  # A reduced set (e.g. a ref.group result with only k - 1 comparisons) would
  # treat the missing pairs as non-significant and produce a misleading display.
  # Count DISTINCT unordered pairs (not rows) so a duplicated comparison can't
  # mask a genuinely missing one.
  n.expected <- choose(length(groups), 2)
  n.present <- length(unique(paste(pmin(group1, group2), pmax(group1, group2))))
  if(n.present < n.expected){
    warning(
      "add_cld(): the input has ", n.present, " of the ", n.expected,
      " pairwise comparisons needed for a complete display of ", length(groups),
      " groups. Missing comparisons are treated as non-significant, which can ",
      "make the compact letter display misleading (e.g. for a `ref.group` result).",
      call. = FALSE
    )
  }
  pvals <- data[[p.col]]
  is.sig <- !is.na(pvals) & pvals < threshold
  sig.pairs <- mapply(
    function(a, b) c(a, b), group1[is.sig], group2[is.sig],
    SIMPLIFY = FALSE, USE.NAMES = FALSE
  )
  letters.map <- .cld_letters(groups, sig.pairs, reversed = reversed)
  tibble(group = groups, cld = unname(letters.map[groups]))
}

# Insert-and-absorb letter assignment (Piepho, 2004). `groups` is a character
# vector (defines the display order); `sig.pairs` a list of c(a, b) significant
# pairs. Returns a named character vector of letters, one per group. Matches
# multcompView::multcompLetters().
.cld_letters <- function(groups, sig.pairs, reversed = FALSE){
  if(length(groups) == 0) return(stats::setNames(character(0), character(0)))
  # Each "column" is a set of groups that are all mutually non-significant.
  columns <- list(groups)
  for(pair in sig.pairs){
    a <- pair[1]; b <- pair[2]
    new.columns <- list()
    for(col in columns){
      if(a %in% col && b %in% col){
        # split: remove a from one copy, b from the other
        new.columns[[length(new.columns) + 1]] <- setdiff(col, a)
        new.columns[[length(new.columns) + 1]] <- setdiff(col, b)
      }
      else {
        new.columns[[length(new.columns) + 1]] <- col
      }
    }
    # drop empty columns and exact duplicates
    new.columns <- Filter(function(x) length(x) > 0, new.columns)
    new.columns <- new.columns[!duplicated(
      lapply(new.columns, function(x) paste(sort(x), collapse = "\\u0001"))
    )]
    # absorb: drop any column whose groups are a strict subset of another column
    keep <- rep(TRUE, length(new.columns))
    for(i in seq_along(new.columns)){
      for(j in seq_along(new.columns)){
        if(i != j && keep[i] && keep[j] &&
           all(new.columns[[i]] %in% new.columns[[j]]) &&
           length(new.columns[[i]]) < length(new.columns[[j]])){
          keep[i] <- FALSE
        }
      }
    }
    columns <- new.columns[keep]
  }
  # order columns by the first-appearing group so letters are stable and read
  # left-to-right with the group order
  first.pos <- sapply(columns, function(col) min(match(col, groups)))
  columns <- columns[order(first.pos)]
  if(reversed) columns <- rev(columns)
  # Letter labels MUST stay single characters so the concatenated cld string
  # remains tokenizable (e.g. "ab" = letters a and b). Extend past z into A..Z
  # for the rare case of > 26 columns (matching multcompView's behavior).
  n.col <- length(columns)
  alphabet <- c(letters, LETTERS)
  if(n.col > length(alphabet)){
    stop("add_cld() supports at most ", length(alphabet), " letter groups; ",
         "this comparison needs ", n.col, ".", call. = FALSE)
  }
  labels <- alphabet[seq_len(n.col)]
  letters.map <- stats::setNames(rep("", length(groups)), groups)
  for(ci in seq_along(columns)){
    this.letter <- labels[ci]
    for(g in columns[[ci]]){
      letters.map[g] <- paste0(letters.map[g], this.letter)
    }
  }
  letters.map
}

context("test-add_cld")

# Build a minimal pairwise-comparison tibble (one .y., explicit p.adj) for a set
# of significant pairs; non-listed pairs are non-significant.
make_pairwise <- function(groups, sig_pairs, y = "y"){
  pairs <- utils::combn(groups, 2, simplify = FALSE)
  is.sig <- vapply(pairs, function(p){
    any(vapply(sig_pairs, function(s) setequal(s, p), logical(1)))
  }, logical(1))
  tibble::tibble(
    .y.    = y,
    group1 = vapply(pairs, `[`, character(1), 1),
    group2 = vapply(pairs, `[`, character(1), 2),
    p.adj  = ifelse(is.sig, 0.001, 0.5)
  )
}

# Expected letters below were cross-checked against
# multcompView::multcompLetters() (a development-time reference, not a dependency).

test_that("add_cld assigns distinct letters when all pairs differ (#110)", {
  res <- make_pairwise(c("A", "B", "C"), list(c("A", "B"), c("A", "C"), c("B", "C"))) %>%
    add_cld()
  expect_equal(res$group, c("A", "B", "C"))
  expect_equal(res$cld, c("a", "b", "c"))
})

test_that("add_cld assigns a shared letter when all pairs are non-significant (#110)", {
  res <- make_pairwise(c("A", "B", "C"), list()) %>% add_cld()
  expect_equal(res$cld, c("a", "a", "a"))
})

test_that("add_cld handles the classic overlapping a/ab/b case (#110)", {
  # A-C significant; A-B and B-C non-significant
  res <- make_pairwise(c("A", "B", "C"), list(c("A", "C"))) %>% add_cld()
  expect_equal(res$cld, c("a", "ab", "b"))
})

test_that("add_cld handles a chain topology a/ab/bc/c (#110)", {
  res <- make_pairwise(
    c("A", "B", "C", "D"),
    list(c("A", "C"), c("A", "D"), c("B", "D"))
  ) %>% add_cld()
  expect_equal(res$cld, c("a", "ab", "bc", "c"))
})

test_that("add_cld handles two clusters (#110)", {
  res <- make_pairwise(
    c("A", "B", "C", "D"),
    list(c("A", "C"), c("A", "D"), c("B", "C"), c("B", "D"))
  ) %>% add_cld()
  expect_equal(res$cld, c("a", "a", "b", "b"))
})

test_that("add_cld matches tukey_hsd / dunn_test on ToothGrowth (#110)", {
  tg <- ToothGrowth; tg$dose <- factor(tg$dose)
  tuk <- tg %>% tukey_hsd(len ~ dose) %>% add_cld()
  expect_equal(tuk$group, c("0.5", "1", "2"))
  expect_equal(tuk$cld, c("a", "b", "c"))     # all doses differ
  dn <- ToothGrowth %>% dunn_test(len ~ dose) %>% add_cld()
  expect_equal(dn$cld, c("a", "b", "c"))
})

test_that("add_cld computes one display per group for grouped tests (#110)", {
  tg <- ToothGrowth; tg$dose <- factor(tg$dose)
  res <- tg %>%
    dplyr::group_by(supp) %>%
    tukey_hsd(len ~ dose) %>%
    add_cld()
  expect_true(all(c("supp", "group", "cld") %in% colnames(res)))
  expect_equal(nrow(res), 6L)                 # 2 supp x 3 doses
  # For OJ, doses 1 and 2 are not significantly different (p.adj = 0.13) -> share a letter
  oj <- res[res$supp == "OJ", ]
  expect_equal(oj$cld, c("a", "b", "b"))
  vc <- res[res$supp == "VC", ]
  expect_equal(vc$cld, c("a", "b", "c"))
})

test_that("add_cld supports p.col and reversed (#110)", {
  dn <- ToothGrowth %>% dunn_test(len ~ dose)
  # use raw p instead of p.adj
  res_p <- dn %>% add_cld(p.col = "p")
  expect_equal(res_p$cld, c("a", "b", "c"))
  # reversed letter assignment
  res_rev <- dn %>% add_cld(reversed = TRUE)
  expect_equal(res_rev$cld, c("c", "b", "a"))
})

test_that("add_cld respects the significance threshold (#110)", {
  # p.adj for 1-vs-2 in OJ is ~0.13; a higher threshold makes it significant
  tg <- ToothGrowth; tg$dose <- factor(tg$dose)
  oj <- tg %>% dplyr::filter(supp == "OJ") %>% tukey_hsd(len ~ dose)
  expect_equal((oj %>% add_cld(threshold = 0.05))$cld, c("a", "b", "b"))
  expect_equal((oj %>% add_cld(threshold = 0.20))$cld, c("a", "b", "c"))
})

test_that("add_cld keeps single-character labels beyond 26 groups (#110)", {
  # 30 mutually-significant groups -> 30 distinct single-character letters; the
  # cld string must stay tokenizable (no multi-char labels that would corrupt it)
  g <- sprintf("G%02d", 1:30)
  res <- make_pairwise(g, utils::combn(g, 2, simplify = FALSE)) %>% add_cld()
  expect_equal(length(unique(res$cld)), 30L)
  expect_true(all(nchar(res$cld) == 1L))
  expect_false(any(duplicated(res$cld)))   # all differ -> no shared letters
})

test_that("add_cld errors clearly beyond 52 letter groups (#110)", {
  g <- sprintf("G%03d", 1:53)
  df <- make_pairwise(g, utils::combn(g, 2, simplify = FALSE))
  expect_error(suppressWarnings(add_cld(df)), "at most 52")
})

test_that("add_cld warns on an incomplete (e.g. ref.group) comparison set (#110)", {
  dn <- ToothGrowth %>% dunn_test(len ~ dose, ref.group = "0.5")   # only k-1 pairs
  expect_warning(dn %>% add_cld(), "all-pairwise|misleading")
})

test_that("add_cld orders groups by factor levels when group columns are factors (#110)", {
  df <- tibble::tibble(
    .y.    = "y",
    group1 = factor(c("lo", "lo", "mid"), levels = c("lo", "mid", "hi")),
    group2 = factor(c("mid", "hi", "hi"), levels = c("lo", "mid", "hi")),
    p.adj  = c(0.001, 0.001, 0.001)
  )
  res <- df %>% add_cld()
  expect_equal(res$group, c("lo", "mid", "hi"))    # factor-level order, not appearance
})

test_that("add_cld errors clearly on invalid input (#110)", {
  expect_error(tibble::tibble(x = 1) %>% add_cld(), "group1")
  expect_error(
    tibble::tibble(group1 = "a", group2 = "b") %>% add_cld(),
    "p-value column"
  )
})

#' @include utilities.R
NULL
#' Replace Correlation Coefficients by Symbols
#'
#' @description Take a correlation matrix and replace the correlation coefficients by symbols according to the
#'   level of the correlation.
#' @param x a correlation matrix. Particularly, an object of class \code{cor_mat}.
#' @param cutpoints numeric vector used for intervals. Default values are
#'  \code{c(0, 0.25, 0.5, 0.75, 1)}.
#' @param symbols character vector, one shorter than cutpoints, used as
#'  correlation coefficient symbols. Default values are \code{c(" ", ".",  "+",
#'  "*")}.
#' @seealso \code{\link{cor_mat}()}
#' @examples
#' # Compute correlation matrix
#' #::::::::::::::::::::::::::::::::::::::::::
#' cor.mat <- mtcars %>%
#'   select(mpg, disp, hp, drat, wt, qsec) %>%
#'   cor_mat()
#'
#' # Replace correlation coefficient by symbols
#' #::::::::::::::::::::::::::::::::::::::::::
#' cor.mat %>%
#'   cor_as_symbols() %>%
#'   pull_lower_triangle()
#'
#' @name cor_as_symbols
#' @export
cor_as_symbols <- function( x, cutpoints = c(0, 0.25, 0.5, 0.75, 1),
                            symbols = c(" ", ".",  "+", "*"))
{


  if(inherits(x, "cor_mat_tri")){
    cor.mat <- x %>%
      replace_empty_by(0) %>%
      as_numeric_triangle() %>%
      as_matrix()
  }
  else{
    cor.mat <- as_matrix(x)
  }

  res <- stats::symnum(
    abs(cor.mat), cutpoints = cutpoints, symbols = symbols,
    abbr.colnames = FALSE
  ) %>%
    structure(class = "matrix") %>% # overwrite "noquote" class
    matrix_to_dataframe() %>%
    add_class(c("data.frame", "tbl_df"))

  pvalue <- attr(x, "pvalue")
  if(!is.null(pvalue)){
    res <- res %>%
      set_attrs(pvalue = pvalue) %>%
      add_class("cor_mat")
  }

  res
}

#' @include utilities.R
#'
NULL
#'Compute Cramer's V
#'@description Compute Cramer's V, which measures the strength of the
#'  association between categorical variables.
#'@inheritParams stats::chisq.test
#'@param ... other arguments passed to the function
#'  \code{\link[stats]{chisq.test}()}.
#'@examples
#'
#' # Data preparation
#' df <- as.table(rbind(c(762, 327, 468), c(484, 239, 477)))
#' dimnames(df) <- list(
#'   gender = c("F", "M"),
#'   party = c("Democrat","Independent", "Republican")
#' )
#' df
#' # Compute cramer's V
#' cramer_v(df)
#'
#'@export
cramer_v <- function(x, y = NULL, correct = TRUE, ...) {
  test <- stats::chisq.test(x, y, correct = correct, ...)
  chi2 <- test$statistic
  N <- sum(test$observed)
  k <- min(dim(test$observed))
  V <- sqrt(chi2/(N * (k - 1)))
  as.numeric(V)
}

#' @include utilities.R utilities_two_sample_test.R
NULL
#'T-test
#'
#'
#'@description Provides a pipe-friendly framework to performs one and two sample
#'  t-tests. Read more: \href{https://www.datanovia.com/en/lessons/t-test-in-r/}{T-test in R}.
#'@inheritParams stats::t.test
#'@param data a data.frame containing the variables in the formula.
#'@param formula a formula of the form \code{x ~ group} where \code{x} is a
#'  numeric variable giving the data values and \code{group} is a factor with
#'  one or multiple levels giving the corresponding groups. For example,
#'  \code{formula = TP53 ~ cancer_group}.
#'@param paired a logical indicating whether you want a paired test.
#'@param ref.group a character string specifying the reference group. If
#'  specified, for a given grouping variable, each of the group levels will be
#'  compared to the reference group (i.e. control group).
#'
#'  If \code{ref.group = "all"}, pairwise two sample tests are performed for
#'  comparing each grouping variable levels against all (i.e. basemean).
#'@param mu a number specifying an optional parameter used to form the null hypothesis.
#'@param comparisons A list of length-2 vectors specifying the groups of
#'  interest to be compared. For example to compare groups "A" vs "B" and "B" vs
#'  "C", the argument is as follow: \code{comparisons = list(c("A", "B"), c("B",
#'  "C"))}
#'@param p.adjust.method method to adjust p values for multiple comparisons.
#'  Used when pairwise comparisons are performed. Allowed values include "holm",
#'  "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
#'  want to adjust the p value (not recommended), use p.adjust.method = "none".
#'@param pool.sd logical value used in the function \code{pairwise_t_test()}.
#'  Switch to allow/disallow the use of a pooled SD.
#'
#'  The \code{pool.sd = TRUE} (default) calculates a common SD for all groups
#'  and uses that for all comparisons (this can be useful if some groups are
#'  small). This method does not actually call t.test, so extra arguments are
#'  ignored. Pooling does not generalize to paired tests so pool.sd and paired
#'  cannot both be TRUE.
#'
#'  If \code{pool.sd = FALSE} the standard two sample t-test is applied to all
#'  possible pairs of groups. This method calls the \code{t.test()}, so extra
#'  arguments, such as \code{var.equal} are accepted.
#'
#'@param detailed logical value. Default is FALSE. If TRUE, a detailed result is
#'  shown.
#'@param ... other arguments to be passed to the function
#'  \code{\link[stats]{t.test}}.
#'
#'@details
#'
#'- If a list of comparisons is specified, the result of the pairwise tests is
#'filtered to keep only the comparisons of interest. The p-value is adjusted
#'after filtering.
#'
#'- For a grouped data, if pairwise test is performed, then the p-values are
#'adjusted for each group level independently.
#'
#'@return return a data frame with some the following columns: \itemize{ \item
#'  \code{.y.}: the y variable used in the test. \item \code{group1,group2}: the
#'  compared groups in the pairwise tests. \item \code{n,n1,n2}: Sample counts.
#'  \item \code{statistic}: Test statistic used to compute the p-value. \item
#'  \code{df}: degrees of freedom. \item \code{p}: p-value. \item \code{p.adj}:
#'  the adjusted p-value. \item \code{method}: the statistical test used to
#'  compare groups. \item \code{p.signif, p.adj.signif}: the significance level
#'  of p-values and adjusted p-values, respectively. \item \code{estimate}:
#'  estimate of the effect size. It corresponds to the estimated mean or
#'  difference in means depending on whether it was a one-sample test or a
#'  two-sample test. \item \code{estimate1, estimate2}: show the mean values of
#'  the two groups, respectively, for independent samples t-tests. \item
#'  \code{alternative}: a character string describing the alternative
#'  hypothesis. \item \code{conf.low,conf.high}: Lower and upper bound on a
#'  confidence interval. }
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#' @examples
#' # Load data
#' #:::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # One-sample test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% t_test(len ~ 1, mu = 0)
#'
#'
#' # Two-samples unpaired test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% t_test(len ~ supp)
#'
#' # Two-samples paired test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% t_test (len ~ supp, paired = TRUE)
#'
#' # Compare supp levels after grouping the data by "dose"
#' #::::::::::::::::::::::::::::::::::::::::
#' df %>%
#'   group_by(dose) %>%
#'   t_test(data =., len ~ supp) %>%
#'   adjust_pvalue(method = "bonferroni") %>%
#'   add_significance("p.adj")
#'
#' # pairwise comparisons
#' #::::::::::::::::::::::::::::::::::::::::
#' # As dose contains more than two levels ==>
#' # pairwise test is automatically performed.
#' df %>% t_test(len ~ dose)
#'
#' # Comparison against reference group
#' #::::::::::::::::::::::::::::::::::::::::
#' # each level is compared to the ref group
#' df %>% t_test(len ~ dose, ref.group = "0.5")
#'
#' # Comparison against all
#' #::::::::::::::::::::::::::::::::::::::::
#' df %>% t_test(len ~ dose, ref.group = "all")
#'
#'@describeIn t_test t test
#'@export
t_test <- function(
  data, formula, comparisons = NULL, ref.group = NULL,
  p.adjust.method = "holm",
  paired = FALSE, var.equal = FALSE, alternative = "two.sided",
  mu = 0, conf.level = 0.95, detailed = FALSE
)
{
  env <- as.list(environment())
  args <- env %>%
    .add_item(method = "t_test")
  params <- env %>%
    remove_null_items() %>%
    add_item(method = "t.test")

  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  number.of.groups <- guess_number_of_groups(data, group)
  if(number.of.groups > 2 & !is.null(ref.group)){
    if(ref.group %in% c("all", ".all.")){
      params$data <- create_data_with_all_ref_group(data, outcome, group)
      params$ref.group <- "all"
    }
  }
  test.func <- two_sample_test
  if(number.of.groups > 2)
    test.func <- pairwise_two_sample_test
  do.call(test.func, params) %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "t_test"))
}



#'@describeIn t_test performs pairwise two sample t-test. Wrapper around the R
#'  base function \code{\link[stats]{pairwise.t.test}}.
#'@export
pairwise_t_test <- function(
  data, formula, comparisons = NULL, ref.group = NULL,
  p.adjust.method = "holm", paired = FALSE, pool.sd = !paired,
  detailed = FALSE, ...) {

  args <- c(as.list(environment()), list(...)) %>%
    .add_item(method = "t_test")
  if(paired) pool.sd <- FALSE
  if(pool.sd){
    res <- pairwise_t_test_psd(
      data, formula, comparisons = comparisons, ref.group = ref.group,
      p.adjust.method = p.adjust.method, detailed = detailed, ...
    )
  }
  else{
    res <- pairwise_two_sample_test(
      data, formula, method = "t.test",
      comparisons = comparisons, ref.group = ref.group,
      p.adjust.method = p.adjust.method, paired = paired,
      detailed = detailed, ...
    )
  }
  res %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "t_test"))
  }


pairwise_t_test_psd <- function(
  data, formula, comparisons = NULL, ref.group = NULL,
  p.adjust.method = "holm", alternative = "two.sided",
  detailed = FALSE
  )
  {
  . <- NULL
  if(is_grouped_df(data)){
    results <- data %>%
      doo(pairwise_t_test_psd, formula, comparisons,
          ref.group, p.adjust.method, alternative = alternative,
          detailed = detailed)
    return(results)
  }

  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  # Convert group into factor if this is not already the case
  data <- data %>% .as_factor(group, ref.group = ref.group)
  outcome.values <- data %>% pull(!!outcome)
  group.values <- data %>% pull(!!group)
  group.size <- data %>% get_group_size(group)

  # Compute pairwise t-test
  group1 <- group2 <- p.value <- NULL
  results <- stats::pairwise.t.test(
    outcome.values, group.values,
    p.adjust.method = "none", pool.sd = TRUE,
    alternative = alternative
    ) %>%
    tidy() %>%
    select(group2, group1, p.value)
  colnames(results) <- c("group1", "group2", "p")
  n1 <- group.size[results$group1]
  n2 <- group.size[results$group2]

  results <- results %>%
    mutate(method = "T-test") %>%
    add_column(.y. = outcome, .before = 1) %>%
    add_column(n1 = n1, n2 = n2, .after = "group2")

  # If ref.group specified, keep only comparisons against reference
  if(!is.null(ref.group)){
    results <- results %>%
      filter(group1 == ref.group)
  }

  # If a comparison list is provided, extract the comparisons of interest
  if(!is.null(comparisons)){
    results <- comparisons %>%
      purrr::map_dfr(~ results %>% filter(group1 %in% .x & group2 %in% .x) )
  }

  p <- p.adj <- NULL
  results <- results %>%
    adjust_pvalue(method = p.adjust.method) %>%
    add_significance("p") %>%
    add_significance("p.adj") %>%
    mutate(
      p = signif(p, digits = 3),
      p.adj = signif(p.adj, digits = 3)
    )
  if(!detailed) results <- remove_details(results, method = "t.test")
  results
}

context("test-p_value_precision")

# p-values must be returned at full precision, not pre-rounded to 3 sig figs
# (#108, #135), and adjusted p-values must be computed from the full-precision
# p-values so downstream re-adjustment is exact (#219).

has_more_than_3_sigfigs <- function(p) {
  p <- p[!is.na(p) & p > 0]
  any(abs(p - signif(p, 3)) > 0)
}

test_that("t_test / wilcox_test return full-precision p (#108, #135)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  expect_true(has_more_than_3_sigfigs((df %>% t_test(len ~ dose))$p))
  expect_true(has_more_than_3_sigfigs((df %>% t_test(len ~ dose))$p.adj))
  expect_true(has_more_than_3_sigfigs((df %>% wilcox_test(len ~ dose))$p))
  expect_true(has_more_than_3_sigfigs((df %>% t_test(len ~ supp))$p))      # 2-group
  expect_true(has_more_than_3_sigfigs((df %>% pairwise_t_test(len ~ dose))$p))
})

test_that("post-hoc and base tests return full-precision p (#108, #135)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  expect_true(has_more_than_3_sigfigs((aov(len ~ dose, df) %>% tukey_hsd())$p.adj))
  expect_true(has_more_than_3_sigfigs((df %>% games_howell_test(len ~ dose))$p.adj))
  expect_true(has_more_than_3_sigfigs((df %>% kruskal_test(len ~ dose))$p))
  expect_true(has_more_than_3_sigfigs((mtcars %>% cor_test(mpg, disp))$p))
})

test_that("omnibus ANOVA p stays rounded (anova_test / welch_anova_test out of scope)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  # ANOVA-family omnibus p keeps the 3-sig-fig summary format (not re-adjusted downstream)
  expect_equal((df %>% welch_anova_test(len ~ dose))$p,
               signif((df %>% welch_anova_test(len ~ dose))$p, 3))
})

test_that("adjusted p-values are computed from full-precision p (#219)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  # pairwise t-test: returned p.adj must equal re-adjusting the returned (full) p
  res <- df %>% t_test(len ~ dose, p.adjust.method = "holm")
  expect_equal(res$p.adj, stats::p.adjust(res$p, method = "holm"))
  # pairwise fisher: same invariant (this previously adjusted rounded p)
  fish <- pairwise_fisher_test(as.table(rbind(c(8, 2), c(1, 5), c(3, 9))),
                               p.adjust.method = "holm")
  expect_equal(fish$p.adj, stats::p.adjust(fish$p, method = "holm"))
})

test_that("dunn_test precision is unchanged (already full) (#219 no-regression)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  res <- df %>% dunn_test(len ~ dose)
  expect_equal(res$p.adj, stats::p.adjust(res$p, method = "holm"))
})

context("test-error-as-na")

test_that("error.as.na = FALSE (default) still errors on a constant comparison (#158)", {
  # both groups constant -> t.test: 'data are essentially constant'
  dd <- data.frame(g = rep(c("x", "y"), each = 4), v = c(rep(7, 4), rep(3, 4)))
  expect_error(dd %>% t_test(v ~ g))
  expect_error(dd %>% t_test(v ~ g, error.as.na = FALSE))
})

test_that("error.as.na = FALSE (default) still errors on too few observations (#208)", {
  d1 <- data.frame(g = c("a", "a", "a", "b"), v = c(1, 2, 3, 5))   # b has 1 obs
  expect_error(d1 %>% t_test(v ~ g))
})

test_that("error.as.na = TRUE returns an NA row for a constant comparison and warns (#158)", {
  dd <- data.frame(g = rep(c("x", "y"), each = 4), v = c(rep(7, 4), rep(3, 4)))
  expect_warning(res <- dd %>% t_test(v ~ g, error.as.na = TRUE), "Could not compute")
  expect_equal(nrow(res), 1L)
  expect_true(is.na(res$statistic) && is.na(res$p))
  expect_equal(res$n1, 4); expect_equal(res$n2, 4)   # n still reported
})

test_that("error.as.na = TRUE returns an NA row for too-few-observations (#208)", {
  d1 <- data.frame(g = c("a", "a", "a", "b"), v = c(1, 2, 3, 5))
  res <- suppressWarnings(d1 %>% t_test(v ~ g, error.as.na = TRUE))
  expect_true(is.na(res$p))
  expect_equal(res$n1, 3); expect_equal(res$n2, 1)
})

test_that("error.as.na = TRUE computes the good comparisons and NA-only the bad pair (#208)", {
  set.seed(1)
  # a,b both constant (a-b is degenerate); c varies -> a-c, b-c computable
  df <- data.frame(g = rep(c("a", "b", "c"), each = 4),
                   v = c(rep(7, 4), rep(3, 4), rnorm(4) + 10))
  res <- suppressWarnings(df %>% t_test(v ~ g, error.as.na = TRUE))
  expect_equal(nrow(res), 3L)
  ab <- res[res$group1 == "a" & res$group2 == "b", ]
  expect_true(is.na(ab$p))                         # degenerate pair -> NA
  others <- res[!(res$group1 == "a" & res$group2 == "b"), ]
  expect_false(any(is.na(others$p)))               # the rest are computed
  # p.adjust is computed over the non-NA comparisons (NA stays NA)
  expect_true(is.na(ab$p.adj))
})

test_that("error.as.na = TRUE keeps good groups in a grouped analysis (#208)", {
  df <- data.frame(
    supp = rep(c("S1", "S2"), each = 8),
    g    = rep(rep(c("x", "y"), each = 4), 2),
    v    = c(rep(2, 8), rnorm(4), rnorm(4) + 3)     # S1 is constant in both groups
  )
  res <- suppressWarnings(df %>% dplyr::group_by(supp) %>% t_test(v ~ g, error.as.na = TRUE))
  expect_equal(nrow(res), 2L)
  expect_true(is.na(res$p[res$supp == "S1"]))       # degenerate group -> NA
  expect_false(is.na(res$p[res$supp == "S2"]))      # good group -> computed
})

test_that("error.as.na = TRUE also works for wilcox_test (#208)", {
  # a group with no finite observations makes wilcox.test() error
  d1 <- data.frame(g = rep(c("a", "b"), each = 2), v = c(NA, NA, 1, 2))
  res <- suppressWarnings(d1 %>% wilcox_test(v ~ g, error.as.na = TRUE))
  expect_true(is.na(res$p))
  expect_equal(res$n1, 0); expect_equal(res$n2, 2)
})

test_that("error.as.na = TRUE keeps the wilcox_test schema (no phantom df column) (#208)", {
  df <- data.frame(g = rep(c("a", "b", "c"), each = 4),
                   v = c(NA, NA, NA, NA, 1, 2, 3, 4, 5, 6, 7, 8))   # a -> NA row
  res <- suppressWarnings(df %>% wilcox_test(v ~ g, error.as.na = TRUE))
  expect_false("df" %in% colnames(res))           # wilcox_test never has df
  # a t_test NA row, by contrast, keeps df
  dt <- data.frame(g = rep(c("a", "b"), each = 4), v = c(rep(7, 4), rep(3, 4)))
  expect_true("df" %in% colnames(suppressWarnings(dt %>% t_test(v ~ g, error.as.na = TRUE))))
})

test_that("error.as.na = TRUE does not alter results on valid data (no-regression)", {
  with.flag <- ToothGrowth %>% t_test(len ~ dose, error.as.na = TRUE)
  without   <- ToothGrowth %>% t_test(len ~ dose)
  expect_equal(with.flag$statistic, without$statistic)
  expect_equal(with.flag$p, without$p)
  expect_equal(colnames(with.flag), colnames(without))
})

context("test-get_n")

test_that("Checking that get_n works for T-test", {
  data("ToothGrowth")
  stat.test <- ToothGrowth %>% t_test(len ~ dose)
  expect_equal(get_n(stat.test), c(40, 40, 40))
})

test_that("Checking that get_n works for grouped T-test", {
  data("ToothGrowth")
  stat.test <- ToothGrowth %>%
    group_by(dose) %>%
    t_test(len ~ supp)
  expect_equal(get_n(stat.test), c(20, 20, 20))
})

test_that("Checking that get_n works for grouped ANOVA", {
  data("ToothGrowth")
  res.aov <- ToothGrowth %>%
    group_by(supp) %>%
    anova_test(len ~ dose)
  expect_equal(get_n(res.aov), c(30, 30))
})

context("test-get_test_label")

test_that("get_test_label includes the sample size n for ANOVA (#150)", {
  one_way <- get_test_label(ToothGrowth %>% anova_test(len ~ dose),
                            detailed = TRUE, type = "text")
  expect_match(one_way, "n = 60")
  # two-way: total sample size
  two_way <- get_test_label(ToothGrowth %>% anova_test(len ~ supp * dose),
                            detailed = TRUE, type = "text")
  expect_match(two_way, "n = 60")
  # repeated measures: n = number of subjects (wid)
  set.seed(1)
  rm <- data.frame(id = factor(rep(1:10, 3)),
                   time = factor(rep(c("t1", "t2", "t3"), each = 10)),
                   score = rnorm(30))
  rm_label <- get_test_label(anova_test(rm, dv = score, wid = id, within = time),
                             detailed = TRUE, type = "text")
  expect_match(rm_label, "n = 10")
  # expression type still builds without error
  expect_silent(get_test_label(ToothGrowth %>% anova_test(len ~ dose), detailed = TRUE))
})

test_that("get_test_label is unchanged for non-ANOVA tests and non-detailed labels (#150)", {
  expect_match(get_test_label(ToothGrowth %>% t_test(len ~ supp), detailed = TRUE, type = "text"), "n = 60")
  expect_match(get_test_label(ToothGrowth %>% kruskal_test(len ~ dose), detailed = TRUE, type = "text"), "n = 60")
  # the non-detailed ANOVA label has no n (only the description + p)
  expect_false(grepl("n =", get_test_label(ToothGrowth %>% anova_test(len ~ dose), type = "text")))
})

context("test-paired-id")

# Long-format paired data: subjects 1..10 measured under groups a, b, c.
make_paired <- function(){
  set.seed(42)
  data.frame(
    y  = rnorm(30),
    g  = rep(c("a", "b", "c"), each = 10),
    id = rep(1:10, 3)
  )
}

test_that("id= matches a manual complete-case paired t-test (#136)", {
  df <- make_paired()
  df <- df[df$g %in% c("a", "b"), ]
  res <- df %>% t_test(y ~ g, paired = TRUE, id = "id")
  x <- df$y[df$g == "a"]; y <- df$y[df$g == "b"]   # already subject-aligned here
  ref <- t.test(x, y, paired = TRUE)
  expect_equal(unname(res$statistic), unname(ref$statistic), tolerance = 1e-7)
  expect_equal(res$p, ref$p.value, tolerance = 1e-7)
  expect_equal(res$n1, 10); expect_equal(res$n2, 10)
})

test_that("id= aligns by subject regardless of row order (#136 silent mis-pairing)", {
  df <- make_paired()
  df <- df[df$g %in% c("a", "b"), ]
  correct <- (df %>% t_test(y ~ g, paired = TRUE, id = "id"))$statistic
  # shuffle the rows of group b: row order no longer matches subject order
  shuffled <- rbind(df[df$g == "a", ], df[df$g == "b", ][c(5, 1, 8, 3, 10, 2, 7, 4, 9, 6), ])
  res <- shuffled %>% t_test(y ~ g, paired = TRUE, id = "id")
  expect_equal(res$statistic, correct, tolerance = 1e-9)   # id pairing is order-invariant
})

test_that("id= handles unequal group sizes from missing observations (#192)", {
  df <- make_paired()
  df <- df[df$g %in% c("a", "b"), ]
  # subject 3 entirely missing from group b -> groups have unequal sizes
  df2 <- df[!(df$g == "b" & df$id == 3), ]
  # the default paired path errors on unequal lengths ...
  expect_error(df2 %>% t_test(y ~ g, paired = TRUE))
  # ... but id= uses the 9 common subjects
  res <- df2 %>% t_test(y ~ g, paired = TRUE, id = "id")
  expect_equal(res$n1, 9); expect_equal(res$n2, 9)
  common <- intersect(df2$id[df2$g == "a"], df2$id[df2$g == "b"])
  xa <- df2$y[df2$g == "a"][match(common, df2$id[df2$g == "a"])]
  yb <- df2$y[df2$g == "b"][match(common, df2$id[df2$g == "b"])]
  ref <- t.test(xa, yb, paired = TRUE)
  expect_equal(unname(res$statistic), unname(ref$statistic), tolerance = 1e-7)
})

test_that("id= NA values are dropped per pair (complete pairs) (#175)", {
  df <- make_paired()
  df <- df[df$g %in% c("a", "b"), ]
  df$y[df$g == "b" & df$id == 4] <- NA   # subject 4 missing in b
  res <- df %>% t_test(y ~ g, paired = TRUE, id = "id")
  expect_equal(res$n1, 9); expect_equal(res$n2, 9)   # the NA pair is dropped
})

test_that("id= does per-comparison pairwise deletion for >2 groups (#192)", {
  df <- make_paired()
  # remove subject 3 from group b, and subject 7 from group c
  df <- df[!(df$g == "b" & df$id == 3), ]
  df <- df[!(df$g == "c" & df$id == 7), ]
  res <- df %>% t_test(y ~ g, paired = TRUE, id = "id")
  expect_equal(nrow(res), 3L)                          # a-b, a-c, b-c
  np <- setNames(res$n1, paste(res$group1, res$group2))
  expect_equal(unname(np["a b"]), 9)                   # a (10) vs b (9) -> 9 pairs
  expect_equal(unname(np["a c"]), 9)                   # a (10) vs c (9) -> 9 pairs
  expect_equal(unname(np["b c"]), 8)                   # b (9, no 3) vs c (9, no 7) -> 8 common
})

test_that("id= works on grouped data, independently per group (#153)", {
  df <- make_paired()
  df$cohort <- rep(c("x", "y"), length.out = nrow(df))
  res <- df %>% dplyr::group_by(cohort) %>% t_test(y ~ g, paired = TRUE, id = "id")
  expect_true("cohort" %in% colnames(res))
  expect_true(nrow(res) >= 1)
})

test_that("id= also works for wilcox_test (#136)", {
  df <- make_paired()
  df <- df[df$g %in% c("a", "b"), ]
  df2 <- df[!(df$g == "b" & df$id == 3), ]
  res <- df2 %>% wilcox_test(y ~ g, paired = TRUE, id = "id")
  expect_equal(res$n1, 9); expect_equal(res$n2, 9)
  common <- intersect(df2$id[df2$g == "a"], df2$id[df2$g == "b"])
  xa <- df2$y[df2$g == "a"][match(common, df2$id[df2$g == "a"])]
  yb <- df2$y[df2$g == "b"][match(common, df2$id[df2$g == "b"])]
  ref <- suppressWarnings(wilcox.test(xa, yb, paired = TRUE))
  expect_equal(unname(res$statistic), unname(ref$statistic), tolerance = 1e-7)
})

test_that("id= errors on duplicated ids within a group (#136)", {
  df <- data.frame(
    y = rnorm(6), g = rep(c("a", "b"), each = 3), id = c(1, 1, 2, 1, 2, 3)
  )
  expect_error(df %>% t_test(y ~ g, paired = TRUE, id = "id"), "unique")
})

test_that("id= drops rows with a missing (NA) id instead of cross-joining them (#136)", {
  # 3 clean pairs (ids 1,2,3) plus 2 unidentified (NA-id) rows per group
  df <- data.frame(
    y  = rnorm(10),
    g  = rep(c("a", "b"), each = 5),
    id = c(1, 2, 3, NA, NA, 1, 2, 3, NA, NA)
  )
  res <- df %>% t_test(y ~ g, paired = TRUE, id = "id")
  expect_equal(res$n1, 3); expect_equal(res$n2, 3)   # only the 3 identified pairs
})

test_that("id= is rejected with ref.group = 'all' (#136)", {
  df <- make_paired()
  expect_error(
    df %>% t_test(y ~ g, paired = TRUE, id = "id", ref.group = "all"),
    "ref.group = 'all'"
  )
  expect_error(
    df %>% wilcox_test(y ~ g, paired = TRUE, id = "id", ref.group = "all"),
    "ref.group = 'all'"
  )
})

test_that("id= errors when the id column is missing (#136)", {
  df <- make_paired(); df <- df[df$g %in% c("a", "b"), ]
  expect_error(df %>% t_test(y ~ g, paired = TRUE, id = "subject"), "not found")
})

test_that("id is ignored for an unpaired test (no behaviour change)", {
  df <- make_paired(); df <- df[df$g %in% c("a", "b"), ]
  with_id <- df %>% t_test(y ~ g, id = "id")          # paired = FALSE
  without  <- df %>% t_test(y ~ g)
  expect_equal(with_id$statistic, without$statistic)
  expect_equal(with_id$n1, without$n1)
})

context("test-facet-x-position")

# Faceted test: facet X has x-levels a,b; facet Y has c,d,e.
facet_df <- function(){
  data.frame(
    y = 1:10,
    x = c("a", "a", "b", "b", "c", "c", "d", "d", "e", "e"),
    group = c("X", "X", "X", "X", "Y", "Y", "Y", "Y", "Y", "Y")
  )
}

test_that("scales = 'free' gives per-facet x positions (#203)", {
  st <- t_test(dplyr::group_by(facet_df(), group), y ~ x) %>%
    add_xy_position(x = "x", scales = "free")
  # facet X: a-b -> 1,2
  x <- st[st$group == "X", ]
  expect_equal(x$xmin, 1); expect_equal(x$xmax, 2)
  # facet Y: c,d,e -> positions 1,2,3 within the facet (not the global 3,4,5)
  y <- st[st$group == "Y", ]
  expect_equal(y$xmin[y$group1 == "c" & y$group2 == "d"], 1)
  expect_equal(y$xmax[y$group1 == "c" & y$group2 == "d"], 2)
  expect_equal(y$xmin[y$group1 == "c" & y$group2 == "e"], 1)
  expect_equal(y$xmax[y$group1 == "c" & y$group2 == "e"], 3)
  expect_equal(y$xmin[y$group1 == "d" & y$group2 == "e"], 2)
  expect_equal(y$xmax[y$group1 == "d" & y$group2 == "e"], 3)
})

test_that("default scales = 'fixed' keeps global x positions (no-regression) (#203)", {
  st <- t_test(dplyr::group_by(facet_df(), group), y ~ x) %>%
    add_xy_position(x = "x")                       # default fixed
  y <- st[st$group == "Y", ]
  # global positions: c,d,e -> 3,4,5
  expect_equal(y$xmin[y$group1 == "c" & y$group2 == "d"], 3)
  expect_equal(y$xmax[y$group1 == "d" & y$group2 == "e"], 5)
})

test_that("scales = 'free_y' does NOT remap x (only y is freed) (#203)", {
  st_fixed <- t_test(dplyr::group_by(facet_df(), group), y ~ x) %>%
    add_xy_position(x = "x")
  st_freey <- t_test(dplyr::group_by(facet_df(), group), y ~ x) %>%
    add_xy_position(x = "x", scales = "free_y")
  expect_equal(st_freey$xmin, st_fixed$xmin)        # x positions unchanged
  expect_equal(st_freey$xmax, st_fixed$xmax)
})

test_that("scales = 'free' does not affect a non-faceted (ungrouped) test (#203)", {
  tg <- ToothGrowth; tg$dose <- factor(tg$dose)
  st_fixed <- tg %>% t_test(len ~ dose) %>% add_xy_position(x = "dose")
  st_free  <- tg %>% t_test(len ~ dose) %>% add_xy_position(x = "dose", scales = "free")
  expect_equal(st_free$xmin, st_fixed$xmin)         # no facet vars -> no remap
  expect_equal(st_free$xmax, st_fixed$xmax)
})

test_that("free-scale remap counts plotted-but-uncompared x levels (#203)", {
  # All three levels a,b,c are plotted in BOTH facets, but only a-vs-c is
  # compared, so b is a plotted box that no comparison touches. The panel x
  # positions must come from the DATA (a,b,c -> 1,2,3), so a-vs-c is 1 -> 3,
  # NOT 1 -> 2 (which a comparison-endpoint remap would wrongly produce).
  df <- data.frame(
    y = c(1, 2, 3, 4, 5, 6, 10, 11, 12, 13, 14, 15),
    x = rep(c("a", "a", "b", "b", "c", "c"), 2),
    group = rep(c("X", "Y"), each = 6)
  )
  st <- t_test(dplyr::group_by(df, group), y ~ x, comparisons = list(c("a", "c"))) %>%
    add_xy_position(x = "x", scales = "free")
  expect_true(all(st$xmin == 1))
  expect_true(all(st$xmax == 3))
})

test_that("free-scale remap shifts a shared comparison by each facet's own levels (#203)", {
  # facet X has only b,c; facet Y has a,b,c. The SAME comparison b-vs-c must land
  # at 1->2 in X but 2->3 in Y (because a occupies panel position 1 in Y). This
  # only holds when the remap is driven by each facet's actual data levels.
  df <- data.frame(
    y = c(1, 2, 3, 4, 10, 11, 12, 13, 14, 15),
    x = c("b", "b", "c", "c", "a", "a", "b", "b", "c", "c"),
    group = c(rep("X", 4), rep("Y", 6))
  )
  st <- t_test(dplyr::group_by(df, group), y ~ x, comparisons = list(c("b", "c"))) %>%
    add_xy_position(x = "x", scales = "free")
  X <- st[st$group == "X", ]
  Y <- st[st$group == "Y", ]
  expect_equal(X$xmin, 1); expect_equal(X$xmax, 2)
  expect_equal(Y$xmin, 2); expect_equal(Y$xmax, 3)
})

test_that("free-scale remap handles non-consecutive global x levels per facet (#203)", {
  # facet Y is missing level 'b' so its present levels are a,c,d (global 1,3,4)
  df <- data.frame(
    y = 1:12,
    x = c("a","a","b","b","c","c", "a","a","c","c","d","d"),
    group = rep(c("X","Y"), each = 6)
  )
  st <- t_test(dplyr::group_by(df, group), y ~ x) %>%
    add_xy_position(x = "x", scales = "free")
  y <- st[st$group == "Y", ]
  # present levels a,c,d -> compressed to 1,2,3 (not 1,3,4)
  expect_setequal(unique(c(y$xmin, y$xmax)), c(1, 2, 3))
})

context("test-t_test")

test_that("pairwise_t_test drops unused factor levels (#133)", {
  set.seed(1)
  d <- data.frame(
    y = rnorm(30),
    g = factor(rep(c("a", "b", "c"), each = 10), levels = c("a", "b", "c", "d"))  # 'd' unused
  )
  res <- d %>% pairwise_t_test(y ~ g)
  expect_equal(nrow(res), 3L)                          # 3 comparisons over a/b/c, not the empty 'd'
  expect_false("d" %in% c(res$group1, res$group2))     # no impossible comparison
  # paired variant (the reported scenario)
  dp <- data.frame(
    id = factor(rep(1:8, 3)),
    g  = factor(rep(c("a", "b", "c"), each = 8), levels = c("a", "b", "c", "d")),
    y  = rnorm(24)
  )
  expect_equal(nrow(dp %>% pairwise_t_test(y ~ g, paired = TRUE)), 3L)
})

test_that("t_test detailed method reports the variant (#124)", {
  expect_equal((ToothGrowth %>% t_test(len ~ supp, detailed = TRUE))$method, "Welch t-test")
  expect_equal((ToothGrowth %>% t_test(len ~ supp, var.equal = TRUE, detailed = TRUE))$method, "T-test")
  expect_equal((ToothGrowth %>% t_test(len ~ supp, paired = TRUE, detailed = TRUE))$method, "Paired t-test")
  expect_equal((ToothGrowth %>% t_test(len ~ 1, mu = 20, detailed = TRUE))$method, "One-sample t-test")
})

test_that("t_test / wilcox_test handle a filtered factor with an unused level (#133)", {
  # 2 species kept, but the 'virginica' factor level is retained (empty) - used to error
  d <- iris %>% filter(Species %in% c("setosa", "versicolor"))
  expect_true(is.data.frame(t_test(d, Sepal.Length ~ Species)))
  expect_true(is.data.frame(wilcox_test(d, Sepal.Length ~ Species)))
})

context("test-ref-group-missing")

# #153: a missing reference group used to surface as a cryptic stats::relevel()
# error ("'<ref>' must be an existing level"). It should now give a clear,
# actionable message that points to the group_by()+filter() idiom.

test_that("missing ref.group gives a clear, actionable error (ungrouped) (#153)", {
  d <- data.frame(g = rep(c("x", "y"), each = 4), y = c(1, 2, 3, 4, 5, 6, 7, 8))
  expect_error(d %>% t_test(y ~ g, ref.group = "ref"), "reference group")
  expect_error(d %>% t_test(y ~ g, ref.group = "ref"), "not present in the data")
  # the available levels are listed to guide the user
  expect_error(d %>% t_test(y ~ g, ref.group = "ref"), "x, y")
})

test_that("missing ref.group in a grouped subset errors clearly (#153)", {
  d <- data.frame(
    blk = c(rep("A", 6), rep("B", 4)),
    g   = c("ref", "ref", "x", "x", "y", "y", "x", "x", "y", "y"),  # B has no 'ref'
    y   = c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
  )
  expect_error(
    d %>% dplyr::group_by(blk) %>% t_test(y ~ g, ref.group = "ref"),
    "reference group"
  )
})

test_that("the group_by()+filter() idiom works after dropping ref-less groups (#153)", {
  d <- data.frame(
    blk = c(rep("A", 6), rep("B", 4)),
    g   = c("ref", "ref", "x", "x", "y", "y", "x", "x", "y", "y"),
    y   = c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
  )
  res <- d %>%
    dplyr::group_by(blk) %>%
    dplyr::filter(any(g == "ref")) %>%
    t_test(y ~ g, ref.group = "ref")
  expect_true(all(res$blk == "A"))          # only the block that has 'ref'
  expect_true(all(res$group1 == "ref"))
})

test_that("a valid ref.group is unaffected (no-regression) (#153)", {
  tg <- ToothGrowth; tg$dose <- factor(tg$dose)
  res <- tg %>% t_test(len ~ dose, ref.group = "0.5")
  expect_equal(nrow(res), 2L)
  expect_true(all(res$group1 == "0.5"))
  # ref.group = "all" still works too
  res_all <- tg %>% t_test(len ~ dose, ref.group = "all")
  expect_equal(nrow(res_all), 3L)
  expect_true(all(res_all$group1 == "all"))
})

test_that("missing ref.group is also clear for wilcox_test (#153)", {
  d <- data.frame(g = rep(c("x", "y"), each = 4), y = c(1, 2, 3, 4, 5, 6, 7, 8))
  expect_error(d %>% wilcox_test(y ~ g, ref.group = "ref"), "reference group")
})

test_that("missing ref.group raises a classed condition for reliable detection (#153)", {
  # The condition carries class 'rstatix_missing_ref_group' so downstream
  # callers (e.g. ggpubr::geom_pwc) can detect it by class, not by matching the
  # translatable message text.
  d <- data.frame(g = rep(c("x", "y"), each = 4), y = c(1, 2, 3, 4, 5, 6, 7, 8))
  expect_error(d %>% t_test(y ~ g, ref.group = "ref"),
               class = "rstatix_missing_ref_group")
  # the class survives the grouped (doo/purrr) path too
  d2 <- data.frame(
    blk = c(rep("A", 6), rep("B", 4)),
    g   = c("ref", "ref", "x", "x", "y", "y", "x", "x", "y", "y"),
    y   = c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
  )
  err <- tryCatch(
    d2 %>% dplyr::group_by(blk) %>% t_test(y ~ g, ref.group = "ref"),
    error = function(e) e
  )
  # in the grouped path the condition is wrapped by dplyr/purrr, so the class
  # lives in the parent chain; rlang::cnd_inherits() walks it.
  expect_true(rlang::cnd_inherits(err, "rstatix_missing_ref_group"))
})

#' @include utilities.R
NULL
#' Adjust P-values for Multiple Comparisons
#' @description A pipe-friendly function to add an adjusted p-value column into
#'   a data frame. Supports grouped data.
#' @param data a data frame containing a p-value column
#' @param p.col column name containing p-values
#' @param output.col the output column name to hold the adjusted p-values
#' @param method method for adjusting p values (see
#'   \code{\link[stats]{p.adjust}}). Allowed values include "holm", "hochberg",
#'   "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't want to
#'   adjust the p value (not recommended), use p.adjust.method = "none".
#' @return a data frame
#'
#' @examples
#' # Perform pairwise comparisons and adjust p-values
#' ToothGrowth %>%
#'  t_test(len ~ dose) %>%
#'  adjust_pvalue()
#'
#' @rdname adjust_pvalue
#' @export
adjust_pvalue <- function(data, p.col = NULL, output.col = NULL, method = "holm"){
  if (is_grouped_df(data)) {
    res <- data %>%
      doo(adjust_pvalue, p.col, output.col, method = method)
    return(res)
  }
  .attributes <- get_test_attributes(data)
  if(!is.null(.attributes$args)){
    .attributes$args$p.adjust.method = method
  }
  p.adjust <- stats::p.adjust
  p.adjust.method <-  method

  # Guess p-value columns if missing
  if(is.null(p.col))
    p.col <- data %>% p_detect("p")
  if(is.null(p.col))
    return(data)
  else if(!(p.col %in% colnames(data)))
    stop("The column ", p.col, "does not exist in the data")
  if(is.null(output.col))
    output.col <- paste0(p.col, ".adj")
  # Adjust p-value
  data %>%
    keep_only_tbl_df_classes() %>%
    mutate(
      !!output.col := p.adjust(!!sym(p.col), method = p.adjust.method)
    ) %>%
    set_test_attributes(.attributes)
}

#' @include utilities.R factorial_design.R anova_summary.R
NULL
#'Anova Test
#'
#'
#'@description Provides a pipe-friendly framework to perform different types of
#'  ANOVA tests, including: \itemize{ \item \strong{\href{https://www.datanovia.com/en/lessons/anova-in-r/}{Independent measures ANOVA}}:
#'  between-Subjects designs, \item \strong{\href{https://www.datanovia.com/en/lessons/repeated-measures-anova-in-r/}{Repeated measures ANOVA}}:
#'  within-Subjects designs \item \strong{\href{https://www.datanovia.com/en/lessons/mixed-anova-in-r/}{Mixed ANOVA}}: Mixed within within- and
#'  between-Subjects designs, also known as split-plot ANOVA and \item \strong{\href{https://www.datanovia.com/en/lessons/ancova-in-r/}{ANCOVA: Analysis of Covariance}}. }
#'
#'  The function is an easy to use wrapper around \code{\link[car]{Anova}()} and
#'  \code{\link[stats]{aov}()}. It makes ANOVA computation handy in R and It's
#'  highly flexible: can support model and formula as input. Variables can be
#'  also specified as character vector using the arguments \code{dv, wid,
#'  between, within, covariate}.
#'
#'  The results include ANOVA table, generalized effect size and some assumption
#'  checks.
#'
#'
#'@param data a data.frame or a model to be analyzed.
#'@param formula a formula specifying the ANOVA model similar to
#'  \link[stats]{aov}. Can be of the form \code{y ~ group} where \code{y} is a
#'  numeric variable giving the data values and \code{group} is a factor with
#'  one or multiple levels giving the corresponding groups. For example,
#'  \code{formula = TP53 ~ cancer_group}.
#'
#'  Examples of supported formula include: \itemize{ \item Between-Ss ANOVA
#'  (independent measures ANOVA): \code{y ~ b1*b2} \item Within-Ss ANOVA
#'  (repeated measures ANOVA): \code{y ~ w1*w2 + Error(id/(w1*w2))} \item Mixed
#'  ANOVA: \code{y ~ b1*b2*w1 + Error(id/w1)} }
#'
#'  If the formula doesn't contain any within vars, a linear model is directly
#'  fitted and  passed to the ANOVA function. For repeated designs, the ANOVA
#'  variables are parsed from the formula.
#'
#'@param dv (numeric) dependent variable name.
#'@param wid (factor) column name containing individuals/subjects identifier.
#'  Should be unique per individual.
#'@param between (optional) between-subject factor variables.
#'@param within (optional) within-subjects factor variables
#'@param covariate (optional) covariate names (for ANCOVA)
#'@param type the type of sums of squares for ANOVA. Allowed values are either
#'  1, 2 or 3. \code{type = 2} is the default because this will yield identical
#'  ANOVA results as type = 1 when data are balanced but type = 2 will
#'  additionally yield various assumption tests where appropriate. When the data
#'  are unbalanced the \code{type = 3} is used by popular commercial softwares
#'  including SPSS.
#'@param effect.size the effect size to compute and to show in the ANOVA
#'  results. Allowed values can be either "ges" (generalized eta squared) or
#'  "pes" (partial eta squared) or both. Default is "ges".
#'@param white.adjust Default is FALSE. If TRUE, heteroscedasticity correction
#'  is applied to the coefficient of covariance matrix. Used only for
#'  independent measures ANOVA.
#'@param error (optional) for a linear model, an lm model object from which the
#'  overall error sum of squares and degrees of freedom are to be calculated.
#'  Read more in \code{\link[car]{Anova}()} documentation.
#'@param observed Variables that are observed (i.e, measured) as compared to
#'  experimentally manipulated. The default effect size reported (generalized
#'  eta-squared) requires correct specification of the observed variables.
#'@param detailed If TRUE, returns extra information (sums of squares columns,
#'  intercept row, etc.) in the ANOVA table.
#'@param x an object of class \code{anova_test}
#'@param correction character. Used only in repeated measures ANOVA test to
#'  specify which correction of the degrees of freedom should be reported for
#'  the within-subject factors. Possible values are:
#'  \itemize{
#'  \item{"GG"}: applies Greenhouse-Geisser correction to all within-subjects factors even if the assumption of sphericity is met (i.e., Mauchly's test is not significant, p > 0.05).
#'  \item{"HF"}: applies Hyunh-Feldt correction to all within-subjects factors even if the assumption of sphericity is met,
#'  \item{"none"}: returns the ANOVA table without any correction and
#'  \item{"auto"}: apply automatically GG correction to only within-subjects factors violating the sphericity assumption (i.e., Mauchly's test p-value is significant, p <= 0.05).
#'  }
#'@seealso \code{\link{anova_summary}()}, \code{\link{factorial_design}()}
#'@return return an object of class \code{anova_test} a data frame containing
#'  the ANOVA table for independent measures ANOVA.
#'
#'  However, for repeated/mixed measures ANOVA, a list containing the following
#'  components are returned: ANOVA table, Mauchly's Test for Sphericity,
#'  Sphericity Corrections. These table are described more in the documentation
#'  of the function \code{\link{anova_summary}()}.
#'
#'  The \strong{returned object has an attribute} called \code{args}, which is a
#'  list holding the arguments used to fit the ANOVA model, including: data, dv,
#'  within, between, type, model, etc.
#'
#'
#'@author Alboukadel Kassambara, \email{alboukadel.kassambara@@gmail.com}
#' @examples
#' # Load data
#' #:::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # One-way ANOVA test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% anova_test(len ~ dose)
#'
#' # Grouped One-way ANOVA test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>%
#'   group_by(supp) %>%
#'   anova_test(len ~ dose)
#'
#' # Two-way ANOVA test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% anova_test(len ~ supp*dose)
#'
#' # Two-way repeated measures ANOVA
#' #:::::::::::::::::::::::::::::::::::::::::
#' df$id <- rep(1:10, 6) # Add individuals id
#' # Use formula
#' \donttest{
#' df %>% anova_test(len ~ supp*dose + Error(id/(supp*dose)))
#' }
#'
#'
#' # or use character vector
#' df %>% anova_test(dv = len, wid = id, within = c(supp, dose))
#'
#' # Extract ANOVA table and apply correction
#' #:::::::::::::::::::::::::::::::::::::::::
#' res.aov <- df %>% anova_test(dv = len, wid = id, within = c(supp, dose))
#' get_anova_table(res.aov, correction = "GG")
#'
#'
#' # Use model as arguments
#' #:::::::::::::::::::::::::::::::::::::::::
#' .my.model <- lm(yield ~ block + N*P*K, npk)
#' anova_test(.my.model)
#'
#'
#'@describeIn anova_test perform anova test
#'@export
anova_test <- function(data, formula, dv, wid, between, within, covariate, type = NULL,
                       effect.size = "ges", error = NULL,
                       white.adjust = FALSE, observed = NULL, detailed = FALSE){
  .args <- rlang::enquos(
    dv = dv, wid = wid, between = between,
    within = within, covariate = covariate) %>%
    select_quo_variables(data) %>%
    add_item(type = type, white.adjust = white.adjust, method = "anova_test")
  if(!missing(formula)) .args$formula <- formula

  .anova_test <- function(data, .args, effect.size = "ges", error = NULL,
                          observed = NULL, detailed = FALSE){
    .args <- .args %>%
      add_item(data = data) %>%
      check_anova_arguments()
    if(.args$type != 1) {
      if(is.null(error)) res.anova <- car_anova(.args)
      else res.anova <- car_anova(.args, error = error)
    }
    else if(.args$type == 1) res.anova <- stats_aov(.args)
    else stop("Something is wrong...")
    results <- res.anova %>%
      anova_summary(
        effect.size = effect.size, detailed = detailed,
        observed = observed
      )
    results
  }
  .append_anova_class <- function(x){
    class(x) <- c("anova_test", class(x), "rstatix_test")
    x
  }

  if(is_grouped_df(data)){
    results <- data %>% doo(
      ~.anova_test(data = ., .args = .args, effect.size = effect.size,
                   error = error, observed = observed, detailed = detailed),
      result = "anova"
    )
    if("anova" %in% colnames(results)){
      results <- results %>%
        mutate(anova = map(.data$anova, .append_anova_class))
    }
  }
  else{
    results <- .anova_test(
      data, .args = .args, effect.size = effect.size,
      error = error, observed = observed, detailed = detailed
      ) %>%
      .append_anova_class()
  }
  results
}




# Extract ANOVA table -----------------------------------------------

#' @describeIn anova_test extract anova table from an object of class
#'   \code{anova_test}. When within-subject factors are present, either
#'   sphericity corrected or uncorrected degrees of freedom can be reported.
#' @export
get_anova_table <- function(x, correction = c("auto", "GG", "HF", "none")){
  correction <- match.arg(correction)
  if(is_grouped_anova_test(x)){
    results <- get_anova_table_from_grouped_test(x, correction = correction)
  }
  else{
    results <- get_anova_table_from_simple_test(x, correction = correction)
  }
  results
}


get_anova_table_from_simple_test <- function(x, correction = "auto"){
  correction.method <- method <- correction
  if(method == "auto") method = "GG"
  # Independent anova
  if(!inherits(x, "list")){
    return(x)
  }
  if(correction.method == "none"){
    res.aov <- x$ANOVA
    attr(res.aov, "args") <- attr(x, "args")
    class(res.aov) <- c("anova_test", class(res.aov), "rstatix_test")
    return(res.aov)
  }
  # repeated/mixed design
  # Get correction table from anova_test
  .args <- attr(x, "args")
  get_corrections_table <- function(x, method = c("GG", "HF")){
    method <- match.arg(method)
    pattern <- paste0("Effect|", method)
    corrections <- x$`Sphericity Corrections` %>%
      select(tidyselect::matches(pattern))
    colnames(corrections) <- c("Effect", "epsilon", "df", "p", "p<.05")
    corrections <- corrections %>%
      tidyr::separate(col = "df", into = c("DFn", "DFd"), sep = ", ", convert = TRUE) %>%
      mutate(method = method)
    corrections
  }
  res.aov <- x$ANOVA
  sphericity <- x$`Mauchly's Test for Sphericity`
  corrections <- get_corrections_table(x, method)
  # If auto apply correction only when sphericity is not assumed (Mauchly p < 0.05)
  if(correction.method == "auto"){
    corrections %<>% filter(sphericity$p <= 0.05)
  }
  if(nrow(corrections) > 0){
    rownames(res.aov) <- res.aov$Effect
    rownames(corrections) <- corrections$Effect
    cols.to.update <- c("DFn", "DFd", "p", "p<.05")
    rows.to.update <- rownames(corrections)
    res.aov[rows.to.update, cols.to.update] <- corrections[rows.to.update, cols.to.update]
    rownames(res.aov) <- 1:nrow(res.aov)
  }
  res.aov <- res.aov %>% set_attrs(args = .args)
  class(res.aov) <- c("anova_test", class(res.aov), "rstatix_test")
  res.aov
}

get_anova_table_from_grouped_test <- function(x, correction = "auto"){
  if(!is_grouped_anova_test(x)){
    return(x)
  }
  extract_table <- function(x, correction){
    get_anova_table_from_simple_test(x, correction = correction) %>%
      remove_class(c("anova_test", "rstatix_test"))
  }
  x %>%
    mutate(anova = map(.data$anova, extract_table, correction = correction)) %>%
    unnest(cols = "anova")
}


is_anova_test <- function(x){
  inherits(x, "anova_test")
}
is_grouped_anova_test <- function(x){
  answer <- FALSE
  if(("anova" %in% colnames(x))){
    if(inherits(x$anova, "list")){
      answer <- inherits(x$anova[[1]], "anova_test")
    }
  }
  answer
}

# Printing anova and plotting model diagnostic -----------------------------------------------

#' @rdname anova_test
#' @method print anova_test
#' @param ... additional arguments
#' @export
print.anova_test <- function(x, ...) {
  .args <- attr(x, "args")
  type <- switch(.args$type, `1` = "I", `2` = "II", `3` =  "III")
  cat("ANOVA Table (type", type, "tests)\n\n")
  if(inherits(x, "data.frame"))
    print.data.frame(x)
  else if(inherits(x, "list")){
    attr(x, "args") <- NULL
    class(x) <- "list"
    print(x)
  }
}

#' @rdname anova_test
#' @method plot anova_test
#' @export
plot.anova_test <- function(x, ...) {
  .args <- attr(x, "args")
  graphics::plot(.args$model,  ...)
}



# Check arguments  -----------------------------------------------

# Check the arguments of ANOVA
# .args is a list
check_anova_arguments <- function(.args){
  if(!is.null(.args$formula)){
    .args <- get_anova_vars_from_formula(.args)
    if(is.null(.args$within)) .args$model <- fit_lm(.args)
  }
  if(inherits(.args$data, "aovlist")){
    stop("A model of class aovlist is not supported.")
  }
  else if(has_model(.args)){
    if(is.null(.args$type)) .args$type <- 2
    return(.args)
  }
  .args <- .args %>%
    check_factorial_design() %>%
    check_anova_type()
  .args
}

get_anova_vars_from_formula <- function(.args){
  formula <- .args$formula
  data <- .args$data
  vars <- all.vars(formula)
  stop_if_multiple_error_terms(formula)
  # Detect transformed responses:
  lhs <- all.names(formula[[2]])
  transf <- setdiff(lhs, all.vars(formula[[2]]))
  if (length(transf) == 0)
    transf = NULL
  if (!is.null(transf)) {
    origdv <- setdiff(lhs, transf)
    dv <- paste0(transf[1], ".", origdv)
    data[[dv]] <- eval(formula[[2]], envir = data)  # add transformed version
    vars <- vars[!(vars %in% lhs)]
  }else {
    dv <- vars[1]
    vars <- vars[-1]
  }
  error.vars <- get_formula_error_vars(formula)
  id <- error.vars[1]
  within <- error.vars[-1]
  between <- vars[!(vars %in% c(id, within))]
  if(length(within) == 0) within <- NULL
  if(length(between) == 0) between <- NULL
  if(is.na(id)) id <- NULL
  .args <- .args %>%
    .add_item(data = data, dv = dv, wid = id, between = between, within = within)
  .args
}

stop_if_multiple_error_terms <- function(formula){
  .terms <- stats::terms(formula, "Error")
  .error.terms <- attr(.terms, "specials")$Error
  if (length(.error.terms) > 1L)
    stop(sprintf("there are %d Error terms: only 1 is allowed", length(.error.terms)))
}


# stop if ancova with repeated variables
stop_if_repeated_ancova <- function(.args){
  if(is_repeated_ancova(.args) | is_mixed_ancova(.args)){
    stop("Don't support ANCOVA with repeated measures")
  }
  .args
}

# Check anova design and type
is_design_balanced <- function(.args){
  res <- .args$data %>%
    group_by(!!!syms(.args$between)) %>%
    summarise(count = n())
  length(unique(res$count)) == 1
}
is_repeated_anova <- function(.args){
  is.null(.args$between) & !is.null(.args$within) & is.null(.args$covariate)
}
is_independent_anova <- function(.args){
  !is.null(.args$between) & is.null(.args$within) & is.null(.args$covariate)
}
is_mixed_anova <- function(.args){
  !is.null(.args$between) & !is.null(.args$within) & is.null(.args$covariate)
}
is_repeated_ancova <- function(.args){
  !is.null(.args$within) & !is.null(.args$covariate) & is.null(.args$between)
}
is_independent_ancova <- function(.args){
  is.null(.args$within) & !is.null(.args$covariate) & !is.null(.args$between)
}
is_mixed_ancova <- function(.args){
  !is.null(.args$between) & !is.null(.args$within) & !is.null(.args$covariate)
}

# Check anova type
check_anova_type <- function(.args){
  n.vars <- length(c(.args$between, .args$within))
  if(is.null(.args$type)){
    .args$type <- 2
    if(is_repeated_anova(.args)) .args$type <- 3
    else if(!is.null(.args$between)) {
      if(!is_design_balanced(.args) & n.vars > 1) .args$type <- 3
    }
  }
  else if (.args$type == 1){
    if(!is_design_balanced(.args) & n.vars > 1){
      warning("Your data are unbalanced and there are more than one variable. ",
              "In this case, using 'type = 1' is not recommended. ",
              "Consider using type 3 ANOVA.", immediate.=TRUE, call.=FALSE)
    }
  }
  .args
}


is_model <- function(object){
  models <- c("lm", "aov", "glm", "multinom", "polr", "mlm", "manova")
  inherits(object,  models)
}
# Get anova model from the list of arguments
get_anova_model <- function(.args){
  if(!is.null(.args$model)) return(.args$model)
  else if(is_model(.args$data)) return(.args$data)
  else stop("No model detected in ANOVA arguments")
}
# Check if ANOVA arguments contain model
has_model <- function(.args){
  !is.null(.args$model) | is_model(.args$data)
}


# Fit lm from formula and data ------------------------------------
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fit_lm <- function(.args){
  .args <- remove_missing_values_in_data(.args)
  lm_data <- droplevels(.args$data)
  lm_formula <- .args$formula
  opt <- options( "contrasts" = c( "contr.sum", "contr.poly" ) )
  results <- stats::lm(lm_formula, lm_data)
  options(opt)
  results
}

# Compute the different types of ANOVA -----------------------------
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
car_anova <- function(.args, ...){
  if(has_model(.args)){
    .model <- get_anova_model(.args)
    res.anova <- car::Anova(
      .model, type = .args$type,
      white.adjust = .args$white.adjust, ...
    )
    .args$model <- .model
  }
  else{
    design <- factorial_design(
      data = .args$data, dv = .args$dv, wid = .args$wid, between = .args$between,
      within = .args$within, covariate = .args$covariate
      )
    if(is_independent_anova(.args)){
      res.anova <- Anova(
        design$model, type = .args$type,
        white.adjust = .args$white.adjust, ...
      )
    }
    else{
      res.anova <- Anova(
        design$model, idata = design$idata,
        idesign = design$idesign, type = .args$type, ...
      )
    }
   .args$model <- design$model
  }
  attr(res.anova, "args") <- .args
  res.anova
}

# R stats aov
stats_aov <- function(.args){
  if(has_model(.args)){
    .model <- get_anova_model(.args)
    res.anova <- stats::aov(.model)
  }
  else{
    aov.formula <- create_aov_formula(.args)
    data <- .args$data
    res.anova <- .model <- stats::aov(aov.formula, data)
  }
  .args$model <- .model
  attr(res.anova, "args") <- .args
  res.anova
}

create_aov_formula <- function(.args){
  between <- paste(.args$between, collapse = "*")
  within <-  paste(.args$within, collapse = "*")
  covariate <- paste(.args$covariate, collapse = "+")
  error <- ifelse(
    within != "",
    error <- paste0("+Error(", .args$wid, "/(", within, "))"),
    ""
  )
  bw.sep <- ifelse(between != "" & within != "", "*", "")  # Between and Within vars separator
  bc.sep <- ifelse(covariate != "", "+", "") # Between and covariate vars separator
  .formula <- paste0(.args$dv, " ~ ", covariate, bc.sep, between, bw.sep, within, error) %>%
    stats::as.formula()
  .formula
}



# Check assumptions (Not used helpers)-------------------------
check_anova_assumptions <- function(data, dv, between){
  . <- NULL
  outliers <- data %>%
    group_by(!!!syms(between)) %>%
    identify_outliers(!!dv)
  groups.normality <- data %>%
    group_by(!!!syms(between)) %>%
    shapiro_test(vars = dv)

  formula <- paste(between, collapse = "*") %>%
    paste(dv, ., sep = " ~ ") %>%
    stats::as.formula()
  model <- stats::lm(formula, data)
  .residuals <- stats::residuals(model)

  variance.homogeneity <- levene_test(data, formula)

  arguments <- list( dv = dv, between = between)

  results <- list(
    outliers = outliers,
    residuals.normality = shapiro_test(.residuals),
    groups.normality = groups.normality,
    variance.homogeneity = variance.homogeneity
  ) %>%
    set_attrs(arguments = arguments)

  results
}


check_repeated_anova_assumptions <- function(data, dv, wid, within){
  . <- NULL
  results <- check_anova_assumptions(data, dv, within)
  results$variance.homogeneity  <- NULL
  arguments <- list( dv = dv, wid = wid, within = within)
  results <- results %>% set_attrs(arguments = arguments)
    within <- paste(within, collapse = ", ") %>%
      paste0("c(", ., ")")
    data.name <- deparse(substitute(data))
    anova.formula <- paste0(
      "anova_test(", data.name, ", dv = ", dv,
      ", wid = ", wid, ", within = ", within, ")"
    )
    res.anova <- eval(parse(text = anova.formula))
    results <- results %>%
      .add_item(sphericity = res.anova$`Mauchly's Test for Sphericity`)
  results
}


check_mixed_anova_assumptions <- function(data, dv, wid , between, within){
  . <- NULL

  arguments <- list( dv = dv, wid = wid, between = between, within = within)


  grouping <- c(between, within)
  outliers <- data %>%
    group_by(!!!syms(grouping)) %>%
    identify_outliers(!!dv)
  groups.normality <- data %>%
    group_by(!!!syms(grouping)) %>%
    shapiro_test(vars = dv)

  formula <- paste(between, collapse = "*") %>%
    paste(dv, ., sep = " ~ ") %>%
    stats::as.formula()
  variance.homogeneity <- data %>%
    group_by(!!!syms(within)) %>%
    levene_test(formula)

  results <- list(
    outliers = outliers,
    groups.normality = groups.normality,
    variance.homogeneity = variance.homogeneity
  ) %>%
    set_attrs(arguments = arguments)

    within <- paste(within, collapse = ", ") %>%
      paste0("c(", ., ")")
    between <- paste(between, collapse = ", ") %>%
      paste0("c(", ., ")")
    data.name <- deparse(substitute(data))
    anova.formula <- paste0(
      "anova_test(", data.name, ", dv = ", dv,
      ", wid = ", wid, ", within = ", within, ", between = ", between, ")"
    )
    res.anova <- eval(parse(text = anova.formula))
    results <- results %>%
      .add_item(sphericity = res.anova$`Mauchly's Test for Sphericity`)

  results

}

context("test-anova_test")

test_that("Checking one-way ANOVA test", {
  data("ToothGrowth")
  res.aov <- ToothGrowth %>% anova_test(len ~ dose)
  expect_equal(res.aov$Effect, "dose")
  expect_equal(res.aov$DFn, 1)
  expect_equal(res.aov$DFd, 58)
  expect_equal(res.aov$F, 105.065)
  expect_equal(res.aov$ges, 0.644)
})

test_that("Checking grouped one-way ANOVA test", {
  data("ToothGrowth")
  res.aov <- ToothGrowth %>%
    group_by(supp) %>%
    anova_test(len ~ dose)
  expect_equal(res.aov$Effect, c("dose", "dose"))
  expect_equal(res.aov$DFn, c(1, 1))
  expect_equal(res.aov$DFd, c(28, 28))
  expect_equal(res.aov$F, c(36.013, 117.948))
  expect_equal(res.aov$ges, c(0.563, 0.808))
})


test_that("Checking two-way ANOVA test", {
  data("ToothGrowth")
  res.aov <- ToothGrowth %>% anova_test(len ~ supp*dose)
  expect_equal(res.aov$Effect, c("supp", "dose", "supp:dose"))
  expect_equal(res.aov$DFn, c(1, 1, 1))
  expect_equal(res.aov$DFd, c(56, 56, 56))
  expect_equal(res.aov$F, c(12.317, 133.415, 5.333))
  expect_equal(res.aov$p, c(8.94e-04, 1.91e-16, 2.50e-02))
  expect_equal(res.aov$ges, c(0.180, 0.704, 0.087))
})


test_that("Checking repeated measures ANOVA test", {
  data("ToothGrowth")
  df <- ToothGrowth
  df$id <- rep(1:10, 6)
  res.aov <- df %>% anova_test(dv = len, wid = id, within = c(supp, dose))
  anova.table <- res.aov$ANOVA
  sphericity <- res.aov$`Mauchly's Test for Sphericity`
  corrections <- res.aov$`Sphericity Corrections`
  expect_equal(anova.table$Effect, c("supp", "dose", "supp:dose"))
  expect_equal(anova.table$DFn, c(1, 2, 2))
  expect_equal(anova.table$DFd, c(9, 18, 18))
  expect_equal(anova.table$F, c(34.866, 106.470, 2.534))
  expect_equal(anova.table$p, c(2.28e-04, 1.06e-10, 1.07e-01))
  expect_equal(anova.table$ges, c(0.224, 0.773, 0.132))
  expect_equal(sphericity$W, c(0.807, 0.934))
  expect_equal(corrections$GGe, c(0.838, 0.938))
  expect_equal(corrections$HFe, c(1.008, 1.176))
})


test_that("Checking that get_anova_table works with any data frame", {
  data("ToothGrowth")
  expect_is(get_anova_table(ToothGrowth), "data.frame")
})

test_that("Checking that get_anova_table works for grouped repeated measures ANOVA", {
  data("ToothGrowth")
  df <- ToothGrowth
  df$id <- rep(1:10, 6)
  res.aov <- df %>%
    group_by(supp) %>%
    anova_test(dv = len, wid = id, within = dose)
  aov.table <- get_anova_table(res.aov)
  expect_equal(aov.table$F, c(23.936, 57.783))
})


test_that("Checking that get_anova_table performs auto sphericity correction", {
  data("ToothGrowth")
  df <- ToothGrowth
  df$id <- rep(1:10, 6)
  res.aov <- df %>% anova_test(dv = len, wid = id, within = c(supp, dose))
  res.aov2 <- res.aov
  res.aov2$`Mauchly's Test for Sphericity`$p[1] <- 0.05 # significant
  # Correction not applied, because there is not significant sphericity test
  auto <- get_anova_table(res.aov, correction = "auto")
  expect_equal(auto$DFn, c(1, 2, 2))
  expect_equal(auto$DFd, c(9, 18, 18))
  expect_equal(auto$F, c(34.866, 106.470, 2.534))
  # Correction automatically applied to the DF of the effect where sphericity is signiica,t
  auto2 <- get_anova_table(res.aov2, correction = "auto")
  expect_equal(auto2$DFn, c(1, 1.68, 2))
  expect_equal(auto2$DFd, c(9, 15.09, 18))
  expect_equal(auto2$F, c(34.866, 106.470, 2.534))
  # Check that GG correction works for all within-subject variables
  gg <- get_anova_table(res.aov2, correction = "GG")
  expect_equal(gg$DFn, c(1, 1.68, 1.88))
  expect_equal(gg$DFd, c(9, 15.09, 16.88))
  expect_equal(gg$p, c(2.28e-04, 2.79e-09, 1.12e-01))
})

test_that("anova_test gives a well-formed error for a single-level factor (#137)", {
  d <- data.frame(id = factor(1:6), grp = factor(rep("a", 6)), score = c(1, 2, 3, 4, 5, 6))
  expect_error(
    anova_test(d, dv = score, wid = id, between = grp),
    "has only one level"
  )
  # regression for the missing space (#137): must read "grp has", not "grphas"
  msg <- tryCatch(
    anova_test(d, dv = score, wid = id, between = grp),
    error = function(e) conditionMessage(e)
  )
  expect_match(msg, "Variable grp has only one level")
})

test_that("anova_test results are dplyr-compatible: rstatix_test before data.frame (#106)", {
  g <- ToothGrowth %>% group_by(supp) %>% anova_test(len ~ dose)
  # class order: the subclasses must come before data.frame (vctrs/dplyr requirement)
  expect_lt(match("rstatix_test", class(g)), match("data.frame", class(g)))
  # grouped: filter / mutate must work (previously errored "must be a vector")
  expect_true(is.data.frame(g %>% dplyr::filter(p < 1)))
  expect_true(is.data.frame(g %>% dplyr::mutate(z = p)))
  # ungrouped and get_anova_table() output too
  u <- ToothGrowth %>% anova_test(len ~ dose)
  expect_true(is.data.frame(u %>% dplyr::filter(p < 1)))
  expect_true(is.data.frame(get_anova_table(g) %>% dplyr::filter(p < 1)))
  # repeated-measures get_anova_table() (with sphericity correction) is dplyr-compatible too
  set.seed(1)
  rm <- data.frame(id = factor(rep(1:10, 3)), time = factor(rep(c("t1","t2","t3"), each = 10)),
                   score = c(rnorm(10, 5), rnorm(10, 6), rnorm(10, 8)))
  rm.aov <- anova_test(rm, dv = score, wid = id, within = time)
  expect_true(is.data.frame(get_anova_table(rm.aov, correction = "GG") %>% dplyr::filter(p < 1)))
  # dispatch is preserved by the reorder
  expect_true(inherits(u, "anova_test"))
  expect_true(inherits(g, "grouped_anova_test"))
})

test_that("anova_test class contract: rstatix_test first, specific class at [2] (#283 revdep)", {
  # Reverse dependencies (e.g. GimmeMyStats, GimmeMyPlot) dispatch on class()[2]
  # (method <- sub('_test', '', class(x)[2])). The class vector must keep
  # 'rstatix_test' first (dplyr/vctrs, #106) AND the specific test class at
  # position 2, so class()[2] resolves to the test name -- do not silently
  # reorder these again (that broke revdeps in the 1.0.0 pretest).
  u <- ToothGrowth %>% anova_test(len ~ dose)
  expect_identical(class(u), c("rstatix_test", "anova_test", "data.frame"))
  expect_identical(class(u)[2], "anova_test")            # class[2] -> "anova"
  # grouped output follows the same contract
  g <- ToothGrowth %>% group_by(supp) %>% anova_test(len ~ dose)
  expect_identical(class(g)[1], "rstatix_test")
  expect_identical(class(g)[2], "grouped_anova_test")
  # #106 invariant restated: rstatix_test strictly before data.frame
  expect_lt(match("rstatix_test", class(u)), match("data.frame", class(u)))
  # the exact downstream idiom must resolve to the intended method
  method <- sub("_test", "", class(u)[2])
  method <- ifelse(method == "data.frame", "anova", method)
  expect_identical(method, "anova")
})

test_that("add_xy_position on an anova_test gives an informative error, not a class error (#111)", {
  g <- ToothGrowth %>% group_by(supp) %>% anova_test(len ~ dose)
  # omnibus ANOVA has no pairwise comparisons: needs a pairwise post-hoc instead
  expect_error(add_xy_position(g), "group1 and group2")
})

test_that("anova_test effect.size = c('pes','ges') returns BOTH columns (#180)", {
  both <- iris %>%
    anova_test(Sepal.Length ~ Sepal.Width + Species, effect.size = c("pes", "ges")) %>%
    get_anova_table()
  expect_true(all(c("pes", "ges") %in% colnames(both)))
  # both-call values match the standalone single-effect-size computations (no regression)
  ges <- iris %>% anova_test(Sepal.Length ~ Sepal.Width + Species, effect.size = "ges") %>% get_anova_table()
  pes <- iris %>% anova_test(Sepal.Length ~ Sepal.Width + Species, effect.size = "pes") %>% get_anova_table()
  expect_equal(both$ges, ges$ges)
  expect_equal(both$pes, pes$pes)
  # single-effect-size output is unchanged: exactly one of the two columns
  expect_false("pes" %in% colnames(ges))
  expect_false("ges" %in% colnames(pes))
})

test_that("repeated-measures anova_test gives clear errors for degenerate designs (#216, #146, #116, #134, #102)", {
  set.seed(1)
  good <- data.frame(
    id = factor(rep(1:8, 3)),
    time = factor(rep(c("t1", "t2", "t3"), each = 8)),
    y = rnorm(24)
  )
  # valid balanced design still works (no false positive)
  expect_silent(suppressMessages(anova_test(good, dv = y, wid = id, within = time)))
  # a subject with >1 observation per within-cell -> clear "duplicated cells" message
  dup <- rbind(good, good[1, ])
  expect_error(anova_test(dup, dv = y, wid = id, within = time), "duplicated cells")
  # wid not repeated across within levels (no complete subject) -> clear message
  notrep <- data.frame(
    id = factor(1:18),
    grp = factor(rep(c("w1", "w2", "w3"), each = 6)),
    y = rnorm(18)
  )
  expect_error(anova_test(notrep, dv = y, wid = id, within = grp), "complete set")
  # a single incomplete subject is NOT rejected (no false positive)
  expect_silent(suppressMessages(anova_test(good[-1, ], dv = y, wid = id, within = time)))
})

context("test-doo")

test_that("doo keeps heterogeneous per-group results in a list-column (#83)", {
  # Regression for #83: a grouped repeated-measures anova_test() can return a
  # plain data frame for one group (no sphericity correction) and a list (ANOVA +
  # Mauchly + sphericity corrections) for others. doo() previously decided whether
  # to unnest from the FIRST element only, then crashed in unnest() trying to
  # combine a data.frame with a list. It must instead keep a list-column.
  df <- data.frame(g = rep(c("a", "b"), each = 4), x = c(1, 2, 3, 4, 5, 6, 7, 8))
  # nest() drops the grouping column, so discriminate on the nested data: group
  # "a" -> data frame result, group "b" -> list result
  f <- function(d){
    if(d$x[1] <= 4) tibble::tibble(v = 1) else list(tab = 1:3)
  }
  res <- df %>% dplyr::group_by(g) %>% doo(f)
  expect_true(".results." %in% colnames(res))   # list-column kept, no crash
  expect_equal(nrow(res), 2L)
  expect_true(is.data.frame(res$.results.[[1]]))
  expect_true(is.list(res$.results.[[2]]) && !is.data.frame(res$.results.[[2]]))
})

test_that("doo still unnests homogeneous data-frame results (#83 no-regression)", {
  res <- ToothGrowth %>%
    dplyr::group_by(dose) %>%
    doo(~broom::tidy(t.test(len ~ supp, data = .)))
  expect_false(".results." %in% colnames(res))          # unnested as before
  expect_true(all(c("estimate", "p.value") %in% colnames(res)))
  expect_equal(nrow(res), 3L)
})

test_that("get_anova_table() handles a grouped rm-ANOVA with mixed corrections (#83)", {
  # End-to-end: a grouped one-way repeated-measures ANOVA where the per-group
  # results differ in whether a sphericity correction is produced. Build one group
  # whose within scores are (near) perfectly linear across blocks (so the
  # sphericity machinery degenerates) and others with ordinary variation.
  set.seed(123)
  k <- 4; n <- 8
  make_group <- function(name, linear){
    block <- factor(rep(seq_len(n), each = k))
    w <- factor(rep(seq_len(k), times = n))
    y <- if(linear) as.numeric(w) + rep(seq_len(n), each = k) * 0.01
         else rnorm(n * k) + as.numeric(w) * 0.4
    data.frame(g = name, id = interaction(name, block), w = w, y = y)
  }
  df <- rbind(make_group("g1", TRUE), make_group("g2", FALSE), make_group("g3", FALSE))
  res <- df %>%
    dplyr::group_by(g) %>%
    anova_test(dv = y, wid = id, within = w)
  tab <- get_anova_table(res)        # must not error and must cover all 3 groups
  expect_equal(nrow(tab), 3L)
  expect_setequal(as.character(tab$g), c("g1", "g2", "g3"))
  expect_false(any(is.na(tab$F)))
  expect_false("data" %in% colnames(tab))
})

context("test-anova_test-ci")

# #18: opt-in confidence intervals for partial eta-squared, computed in base R
# from the noncentral F distribution, matching effectsize::eta_squared().

tg <- function(){ d <- ToothGrowth; d$dose <- factor(d$dose); d }

test_that("ci = NULL (default) leaves the output unchanged (no-regression) (#18)", {
  d <- tg()
  res <- d %>% anova_test(len ~ supp * dose)
  expect_false(any(c("conf.low", "conf.high") %in% colnames(res)))
  res_pes <- d %>% anova_test(len ~ supp * dose, effect.size = "pes")
  expect_false(any(c("conf.low", "conf.high") %in% colnames(res_pes)))
})

test_that("ci adds conf.low/conf.high bracketing pes (#18)", {
  d <- tg()
  res <- d %>% anova_test(len ~ supp * dose, effect.size = "pes", ci = 0.95)
  expect_true(all(c("pes", "conf.low", "conf.high") %in% colnames(res)))
  expect_true(all(res$conf.low <= res$pes + 1e-8))
  expect_true(all(res$pes <= res$conf.high + 1e-8))
  expect_true(all(res$conf.low >= 0 & res$conf.high <= 1))
})

test_that("ci reproduces the known effectsize partial-eta-squared interval (#18)", {
  # Expected bounds validated during development against
  # effectsize::eta_squared(model, partial = TRUE, ci = 0.95,
  # alternative = "two.sided") on aov(len ~ supp * dose). Hard-coded here so the
  # test needs no dependency on effectsize (which is not imported/suggested).
  d <- tg()
  res <- d %>% anova_test(len ~ supp * dose, effect.size = "pes", ci = 0.95)
  res <- res[order(res$Effect), ]
  expect_equal(res$conf.low[res$Effect == "supp"],      0.059, tolerance = 0.002)
  expect_equal(res$conf.high[res$Effect == "supp"],     0.402, tolerance = 0.002)
  expect_equal(res$conf.low[res$Effect == "dose"],      0.662, tolerance = 0.002)
  expect_equal(res$conf.high[res$Effect == "dose"],     0.838, tolerance = 0.002)
  expect_equal(res$conf.low[res$Effect == "supp:dose"], 0.001, tolerance = 0.002)
  expect_equal(res$conf.high[res$Effect == "supp:dose"],0.295, tolerance = 0.002)
})

test_that("ci works for one-way and repeated-measures designs (#18)", {
  d <- tg()
  ow <- d %>% anova_test(len ~ dose, effect.size = "pes", ci = 0.95)
  expect_true(all(c("conf.low", "conf.high") %in% colnames(ow)))
  set.seed(1)
  dr <- data.frame(
    id = factor(rep(1:12, 3)),
    time = factor(rep(c("t1", "t2", "t3"), each = 12)),
    score = c(rnorm(12, 5), rnorm(12, 6), rnorm(12, 8))
  )
  rm <- dr %>% anova_test(dv = score, wid = id, within = time, effect.size = "pes", ci = 0.95)
  rm <- get_anova_table(rm)
  expect_true(all(c("conf.low", "conf.high") %in% colnames(rm)))
  expect_true(all(rm$conf.low <= rm$pes & rm$pes <= rm$conf.high))
})

test_that("ci requires effect.size to include 'pes' and a valid level (#18)", {
  d <- tg()
  expect_error(d %>% anova_test(len ~ supp * dose, ci = 0.95), "partial eta-squared")
  expect_error(d %>% anova_test(len ~ supp * dose, effect.size = "pes", ci = 95),
               "between 0 and 1")
  expect_error(d %>% anova_test(len ~ supp * dose, effect.size = "pes", ci = -1),
               "between 0 and 1")
})

test_that("a narrower confidence level gives a narrower interval (#18)", {
  d <- tg()
  r90 <- d %>% anova_test(len ~ dose, effect.size = "pes", ci = 0.90)
  r99 <- d %>% anova_test(len ~ dose, effect.size = "pes", ci = 0.99)
  expect_true((r90$conf.high - r90$conf.low) < (r99$conf.high - r99$conf.low))
})

#' @importFrom tibble tibble
#' @export
tibble::tibble
#' @importFrom dplyr mutate
#' @export
dplyr::mutate
#' @importFrom dplyr filter
#' @export
dplyr::filter
#' @importFrom dplyr group_by
#' @export
dplyr::group_by
#' @importFrom dplyr select
#' @export
dplyr::select
#' @importFrom dplyr desc
#' @export
dplyr::desc
#' @importFrom tidyr drop_na
#' @export
tidyr::drop_na
#' @importFrom tidyr gather
#' @export
tidyr::gather
#' @importFrom tidyr spread
#' @export
tidyr::spread
#' @importFrom generics tidy
#' @export
generics::tidy
#' @importFrom generics augment
#' @export
generics::augment
#' @importFrom car Anova
#' @export
car::Anova

#' @include utilities.R
NULL
#' Sample n Rows By Group From a Table
#'
#' @description sample n rows by group from a table using the \code{\link[dplyr]{sample_n}()} function.
#'
#' @param data a data frame
#' @param ... Variables to group by
#' @param size the number of rows to select
#' @param replace with or without replacement?
#'
#' @examples
#' ToothGrowth %>% sample_n_by(dose, supp, size = 2)
#' @name sample_n_by
#' @export
sample_n_by <- function(data, ..., size = 1, replace = FALSE){
  data %>%
    group_by(...) %>%
    dplyr::sample_n(size = size, replace = replace) %>%
    dplyr::ungroup()
}

#' @include utilities.R
NULL
#'Exact Multinomial Test
#'
#'@description Performs an exact multinomial test. Alternative to the chi-square test of goodness-of-fit-test when the sample
#'  size is small.
#'
#'@inheritParams binom_test
#'
#'@seealso \link{binom_test}
#'@return return a data frame containing the p-value and its significance.
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#'
#' @examples
#' # Data
#' tulip <- c(red = 81, yellow = 50, white = 27)
#'
#' # Question 1: are the color equally common ?
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # this is a test of homogeneity
#' res <- multinom_test(tulip)
#' res
#'
#' attr(res, "descriptives")
#'
#' # Pairwise comparisons between groups
#' pairwise_binom_test(tulip, p.adjust.method = "bonferroni")
#'
#'
#' # Question 2: comparing observed to expected proportions
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # this is a goodness-of-fit test
#' expected.p <- c(red = 0.5, yellow = 0.33, white = 0.17)
#' res <- multinom_test(tulip, expected.p)
#' res
#' attr(res, "descriptives")
#'
#' # Pairwise comparisons against a given probabilities
#' pairwise_binom_test_against_p(tulip, expected.p)
#' @export
multinom_test <- function (x, p = rep(1/length(x), length(x)), detailed = FALSE)
{
  args <- as.list(environment()) %>%
    add_item(method = "exact_multinom_test")
  if (!is.vector(x)) {
    stop("'x' must be a vector")
  }
  if (sum(p) != 1) {
    stop("sum of probabilities must be 1")
  }
  if (length(x) != length(p)) {
    stop("'x' and 'p' lengths differ")
  }
  if(is.null(names(x))){
    names(x) <- paste0("grp", 1:length(x))
  }
  size <- sum(x)
  groups <- length(x)
  numEvents <- choose(size + groups - 1, groups - 1)
  pObs <- stats::dmultinom(x, size, p)
  findVectors <- function(groups, size) {
    if (groups == 1) {
      mat <- size
    }
    else {
      mat <- matrix(rep(0, groups - 1), nrow = 1)
      for (i in 1:size) {
        mat <- rbind(mat, findVectors(groups - 1, i))
      }
      mat <- cbind(mat, size - rowSums(mat))
    }
    mat
  }
  eventMat <- findVectors(groups, size)
  eventProb <- apply(eventMat, 1, function(x) stats::dmultinom(x, size, p))
  p.val <- sum(eventProb[eventProb <= pObs])
  results <- tibble(
    p = p.val, method = "Exact multinomial test"
  ) %>%
    add_significance() %>%
    select(.data$p, .data$p.signif, .data$method)
  descriptives <- tibble(
    group = names(x),
    observed = x, expected = p*size
  )
  if(!detailed){
    results <- results[, c("p", "p.signif")]
  }
  results %>%
    set_attrs(args = args, descriptives = descriptives) %>%
    add_class(c("rstatix_test", "exact_multinom_test"))
}

#' @include utilities.R
NULL
#'Fligner-Killeen Test
#'
#'
#'@description Provides a pipe-friendly framework to perform the Fligner-Killeen
#'  test, a non-parametric (rank-based) test of the homogeneity of group
#'  variances. It is robust against departures from normality and is a useful
#'  alternative to \code{\link{levene_test}()}. Wrapper around the function
#'  \code{\link[stats]{fligner.test}()}.
#'@param data a data.frame containing the variables in the formula.
#'@param formula a formula of the form \code{x ~ group} where \code{x} is a
#'  numeric variable giving the data values and \code{group} is a factor with
#'  one or multiple levels giving the corresponding groups. For example,
#'  \code{formula = TP53 ~ cancer_group}.
#'@param ... other arguments to be passed to the function
#'  \code{\link[stats]{fligner.test}}.
#'
#'@return return a data frame with the following columns: \itemize{ \item
#'  \code{.y.}: the y variable used in the test. \item \code{n}: sample count.
#'  \item \code{statistic}: the Fligner-Killeen test statistic (a chi-squared
#'  statistic) used to compute the p-value. \item \code{df}: the degrees of
#'  freedom. \item \code{p}: p-value. \item \code{method}: the statistical test
#'  used to compare groups.}
#'@seealso \code{\link{levene_test}}
#' @examples
#' # Load data
#' #:::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Fligner-Killeen test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% fligner_test(len ~ dose)
#'
#' # Grouped data
#' df %>%
#'   group_by(supp) %>%
#'   fligner_test(len ~ dose)
#'@name fligner_test
#'@export
fligner_test <- function(data, formula, ...){
  args <- c(as.list(environment()), list(...)) %>%
    .add_item(method = "fligner_test")
  if(is_grouped_df(data)){
    results <- data %>% doo(.fligner_test, formula, ...)
  }
  else{
    results <- .fligner_test(data, formula, ...)
  }
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "fligner_test"))
}

.fligner_test <- function(data, formula, ...)
{
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  term <- statistic <- p <- df <- method <- NULL
  # Report the number of observations actually used by the test. fligner.test()
  # drops rows with missing outcome/group values via complete.cases(), so n must
  # be the complete-case count, not nrow(data) which would be inflated when the
  # data contain NAs. Forcing na.action = na.omit makes n match the test's
  # effective sample size; for data without NAs it equals nrow(data).
  n <- nrow(stats::model.frame(formula, data = data, na.action = stats::na.omit))
  stats::fligner.test(formula, data = data, ...) %>%
    as_tidy_stat() %>%
    select(statistic, df, p, method) %>%
    add_column(.y. = outcome, n = n, .before = "statistic")
}

#' @include utilities.R
NULL

#' Extract Label Information from Statistical Tests
#' @description Extracts label information from statistical tests. Useful for
#'   labelling plots with test outputs.
#' @param stat.test statistical test results returned by \code{rstatix}
#'   functions.
#' @param description the test description used as the prefix of the label.
#'   Examples of description are "ANOVA", "Two Way ANOVA". To remove the default
#'   description, specify \code{description = NULL}. If missing, we'll try to
#'   guess the statistical test default description.
#' @param p.col character specifying the column containing the p-value. Default
#'   is \code{"p"}, can be \code{"p.adj"}.
#' @param type the label type. Can be one of "text" and "expression". Partial
#'   match allowed. If you want to add the label onto a ggplot, it might be
#'   useful to specify \code{type = "expresion"}.
#' @param correction character, considered only in the case of ANOVA test. Which sphericity
#'   correction of the degrees of freedom should be reported for the
#'   within-subject factors (repeated measures). The default is set to
#'   \code{"GG"} corresponding to the Greenhouse-Geisser correction. Possible
#'   values are \code{"GG"}, \code{"HF"} (i.e., Hyunh-Feldt correction),
#'   \code{"none"} (i.e., no correction) and \code{"auto"} (apply automatically
#'   GG correction if the sphericity assumption is not for within-subject
#'   design.
#' @param row numeric, the row index to be considered. If NULL, the last row is
#'   automatically considered for ANOVA test.
#' @param statistic.text character specifying the test statistic. For example
#'   \code{statistic.text = "F"} (for ANOVA test ); \code{statistic.text = "t"}
#'   (for t-test ).
#' @param statistic the numeric value of a statistic.
#' @param p the p-value of the test.
#' @param parameter string containing the degree of freedom (if exists). Default
#'   is \code{NA} to accommodate non-parametric tests. For example
#'   \code{parameter = "1,9"} (for ANOVA test. Two parameters exist: DFn and
#'   DFd); \code{sparameter = "9"} (for t-test ).
#' @param n sample count, example: \code{n = 10}.
#' @param effect.size the effect size value
#' @param effect.size.text a character specifying the relevant effect size. For
#'   example, for \code{Cohens d} statistic, \code{effect.size.text = "d"}. You
#'   can also use plotmath expression as follow \code{quote(italic("d"))}.
#' @param detailed logical value. If TRUE, returns detailed label.
#' @return a text label or an expression to pass to a plotting function.
#' @examples
#' # Load data
#' #:::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # One-way ANOVA test
#' #:::::::::::::::::::::::::::::::::::::::::
#' anov <- df %>% anova_test(len ~ dose)
#' get_test_label(anov, detailed = TRUE, type = "text")
#'
#' # Two-way ANOVA test
#' #:::::::::::::::::::::::::::::::::::::::::
#' anov <- df %>% anova_test(len ~ supp*dose)
#' get_test_label(anov, detailed = TRUE, type = "text",
#'    description = "Two Way ANOVA")
#'
#'
#' # Kruskal-Wallis test
#' #:::::::::::::::::::::::::::::::::::::::::
#' kruskal<- df %>% kruskal_test(len ~ dose)
#' get_test_label(kruskal, detailed = TRUE, type = "text")
#'
#' # Wilcoxon test
#' #:::::::::::::::::::::::::::::::::::::::::
#' # Unpaired test
#' wilcox <- df %>% wilcox_test(len ~ supp)
#' get_test_label(wilcox, detailed = TRUE, type = "text")
#'# Paired test
#' wilcox <- df %>% wilcox_test(len ~ supp, paired = TRUE)
#' get_test_label(wilcox, detailed = TRUE, type = "text")
#'
#' # T test
#' #:::::::::::::::::::::::::::::::::::::::::
#' ttest <- df %>% t_test(len ~ dose)
#' get_test_label(ttest, detailed = TRUE, type = "text")
#'
#'
#' # Pairwise comparisons labels
#' #:::::::::::::::::::::::::::::::::::::::::
#' get_pwc_label(ttest, type = "text")
#'
#'
#' # Create test labels
#' #:::::::::::::::::::::::::::::::::::::::::
#' create_test_label(
#'   statistic.text = "F", statistic = 71.82,
#'   parameter = "4, 294",
#'   p = "<0.0001",
#'   description = "ANOVA",
#'   type = "text"
#' )
#'
#' @describeIn get_test_label Extract label from pairwise comparisons.
#' @export
get_pwc_label <- function(stat.test, type = c("expression", "text")){
  methods <- get_pairwise_comparison_methods()
  stat.test %>% stop_ifnot_class(names(methods))
  type <- match.arg(type)
  args <- attr(stat.test, "args")
  stat.method <- methods[args$method]
  p.adjust.method <- args$p.adjust.method  %>%
    to_uppercase_first_letter()
  if(! "p.adj" %in% colnames(stat.test)){
    p.adjust.method <- "None"
  }
  if(type == "text"){
    paste0("pwc: ", stat.method, "; p.adjust: ", p.adjust.method)
  }
  else if(type == "expression"){
    substitute(
      expr = paste(
        "pwc: ", bold(stat.method),
          "; p.adjust: ", bold(p.adjust.method)
        ),
      env = list(stat.method = stat.method, p.adjust.method = p.adjust.method)
    )
  }
}

#' @describeIn get_test_label Extract labels for statistical tests.
#' @export
get_test_label <- function(stat.test, description = NULL, p.col = "p",
                           type = c("expression", "text"),
                           correction = c("auto", "GG", "HF", "none"), row = NULL, detailed = FALSE){
  type = match.arg(type)
  allowed.tests <- c(
    get_pairwise_comparison_methods(),
    kruskal_test = "Kruskal-Wallis",
    friedman_test = "Friedman test",
    anova_test = "Anova",
    welch_anova_test = "Welch ANOVA",
    chisq_test = "Chi-square test",
    exact_multinom_test = "Exact multinomial test",
    exact_binom_test = "Exact binomial test",
    cochran_qtest = "Cochran Q test",
    chisq_trend_test = "Chi-square trend test"
  )
  stop_ifnot_class(stat.test, .class = names(allowed.tests))
  is_anova_test <- inherits(stat.test, "anova_test")
  if(is_anova_test){
    stat.test <- get_anova_table(stat.test, correction = correction)
    if(is.null(row)) row <-  nrow(stat.test) # consider the last row
  }
  if(!is.null(row)) {
    stat.test <- stat.test %>%
      keep_only_tbl_df_classes() %>%
      dplyr::slice(row)
  }

  statistic.text <- get_statistic_text(stat.test, type = type)
  statistic <- get_statistic(stat.test)
  df <- get_df(stat.test)
  n <- get_n(stat.test)
  effect <- get_effect_size(stat.test, type)
  effect.size <- effect$value
  effect.size.text <- effect$text

  if(missing(description)){
    description <- get_description(stat.test)
  }
  if(!is.null(description)){
    if(description != ""){
      description <- paste0(description, ", ")
    }
  }
  if(!(p.col %in% colnames(stat.test))){
    # automatic detection of p.col
    p.col <- p_detect(stat.test)
  }
  stat.test <- stat.test %>%
    keep_only_tbl_df_classes() %>%
    select(!!sym(p.col)) %>%
    rename(p = p.col) %>%
    mutate(
      row.id = 1:nrow(stat.test), n = n,
      statistic = statistic, parameter = df,
      effect.size = effect.size
    )
  if(is.numeric(stat.test$p)){
    stat.test$p <- p_format(stat.test$p, 3)
  }

  get_label_func <- switch (
    type,
    expression = create_test_label.expression,
    text = create_test_label.text
  )
  get_label_func_df <- function(df){
    get_label_func(
      description, statistic.text = statistic.text,
      statistic = df$statistic, parameter = df$parameter,
      p = df$p, n = df$n,  effect.size = df$effect.size,
      effect.size.text = effect.size.text, detailed = detailed
    )
  }
  if(nrow(stat.test) > 1){
    results <- stat.test %>%
      group_by(.data$row.id) %>%
      doo(get_label_func_df) %>%
      pull(.data$.results.)
  }
  else{
    results <- get_label_func_df(stat.test)
  }
  results
}

#' @describeIn get_test_label Create labels from user specified test results.
#' @export
create_test_label <- function(
  statistic.text, statistic, p, parameter = NA, description = NULL, n = NA, effect.size = NA, effect.size.text = NA,
  type = c("expression", "text"), detailed = FALSE)
{
  type <- match.arg(type)
  if(!is.null(description)){
    if(description != ""){
      description <- paste0(description, ", ")
    }
  }
  else description <- ""
  label_func <- switch(
    type,
    text = create_test_label.text,
    expression = create_test_label.expression,
    create_test_label.text
  )
  label_func(
    description = description, statistic.text = statistic.text,
    statistic = statistic, parameter = parameter,
    p = p, n = n,  effect.size = effect.size,
    effect.size.text = effect.size.text, detailed = detailed
  )
}

# Build test labeles
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# description: stat test description, e.g "T test"
# statistic.text: statistic text, example: "t",
# statistic: statistic value, example: 10
# parameter: string containing the degree of freedom,
#   ex: "9"  for t-test or "1,9" for ANOVA (DFn = 1 and DFd = 9)
# p: p value
# n: sample count
create_test_label.expression <- function(
  description, statistic.text, statistic, parameter, p,  n = NA,
  effect.size = NA, effect.size.text = NA, detailed = FALSE)
  {

  if(is.na(parameter)) parameter <- ""
  else parameter <- paste0("(", parameter, ")")
  # Sample count
  if(is.na(n)) {
    n <- ""
  }
  else{
    n <- substitute(
      expr = paste(", ", italic("n"), " = ", n),
      env = list(n = n)
    )
  }
  # Effect size
  if(is.na(effect.size)){
    effect.size <- ""
  }
  else{
    effect.size <- round_value(effect.size, 2)
    effect.size <- substitute(
      expr = paste(", ", effect.size.text, " = ", effect.size),
      env = list(effect.size.text = effect.size.text, effect.size = effect.size)
    )
  }
  # Create label
  statistic <- round_value(statistic, 2)
  equal <- " = "
  if(is.na(statistic))
    statistic.text <- equal <- statistic <- ""
  else
    statistic <- paste0(statistic, ", ")
  env <- as.list(environment())
  if(detailed){
    substitute(
      expr = paste(
        description, statistic.text, parameter, equal, statistic,
        italic("p"), " = ", p, effect.size, n
      ),
      env = env
    )
  }
  else{
    substitute(
      expr = paste(description, italic("p"), " = ", p),
      env = env
      )
  }
}

create_test_label.text <- function(description, statistic.text,
                                statistic, parameter, p,  n = NA,
                                effect.size = NA, effect.size.text = NA,  detailed = FALSE){
  if(is.na(parameter)) parameter <- ""
  else parameter <- paste0("(", parameter, ")")
  if(is.na(effect.size)) effect.size <- ""
  else effect.size <- paste0(", ", effect.size.text, " = ", effect.size)
  if(is.na(n)) n <- ""
  else n <- paste0(", ", "n", " = ", n)
  if(!is.na(statistic)){
    statistics <- paste0(statistic.text, parameter, " = ", round_value(statistic, 2), ", ")
  }
  else statistics <- ""
  if(detailed){
    paste0(
      description, statistics,
      "p", " = ", p, effect.size, n
    )
  }
  else{
    paste0(description, "p = ", p)
  }
}

# Get label parameters
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

# Statical test text: F, t, W, V, X2, -------------------------------------------
get_statistic_text <- function(stat.test, type = c("expression", "text")){
  type <- match.arg(type)
  args <- attr(stat.test, "args")
  stat.method <- args$method
  is.paired <- args$paired
  if(!is.null(is.paired)){
    if(is.paired & stat.method == "wilcox_test"){
      stat.method = "wilcox_test_paired"
    }
  }
  if(is.null(is.paired)) is.paired <- FALSE
  if(type == "expression"){
    statistic.text <- switch(
      stat.method,
      t_test = quote(italic("t")),
      wilcox_test = quote(italic("W")),
      wilcox_test_paired = quote(italic("V")),
      sign_test = quote(italic("S")),
      dunn_test = quote(italic("Z")),
      emmeans_test = quote(italic("t")),
      tukey_hsd = quote(italic("t")),
      games_howell_test = quote(italic("t")),
      kruskal_test = quote(italic(chi)^2),
      friedman_test = quote(italic(chi)^2),
      anova_test = quote(italic("F")),
      welch_anova_test = quote(italic("F")),
      chisq_test = quote(italic(chi)^2),
      mcnemar_test = quote(italic(chi)^2),
      prop_test = quote(italic(chi)^2),
      cochran_qtest = quote(italic(chi)^2),
      chisq_trend_test = quote(italic(chi)^2),
      quote(italic("Stat"))
    )
  }
  else{
    statistic.text <- switch(
      stat.method,
      t_test = "t",
      wilcox_test = "W",
      wilcox_test_paired = "V",
      sign_test = "S",
      dunn_test = "Z",
      emmeans_test = "t",
      tukey_hsd = "t",
      games_howell_test = "t",
      kruskal_test = "X2",
      friedman_test = "X2",
      anova_test = "F",
      welch_anova_test = "F",
      chisq_test = "X2",
      mcnemar_test = "X2",
      prop_test = "X2",
      cochran_qtest = "X2",
      chisq_trend_test = "X2",
      "Stat"
    )
  }
  statistic.text
}

# Statistic values -------------------------------------------------
get_statistic <- function(stat.test){
  stat.cols <- colnames(stat.test)
  if("statistic" %in% stat.cols){
    result <- stat.test$statistic
  }
  else if ("F" %in% stat.cols){
    result <- stat.test$F
  }
  else{
    # statistic column not found
    result <- rep(NA, nrow(stat.test))
  }
  result
}

# Degree of freedom-------------------------------------------------
get_df <- function(stat.test){
  args <- attr(stat.test, "args")
  df.cols <- c("df", "DFn", "DFd")
  if(!any(df.cols %in% colnames(stat.test))){
    return(NA)
  }
  if(all(c("DFn", "DFd") %in% colnames(stat.test))){
    dfn <- round_value(stat.test$DFn, 2)
    dfd <- round_value(stat.test$DFd, 2)
    df <- paste(dfn, dfd, sep = ",")
  }
  else{
    df <- round_value(stat.test$df, 2)
  }
  df
}

# Sample count-------------------------------------------------
get_n <- function(stat.test){
  if(inherits(stat.test, "anova_test")){
    .args <- attr(stat.test, "args")
    wid <- .args$wid
    if(is.null(wid)) n <- nrow(.args$data)
    else n <- .args$data %>% pull(!!wid) %>% unique() %>% length()
    stat.test %<>% mutate(n = !!n)
  }
  n.cols <- c("n", "n1", "n2")
  if(!any(n.cols %in% colnames(stat.test))){
    return(NA)
  }
  if("n" %in% colnames(stat.test)){
    stat.test$n
  }
  else if(all(c("n1", "n2") %in% colnames(stat.test))){
    if(is_paired(stat.test)) stat.test$n1
    else stat.test$n1 + stat.test$n2
  }
}

# Statistical test description ---------------------------------
get_description <- function(stat.test){
  tests <- c(
    t_test = "T test",
    wilcox_test = "Wilcoxon test",
    sign_test = "Sign test",
    dunn_test = "Dunn test",
    emmeans_test = "Emmeans test",
    tukey_hsd = "Tukey HSD",
    anova_test = "Anova",
    welch_anova_test = "Welch Anova",
    kruskal_test = "Kruskal-Wallis",
    friedman_test = "Friedman test",
    cor_test = "Correlation",
    prop_test = "Z-Prop test",
    fisher_test = "Fisher's exact test",
    chisq_test = "Chi-square test",
    exact_multinom_test = "Exact multinomial test",
    exact_binom_test = "Exact binomial test",
    mcnemar_test = "McNemar test",
    cochran_qtest = "Cochran Q test",
    chisq_trend_test = "Chi-square trend test"
  )
  args <- attr(stat.test, "args")
  stat.method <- args$method
  if(stat.method %in% names(tests)){
    description <- tests[stat.method]
  }
  else{
    description  <- stat.method
  }
  as.character(description)
}

# Efect size ---------------------------------
get_effect_size <- function(stat.test, type = "text"){
  stat.method <- attr(stat.test, "args")$method
  value <- text <- NA

  if("ges" %in% colnames(stat.test)) {
    value <- stat.test$ges
    if(type == "expression") text <- quote(eta["g"]^2)
    else text <- "eta2[g]"
  }
  else if("pes" %in% colnames(stat.test)) {
    if(type == "expression") text <- quote(eta["p"]^2)
    else text <- "eta2[p]"
    value <- stat.test$pes
  }
  else if("effsize" %in% colnames(stat.test)){
    value <- stat.test$effsize
    if(type == "expression"){
      text <- switch(
        stat.method,
        t_test = quote(italic("d")),
        wilcox_test = quote(italic("r")),
        kruskal_test = quote(eta["H"]^2),
        friedman_test = quote(italic("W")["Kendall"]),
        quote(italic("effsize"))
      )
    }
    else{
      text <- switch(
        stat.method,
        t_test = "d",
        wilcox_test = "r",
        kruskal_test = "eta2[H]",
        friedman_test = "W[Kendall]",
        "effsize"
      )
    }
  }
  list(value = value, text = text)
}

# Check if paired stat test--------------------------------------------
is_paired <- function(stat.test){
  args <- attr(stat.test, "args")
  is.paired <- args$paired
  if(is.null(is.paired)) is.paired <- FALSE
  is.paired
}

context("test-friedman_conover_test")

# A fixed, non-degenerate balanced complete block design (3 treatments, 6 blocks).
demo_df <- data.frame(
  id        = factor(rep(1:6, 3)),
  treatment = factor(rep(c("A", "B", "C"), each = 6)),
  score     = c(4, 6, 3, 5, 4, 5,   7, 8, 6, 7, 9, 6,   6, 9, 7, 8, 8, 9)
)

# Self-contained base-R re-derivation of the Durbin-Conover pairwise statistic,
# to validate without depending on an external package.
durbin_conover_ref <- function(data, dv, within, wid){
  data[[within]] <- factor(data[[within]])
  trt <- levels(data[[within]])
  m <- tapply(data[[dv]], list(data[[wid]], factor(data[[within]], levels = trt)),
              function(z) z[1])
  m <- m[, trt, drop = FALSE]
  b <- nrow(m); k <- ncol(m)
  R <- t(apply(m, 1, rank)); Rj <- colSums(R); names(Rj) <- trt
  df <- (b - 1) * (k - 1)
  denom <- sqrt(2 * (b * sum(R^2) - sum(Rj^2)) / df)
  out <- list()
  for(a in seq_len(k)) for(bb in seq_len(a - 1)){
    t <- (Rj[trt[a]] - Rj[trt[bb]]) / denom
    out[[paste(trt[bb], trt[a])]] <- data.frame(
      group1 = trt[bb], group2 = trt[a], statistic = unname(t),
      p = 2 * stats::pt(abs(t), df, lower.tail = FALSE), df = df,
      stringsAsFactors = FALSE
    )
  }
  do.call(rbind, out)
}

test_that("friedman_conover_test matches the Durbin-Conover formula (#8)", {
  res <- demo_df %>% friedman_conover_test(score ~ treatment | id, detailed = TRUE)
  ref <- durbin_conover_ref(demo_df, "score", "treatment", "id")
  res <- res[order(res$group1, res$group2), ]
  ref <- ref[order(ref$group1, ref$group2), ]
  expect_equal(res$statistic, ref$statistic, tolerance = 1e-6)
  expect_equal(res$p, ref$p, tolerance = 1e-9)
  expect_equal(unique(res$df), (6 - 1) * (3 - 1))      # (b-1)(k-1)
})

test_that("friedman_conover_test reproduces PMCMRplus::frdAllPairsConoverTest values (#8)", {
  # Reference produced by PMCMRplus::frdAllPairsConoverTest(...); hard-coded so the
  # test has no external dependency. PMCMRplus reports unsigned t and raw p.
  res <- demo_df %>% friedman_conover_test(score ~ treatment | id, p.adjust.method = "none", detailed = TRUE)
  res <- res[order(res$group1, res$group2), ]
  expect_equal(abs(res$statistic), c(4.472136, 5.590170, 1.118034), tolerance = 1e-5)
  expect_equal(res$p, c(0.0011934670, 0.0002308192, 0.2896916), tolerance = 1e-6)
})

test_that("friedman_conover_test default (non-detailed) returns the expected columns (#8)", {
  res <- demo_df %>% friedman_conover_test(score ~ treatment | id)
  expect_equal(
    colnames(res),
    c(".y.", "group1", "group2", "n1", "n2", "statistic", "df", "p", "p.adj", "p.adj.signif")
  )
  expect_equal(nrow(res), 3L)                          # k(k-1)/2 comparisons
  expect_equal(unique(c(res$n1, res$n2)), 6L)          # number of blocks
})

test_that("friedman_conover_test detailed adds estimate/method columns (#8)", {
  res <- demo_df %>% friedman_conover_test(score ~ treatment | id, detailed = TRUE)
  expect_true(all(c("estimate", "estimate1", "estimate2", "method") %in% colnames(res)))
  expect_equal(unique(res$method), "Durbin-Conover")
  # estimate is the rank-sum difference, oriented as group2 - group1
  expect_equal(as.numeric(res$estimate), as.numeric(res$estimate2 - res$estimate1), tolerance = 1e-9)
})

test_that("friedman_conover_test ref.group adjusts over only k-1 comparisons (#8)", {
  res <- demo_df %>% friedman_conover_test(score ~ treatment | id, ref.group = "A")
  expect_equal(nrow(res), 2L)
  expect_true(all(res$group1 == "A"))
  expect_equal(res$p.adj, p.adjust(res$p, method = "holm"))
})

test_that("friedman_conover_test all-pairs p.adj uses all comparisons (#8)", {
  res <- demo_df %>% friedman_conover_test(score ~ treatment | id)
  expect_equal(res$p.adj, p.adjust(res$p, method = "holm"))
})

test_that("friedman_conover_test works on grouped data (#8)", {
  df2 <- rbind(
    cbind(demo_df, cohort = "g1"),
    cbind(transform(demo_df, score = score + c(0, 1, -1, 2, 0, 1)), cohort = "g2")
  )
  res <- df2 %>% dplyr::group_by(cohort) %>% friedman_conover_test(score ~ treatment | id)
  expect_true("cohort" %in% colnames(res))
  expect_equal(nrow(res), 6L)                          # 2 cohorts x 3 comparisons
})

test_that("friedman_conover_test errors on an incomplete/unbalanced block design (#8)", {
  inc <- demo_df[-1, ]                                  # drop one cell -> unbalanced
  expect_error(inc %>% friedman_conover_test(score ~ treatment | id), "complete block")
})

test_that("friedman_conover_test errors when the design has no rank variability (#8)", {
  # every subject ranks the treatments in the same order -> denominator is zero
  degen <- data.frame(
    id        = factor(rep(1:4, each = 3)),
    treatment = factor(rep(c("A", "B", "C"), 4)),
    score     = rep(c(1, 2, 3), 4)
  )
  expect_error(degen %>% friedman_conover_test(score ~ treatment | id), "undefined")
})

test_that("friedman_conover_test errors for an invalid ref.group (#8)", {
  expect_error(
    demo_df %>% friedman_conover_test(score ~ treatment | id, ref.group = "Z"),
    "not a level"
  )
})

test_that("friedman_conover_test rejects missing outcome values (#8)", {
  na_df <- demo_df
  na_df$score[1] <- NA
  expect_error(
    na_df %>% friedman_conover_test(score ~ treatment | id),
    "missing values"
  )
})

test_that("friedman_conover_test output works with add_xy_position (ggpubr brackets) (#8)", {
  res <- demo_df %>% friedman_conover_test(score ~ treatment | id)
  pos <- res %>% add_xy_position(x = "treatment")
  expect_true(all(c("xmin", "xmax", "y.position") %in% colnames(pos)))
  expect_equal(nrow(pos), 3L)
  expect_false(any(is.na(pos$y.position)))
})

test_that("drop_formula_block_term strips '| subject' but leaves simple formulas intact", {
  # repeated-measures formula -> only the within term remains
  expect_equal(
    deparse(drop_formula_block_term(score ~ treatment | id)),
    "score ~ treatment"
  )
  # ordinary pairwise formulas are returned unchanged (no-regression):
  # the exact same object is returned, attributes/environment preserved
  expect_true(identical(len ~ dose, drop_formula_block_term(len ~ dose)))
  expect_equal(deparse(drop_formula_block_term(len ~ supp)), "len ~ supp")
  # non-syntactic (backticked) names and compound within terms are preserved
  expect_equal(deparse(drop_formula_block_term(`my score` ~ trt | id)), "`my score` ~ trt")
  expect_equal(deparse(drop_formula_block_term(y ~ a + b | id)), "y ~ a + b")
})

test_that("add_xy_position works on results with non-syntactic outcome names (#8)", {
  df <- data.frame(
    `my score` = c(10, 12, 9, 22, 18, 20, 28, 33, 30, 17, 14, 15, 5, 9, 7),
    trt        = factor(rep(c("a", "b", "c"), 5)),
    id         = factor(rep(1:5, each = 3)),
    check.names = FALSE
  )
  res <- df %>% friedman_conover_test(`my score` ~ trt | id)
  pos <- res %>% add_xy_position(x = "trt")
  expect_true(all(c("xmin", "xmax", "y.position") %in% colnames(pos)))
  expect_false(any(is.na(pos$y.position)))
})

test_that("get_description gives a friendly friedman_conover_test label (#8)", {
  res <- demo_df %>% friedman_conover_test(score ~ treatment | id)
  expect_equal(get_description(res), "Durbin-Conover test")
  lab <- unlist(get_test_label(res, type = "text"))
  expect_true(all(grepl("Durbin-Conover test", lab)))
})

context("test-conover_test")

# Self-contained base-R re-derivation of the Conover-Iman pairwise statistic,
# used to validate conover_test() without depending on an external package.
conover_ref <- function(x, g){
  g <- factor(g)
  N <- length(x); k <- nlevels(g)
  r <- rank(x)
  Rbar <- tapply(r, g, mean); nj <- tapply(x, g, length)
  rs <- tapply(r, g, sum)
  H.unc <- (12 / (N * (N + 1))) * sum(rs^2 / nj) - 3 * (N + 1)
  ties <- table(r)
  H <- H.unc / (1 - sum(ties^3 - ties) / (N^3 - N))
  S2 <- (1 / (N - 1)) * (sum(r^2) - N * (N + 1)^2 / 4)
  scale <- S2 * ((N - 1 - H) / (N - k))
  lev <- levels(g)
  out <- list()
  for(a in seq_len(k)) for(b in seq_len(a - 1)){
    # match pairwise.table orientation: group1 = lower-index level (b)
    t <- (Rbar[lev[a]] - Rbar[lev[b]]) / sqrt(scale * (1/nj[lev[a]] + 1/nj[lev[b]]))
    out[[paste(lev[b], lev[a])]] <- list(
      group1 = lev[b], group2 = lev[a],
      statistic = unname(t), p = 2 * stats::pt(abs(t), N - k, lower.tail = FALSE),
      df = N - k
    )
  }
  do.call(rbind.data.frame, lapply(out, function(o) data.frame(o, stringsAsFactors = FALSE)))
}

test_that("conover_test matches the Conover-Iman formula (#222, #17)", {
  res <- ToothGrowth %>% conover_test(len ~ dose, detailed = TRUE)
  ref <- conover_ref(ToothGrowth$len, ToothGrowth$dose)
  ref <- ref[order(ref$group1, ref$group2), ]
  res <- res[order(res$group1, res$group2), ]
  expect_equal(res$statistic, ref$statistic, tolerance = 1e-6)
  expect_equal(res$p, ref$p, tolerance = 1e-9)
  expect_equal(unique(res$df), 60L - 3L)            # df = N - k
})

test_that("conover_test reproduces PMCMRplus::kwAllPairsConoverTest reference values (#222)", {
  # Reference produced by PMCMRplus::kwAllPairsConoverTest(len ~ factor(dose),
  # ToothGrowth, p.adjust.method = "none"); hard-coded so the test has no external
  # dependency. PMCMRplus reports the unsigned t and the raw (unadjusted) p.
  res <- ToothGrowth %>% conover_test(len ~ dose, p.adjust.method = "none", detailed = TRUE)
  res <- res[order(res$group1, res$group2), ]
  expect_equal(abs(res$statistic), c(6.268626, 11.219642, 4.951016), tolerance = 1e-5)
  expect_equal(res$p, c(5.203093e-08, 4.683820e-16, 6.915398e-06), tolerance = 1e-6)
})

test_that("conover_test default (non-detailed) returns the expected columns (#222)", {
  res <- ToothGrowth %>% conover_test(len ~ dose)
  expect_equal(
    colnames(res),
    c(".y.", "group1", "group2", "n1", "n2", "statistic", "df", "p", "p.adj", "p.adj.signif")
  )
  expect_equal(nrow(res), 3L)                        # k(k-1)/2 comparisons
  expect_false(any(is.na(res$statistic)))
})

test_that("conover_test detailed adds estimate/method columns (#222)", {
  res <- ToothGrowth %>% conover_test(len ~ dose, detailed = TRUE)
  expect_true(all(c("estimate", "estimate1", "estimate2", "method") %in% colnames(res)))
  expect_equal(unique(res$method), "Conover test")
  # estimate is the mean-rank difference, oriented like dunn_test (group2 - group1)
  expect_equal(as.numeric(res$estimate), as.numeric(res$estimate2 - res$estimate1), tolerance = 1e-9)
})

test_that("conover_test ref.group compares each group to the control and adjusts over k-1 (#222)", {
  res <- ToothGrowth %>% conover_test(len ~ dose, ref.group = "0.5")
  expect_equal(nrow(res), 2L)
  expect_true(all(res$group1 == "0.5"))
  expect_equal(res$p.adj, p.adjust(res$p, method = "holm"))   # adjusted over only k-1
})

test_that("conover_test p.adj uses all pairs without ref.group (#222)", {
  res <- ToothGrowth %>% conover_test(len ~ dose)
  expect_equal(res$p.adj, p.adjust(res$p, method = "holm"))
})

test_that("conover_test works on grouped data (#222)", {
  res <- ToothGrowth %>% dplyr::group_by(supp) %>% conover_test(len ~ dose)
  expect_true("supp" %in% colnames(res))
  expect_equal(nrow(res), 6L)                        # 2 supp x 3 comparisons
})

test_that("conover_test errors when all observations are in one group (#222)", {
  df <- data.frame(y = rnorm(10), g = rep("a", 10))
  expect_error(df %>% conover_test(y ~ g), "same group")
})

test_that("conover_test fails cleanly on degenerate (undefined) data (#222)", {
  # one observation per group -> N - k = 0 residual df
  singletons <- data.frame(y = 1:6, g = factor(1:6))
  expect_error(singletons %>% conover_test(y ~ g), "undefined")
  # all values identical -> no rank variability
  constant <- data.frame(y = rep(5, 9), g = factor(rep(c("a", "b", "c"), each = 3)))
  expect_error(constant %>% conover_test(y ~ g), "undefined")
})

test_that("conover_test guard does NOT fire for ordinary inputs (#222)", {
  # representative valid inputs must still run and return finite statistics
  expect_silent_run <- function(expr) {
    res <- expr
    expect_false(any(is.na(res$statistic)))
    expect_true(all(is.finite(res$statistic)))
  }
  expect_silent_run(ToothGrowth %>% conover_test(len ~ dose))
  expect_silent_run(iris %>% conover_test(Sepal.Length ~ Species))
  # two groups, and a group with a single observation (still has residual df)
  df2 <- data.frame(y = c(1, 2, 3, 10, 11, 12), g = factor(rep(c("a", "b"), each = 3)))
  expect_silent_run(df2 %>% conover_test(y ~ g))
})

test_that("get_description gives a friendly conover_test label (#222)", {
  res <- ToothGrowth %>% conover_test(len ~ dose)
  expect_equal(get_description(res), "Conover test")   # not the raw "conover_test"
  # multi-row pairwise output -> get_test_label returns one label per comparison
  lab <- unlist(get_test_label(res, type = "text"))
  expect_true(all(grepl("Conover test", lab)))
})

context("test-fligner_test")

test_that("fligner_test matches stats::fligner.test (#179)", {
  res <- ToothGrowth %>% fligner_test(len ~ dose)
  ref <- stats::fligner.test(len ~ dose, data = ToothGrowth)
  expect_equal(unname(res$statistic), unname(ref$statistic))
  expect_equal(unname(res$df), unname(ref$parameter))
  expect_equal(res$p, ref$p.value)
})

test_that("fligner_test returns the expected columns (#179)", {
  res <- ToothGrowth %>% fligner_test(len ~ dose)
  expect_equal(colnames(res), c(".y.", "n", "statistic", "df", "p", "method"))
  expect_equal(nrow(res), 1L)
  expect_match(unique(res$method), "Fligner")
})

test_that("fligner_test works on grouped data (#179)", {
  res <- ToothGrowth %>% dplyr::group_by(supp) %>% fligner_test(len ~ dose)
  expect_true("supp" %in% colnames(res))
  expect_equal(nrow(res), 2L)
})

test_that("fligner_test reports the complete-case n with NAs (#179)", {
  df <- ToothGrowth
  df$len[1:5] <- NA
  res <- df %>% fligner_test(len ~ dose)
  expect_equal(res$n, 55)                 # complete cases, not nrow = 60
  # statistic still matches base fligner.test on the same data
  ref <- stats::fligner.test(len ~ dose, data = df)
  expect_equal(unname(res$statistic), unname(ref$statistic))
})

test_that("fligner_test n is unchanged for data without NAs (no-regression) (#179)", {
  expect_equal((ToothGrowth %>% fligner_test(len ~ dose))$n, nrow(ToothGrowth))
})

test_that("fligner_test surfaces the base-R error for a single group (#179)", {
  df <- data.frame(y = rnorm(10), g = factor(rep("a", 10)))
  # base fligner.test() errors (message is locale-translated, so don't match text)
  expect_error(df %>% fligner_test(y ~ g))
})

test_that("fligner_test grouped reports per-group n (#179)", {
  res <- ToothGrowth %>% dplyr::group_by(supp) %>% fligner_test(len ~ dose)
  expect_equal(res$n, c(30, 30))
})

test_that("get_description/get_test_label give a friendly fligner_test label (#179)", {
  res <- ToothGrowth %>% fligner_test(len ~ dose)
  expect_equal(get_description(res), "Fligner-Killeen")
  lab <- get_test_label(res, type = "text")
  expect_true(is.character(lab) && grepl("Fligner-Killeen", lab))
})

context("test-friedman_nemenyi_test")

# A fixed, non-degenerate balanced complete block design (3 treatments, 6 blocks).
demo_df <- data.frame(
  id        = factor(rep(1:6, 3)),
  treatment = factor(rep(c("A", "B", "C"), each = 6)),
  score     = c(4, 6, 3, 5, 4, 5,   7, 8, 6, 7, 9, 6,   6, 9, 7, 8, 8, 9)
)

# Self-contained base-R re-derivation of the Nemenyi pairwise statistic and p.
nemenyi_ref <- function(data, dv, within, wid){
  data[[within]] <- factor(data[[within]])
  trt <- levels(data[[within]])
  m <- tapply(data[[dv]], list(data[[wid]], factor(data[[within]], levels = trt)),
              function(z) z[1])
  m <- m[, trt, drop = FALSE]
  b <- nrow(m); k <- ncol(m)
  R <- t(apply(m, 1, rank)); Rj <- colSums(R); names(Rj) <- trt
  denom <- sqrt(b * k * (k + 1) / 12)
  out <- list()
  for(a in seq_len(k)) for(bb in seq_len(a - 1)){
    q <- (Rj[trt[a]] - Rj[trt[bb]]) / denom
    out[[paste(trt[bb], trt[a])]] <- data.frame(
      group1 = trt[bb], group2 = trt[a], statistic = unname(q),
      p.adj = stats::ptukey(abs(q), nmeans = k, df = Inf, lower.tail = FALSE),
      stringsAsFactors = FALSE
    )
  }
  do.call(rbind, out)
}

test_that("friedman_nemenyi_test matches the Nemenyi formula (#141)", {
  res <- demo_df %>% friedman_nemenyi_test(score ~ treatment | id, detailed = TRUE)
  ref <- nemenyi_ref(demo_df, "score", "treatment", "id")
  res <- res[order(res$group1, res$group2), ]
  ref <- ref[order(ref$group1, ref$group2), ]
  expect_equal(res$statistic, ref$statistic, tolerance = 1e-6)
  expect_equal(res$p.adj, ref$p.adj, tolerance = 1e-9)
})

test_that("friedman_nemenyi_test reproduces PMCMRplus::frdAllPairsNemenyiTest values (#141)", {
  # Reference produced by PMCMRplus::frdAllPairsNemenyiTest(...); hard-coded so
  # the test has no external dependency.
  res <- demo_df %>% friedman_nemenyi_test(score ~ treatment | id, detailed = TRUE)
  res <- res[order(res$group1, res$group2), ]
  expect_equal(abs(res$statistic), c(3.2659863, 4.0824829, 0.8164966), tolerance = 1e-5)
  expect_equal(res$p.adj, c(0.05450063, 0.01086516, 0.83222922), tolerance = 1e-6)
})

test_that("friedman_nemenyi_test default returns the expected columns (tukey-style p.adj) (#141)", {
  res <- demo_df %>% friedman_nemenyi_test(score ~ treatment | id)
  expect_equal(
    colnames(res),
    c(".y.", "group1", "group2", "n1", "n2", "statistic", "p.adj", "p.adj.signif")
  )
  expect_false("p" %in% colnames(res))                 # studentized range -> only p.adj
  expect_equal(nrow(res), 3L)
  expect_equal(unique(c(res$n1, res$n2)), 6L)
})

test_that("friedman_nemenyi_test detailed adds estimate/method columns (#141)", {
  res <- demo_df %>% friedman_nemenyi_test(score ~ treatment | id, detailed = TRUE)
  expect_true(all(c("estimate", "estimate1", "estimate2", "method") %in% colnames(res)))
  expect_equal(unique(res$method), "Nemenyi test")
  expect_equal(as.numeric(res$estimate), as.numeric(res$estimate2 - res$estimate1), tolerance = 1e-9)
})

test_that("friedman_nemenyi_test works on grouped data (#141)", {
  df2 <- rbind(
    cbind(demo_df, cohort = "g1"),
    cbind(transform(demo_df, score = score + c(0, 1, -1, 2, 0, 1)), cohort = "g2")
  )
  res <- df2 %>% dplyr::group_by(cohort) %>% friedman_nemenyi_test(score ~ treatment | id)
  expect_true("cohort" %in% colnames(res))
  expect_equal(nrow(res), 6L)                          # 2 cohorts x 3 comparisons
})

test_that("friedman_nemenyi_test rejects missing outcome values (#141)", {
  na_df <- demo_df
  na_df$score[1] <- NA
  expect_error(na_df %>% friedman_nemenyi_test(score ~ treatment | id), "missing values")
})

test_that("friedman_nemenyi_test errors on an incomplete/unbalanced design (#141)", {
  expect_error(demo_df[-1, ] %>% friedman_nemenyi_test(score ~ treatment | id), "complete block")
})

test_that("friedman_nemenyi_test requires at least two blocks (#141)", {
  one_block <- data.frame(
    id = factor(c(1, 1, 1)), treatment = factor(c("A", "B", "C")), score = c(3, 5, 8)
  )
  expect_error(one_block %>% friedman_nemenyi_test(score ~ treatment | id), "two blocks")
  # the valid design (6 blocks) is unaffected
  expect_silent(suppressWarnings(demo_df %>% friedman_nemenyi_test(score ~ treatment | id)))
})

test_that("friedman_nemenyi_test output works with add_xy_position (ggpubr brackets) (#141)", {
  res <- demo_df %>% friedman_nemenyi_test(score ~ treatment | id)
  pos <- res %>% add_xy_position(x = "treatment")
  expect_true(all(c("xmin", "xmax", "y.position") %in% colnames(pos)))
  expect_false(any(is.na(pos$y.position)))
})

test_that("get_description gives a friendly friedman_nemenyi_test label (#141)", {
  res <- demo_df %>% friedman_nemenyi_test(score ~ treatment | id)
  expect_equal(get_description(res), "Nemenyi test")
  lab <- unlist(get_test_label(res, type = "text"))
  expect_true(all(grepl("Nemenyi test", lab)))
})

context("test-ks_test")

test_that("ks_test two-sample matches stats::ks.test (#92, #168)", {
  res <- ToothGrowth %>% ks_test(len ~ supp, detailed = TRUE)
  oj <- ToothGrowth$len[ToothGrowth$supp == "OJ"]
  vc <- ToothGrowth$len[ToothGrowth$supp == "VC"]
  ref <- suppressWarnings(stats::ks.test(oj, vc))
  expect_equal(nrow(res), 1L)
  expect_equal(res$group1, "OJ")
  expect_equal(res$group2, "VC")
  expect_equal(res$n1, 30); expect_equal(res$n2, 30)
  expect_equal(unname(res$statistic), unname(ref$statistic))   # D
  expect_equal(res$p, ref$p.value)
})

test_that("ks_test default (non-detailed) keeps n1, n2 and statistic, like wilcox_test (#92)", {
  res <- ToothGrowth %>% ks_test(len ~ supp)
  expect_equal(
    colnames(res),
    c(".y.", "group1", "group2", "n1", "n2", "statistic", "p")
  )
  expect_equal(res$n1, 30); expect_equal(res$n2, 30)
  expect_false(is.na(res$statistic))
})

test_that("get_test_label/get_description give a friendly ks_test label (#92)", {
  res <- ToothGrowth %>% ks_test(len ~ supp)
  expect_equal(get_description(res), "Kolmogorov-Smirnov test")   # not the raw "ks_test"
  lab <- get_test_label(res, type = "text")
  expect_true(is.character(lab) && grepl("Kolmogorov-Smirnov test", lab))
})

test_that("ks_test performs pairwise comparisons for >2 groups (#92)", {
  res <- ToothGrowth %>% ks_test(len ~ dose)
  expect_equal(nrow(res), 3L)
  expect_true(all(c("group1", "group2", "p", "p.adj", "p.adj.signif") %in% colnames(res)))
  # p.adj is computed over the 3 comparisons (holm)
  expect_equal(res$p.adj, p.adjust(res$p, method = "holm"))
})

test_that("ks_test ref.group compares each group to the control (#92)", {
  res <- ToothGrowth %>% ks_test(len ~ dose, ref.group = "0.5")
  expect_equal(nrow(res), 2L)
  expect_true(all(res$group1 == "0.5"))
  expect_equal(res$p.adj, p.adjust(res$p, method = "holm"))   # adjusted over k-1
})

test_that("ks_test detailed returns the statistic and method (#92)", {
  res <- ToothGrowth %>% ks_test(len ~ supp, detailed = TRUE)
  expect_true(all(c("statistic", "method", "alternative") %in% colnames(res)))
  expect_match(unique(res$method), "Kolmogorov-Smirnov")
})

test_that("ks_test works on grouped data (#92)", {
  res <- ToothGrowth %>% dplyr::group_by(dose) %>% ks_test(len ~ supp)
  expect_true("dose" %in% colnames(res))
  expect_equal(nrow(res), 3L)               # one OJ-vs-VC test per dose
})

test_that("ks_test errors clearly without a grouping variable (#92)", {
  expect_error(ToothGrowth %>% ks_test(len ~ 1), "at least two levels")
})

#' @include utilities.R
#' @importFrom  stats cov
#' @importFrom stats pchisq
NULL
#' Box's M-test for Homogeneity of Covariance Matrices
#' @description Performs the Box's M-test for homogeneity of covariance matrices
#'   obtained from multivariate normal data according to one grouping variable.
#'   The test is based on the chi-square approximation.
#' @param data a numeric data.frame or matrix containing n observations of p
#'   variables; it is expected that n > p.
#' @param group a vector of length n containing the class of each
#'   observation; it is usually a factor.
#' @return A data frame containing the following components:
#' \itemize{
#' \item{statistic }{an approximated value of the chi-square distribution.}
#' \item{parameter }{the degrees of freedom related of the test statistic in this case that it follows a Chi-square distribution.}
#' \item{p.value }{the p-value of the test.}
#' \item{method }{the character string "Box's M-test for Homogeneity of Covariance Matrices".}
#' }
#' @examples
#' data(iris)
#' box_m(iris[, -5], iris[, 5])
#' @export
box_m <-function(data, group)
  {
    if (!inherits(data, c("data.frame", "matrix")))
      stop("'data' must be a numeric data.frame or matrix!")
    if (length(group) != nrow(data))
      stop("incompatible dimensions!")
    dname <- deparse(substitute(data))
    data <- as.matrix(data)
    group <- as.factor(as.character(group))
    p <- ncol(data)
    nlev <- nlevels(group)
    lev <- levels(group)
    dfs <- tapply(group, group, length) - 1
    if (any(dfs < p))
      warning("there are one or more levels with less observations than variables!")
    mats <- aux <- list()
    for(i in 1:nlev) {
      mats[[i]] <- cov(data[group == lev[i], , drop = FALSE])
      aux[[i]] <- mats[[i]] * dfs[i]
    }
    names(mats) <- lev
    pooled <- Reduce("+", aux) / sum(dfs)
    logdet <- log(unlist(lapply(mats, det)))
    minus2logM <- sum(dfs) * log(det(pooled)) - sum(logdet * dfs)
    sum1 <- sum(1 / dfs)
    Co <- (((2 * p^2) + (3 * p) - 1) / (6 * (p + 1) *
                                          (nlev - 1))) * (sum1 - (1 / sum(dfs)))
    X2 <- minus2logM * (1 - Co)
    dfchi <- (choose(p, 2) + p) * (nlev - 1)
    pval <- pchisq(X2, dfchi, lower.tail = FALSE)
    out <- structure(
      list(statistic = c("Chi-Sq (approx.)" = X2),
           parameter = c(df = dfchi),
           p.value = pval,
           cov = mats, pooled = pooled, logDet = logdet,
           data.name = dname,
           method = "Box's M-test for Homogeneity of Covariance Matrices"
      ),
      class = c("htest", "boxM")
    )
    out <- broom::tidy(out)
    return(out)
  }

#' @include utilities.R utilities_two_sample_test.R
NULL

#'Wilcoxon Effect Size
#'@description Compute Wilcoxon effect size (\code{r}) for: \itemize{ \item
#'  one-sample test (Wilcoxon one-sample signed-rank test); \item paired
#'  two-samples test (Wilcoxon two-sample paired signed-rank test) and \item
#'  independent two-samples test ( Mann-Whitney, two-sample rank-sum test). }
#'
#'  It can also returns confidence intervals by bootstap.
#'
#'  The effect size \code{r} is calculated as \code{Z} statistic divided by
#'  square root of the sample size (N) (\eqn{Z/\sqrt{N}}). The \code{Z} value is
#'  extracted from either \code{coin::wilcoxsign_test()} (case of one- or
#'  paired-samples test) or \code{coin::wilcox_test()} (case of independent
#'  two-samples test).
#'
#'  Note that \code{N} corresponds to total sample size for independent samples
#'  test and to total number of pairs for paired samples test.
#'
#'  The \code{r} value varies from 0 to close to 1. The interpretation values
#'  for r commonly in published litterature and on the internet are: \code{0.10
#'  - < 0.3} (small effect), \code{0.30 - < 0.5} (moderate effect) and \code{>=
#'  0.5} (large effect).
#'
#'@inheritParams wilcox_test
#'@param ci If TRUE, returns confidence intervals by bootstrap. May be slow.
#'@param conf.level The level for the confidence interval.
#'@param ci.type The type of confidence interval to use. Can be any of "norm",
#'  "basic", "perc", or "bca". Passed to \code{boot::boot.ci}.
#'@param nboot The number of replications to use for bootstrap.
#'@param ... Additional arguments passed to the functions
#'  \code{coin::wilcoxsign_test()} (case of one- or paired-samples test) or
#'  \code{coin::wilcox_test()} (case of independent two-samples test).
#'@return return a data frame with some of the following columns: \itemize{
#'  \item \code{.y.}: the y variable used in the test. \item
#'  \code{group1,group2}: the compared groups in the pairwise tests. \item
#'  \code{n,n1,n2}: Sample counts. \item \code{effsize}: estimate of the effect
#'  size (\code{r} value). \item \code{magnitude}: magnitude of effect size.
#'  \item \code{conf.low,conf.high}: lower and upper bound of the effect size
#'  confidence interval.}
#'@references Maciej Tomczak and Ewa Tomczak. The need to report effect size
#'  estimates revisited. An overview of some recommended measures of effect
#'  size. Trends in Sport Sciences. 2014; 1(21):19-25.
#' @examples
#' if(require("coin")){
#'
#' # One-sample Wilcoxon test effect size
#' ToothGrowth %>% wilcox_effsize(len ~ 1, mu = 0)
#'
#' # Independent two-samples wilcoxon effect size
#' ToothGrowth %>% wilcox_effsize(len ~ supp)
#'
#'
#' # Paired-samples wilcoxon effect size
#' ToothGrowth %>% wilcox_effsize(len ~ supp, paired = TRUE)
#'
#' # Pairwise comparisons
#' ToothGrowth %>% wilcox_effsize(len ~ dose)
#'
#' # Grouped data
#' ToothGrowth %>%
#'   group_by(supp) %>%
#'   wilcox_effsize(len ~ dose)
#'
#' }
#'@export
wilcox_effsize <- function(data, formula, comparisons = NULL, ref.group = NULL,
                                paired = FALSE, alternative = "two.sided",
                                mu = 0, ci = FALSE, conf.level = 0.95, ci.type = "perc",
                                nboot = 1000,  ...){

  env <- as.list(environment())
  args <- env %>% .add_item(method = "wilcox_effsize")
  params <- c(env, list(...)) %>%
    remove_null_items() %>%
    add_item(method = "coin.wilcox.test", detailed = FALSE)

  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  number.of.groups <- guess_number_of_groups(data, group)
  if(number.of.groups > 2 & !is.null(ref.group)){
    if(ref.group %in% c("all", ".all.")){
      params$data <- create_data_with_all_ref_group(data, outcome, group)
      params$ref.group <- "all"
    }
  }
  test.func <- two_sample_test
  if(number.of.groups > 2) test.func <- pairwise_two_sample_test
  res <- do.call(test.func, params) %>%
    select(.data$.y., .data$group1, .data$group2, .data$estimate, everything()) %>%
    rename(effsize = .data$estimate) %>%
    mutate(magnitude = get_wilcox_effsize_magnitude(.data$effsize)) %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "wilcox_effsize"))
  res
}



# Wilcoxon test using coin R package; returns effect size
coin.wilcox.test <- function(x, y = NULL, mu = 0, paired = FALSE, alternative = c("two.sided", "less", "greater"),
                     ci = FALSE, conf.level = 0.95,  ci.type = "perc", nboot = 1000, ...){
  required_package("coin")

  alternative <- match.arg(alternative)
  check_two_samples_test_args(
    x = x, y = y, mu = mu, paired = paired,
    conf.level = conf.level
    )

  if (!is.null(y)) {
    DNAME <- paste(deparse(substitute(x)), "and", deparse(substitute(y)))
    if (paired) {
      # Transform paired test into one-sample test problem
      OK <- complete.cases(x, y)
      x <- x[OK] - y[OK]
      y <- NULL
      METHOD <- "Paired Wilcoxon test (coin)"
    }
    else {
      x <- x[is.finite(x)]
      y <- y[is.finite(y)]
      METHOD <- "Independent Wilcoxon test (coin)"
    }
  }
  else {
    DNAME <- deparse(substitute(x))
    METHOD <- "One-sample Wilcoxon test (coin)"
    x <- x[is.finite(x)]
  }

  if(is.null(y)){
    y <- rep(mu, length(x))
    test.type <- "symmetry"
  }
  else{
    group <- rep(c("grp1", "grp2"), times = c(length(x), length(y))) %>%
      factor()
    x <- c(x, y)
    y <- group
    test.type <- "independence"
  }
  data <- data.frame(x, y)
  results <- coin_wilcox_test(
    data, x ~ y, type = test.type,
    alternative = alternative,  ...
    )
  # Confidence interval of the effect size r
  if (ci == TRUE) {
    stat.func <- function(data, subset) {
      coin_wilcox_test(
        data, formula = x ~ y, subset = subset,
        type = test.type, alternative = alternative, ...
        )$r
    }
    CI <- get_boot_ci(
      data, stat.func, conf.level = conf.level,
      type = ci.type, nboot = nboot
      )
    results <- results %>% mutate(conf.low = CI[1], conf.high = CI[2])
  }
  RVAL <- list(statistic = results$z, parameter = results$n, p.value = results$p,
               null.value = mu, alternative = alternative, method = METHOD,
               data.name = DNAME, estimate = results$r)
  if (ci) {
    attr(CI, "conf.level") <- conf.level
    RVAL <- c(RVAL, list(conf.int = CI))
  }
  names(RVAL$statistic) <- "Z"
  names(RVAL$parameter) <- "n"
  names(RVAL$estimate) <- "Effect size (r)"
  class(RVAL) <- "htest"
  RVAL
}

# Perform wilcoxon test using coin package
coin_wilcox_test <- function(data, formula, subset = NULL, type = c("independence", "symmetry"),  ...){
  type <- match.arg(type)
  coin_wilcox_test_func <- switch (
    type,
    independence = coin::wilcox_test,
    symmetry = coin::wilcoxsign_test
  )
  if(!is.null(subset)) data <- data[subset, ]
  res.wilcox <-suppressWarnings(coin_wilcox_test_func(formula, data = data,...))
  n <- nrow(data)
  z <- as.vector(coin::statistic(res.wilcox, type = "standardized"))
  p <- coin::pvalue(res.wilcox)
  r <- abs(z)/sqrt(n) # Effect size
  tibble(n = n, z = z, r = r, p = p)
}

get_wilcox_effsize_magnitude <- function(d){
  magnitude.levels = c(0.3, 0.5, Inf)
  magnitude = c("small","moderate","large")
  d.index <- findInterval(abs(d), magnitude.levels)+1
  magnitude <- factor(magnitude[d.index], levels = magnitude, ordered = TRUE)
  magnitude
}

context("test-wilcox_effsize")

test_that("wilcox_effsize detailed exposes the Z statistic (#122)", {
  skip_if_not_installed("coin")
  res <- ToothGrowth %>% wilcox_effsize(len ~ supp, paired = TRUE, detailed = TRUE)
  expect_true(all(c("effsize", "statistic", "p") %in% colnames(res)))
  # statistic is the Z used to compute r = |Z| / sqrt(N), N = number of pairs (30)
  expect_equal(unname(abs(res$statistic) / sqrt(30)), unname(res$effsize), tolerance = 1e-7)
  # no spurious 'df' column (Wilcoxon has no degrees of freedom) (#122 review)
  expect_false("df" %in% colnames(res))
  # method correctly labels the paired case as signed-rank (#122 review)
  expect_equal(unique(res$method), "Wilcoxon signed rank test")
})

test_that("wilcox_effsize detailed labels the method correctly per case (#122)", {
  skip_if_not_installed("coin")
  expect_equal(unique((ToothGrowth %>% wilcox_effsize(len ~ supp, detailed = TRUE))$method),
               "Wilcoxon rank sum test")                                   # independent
  expect_equal(unique((ToothGrowth %>% wilcox_effsize(len ~ 1, mu = 20, detailed = TRUE))$method),
               "Wilcoxon signed rank test")                               # one-sample
})

test_that("wilcox_effsize detailed works for the independent and pairwise cases (#122)", {
  skip_if_not_installed("coin")
  ind <- ToothGrowth %>% wilcox_effsize(len ~ supp, detailed = TRUE)
  expect_true("statistic" %in% colnames(ind))
  pw <- ToothGrowth %>% wilcox_effsize(len ~ dose, detailed = TRUE)
  expect_equal(nrow(pw), 3L)
  expect_true(all(c("statistic", "p", "p.adj") %in% colnames(pw)))
})

test_that("wilcox_effsize default output is unchanged (#122 no-regression)", {
  skip_if_not_installed("coin")
  res <- ToothGrowth %>% wilcox_effsize(len ~ supp, paired = TRUE)
  # default (detailed = FALSE) must not gain the Z/p columns
  expect_equal(
    colnames(res),
    c(".y.", "group1", "group2", "effsize", "n1", "n2", "magnitude")
  )
  expect_false("statistic" %in% colnames(res))
})

#' @include utilities.R
NULL
#'Fisher's Exact Test for Count Data
#'@description Performs Fisher's exact test for testing the null of independence
#'  of rows and columns in a contingency table.
#'
#'  Wrappers around the R base function \code{\link[stats]{fisher.test}()} but
#'  have the advantage of performing pairwise and row-wise fisher tests, the
#'  post-hoc tests following a significant chi-square test of homogeneity for 2xc
#'  and rx2 contingency tables.
#'@inheritParams stats::fisher.test
#'@param xtab a contingency table in a matrix form.
#'@param p.adjust.method method to adjust p values for multiple comparisons.
#'  Used when pairwise comparisons are performed. Allowed values include "holm",
#'  "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
#'  want to adjust the p value (not recommended), use p.adjust.method = "none".
#'@param detailed logical value. Default is FALSE. If TRUE, a detailed result is
#'  shown.
#'@param ... Other arguments passed to the function \code{fisher_test()}.
#'
#'@return return a data frame with some the following columns: \itemize{ \item
#'  \code{group}: the categories in the row-wise proportion tests. \item
#'  \code{p}: p-value. \item \code{p.adj}: the adjusted p-value. \item
#'  \code{method}: the used statistical test. \item \code{p.signif,
#'  p.adj.signif}: the significance level of p-values and adjusted p-values,
#'  respectively. \item \code{estimate}: an estimate of the odds ratio. Only
#'  present in the 2 by 2 case. \item \code{alternative}: a character string
#'  describing the alternative hypothesis. \item \code{conf.low,conf.high}: a
#'  confidence interval for the odds ratio. Only present in the 2 by 2 case and
#'  if argument conf.int = TRUE.}
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#' @examples
#'
#' # Comparing two proportions
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Data: frequencies of smokers between two groups
#' xtab <- as.table(rbind(c(490, 10), c(400, 100)))
#' dimnames(xtab) <- list(
#'   group = c("grp1", "grp2"),
#'   smoker = c("yes", "no")
#' )
#' xtab
#' # compare the proportion of smokers
#' fisher_test(xtab, detailed = TRUE)
#'
#' # Homogeneity of proportions between groups
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # H0: the proportion of smokers is similar in the four groups
#' # Ha:  this proportion is different in at least one of the populations.
#' #
#' # Data preparation
#' grp.size <- c( 106, 113, 156, 102 )
#' smokers  <- c( 50, 100, 139, 80 )
#' no.smokers <- grp.size - smokers
#' xtab <- as.table(rbind(
#'   smokers,
#'   no.smokers
#' ))
#' dimnames(xtab) <- list(
#'   Smokers = c("Yes", "No"),
#'   Groups = c("grp1", "grp2", "grp3", "grp4")
#' )
#' xtab
#'
#' # Compare the proportions of smokers between groups
#' fisher_test(xtab, detailed = TRUE)
#'
#' # Pairwise comparison between groups
#' pairwise_fisher_test(xtab)
#'
#'
#' # Pairwise proportion tests
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Data: Titanic
#' xtab <- as.table(rbind(
#'   c(122, 167, 528, 673),
#'   c(203, 118, 178, 212)
#' ))
#' dimnames(xtab) <- list(
#'   Survived = c("No", "Yes"),
#'   Class = c("1st", "2nd", "3rd", "Crew")
#' )
#' xtab
#' # Compare the proportion of survived between groups
#' pairwise_fisher_test(xtab)
#'
#' # Row-wise proportion tests
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Data: Titanic
#' xtab <- as.table(rbind(
#'   c(180, 145), c(179, 106),
#'   c(510, 196), c(862, 23)
#' ))
#' dimnames(xtab) <- list(
#'   Class = c("1st", "2nd", "3rd", "Crew"),
#'   Gender = c("Male", "Female")
#' )
#' xtab
#' # Compare the proportion of males and females in each category
#' row_wise_fisher_test(xtab)
#'
#' # A r x c table  Agresti (2002, p. 57) Job Satisfaction
#' Job <- matrix(c(1,2,1,0, 3,3,6,1, 10,10,14,9, 6,7,12,11), 4, 4,
#'               dimnames = list(income = c("< 15k", "15-25k", "25-40k", "> 40k"),
#'                              satisfaction = c("VeryD", "LittleD", "ModerateS", "VeryS")))
#' fisher_test(Job)
#' fisher_test(Job, simulate.p.value = TRUE, B = 1e5)


#' @describeIn fisher_test performs Fisher's exact test for testing the null of
#'   independence of rows and columns in a contingency table with fixed
#'   marginals. Wrapper around the function \code{\link[stats]{fisher.test}()}.
#' @export
fisher_test <- function(xtab, workspace = 200000, alternative = "two.sided",
                      conf.int = TRUE, conf.level = 0.95, simulate.p.value = FALSE,
                      B = 2000, detailed = FALSE, ...){
  if(is.data.frame(xtab)) xtab <- as.matrix(xtab)
  args <- as.list(environment()) %>% add_item(method = "fisher_test")
  results <- stats::fisher.test(
    xtab, workspace = workspace, alternative = alternative,
    conf.int = conf.int, conf.level = conf.level,
    simulate.p.value = simulate.p.value, B = B, ...
    ) %>%
    as_tidy_stat() %>%
    add_significance("p") %>%
    add_columns(n = sum(xtab), .before = 1) %>%
    mutate(method = "Fisher's Exact test")
  if(!detailed) results <- remove_details(results, method = "prop.test")
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "fisher_test"))
}

#' @describeIn fisher_test pairwise comparisons between proportions, a post-hoc
#'   tests following a significant Fisher's exact test of homogeneity for 2xc
#'   design.
#' @export
pairwise_fisher_test <- function(xtab, p.adjust.method = "holm", detailed = FALSE, ...){

  if(is.data.frame(xtab)) xtab <- as.matrix(xtab)
  if(ncol(xtab) > 2 & nrow(xtab) == 2)
    xtab <- t(xtab)
  if (is.null(colnames(xtab)) | any(0 %in% nchar(colnames(xtab)))) {
    colnames(xtab) <- paste0("col", 1:ncol(xtab))
  }
  if (is.null(rownames(xtab)) | any(0 %in% nchar(rownames(xtab)))) {
    rownames(xtab) <- paste0("row", 1:nrow(xtab))
  }
  if(ncol(xtab) > 2){
    stop("A two-dimensionnal contingency table required.")
  }
  compare_pair <- function(rows, xtab, ...){
    rows <- as.character(rows)
    fisher_test(xtab[rows, ], detailed = detailed, ...) %>%
      add_columns(group1 = rows[1], group2 = rows[2], .before = 1) %>%
      keep_only_tbl_df_classes()
  }
  args <- c(as.list(environment()), list(...)) %>%
    add_item(method = "fisher_test")

  comparisons <- rownames(xtab) %>%
    .possible_pairs()
  results <- comparisons %>%
    map(compare_pair, xtab, ...) %>%
    bind_rows() %>%
    adjust_pvalue("p", method = p.adjust.method) %>%
    add_significance("p.adj") %>%
    mutate(p.adj = signif(.data$p.adj, digits = 3)) %>%
    select(-.data$p.signif)
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "fisher_test"))
}


#' @describeIn fisher_test performs row-wise Fisher's exact test of count data, a post-hoc tests following a significant chi-square test
#'   of homogeneity for rx2 contingency table. The test is conducted for each category (row).
#' @export
row_wise_fisher_test <- function(xtab, p.adjust.method = "holm", detailed = FALSE, ...){
  if(is.data.frame(xtab))
    xtab <- as.matrix(xtab)
  if(!inherits(xtab, c("matrix", "table"))){
    stop("An object of class 'matrix' or 'table' required")
  }
  if(ncol(xtab) !=2){
    stop("A cross-tabulation with two columns required")
  }
  args <- c(as.list(environment()), list(...)) %>%
    add_item(method = "fisher_test")
  # Create xtab for each category (row)
  columns.total <- margin.table(xtab, 2)
  create_xtab <- function(x, n){
    as.data.frame(rbind(x, n-x))
  }
  xtab.list <- apply(xtab, 1, create_xtab, columns.total )
  results <- xtab.list %>%
    map(fisher_test, detailed = detailed, ...) %>%
    map(keep_only_tbl_df_classes) %>%
    bind_rows(.id = "group") %>%
    adjust_pvalue(method = p.adjust.method) %>%
    add_significance("p.adj") %>%
    mutate(
      p = signif(.data$p, digits = 3),
      p.adj = signif(.data$p.adj, digits = 3)
    ) %>%
    select(-.data$p.signif)
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "fisher_test"))
}

#' @include utilities.R utilities_two_sample_test.R
#' @importFrom stats ks.test
NULL
#'Two-Sample Kolmogorov-Smirnov Test
#'
#'
#'@description Provides a pipe-friendly framework to perform the two-sample
#'  Kolmogorov-Smirnov test, comparing the (empirical) distributions of a numeric
#'  variable between two groups. Wrapper around the R base function
#'  \code{\link[stats]{ks.test}()}.
#'
#'  When the grouping factor contains more than two levels, pairwise
#'  Kolmogorov-Smirnov tests are automatically performed, with p-value
#'  adjustment.
#'@inheritParams stats::ks.test
#'@param data a data.frame containing the variables in the formula.
#'@param formula a formula of the form \code{x ~ group} where \code{x} is a
#'  numeric variable giving the data values and \code{group} is a factor with two
#'  or more levels giving the corresponding groups.
#'@param comparisons A list of length-2 vectors specifying the groups of interest
#'  to be compared. For example to compare groups "A" vs "B" and "B" vs "C", the
#'  argument is as follow: \code{comparisons = list(c("A", "B"), c("B", "C"))}.
#'@param ref.group a character string specifying the reference group. If
#'  specified, for a given grouping variable, each of the group levels will be
#'  compared to the reference group (i.e. control group). If \code{ref.group =
#'  "all"}, pairwise two sample tests are performed for comparing each grouping
#'  variable level against all (i.e. basemean).
#'@param p.adjust.method method to adjust p values for multiple comparisons. Used
#'  when pairwise comparisons are performed. Allowed values include "holm",
#'  "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
#'  want to adjust the p value (not recommended), use p.adjust.method = "none".
#'@param detailed logical value. Default is FALSE. If TRUE, a detailed result is
#'  shown.
#'@return return a data frame with some of the following columns: \itemize{ \item
#'  \code{.y.}: the y variable used in the test. \item \code{group1,group2}: the
#'  compared groups in the pairwise tests. \item \code{n1,n2}: sample counts.
#'  \item \code{statistic}: the value of the test statistic \code{D} (the maximum
#'  difference between the two empirical cumulative distribution functions). \item
#'  \code{p}: p-value. \item \code{p.adj}: the adjusted p-value. \item
#'  \code{method}: the statistical test used to compare groups. \item
#'  \code{p.signif, p.adj.signif}: the significance level of p-values and adjusted
#'  p-values, respectively. \item \code{alternative}: the alternative hypothesis.}
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#'@seealso \code{\link{wilcox_test}()}, \code{\link{t_test}()}
#' @examples
#' # Two-samples test
#' #:::::::::::::::::::::::::::::::::::::::::
#' ToothGrowth %>% ks_test(len ~ supp)
#'
#' # Pairwise comparisons (more than two groups)
#' #:::::::::::::::::::::::::::::::::::::::::
#' ToothGrowth %>% ks_test(len ~ dose)
#'
#' # Comparison against a reference group
#' #:::::::::::::::::::::::::::::::::::::::::
#' ToothGrowth %>% ks_test(len ~ dose, ref.group = "0.5")
#'@export
ks_test <- function(
  data, formula, comparisons = NULL, ref.group = NULL,
  p.adjust.method = "holm", alternative = "two.sided",
  exact = NULL, detailed = FALSE
)
{
  env <- as.list(environment())
  args <- env %>%
    add_item(method = "ks_test")
  params <- env %>%
    remove_null_items() %>%
    add_item(method = "ks.test")

  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  if(.is_empty(group)){
    stop("The Kolmogorov-Smirnov test requires a grouping variable with at ",
         "least two levels; use a formula of the form 'outcome ~ group'.",
         call. = FALSE)
  }
  number.of.groups <- guess_number_of_groups(data, group)
  if(number.of.groups == 1){
    stop("The grouping variable must have at least two levels.", call. = FALSE)
  }
  if(number.of.groups > 2 & !is.null(ref.group)){
    if(ref.group %in% c("all", ".all.")){
      params$data <- create_data_with_all_ref_group(data, outcome, group)
      params$ref.group <- "all"
    }
  }
  test.func <- two_sample_test
  if(number.of.groups > 2) test.func <- pairwise_two_sample_test
  do.call(test.func, params) %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "ks_test"))
}

#' @include utilities.R
NULL
#' Visualize Correlation Matrix Using Base Plot
#' @description Provide a tibble-friendly framework to visualize a correlation
#'   matrix. Wrapper around the R base function
#'   \code{\link[corrplot]{corrplot}()}. Compared to
#'   \code{\link[corrplot]{corrplot}()}, it can handle directly the output of the
#'   functions \code{\link{cor_mat}() (in rstatix)}, \code{rcorr() (in Hmisc)},
#'   \code{correlate() (in corrr)} and \code{cor() (in stats)}.
#'
#'   The p-values contained in the outputs of the functions
#'   \code{\link{cor_mat}()} and \code{rcorr()} are automatically detected and
#'   used in the visualization.
#' @inheritParams corrplot::corrplot
#' @param cor.mat the correlation matrix to visualize
#' @param palette character vector containing the color palette.
#' @param p.mat matrix of p-value corresponding to the correlation matrix.
#' @param significant.level significant level, if the p-value is bigger than
#'   \code{significant.level}, then the corresponding correlation coefficient is
#'   regarded as insignificant.
#' @param insignificant character, specialized insignificant correlation
#'   coefficients, "cross" (default), "blank". If "blank", wipe away the
#'   corresponding glyphs; if "cross", add crosses (X) on corresponding glyphs.
#' @param label logical value. If TRUE, shows the correlation coefficient
#'   labels.
#' @param font.label a list with one or more of the following elements: size
#'   (e.g., 1), color (e.g., "black") and style (e.g., "bold"). Used to
#'   customize the correlation coefficient labels. For example \code{font.label
#'   = list(size = 1, color = "black", style = "bold")}.
#' @seealso \code{\link{cor_as_symbols}()}
#' @examples
#' # Compute correlation matrix
#' #::::::::::::::::::::::::::::::::::::::::::
#' cor.mat <- mtcars %>%
#'   select(mpg, disp, hp, drat, wt, qsec) %>%
#'   cor_mat()
#'
#' # Visualize correlation matrix
#' #::::::::::::::::::::::::::::::::::::::::::
#' # Full correlation matrix,
#' # insignificant correlations are marked by crosses
#' cor.mat %>% cor_plot()
#'
#' # Reorder by correlation coefficient
#' # pull lower triangle and visualize
#' cor.lower.tri <- cor.mat %>%
#'   cor_reorder() %>%
#'   pull_lower_triangle()
#' cor.lower.tri %>% cor_plot()
#'
#' # Change visualization methods
#' #::::::::::::::::::::::::::::::::::::::::::
#' cor.lower.tri %>%
#'   cor_plot(method = "pie")
#'
#' cor.lower.tri %>%
#'   cor_plot(method = "color")
#'
#' cor.lower.tri %>%
#'   cor_plot(method = "number")
#'
#' # Show the correlation coefficient: label = TRUE
#' # Blank the insignificant correlation
#' #::::::::::::::::::::::::::::::::::::::::::
#' cor.lower.tri %>%
#'   cor_plot(
#'     method = "color",
#'     label = TRUE,
#'     insignificant = "blank"
#'   )
#'
#' # Change the color palettes
#' #::::::::::::::::::::::::::::::::::::::::::
#'
#' # Using custom color palette
#' # Require ggpubr: install.packages("ggpubr")
#' if(require("ggpubr")){
#'   my.palette <- get_palette(c("red", "white", "blue"), 200)
#'   cor.lower.tri %>%
#'    cor_plot(palette = my.palette)
#' }
#'
#' # Using RcolorBrewer color palette
#' if(require("ggpubr")){
#'   my.palette <- get_palette("PuOr", 200)
#'   cor.lower.tri %>%
#'    cor_plot(palette = my.palette)
#' }
#'
#' @export
cor_plot <- function(cor.mat, method = "circle", type = "full", palette = NULL,
                     p.mat = NULL, significant.level = 0.05, insignificant = c("cross", "blank"),
                     label = FALSE, font.label = list()) {


  insignificant <- match.arg(insignificant)
  if(insignificant == "cross")
    insignificant <- "pch"

  # Outline color of circle, ellipse, ....
  outline <- ifelse(method == "color", "white", FALSE)

  # Correlation coefficients label parameters
  font <- parse_font(font.label)
  addCoef.col <- NULL
  if(label) addCoef.col <- font$color

  # Correlation matrix data
  show.diagonal <- TRUE
  if(inherits(cor.mat, "cor_mat")){
    cor.value <- cor.mat %>% as_matrix()
    p.mat <- cor.mat %>% cor_get_pval() %>% as_matrix()
  }
  else if(inherits(cor.mat, "cor_mat_tri")){

    cor.value <- cor.mat %>%
      as_numeric_triangle() %>%
      as_matrix()
    p.mat <- cor.mat %>%
      cor_get_pval() %>%
      as_numeric_triangle() %>%
      as_matrix()

    if(inherits(cor.mat, "lower_tri"))
      type <- "lower"
    else
      type <- "upper"

    cor.diagonal <- diag(cor.value)
    cor.diagonal.is.na <- all(is.na(cor.diagonal))
    if(cor.diagonal.is.na)
      show.diagonal <- FALSE
    else
      show.diagonal <- TRUE
  }
  else if(inherits(cor.mat, "rcorr")){
    cor.value <- cor.mat$r
    p.mat <- cor.mat$P
  }
  else {
    cor.value <- cor.mat %>% as_matrix()
  }

  # Correlation matrix p-value
  if(inherits(p.mat, "tbl_df"))
    p.mat <- p.mat %>% as_matrix()

  corrplot <- corrplot::corrplot
  corrplot(
    cor.value,  method = method, type = type,
    tl.col="black", tl.srt = 45,
    col = palette, diag = show.diagonal,
    p.mat = p.mat, sig.level = significant.level,
    insig = insignificant, pch.cex = 2,
    outline = outline, addCoef.col = addCoef.col,
    number.cex = font$size, number.font = font$style
    )

}


# Parse label font
parse_font <- function(font){

  if(.is_empty(font)){
    font <- list(size = 1, color = "black", style = "plain")
  }
  else if(!is.list(font)){
    stop("The argument font should be a list. ",
         "Example: font <- list(size = 1, color = 'black', style = 2)")
  }
  else{

    font$size <- ifelse(is.null(font$size), 1, font$size)
    font$color <- ifelse(is.null(font$color), "black", font$color)
    font$style <- ifelse(is.null(font$style), "plain", font$style)
  }

  # convert fon style to numeric
  available.styles <- c(1, 2, 3, 4) %>%
    rlang::set_names(c("plain",  "bold", "italic", "bold.italic"))
  font$style <- available.styles[font$style]

  font
}

#' @include utilities.R utilities_two_sample_test.R factorial_design.R
NULL
#'Create Nice Summary Tables of ANOVA Results
#'
#'
#'@description Create beautiful summary tables of ANOVA test results obtained
#'  from either \code{\link[car]{Anova}()} or \code{\link[stats]{aov}()}.
#'
#'  The results include ANOVA table, generalized effect size and some assumption
#'  checks.
#'
#'@param effect.size the effect size to compute and to show in the ANOVA
#'  results. Allowed values can be either "ges" (generalized eta squared) or
#'  "pes" (partial eta squared) or both. Default is "ges".
#'@param observed Variables that are observed (i.e, measured) as compared to
#'  experimentally manipulated. The default effect size reported (generalized
#'  eta-squared) requires correct specification of the observed variables.
#'@param detailed If TRUE, returns extra information (sums of squares columns,
#'  intercept row, etc.) in the ANOVA table.
#'@param object an object of returned by either \code{\link[car]{Anova}()}, or
#'  \code{\link[stats]{aov}()}.
#'
#'@return return an object of class \code{anova_test} a data frame containing
#'  the ANOVA table for independent measures ANOVA. However, for repeated/mixed
#'  measures ANOVA, it is a list containing the following components are
#'  returned:
#'
#'  \itemize{ \item \strong{ANOVA}: a data frame containing ANOVA results \item
#'  \strong{Mauchly's Test for Sphericity}: If any within-Ss variables with more
#'  than 2 levels are present, a data frame containing the results of Mauchly's
#'  test for Sphericity. Only reported for effects that have more than 2 levels
#'  because sphericity necessarily holds for effects with only 2 levels. \item
#'  \strong{Sphericity Corrections}: If any within-Ss variables are present, a
#'  data frame containing the Greenhouse-Geisser and Huynh-Feldt epsilon values,
#'  and corresponding corrected p-values. }
#'
#'  The \strong{returned object might have an attribute} called \code{args} if
#'  you compute ANOVA using the function \code{\link{anova_test}()}. The attribute \code{args} is a
#'  list holding the arguments used to fit the ANOVA model, including: data, dv,
#'  within, between, type, model, etc.
#'
#'
#'  The following abbreviations are used in the different results tables:
#'
#'  \itemize{ \item DFn	Degrees of Freedom in the numerator (i.e. DF effect).
#'  \item DFd	Degrees of Freedom in the denominator (i.e., DF error). \item
#'  SSn	Sum of Squares in the numerator (i.e., SS effect). \item SSd	Sum of
#'  Squares in the denominator (i.e.,SS error). \item F	F-value. \item p	p-value
#'  (probability of the data given the null hypothesis). \item p<.05	Highlights
#'  p-values less than the traditional alpha level of .05. \item ges	Generalized
#'  Eta-Squared measure of effect size. \item GGe	Greenhouse-Geisser epsilon.
#'  \item p[GGe]	p-value after correction using Greenhouse-Geisser epsilon.
#'  \item p[GGe]<.05	Highlights p-values (after correction using
#'  Greenhouse-Geisser epsilon) less than the traditional alpha level of .05.
#'  \item HFe	Huynh-Feldt epsilon. \item p[HFe]	p-value after correction using
#'  Huynh-Feldt epsilon. \item p[HFe]<.05	Highlights p-values (after correction
#'  using Huynh-Feldt epsilon) less than the traditional alpha level of .05.
#'  \item W	Mauchly's W statistic }
#'
#'@author Alboukadel Kassambara, \email{alboukadel.kassambara@@gmail.com}
#'@seealso \code{\link{anova_test}()}, \code{\link{factorial_design}()}
#' @examples
#'# Load data
#'#:::::::::::::::::::::::::::::::::::::::
#'data("ToothGrowth")
#'df <- ToothGrowth
#'df$dose <- as.factor(df$dose)
#'
#'# Independent measures ANOVA
#'#:::::::::::::::::::::::::::::::::::::::::
#'# Compute ANOVA and display the summary
#' res.anova <- Anova(lm(len ~ dose*supp, data = df))
#' anova_summary(res.anova)
#'
#'# Display both SSn and SSd using detailed = TRUE
#'# Show generalized eta squared using effect.size = "ges"
#'anova_summary(res.anova, detailed = TRUE, effect.size = "ges")
#'
#'# Show partial eta squared using effect.size = "pes"
#'anova_summary(res.anova, detailed = TRUE, effect.size = "pes")
#'
#'# Repeated measures designs using car::Anova()
#'#:::::::::::::::::::::::::::::::::::::::::
#'# Prepare the data
#'df$id <- as.factor(rep(1:10, 6)) # Add individuals ids
#'head(df)
#'
#'# Easily perform repeated measures ANOVA using the car package
#' design <- factorial_design(df, dv = len, wid = id, within = c(supp, dose))
#' res.anova <- Anova(design$model, idata = design$idata, idesign = design$idesign, type = 3)
#' anova_summary(res.anova)
#'
#'# Repeated measures designs using stats::Aov()
#'#:::::::::::::::::::::::::::::::::::::::::
#' res.anova <- aov(len ~ dose*supp + Error(id/(supp*dose)), data = df)
#' anova_summary(res.anova)

#'@name anova_summary
#'@export
anova_summary <- function(object, effect.size = "ges", detailed = FALSE, observed = NULL){
  if(inherits(object, "Anova.mlm")){
    results <- repeated_anova_summary(object)
  }
  else if(inherits(object, "anova")){
    results <- summary_independent_anova(object)
  }
  else if(inherits(object, c("aov", "aovlist"))){
    results <- summary_aov(object)
  }
  else{
    stop("Non-supported object passed: ",
         paste(class(object), collapse = ", "), ". ",
         "Object needs to be of class 'Anova.mlm' or 'anova'.")
  }
  .args <- attr(object, "args") # exist only in anova_test()
  results <- results %>%
    add_anova_effect_size(effect.size, observed)

  if(!detailed){
    results <- remove_details(results, method = "anova")
  }
  results$ANOVA <- order_by_interaction_levels(results$ANOVA)
  results <- results %>% map(~dplyr::mutate_if(., is.numeric, round_value, 3))
  if(length(results) == 1) results <- results[[1]]
  results %>% set_attrs(args = .args)
}




# Summary of Anova.mlm object: summary_anova_mlm
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# this function is used for repeated and mixed anova
repeated_anova_summary <- function(res.anova, detailed = FALSE){
  .summary <- suppressWarnings(summary(res.anova))
  # Anova table converted into data frame
  aov.table <- .summary$univariate.tests %>%
    convert_anova_object_as_data_frame() %>%
    set_colnames(c("Effect", "SSn", "DFn", "SSd", "DFd", "F", "p")) %>%
    select(
      .data$Effect, .data$DFn, .data$DFd,
      .data$SSn, .data$SSd, .data$F, .data$p
    ) %>%
    mutate(`p<.05` = ifelse(.data$p < 0.05, "*",''))
  sphericity.test <- corrections <- NULL
  # Mauchly's Test for Sphericity
  if(nrow(.summary$sphericity.tests) > 0){
    sphericity.test <- .summary$sphericity.tests %>%
      convert_anova_object_as_data_frame() %>%
      set_colnames(c("Effect", "W", "p")) %>%
      mutate(`p<.05` = ifelse(.data$p < 0.05, "*",''))
  }
  # Sphericity corrections
  if(nrow(.summary$sphericity.tests) > 0){
    corrections <- .summary$pval.adjustments %>%
      as.data.frame() %>%
      set_colnames(c("GGe", "p[GG]", "HFe", "p[HF]")) %>%
      tibble::rownames_to_column("Effect")
    p.gg.signif <- ifelse(corrections[["p[GG]"]] < 0.05, "*",'')
    p.hf.signif <- ifelse(corrections[["p[HF]"]] < 0.05, "*",'')
    corrections <- corrections %>%
      add_column(`p[GG]<.05` = p.gg.signif, .after = "p[GG]") %>%
      add_column(`p[HF]<.05` = p.hf.signif, .after = "p[HF]")
  }
  # Results
  results <- list(ANOVA = aov.table)
  if(!is.null(sphericity.test)){
    results $`Mauchly's Test for Sphericity` <- sphericity.test
    results$`Sphericity Corrections` <-  corrections
    results <- results %>% add_corrected_df()
  }
  results
}
convert_anova_object_as_data_frame <- function(aov.table){
  aov.table.list <- list(Effect = rownames(aov.table))
  for(col in colnames(aov.table)){
    aov.table.list[[col]] <- aov.table[, col]
  }
  aov.table <- as.data.frame(aov.table.list, stringsAsFactors = FALSE)
  rownames(aov.table) <- 1:nrow(aov.table)
  aov.table
}

add_corrected_df <- function(.summary){
  aov.table <- .summary$ANOVA %>%
    select(.data$Effect, .data$DFn, .data$DFd)
  corrections <- .summary$`Sphericity Corrections` %>%
    dplyr::left_join(aov.table, by = "Effect") %>%
    mutate(
      df.gg = paste(round_value(.data$GGe*.data$DFn, 2), round_value(.data$GGe*.data$DFd, 2), sep = ", "),
      df.hf = paste(round_value(.data$HFe*.data$DFn, 2), round_value(.data$HFe*.data$DFd, 2), sep = ", ")
    ) %>%
    select(-.data$DFd, -.data$DFn)
  df.gg <- corrections$df.gg
  df.hf <- corrections$df.hf
  .summary$`Sphericity Corrections` <- corrections %>%
    select(-.data$df.gg, -.data$df.hf) %>%
    add_column(`DF[GG]` = df.gg, .after = "GGe") %>%
    add_column(`DF[HF]` = df.hf, .after = "HFe")
  .summary
}

# Summary of independent anova
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
summary_independent_anova <- function(res.anova){
  res.anova <- as.data.frame(res.anova)
  .residuals <- res.anova["Residuals", 1:2]
  if('Mean Sq' %in% colnames(res.anova)){
    # exists when res.anova is from stats::anova
    res.anova <- select(res.anova, -.data$`Mean Sq`)
  }
  if('Sum Sq' %in% colnames(res.anova)){
    # in stats::anova, Sum Sq is not the first column, so do select
    res.anova <- res.anova %>% select(.data$`Sum Sq`, dplyr::everything())
    colnames(res.anova) <- c('SSn','DFn','F','p')
    ss.exists <- TRUE
  }
  else{
    # case of white.adjust = TRUE. SS doesnt exist in the results
    colnames(res.anova) <- c('DFn','F','p')
    ss.exists <- FALSE
  }
  res.anova <- res.anova %>%
    tibble::rownames_to_column("Effect")  %>%
    add_column(DFd = .residuals$Df, .after = "DFn") %>%
    mutate(`p<.05` = ifelse(.data$p < 0.05, "*",'')) %>%
    filter(.data$Effect != "Residuals")
  if(ss.exists){
    res.anova <- res.anova %>%
      add_column(SSd = .residuals$`Sum Sq`, .after = "SSn")
  }
  results <- list(ANOVA = res.anova)
  results
}

# Summary of anova from stats::aov
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
summary_aov <- function(res.anova){
  remove_empty_space <- function(x){
    sapply(x, function(x){strsplit(x, " ")[[1]][1]})
  }
  reformat_aov_summary <- function(aov.summary){
    if(inherits(aov.summary, "listof"))
      aov.summary <- as.data.frame(aov.summary[[1]])
    else as.data.frame(aov.summary)
    .residuals <- aov.summary["Residuals", 1:2]
    aov.summary <- aov.summary %>%
      set_colnames(c("DFn", "SSn", "MS", "F", "p")) %>%
      tibble::rownames_to_column("Effect")  %>%
      add_column(DFd = .residuals$Df, .after = "DFn") %>%
      add_column(SSd = .residuals$`Sum Sq`, .after = "SSn") %>%
      mutate(`p<.05` = as.character(ifelse(.data$p < 0.05, "*",''))) %>%
      mutate(Effect = remove_empty_space(.data$Effect)) %>%
      filter(!is.na(.data$p)) %>%
      select(-.data$MS)
    aov.summary
  }
  res.anova <- summary(res.anova) %>%
    map(reformat_aov_summary) %>%
    dplyr::bind_rows() %>%
    order_by_interaction_levels()
  results <- list(ANOVA = res.anova)
  results
}


# Reorder ANOVA table by interaction levels in the term
order_by_interaction_levels <- function(aov.table){
  .terms <- aov.table$Effect
  nb.interaction <- str_count(.terms, ":")
  aov.table %>% dplyr::arrange(nb.interaction)
}


# Add effect size
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
add_anova_effect_size <- function(res.anova.summary, effect.size = "ges",  observed = NULL){
  ss.exists <- "SSn" %in% colnames(res.anova.summary$ANOVA)
  if(!ss.exists){
    return(res.anova.summary)
  }
  if("pes" %in% effect.size){
    res.anova.summary <- res.anova.summary %>%
      add_partial_eta_squared()
  }
  else {
    res.anova.summary <- res.anova.summary %>%
      add_generalized_eta_squared(observed)
  }
  res.anova.summary
}

# Generalized eta squared
add_generalized_eta_squared <- function(res.anova.summary, observed = NULL){
  aov.table <- res.anova.summary$ANOVA
  if(!is.null(observed)){
    obs <- rep(FALSE, nrow(aov.table))
    for(i in observed){
      if (!any(grepl(paste0("\\b",i,"\\b"), aov.table$Effect)))
        stop("Specified observed variable not found in data: ", i)
      obs <- obs | grepl(paste0("\\b",i,"\\b"), aov.table$Effect)
    }
    obs.SSn1 = sum(aov.table$SSn*obs)
    obs.SSn2 = aov.table$SSn*obs
  }
  else{
    obs.SSn1 <- 0
    obs.SSn2 <- 0
  }
  aov.table <- aov.table %>%
    mutate(ges = .data$SSn / (.data$SSn + sum(unique(.data$SSd)) + obs.SSn1 - obs.SSn2))
  res.anova.summary$ANOVA <- aov.table
  res.anova.summary
}
# Partial eta squared
add_partial_eta_squared <- function(res.anova.summary){
  res.anova.summary$ANOVA <- res.anova.summary$ANOVA %>%
    mutate(pes = .data$SSn/(.data$SSn + .data$SSd))
  res.anova.summary
}

#' @include utilities.R friedman_test.R
NULL
#'Conover's All-Pairs Comparisons Test for Friedman Rank Sums
#'
#'@description Performs Conover's all-pairs comparison test (also known as the
#'  Durbin-Conover test) for a two-way balanced complete block design, following
#'  a significant Friedman rank sum test. It is the repeated-measures analogue of
#'  Conover's test for the Kruskal-Wallis design: the
#'  within-block ranks are compared pairwise using the pooled rank variance and
#'  the statistic is referred to a \emph{t}-distribution with \eqn{(b - 1)(k - 1)}
#'  degrees of freedom (\eqn{b} blocks, \eqn{k} treatments). It should only be
#'  used as a post-hoc procedure when the Friedman test is itself significant
#'  (Conover, 1999).
#'
#'  If a reference group is specified (via \code{ref.group}), then each of the
#'  remaining treatments is compared only to the reference (control) treatment,
#'  and the p-value adjustment for multiple comparisons is computed over only
#'  these \code{k - 1} comparisons (as for \code{\link{dunn_test}()}).
#'
#'@param data a data.frame containing the variables in the formula.
#'@param formula a formula of the form \code{a ~ b | c}, where \code{a}
#'  (numeric) is the dependent variable name; \code{b} is the within-subjects
#'  factor variable (the treatment); and \code{c} is the column name containing
#'  the individuals/subjects (block) identifier. Should be unique per individual.
#'@param p.adjust.method method to adjust p-values for multiple comparisons. Used
#'  when pairwise comparisons are performed. Allowed values include "holm",
#'  "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". Default is
#'  "holm".
#'@param ref.group a character string specifying the reference treatment. If
#'  specified, each of the treatment levels is compared to the reference
#'  (control), and the p-value adjustment is computed over only these
#'  comparisons.
#'@param detailed logical value. If TRUE, returns the rank-sum estimate and the
#'  test method in the output.
#'@return return a data frame with some of the following columns: \itemize{
#'  \item \code{.y.}: the y (outcome) variable used in the test. \item
#'  \code{group1,group2}: the compared treatments in the pairwise tests. \item
#'  \code{n1,n2}: the number of blocks (subjects) contributing to each treatment.
#'  \item \code{estimate}: the rank-sum difference. \item \code{estimate1,
#'  estimate2}: the rank sums of the two treatments, respectively. \item
#'  \code{statistic}: Test statistic (t-value) used to compute the p-value. \item
#'  \code{df}: degrees of freedom (\eqn{(b - 1)(k - 1)}). \item \code{p}: p-value.
#'  \item \code{p.adj}: the adjusted p-value. \item \code{method}: the
#'  statistical test used to compare groups. \item \code{p.adj.signif}: the
#'  significance level of the adjusted p-values. }
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#'@details For a balanced complete block design with \eqn{b} blocks and \eqn{k}
#'  treatments, the observations within each block are ranked. Let \eqn{R_j} be
#'  the sum of the within-block ranks for treatment \eqn{j} and let \eqn{A =
#'  \sum r^2} be the sum of the squared within-block ranks. The pairwise
#'  statistic for treatments \eqn{i} and \eqn{j} is \deqn{t_{ij} =
#'  \frac{R_i - R_j}{\sqrt{\dfrac{2\,(b\,A - \sum_j R_j^2)}{(b - 1)(k - 1)}}}}
#'  referred to a \emph{t}-distribution with \eqn{(b - 1)(k - 1)} degrees of
#'  freedom. This is the Conover (1999) post-hoc, also known as the
#'  Durbin-Conover test.
#'
#'@references Conover, W. J. (1999) Practical Nonparametric Statistics, 3rd
#'  edition. Wiley.
#' @seealso \code{\link{friedman_test}}, \code{\link{friedman_nemenyi_test}},
#'   \code{\link{friedman_effsize}}
#' @examples
#' # A balanced complete block design: 3 treatments measured on 6 subjects
#' df <- data.frame(
#'   id        = factor(rep(1:6, 3)),
#'   treatment = factor(rep(c("A", "B", "C"), each = 6)),
#'   score     = c(4, 6, 3, 5, 4, 5,    7, 8, 6, 7, 9, 6,    6, 9, 7, 8, 8, 9)
#' )
#'
#' # Omnibus Friedman test
#' df %>% friedman_test(score ~ treatment | id)
#'
#' # Conover (Durbin-Conover) all-pairs post-hoc
#' df %>% friedman_conover_test(score ~ treatment | id)
#'
#' # Comparison against a reference (control) treatment
#' df %>% friedman_conover_test(score ~ treatment | id, ref.group = "A")
#'@name friedman_conover_test
#'@export
friedman_conover_test <- function(data, formula, p.adjust.method = "holm", ref.group = NULL, detailed = FALSE){
  if(!is.null(ref.group)) ref.group <- as.character(ref.group)
  args <- as.list(environment()) %>%
    .add_item(method = "friedman_conover_test")
  if(is_grouped_df(data)){
    results <- data %>%
      doo(.friedman_conover_test, formula, p.adjust.method, ref.group = ref.group)
  }
  else{
    results <- .friedman_conover_test(data, formula, p.adjust.method, ref.group = ref.group)
  }
  if(!detailed){
    results <- results %>%
      select(-any_of(c("method", "estimate", "estimate1", "estimate2")))
  }
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "friedman_conover_test"))
}


.friedman_conover_test <- function(data, formula, p.adjust.method = "holm", ref.group = NULL){
  vars <- get_friedman_vars(formula)
  outcome <- vars$dv; treatment <- vars$within; block <- vars$wid

  data <- data %>%
    select(!!!syms(c(outcome, treatment, block))) %>%
    .as_factor(treatment)
  treatments <- levels(data %>% pull(!!treatment))
  k <- length(treatments)
  if(k < 2){
    stop("the treatment (within-subjects) factor must have at least two levels")
  }
  if(!is.null(ref.group)){
    if(!(ref.group %in% treatments)){
      stop(
        "Specified reference group ('", ref.group, "') is not a level of the ",
        "treatment variable. Valid levels are: ",
        paste(treatments, collapse = ", "), ".",
        call. = FALSE
      )
    }
  }

  # Build the balanced complete-block matrix (rows = blocks, cols = treatments).
  # The Durbin-Conover post-hoc requires every block to have exactly one
  # observation per treatment (no missing or duplicated cells).
  block.values <- data %>% pull(!!block)
  treat.values <- factor(data %>% pull(!!treatment), levels = treatments)
  outcome.values <- data %>% pull(!!outcome)
  # A missing outcome would still count as a present cell below (table() counts
  # rows, not non-NA values) and rank() would then assign the missing cell a
  # rank, silently corrupting the rank sums. Reject NA outcomes outright, as the
  # omnibus friedman_test() does (an unreplicated complete block design has no
  # missing cells).
  if(any(is.na(outcome.values))){
    stop(
      "friedman_conover_test() does not allow missing values in the outcome: ",
      "an unreplicated complete block design must have one non-missing ",
      "observation per subject and treatment.",
      call. = FALSE
    )
  }
  counts <- table(block.values, treat.values)
  if(any(counts != 1)){
    stop(
      "friedman_conover_test() requires a balanced complete block design: ",
      "each subject (block) must have exactly one observation per treatment ",
      "(no missing or replicated cells).",
      call. = FALSE
    )
  }
  m <- tapply(outcome.values, list(block.values, treat.values), function(z) z[1])
  m <- m[, treatments, drop = FALSE]
  b <- nrow(m)

  R <- t(apply(m, 1, rank))                 # within-block ranks (ties: average)
  rank.sums <- colSums(R)
  names(rank.sums) <- treatments
  A <- sum(R^2)
  df.value <- (b - 1) * (k - 1)
  pooled <- 2 * (b * A - sum(rank.sums^2)) / df.value

  # The statistic is undefined when there is no residual rank variability
  # (e.g. every block ranks the treatments in exactly the same order, so the
  # denominator collapses to zero) or when there is only a single block.
  if(df.value < 1 || !is.finite(pooled) || pooled <= 0){
    stop(
      "Conover (Durbin-Conover) test is undefined for these data: there is no ",
      "residual variability in the within-block ranks (e.g. every subject ranks ",
      "the treatments in the same order), or there are too few blocks.",
      call. = FALSE
    )
  }
  denom <- sqrt(pooled)

  compare.estimate <- function(i, j){
    rank.sums[i] - rank.sums[j]
  }
  compare.stats <- function(i, j){
    (rank.sums[i] - rank.sums[j]) / denom
  }
  compare.levels <- function(i, j){
    2 * stats::pt(abs(compare.stats(i, j)), df = df.value, lower.tail = FALSE)
  }
  ESTIMATE <- stats::pairwise.table(
    compare.estimate, treatments, p.adjust.method = "none"
  ) %>% tidy_squared_matrix("diff")
  PSTAT <- stats::pairwise.table(
    compare.stats, treatments, p.adjust.method = "none"
  ) %>% tidy_squared_matrix("statistic")

  # Assemble the full per-pair table before adjusting p-values, so that with a
  # reference group the adjustment is applied over only the retained k - 1
  # comparisons (cf. dunn_test()/conover_test()).
  PVAL <- stats::pairwise.table(
    compare.levels, treatments, p.adjust.method = "none"
  ) %>%
    tidy_squared_matrix("p") %>%
    mutate(method = "Durbin-Conover", .y. = outcome, df = df.value) %>%
    add_column(statistic = PSTAT$statistic, .before = "p") %>%
    add_column(estimate = ESTIMATE$diff, .before = "group1") %>%
    select(all_of(c(".y.", "group1", "group2", "estimate")), everything())

  if(!is.null(ref.group)){
    PVAL <- PVAL %>%
      filter(.data$group1 == ref.group | .data$group2 == ref.group)
    to.flip <- PVAL$group2 == ref.group
    if(any(to.flip)){
      g1 <- PVAL$group1; g2 <- PVAL$group2
      PVAL$group1[to.flip] <- g2[to.flip]
      PVAL$group2[to.flip] <- g1[to.flip]
      PVAL$estimate[to.flip]  <- -PVAL$estimate[to.flip]
      PVAL$statistic[to.flip] <- -PVAL$statistic[to.flip]
    }
  }

  PVAL <- PVAL %>%
    adjust_pvalue(method = p.adjust.method) %>%
    add_significance("p.adj")

  rank.sums1 <- rank.sums[PVAL$group1]
  rank.sums2 <- rank.sums[PVAL$group2]
  PVAL %>%
    add_column(n1 = b, n2 = b, .after = "group2") %>%
    add_column(estimate1 = rank.sums1, estimate2 = rank.sums2, .after = "estimate") %>%
    select(all_of(c(".y.", "group1", "group2", "n1", "n2", "estimate", "estimate1", "estimate2", "statistic", "df", "p", "p.adj", "p.adj.signif", "method")))
}

#' Convert a Table of Counts into a Data Frame of cases
#' @description converts a contingency table or a data frame of counts into a
#'   data frame of individual observations.
#' @param x a contingency table or a data frame
#' @param count.col the name of the column containing the counts. Default is "Freq".
#' @return a data frame of cases
#'
#' @examples
#' # Create a cross-tabulation demo data
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' xtab <- as.table(
#'   rbind(c(20, 5), c(16,9))
#' )
#' dimnames(xtab) <- list(
#'   before = c("non.smoker", "smoker"),
#'   after = c("non.smoker", "smoker")
#' )
#' xtab
#'
#' # Convert into a data frame of cases
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' df <- counts_to_cases(xtab)
#' head(df)
#'
#' @export
counts_to_cases <- function(x, count.col = "Freq") {
  if(!inherits(x, "table")) x <- as.table(as.matrix(x))
  x <- as.data.frame(x)
  # Get the row indices to pull from x
  idx <- rep.int(seq_len(nrow(x)), x[[count.col]])
  # Drop count column
  x[[count.col]] <- NULL
  # Get the rows from x
  x <- x[idx, ]
  rownames(x) <- 1:nrow(x)
  x
}

#' @importFrom magrittr %>%
#' @importFrom magrittr %<>%
#' @importFrom magrittr extract
#' @importFrom magrittr set_colnames
#' @importFrom dplyr mutate
#' @importFrom dplyr mutate_at
#' @importFrom dplyr mutate_all
#' @importFrom dplyr pull
#' @importFrom dplyr select
#' @importFrom dplyr rename
#' @importFrom dplyr as_data_frame
#' @importFrom dplyr bind_rows
#' @importFrom dplyr group_by
#' @importFrom dplyr summarise
#' @importFrom dplyr n
#' @importFrom dplyr is_grouped_df
#' @importFrom dplyr ungroup
#' @importFrom dplyr do
#' @importFrom dplyr filter
#' @importFrom dplyr tibble
#' @importFrom dplyr everything
#' @importFrom dplyr left_join
#' @importFrom purrr map map2
#' @importFrom broom tidy
#' @importFrom stats t.test
#' @importFrom rlang sym
#' @importFrom rlang !!
#' @importFrom rlang :=
#' @importFrom rlang .data
#' @importFrom rlang syms
#' @importFrom rlang !!!
#' @importFrom rlang quos
#' @importFrom rlang quo_name
#' @importFrom tibble add_column
#' @importFrom tibble as_tibble
#' @importFrom tidyr spread
#' @importFrom tidyr gather
#' @importFrom tidyr nest


# Unnesting, adapt to tidyr 1.0.0
unnest <- function(data, cols = "data", ...){
  if(is_pkg_version_sup("tidyr", "0.8.3")){
   results <- tidyr::unnest(data, cols = cols, ...)
  }
  else {results <- tidyr::unnest(data, ...)}
  results
}

# Check if an installed package version is superior to a specified version
# Version, pkg: character vector
is_pkg_version_sup<- function(pkg, version){
  vv <- as.character(utils::packageVersion(pkg))
  cc <- utils::compareVersion(vv, version) > 0
  cc
}

# Rounding values --------------------------------------
# Round a vector, conditionnaly rounding
round_value <- function(x, digits = 0){
  sapply(
    x,
    function(x, digits){
      if(is.na(x))
        x
      else if(abs(x) > 10^-digits)
        round(x, digits)
      else
        signif(x, digits)
    },
    digits
  )
}
# Round a whole data frame or selected columns
round_column <- function(data, ...,  digits = 0){
  dot.vars <- get_existing_dot_vars(data, ...)
  if(.is_empty(dot.vars)){
    data %<>% dplyr::mutate_if(is.numeric, round_value, digits = digits)
  }
  data %<>% dplyr::mutate_at(dot.vars, round_value, digits = digits)
  data
}

# Extract or replace number from character string
extract_number <- function(x){
  as.numeric(gsub("[^0-9.-]+", "", as.character(x)))
}
replace_number <- function(x, replacement = ""){
  gsub("[0-9.]", replacement, as.character(x))
}

# Add columns into data frame
# If specified before or after columns does not exist, columns are appended at the end
add_columns <- function(.data, ..., .before = NULL, .after = NULL){
  if(is.character(.before)){
    if(!(.before %in% colnames(.data))){
      .before <- NULL
    }
  }
  if(is.character(.after)){
    if(!(.after %in% colnames(.data))){
      .after <- NULL
    }
  }
  tibble::add_column(.data, ..., .before = .before, .after = .after)
}








# Check if required package is installed
required_package <- function(pkg){
  if (!requireNamespace(pkg, quietly = TRUE)) {
    stop(
      pkg, " package needed to be installed before using this function. ",
      "Type this in R: install.packages('", pkg, "')"
    )
  }
}


# Check if a given column name is in the data
assertthat_column_exists <-function(data, cols){
  .diff <- setdiff(cols, colnames(data))
  if(!.is_empty(.diff)){
    stop("Can't find the following variable(s) in the data: ",
         paste(col, collapse = ", "))
  }
}
# remove null elements from a list
remove_null_items <- function(.list){
  Filter(Negate(is.null), .list)
}

# Count a pattern in a string
str_count <- function(x, pattern){
  lengths(regmatches(x, gregexpr(pattern, x)))
}

# Check if all columns in a data frame are numeric
is_all_columns_numeric <- function(data){
  data %>%
    map(is.numeric) %>%
    unlist() %>%
    all()
}

is_lm <- function(object){
  inherits(object, "lm")
}

# Check odd and even numbers
is_odd_number <- function(x){
  x %% 2 != 0
}
is_even_number <- function(x){
  x %% 2 == 0
}


# Set diff that can keep duplicates
set_diff <- function(x, y, keep.dup = FALSE){
  if(!keep.dup)
    res <- setdiff(x, y)
  else{
    ins <- x %in% y
    res <- x[!ins]
  }
  res
}


# NSE
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

# Get the value of enquo variables. Usage:
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# .args <- rlang::enquos(x = x, y = y, ...) %>%
#  map(~get_quo_vars(data, .))
get_quo_vars <- function (data, vars)
{
  if(rlang::quo_is_missing(vars)){
    return(NULL)
  }
  names(data) %>%
    tidyselect::vars_select(!!vars) %>%
    magrittr::set_names(NULL)
}
# .args <- rlang::enquos(x = x, y = y, ...) %>%
#   get_quo_vars_list(data, .)
get_quo_vars_list <- function(data, .enquos){
  . <- NULL
  res <- .enquos %>% map(~get_quo_vars(data, .))
  res <- map(res, set_empty_to_null )
  res
}
# pipe friendly alias of get_quo_vars_list
select_quo_variables <- function(.enquos, data){
  get_quo_vars_list(data, .enquos)
}


set_empty_to_null <- function(x){
  if(.is_empty(x)) x <- NULL
  x
}


# Extract variables used in a formula
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

get_formula_left_hand_side <- function(formula){
  deparse(formula[[2]])
}
get_formula_right_hand_side <- function(formula){
  attr(stats::terms(formula), "term.labels")
}
.extract_formula_variables <- function(formula){
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  list(outcome = outcome, group = group)
}

# Get formula error term: len ~ dose*diet + Error(id/diet)
# retruns Error(id/diet)
get_formula_error_term <- function(formula){
  rhs <- get_formula_right_hand_side(formula)
  error.term <- rhs[grepl("^Error\\(", rhs)]
  error.term
}
# Get variables included in the error terms
get_formula_error_vars <- function(formula){
  error.term <- get_formula_error_term(formula)
  all.vars(parse(text = error.term))
}

is_error_term_in_formula <- function(formula){
  error.term <- get_formula_error_term(formula)
  length(error.term) > 0
}

# Grouping variables manipulation
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
guess_number_of_groups <- function(data, group){
  if(.is_empty(group)){
    number.of.groups <- 1  # Null model
  }
  else{
    number.of.groups <- data %>%
      pull(!!group) %>% unique() %>% length()
  }
  number.of.groups
}

get_levels <- function(data, group){
  data %>% pull(!!group) %>% levels()
}


# Convert a group column into a factor if this is not already the case
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# group.col column name containing groups
# ref.group: optional reference group
# ToothGrowth %>% .as_factor("dose", ref.group = "0.5") %>% pull("dose")
.as_factor <- function (data, group.col, ref.group = NULL){
  if(.is_empty(group.col))
    return(data)
  group.values <- data %>% pull(group.col)
  if(!is.factor(group.values))
    group.values <- as.factor(group.values)
  if(!is.null(ref.group)){
    if(ref.group != "")
      group.values <- stats::relevel(group.values, ref.group)
  }
  data %>% mutate(!!group.col := group.values)
}



# Guess p-value column name from a statistical test output
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# transform vector into regular expression
as_regexp <- function(x){
  . <- NULL
  gsub(".", "\\.", x, fixed = TRUE) %>%
    paste(collapse = "$|^") %>%
    paste("^", ., "$", sep = "")
}


# Generate all possible pairs of a factor levels
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# if ref.group is specified, then all possible pairs,
# against reference group, are generated
.possible_pairs <- function(group.levels, ref.group = NULL){
  # Ref group should be always the first group
  if(!is.null(ref.group))
    group.levels <- c(ref.group,  group.levels)
  group.levels <- unique(group.levels)
  # Generate possible pairs
  possible.pairs <- utils::combn(group.levels, 2) %>%
    as.data.frame()
  mate1 <- possible.pairs[1,]
  # select only comparisons against ref.group (if specified)
  if(!is.null(ref.group))
    possible.pairs <- possible.pairs %>%
    select(which(mate1 == ref.group))
  possible.pairs %>% as.list()
}



# Create a tidy statistical output
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Generic function to create a tidy statistical output
as_tidy_stat <- function(x, round.p = TRUE, digits = 3, stat.method = NULL){
  estimate <- estimate1 <- estimate2 <- p.value <-
    alternative <- p <- NULL
  res <- tidy(x)
  if(!is.null(stat.method)){
    res %<>% mutate(method = stat.method)
  }
  else if("method" %in% colnames(res)){
    stat.method <- get_stat_method(x)
    res %<>% mutate(method = stat.method)
  }
  if("p.value" %in% colnames(res)){
    res<- res %>% rename(p = p.value)
    if(round.p) res <- res %>% mutate(p = signif(p, digits))
  }
  if("parameter" %in% colnames(res)){
    res <- res %>% rename(df = .data$parameter)
  }
  res
}

get_stat_method <- function(x){
  if(inherits(x, c("aov", "anova"))){
    return("Anova")
  }
  available.methods <- c(
    "T-test", "Wilcoxon", "Kruskal-Wallis",
    "Pearson", "Spearman", "Kendall", "Sign-Test",
    "Cohen's d", "Chi-squared test"
  )
  used.method <- available.methods %>%
    map(grepl, x$method, ignore.case = TRUE) %>%
    unlist()
  if(sum(used.method) > 0){
    results <- available.methods %>% extract(used.method)
  if(length(results) >= 2)
    results <- paste(results, collapse = " ")
  }
  else
    results <- x$method
  results
}

# Check if en object is empty
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.is_empty <- function(x){
  length(x) == 0
}

# Check if is a list
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.is_list <- function(x){
  inherits(x, "list")
}

# Returns the levels of a factor variable
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.levels <- function(x){
  if(!is.factor(x)) x <- as.factor(x)
  levels(x)
}

# Add/remove items in a list
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
add_item <- function(.list, ...){
  pms <- list(...)
  for(pms.names in names(pms)){
    .list[[pms.names]] <- pms[[pms.names]]
  }
  .list
}
remove_item <- function(.list, items){
  for(item in items)
    .list[[item]] <- NULL
  .list
}
remove_null_items <- function(.list){
  Filter(Negate(is.null), .list)
}


# depreciated
.add_item <- function(.list, ...){
  add_item(.list, ...)
}



# First letter uppercase
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
to_uppercase_first_letter <- function(x) {
  if(is.null(x)) return(x)
  substr(x, 1, 1) <- toupper(substr(x, 1, 1))
  x
}


# Data conversion
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

as_matrix <- function(x){

  if(inherits(x, "tbl_df")){
    tibble_to_matrix(x)
  }
  else if(inherits(x, "matrix")){
    x
  }
  else if(is.data.frame(x)){
    if("rowname" %in% colnames(x)){
      x %>%
        tibble::remove_rownames() %>%
        tibble::column_to_rownames("rowname") %>%
        as_matrix()
    }
    else {
      as.matrix(x)
    }
  }
  else{
    as.matrix(x)
  }
}


# Convert a tbl to matrix
tibble_to_matrix <- function(x){
  x <-  as.data.frame(x)
  rownames(x) <- x[, 1]
  x <- x[, -1]
  as.matrix(x)
}

# Convert a matrix to standard data frame
matrix_to_dataframe <- function(x){
  x <- as.data.frame(x, stringsAsFactors = FALSE) %>%
    add_column(rowname = rownames(x), .before = 1)
  rownames(x) <- NULL
  x
}

# Convert a matrix to tibble
matrix_to_tibble <- function(x){
  as_tibble(x, rownames = "rowname")
}

# Replace empty space as na
replace_empty_by <- function(x, replacement = NA){
  x %>%
    keep_only_tbl_df_classes() %>%
    dplyr::mutate_all(
      function(x){x[x==""] <- replacement; x}
      )
}


# Correlation analysis
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

# Stop if not an object of class cor_mat
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++
stop_ifnot_cormat <- function(x){
  if(!inherits(x, "cor_mat")){
    stop("An object of class cor_mat is required")
  }
}

# Subset a correlation matrix, return a tibble
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++
subset_matrix <- function(x, vars, row.vars = vars,
                          col.vars = vars){

  if(inherits(x, c("tbl_df", "data.frame"))){
    . <- NULL
    x %>% as_matrix() %>%
      .[row.vars, col.vars, drop = FALSE] %>%
      as_tibble(rownames = "rowname")
  }
  else if(inherits(x, "matrix")){
    x[row.vars, col.vars, drop = FALSE] %>%
      as_tibble(rownames ="rowname")
  }
  else{
    stop("x should be a data frame or rownames")
  }
}


# Tidy Select
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

# Collect variables provided by users; get selected variables
get_selected_vars <- function(x, ..., vars = NULL){

  if(is_grouped_df(x))
    x <- x %>% dplyr::ungroup()
  dot.vars <- rlang::quos(...)

  if(length(vars) > 0){
    return(vars)
  }
  if (length(dot.vars) == 0) selected <- colnames(x)
  else selected <- tidyselect::vars_select(names(x), !!! dot.vars)
  selected %>% as.character()
}

# Return dot variables -----------------------
get_dot_vars <- function(...){
  rlang::quos(...) %>%
    map(rlang::quo_text) %>%
    unlist()
}
get_existing_dot_vars <- function(data, ...){
  tidyselect::vars_select(colnames(data), !!!rlang::quos(...))
}





# Select numeric columns
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
select_numeric_columns <- function(data){
  if(is_grouped_df(data))
    data <- data %>% dplyr::ungroup()
  data %>% dplyr::select_if(is.numeric)
}

# Add a class to an object
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
add_class <- function(x, .class){
  class(x) <- unique(c(.class, class(x)))
  x
}

prepend_class <- function(x, .class){
  current.class <- class(x)
  diff.class <- setdiff(class(x), .class)
  x <- structure(x, class = c(.class, diff.class))
  x
}

remove_class <- function(x, toremove){
  class(x) <- setdiff(class(x), toremove)
  x
}

keep_only_tbl_df_classes <- function(x){
  toremove <- setdiff(class(x), c("tbl_df", "tbl", "data.frame"))
  if(length(toremove) > 0){
    x <- remove_class(x, toremove)
  }
  x
}

# Add/set attributes
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
set_attrs <- function (x, ...)
{
  attrs <- list(...)
  attributes(x) <- c(attributes(x), attrs)
  x
}

# Correlation analysis
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

# Check classes
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.is_cor_mat <- function(x){
  inherits(x, "cor_mat")
}

.is_cor_test <- function(x){
  inherits(x, "cor_test")
}

# Convert a cor_mat_tri to numeric data
as_numeric_triangle <- function(x){
  rrowname <- x %>% pull(1)
  res <- x %>%
    replace_empty_by(NA) %>%
    select(-1) %>%
    mutate_all(as.numeric) %>%
    add_column(rowname = rrowname, .before = 1)
  res
}


# Create label for each row in a grouped data
#::::::::::::::::::::::::::::::::::::::::::::::::
# Labels are the combination of the levels of the grouping variables
# ex: dose:0.5,supp:VC
create_grouped_data_label <- function(data){
  if(!is_grouped_df(data)){
    stop("data should be a grouped data")
  }
  .nested <- nest(data)
  .vars <- dplyr::group_vars(data)
  .data <- .nested %>% select(!!!syms(.vars))
  for(.var in .vars){
    values <- .data %>% pull(!!.var)
    .data  <- .data %>%
      mutate(!!.var := paste0(.var, ":", values))
  }
  .results <- .data %>%
    purrr::pmap(paste, sep = ",") %>%
    unlist()
  .results
}

#::::::::::::::::::::::::::::::::::::::::::::::::
# Helper functions to process rstatix test results
#::::::::::::::::::::::::::::::::::::::::::::::::
is_rstatix_test <- function(x){
  inherits(x, "rstatix_test")
}
# get rstatix test arguments
get_test_arguments <- function(test){
  attr(test, "args")
}

# get test grouping variables
# exist when tests are performed on grouped data
get_test_grouping_vars <- function(test){
  args <- get_test_arguments(test)
  grouping.vars <- dplyr::group_vars(args$data)
  if(.is_empty(grouping.vars))
    grouping.vars <- NULL
  grouping.vars
}
# Get  and set the test attributes: class and attr
# used to propagate attributes
get_test_attributes <- function(test){
  .attributes <- attributes(test)
  .attributes$names <- .attributes$row.names <- NULL
  .attributes
}
set_test_attributes <- function(test, .attributes){
  class(test) <- .attributes$class
  .attributes$class <- NULL
  for (attr.name in names(.attributes)){
    attr(test, attr.name ) <- .attributes[[attr.name]]
  }
  test
}

get_group_size <- function(data, group){
  result <- data %>%
    group_by(!!sym(group)) %>%
    dplyr::count()
  n <- result$n
  group.levels <- result %>% pull(1)
  names(n) <- group.levels
  n
}

stop_ifnot_class <- function(x, .class){
  object.name <- deparse(substitute(x))
  if(!inherits(x, .class)){
    stop(object.name, " should be an object of class: ",
         paste(.class, collapse = ", "))
  }
}

# Allowed pairwise comparison tests
get_pairwise_comparison_methods <- function(){
  c(
    t_test = "T test",
    wilcox_test = "Wilcoxon test",
    sign_test = "Sign test",
    dunn_test = "Dunn test",
    emmeans_test = "Emmeans test",
    tukey_hsd = "Tukey HSD",
    games_howell_test = "Games Howell",
    prop_test = "Z-Prop test",
    fisher_test = "Fisher's exact test",
    chisq_test = "Chi-square test",
    exact_binom_test = "Exact binomial test",
    mcnemar_test = "McNemar test"
  )
}

# Bootstrap confidence intervals -------------------------
get_boot_ci <- function(data, stat.func, conf.level = 0.95, type = "perc", nboot = 500){
  required_package("boot")
  Boot = boot::boot(data, stat.func, R = nboot)
  BCI = boot::boot.ci(Boot, conf = conf.level, type = type, parallel = "multicore")
  type <- switch(
    type, norm = "normal", perc = "percent",
    basic = "basic", bca = "bca", stud = "student", type
  )
  CI <- as.vector(BCI[[type]]) %>%
    utils::tail(2) %>% round_value(digits = 2)
  CI
}



get_complete_cases <- function(data){
  data %>%
    filter(complete.cases(data))
}


# transform squared matrix into tidy data frame
tidy_squared_matrix <- function(data, value = "value"){
  data %>%
    as_tibble(rownames = "group2") %>%
    gather(key = "group1", value = !!value, -.data$group2) %>%
    stats::na.omit() %>% as_tibble() %>%
    select(.data$group1, everything())
}


# Binomial proportion confidence interval
get_prop_conf_int <- function(x, n, p = 0.5, conf.level = 0.95,
                              alternative = "two.sided"){
  .get_conf <- function(x, n, p, alternative, conf.level){
    res <- stats::binom.test(x, n, p, alternative, conf.level)$conf.int
    tibble(conf.low = res[1], conf.high = res[2])
  }
  results <- list(x = x, n = n, p = p) %>%
    purrr::pmap(
      .get_conf, conf.level = conf.level,
      alternative = alternative
    ) %>%
    dplyr::bind_rows()
}

#' @include utilities.R
NULL
#'Proportion Test
#'@description Performs proportion tests to either evaluate the homogeneity of
#'  proportions (probabilities of success) in several groups or to test that the
#'  proportions are equal to certain given values.
#'
#'  Wrappers around the R base function \code{\link[stats]{prop.test}()} but have
#'  the advantage of performing pairwise and row-wise z-test of two proportions,
#'  the post-hoc tests following a significant chi-square test of homogeneity
#'  for 2xc and rx2 contingency tables.
#'@inheritParams stats::prop.test
#'@param xtab a cross-tabulation (or contingency table) with two columns and
#'  multiple rows (rx2 design). The columns give the counts of successes and
#'  failures respectively.
#'@param p.adjust.method method to adjust p values for multiple comparisons.
#'  Used when pairwise comparisons are performed. Allowed values include "holm",
#'  "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
#'  want to adjust the p value (not recommended), use p.adjust.method = "none".
#'@param detailed logical value. Default is FALSE. If TRUE, a detailed result is
#'  shown.
#'@param ... Other arguments passed to the function \code{prop_test()}.
#'
#'@return return a data frame with some the following columns: \itemize{
#' \item \code{n}: the number of participants.
#'\item \code{group}: the categories in the row-wise proportion tests. \item
#'  \code{statistic}: the value of Pearson's chi-squared test statistic. \item
#'  \code{df}: the degrees of freedom of the approximate chi-squared
#'  distribution of the test statistic. \item \code{p}: p-value. \item
#'  \code{p.adj}: the adjusted p-value. \item \code{method}: the used
#'  statistical test. \item \code{p.signif, p.adj.signif}: the significance
#'  level of p-values and adjusted p-values, respectively. \item
#'  \code{estimate}: a vector with the sample proportions x/n. \item
#'  \code{estimate1, estimate2}: the proportion in each of the two populations.
#'  \item \code{alternative}: a character string describing the alternative
#'  hypothesis. \item \code{conf.low,conf.high}: Lower and upper bound on a
#'  confidence interval. a confidence interval for the true proportion if there
#'  is one group, or for the difference in proportions if there are 2 groups and
#'  p is not given, or NULL otherwise. In the cases where it is not NULL, the
#'  returned confidence interval has an asymptotic confidence level as specified
#'  by conf.level, and is appropriate to the specified alternative hypothesis.}
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#' @examples
#' # Comparing an observed proportion to an expected proportion
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' prop_test(x = 95, n = 160, p = 0.5, detailed = TRUE)
#'
#' # Comparing two proportions
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Data: frequencies of smokers between two groups
#' xtab <- as.table(rbind(c(490, 10), c(400, 100)))
#' dimnames(xtab) <- list(
#'   group = c("grp1", "grp2"),
#'   smoker = c("yes", "no")
#' )
#' xtab
#' # compare the proportion of smokers
#' prop_test(xtab, detailed = TRUE)
#'
#' # Homogeneity of proportions between groups
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # H0: the proportion of smokers is similar in the four groups
#' # Ha:  this proportion is different in at least one of the populations.
#' #
#' # Data preparation
#' grp.size <- c( 106, 113, 156, 102 )
#' smokers  <- c( 50, 100, 139, 80 )
#' no.smokers <- grp.size - smokers
#' xtab <- as.table(rbind(
#'   smokers,
#'   no.smokers
#' ))
#' dimnames(xtab) <- list(
#'   Smokers = c("Yes", "No"),
#'   Groups = c("grp1", "grp2", "grp3", "grp4")
#' )
#' xtab
#'
#' # Compare the proportions of smokers between groups
#' prop_test(xtab, detailed = TRUE)
#'
#' # Pairwise comparison between groups
#' pairwise_prop_test(xtab)
#'
#'
#' # Pairwise proportion tests
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Data: Titanic
#' xtab <- as.table(rbind(
#'   c(122, 167, 528, 673),
#'   c(203, 118, 178, 212)
#' ))
#' dimnames(xtab) <- list(
#'   Survived = c("No", "Yes"),
#'   Class = c("1st", "2nd", "3rd", "Crew")
#' )
#' xtab
#' # Compare the proportion of survived between groups
#' pairwise_prop_test(xtab)
#'
#' # Row-wise proportion tests
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Data: Titanic
#' xtab <- as.table(rbind(
#'   c(180, 145), c(179, 106),
#'   c(510, 196), c(862, 23)
#' ))
#' dimnames(xtab) <- list(
#'   Class = c("1st", "2nd", "3rd", "Crew"),
#'   Gender = c("Male", "Female")
#' )
#' xtab
#' # Compare the proportion of males and females in each category
#' row_wise_prop_test(xtab)


#' @describeIn prop_test performs one-sample and two-samples z-test of
#'   proportions. Wrapper around the function \code{\link[stats]{prop.test}()}.
#' @export
prop_test <- function(x, n, p = NULL, alternative = c("two.sided", "less", "greater"),
                      correct = TRUE, conf.level = 0.95, detailed = FALSE){
  args <- as.list(environment()) %>%
    add_item(method = "prop_test")
  if(is.data.frame(x)) x <- as.matrix(x)

  if(inherits(x, c("matrix", "table"))){
    if(ncol(x) > 2 & nrow(x) == 2) x <- t(x)
    nb.grp <- nrow(x)
    row.sums <- rowSums(x)
    n <- sum(x)
    Ns <- matrix(c(n, row.sums), nrow = 1, ncol = nb.grp+1)
    colnames(Ns) <- c("n", paste0("n", 1:nb.grp))
  }
  else{
    row.sums <- x
    nb.grp <- length(x)
    Ns <- matrix(c(sum(n), row.sums), nrow = 1, ncol = nb.grp+1)
    colnames(Ns) <- c("n", paste0("n", 1:nb.grp))
  }
  Ns <- as_tibble(Ns)
  results <- stats::prop.test(x, n, p, alternative, conf.level, correct) %>%
    as_tidy_stat() %>%
    add_significance("p") %>%
    mutate(method = "Prop test")

  results <- dplyr::bind_cols(Ns, results)

  if(!detailed) results <- remove_details(results, method = "prop.test")
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "prop_test"))
}

#' @describeIn prop_test pairwise comparisons between proportions, a post-hoc
#'   tests following a significant chi-square test of homogeneity for 2xc
#'   design. Wrapper around \code{\link[stats]{pairwise.prop.test}()}
#' @export
pairwise_prop_test <- function(xtab, p.adjust.method = "holm", ...){
  if(is.data.frame(xtab)) xtab <- as.matrix(xtab)
  if(ncol(xtab) > 2 & nrow(xtab) == 2)
    xtab <- t(xtab)
  args <- c(as.list(environment()), list(...)) %>%
    add_item(method = "prop_test")
  results <- stats::pairwise.prop.test(
    xtab, p.adjust.method = "none",
    ...
    ) %>%
    tidy() %>%
    select(.data$group2, .data$group1, .data$p.value)
  colnames(results) <- c("group1", "group2", "p")
  results <- results %>%
    adjust_pvalue(method = p.adjust.method) %>%
    add_significance("p.adj") %>%
    mutate(
      p = signif(.data$p, digits = 3),
      p.adj = signif(.data$p.adj, digits = 3)
      )
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "prop_test"))
}


#' @describeIn prop_test performs row-wise z-test of two proportions, a post-hoc tests following a significant chi-square test
#'   of homogeneity for rx2 contingency table. The z-test of two proportions is calculated for each category (row).
#' @export
row_wise_prop_test <- function(xtab, p.adjust.method = "holm", detailed = FALSE, ...){
  if(is.data.frame(xtab))
    xtab <- as.matrix(xtab)
  if(!inherits(xtab, c("matrix", "table"))){
    stop("An object of class 'matrix' or 'table' required")
  }
  if(ncol(xtab) !=2){
    stop("A cross-tabulation with two columns required")
  }
  args <- c(as.list(environment()), list(...)) %>%
    add_item(method = "prop_test")
  columns.total <- margin.table(xtab, 2)
  results <- apply(
    xtab, MARGIN = 1, FUN = prop_test,
    n = columns.total, detailed = detailed, ...
    ) %>%
    map(keep_only_tbl_df_classes) %>%
    bind_rows(.id = "group") %>%
    adjust_pvalue(method = p.adjust.method) %>%
    add_significance("p.adj") %>%
    mutate(
      p = signif(.data$p, digits = 3),
      p.adj = signif(.data$p.adj, digits = 3)
    ) %>%
    select(-.data$p.signif)
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "prop_test"))
}

#' @include utilities.R
NULL
#'Make Clean Names
#'
#'
#'@description Pipe-friendly function to make syntactically valid names out of
#'  character vectors.
#'
#' @param data a data frame or vector
#' @return a data frame or a vector depending on the input data
#'
#' @examples
#'
#' # Vector
#' make_clean_names(c("a and b", "a-and-b"))
#' make_clean_names(1:10)
#'
#' # data frame
#' df <- data.frame(
#' `a and b` = 1:4,
#' `c and d` = 5:8,
#'  check.names = FALSE
#' )
#' df
#' make_clean_names(df)
#'
#' @export
make_clean_names <- function(data){
  if(is.vector(data)){
    data <- make.names(data)
  }
  else if(inherits(data, c("data.frame", "matrix"))){
    .colnames <- colnames(data) %>% make.names()
    colnames(data) <- .colnames
  }
  data
}

#' @include utilities.R
#' @importFrom stats TukeyHSD
#' @importFrom dplyr everything
#' @importFrom tidyr separate
NULL
#'Tukey Honest Significant Differences
#'
#'
#'@description Provides a pipe-friendly framework to performs Tukey post-hoc
#'  tests. Wrapper around the function \code{\link[stats]{TukeyHSD}()}. It is
#'  essentially a t-test that corrects for multiple testing.
#'
#'  Can handle different inputs formats: aov, lm, formula.
#'@param x an object of class \code{aov}, \code{lm} or \code{data.frame}
#'  containing the variables used in the formula.
#'@param data a data.frame containing the variables in the formula.
#'@param formula a formula of the form \code{x ~ group} where \code{x} is a
#'  numeric variable giving the data values and \code{group} is a factor with
#'  one or multiple levels giving the corresponding groups. For example,
#'  \code{formula = TP53 ~ cancer_group}.
#'@param ... other arguments passed to the function
#'  \code{\link[stats]{TukeyHSD}()}. These include: \itemize{ \item
#'  \strong{which}: A character vector listing terms in the fitted model for
#'  which the intervals should be calculated. Defaults to all the terms. \item
#'  \strong{ordered}: A logical value indicating if the levels of the factor
#'  should be ordered according to increasing average in the sample before
#'  taking differences. If ordered is true then the calculated differences in
#'  the means will all be positive. The significant differences will be those
#'  for which the lwr end point is positive. }
#'@return a tibble data frame containing the results of the different
#'  comparisons.
#' @examples
#' # Data preparation
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#' # Tukey HSD from ANOVA results
#' aov(len ~ dose, data = df) %>% tukey_hsd()
#'
#' # two-way anova with interaction
#' aov(len ~ dose*supp, data = df) %>% tukey_hsd()
#'
#' # Tukey HSD from lm() results
#' lm(len ~ dose, data = df) %>% tukey_hsd()
#'
#' # Tukey HSD from data frame and formula
#' tukey_hsd(df, len ~ dose)
#'
#' # Tukey HSD using grouped data
#' df %>%
#'   group_by(supp) %>%
#'   tukey_hsd(len ~ dose)
#'
#'@export
tukey_hsd <- function(x, ...){
  UseMethod("tukey_hsd", x)
}


#' @export
#' @describeIn tukey_hsd performs tukey post-hoc test from \code{aov()} results.
tukey_hsd.default <- function(x, ...)
{
  tukey_hsd_of_model(x, ...) %>%
    set_attrs(args = list(x = x, p.adjust.method = "Tukey", method = "tukey_hsd")) %>%
    add_class(c("rstatix_test", "tukey_hsd"))
}

#' @export
#' @describeIn tukey_hsd performs tukey post-hoc test from \code{lm()} model.
tukey_hsd.lm <- function(x, ...)
{
  stats::aov(x) %>% tukey_hsd.default(...)
}


#' @describeIn tukey_hsd performs tukey post-hoc tests using data and formula as
#'   inputs. ANOVA will be automatically performed using the function
#'   \code{\link[stats]{aov}()}
#' @export
tukey_hsd.data.frame <- function(x, formula, ...){
  args <- list(
    data = x, formula = formula, method = "tukey_hsd",
    p.adjust.method = "Tukey"
    )
  if(is_grouped_df(x)){
    results <- x %>%
      doo(tukey_hsd_core, formula, ...) %>%
      set_attrs(args = args) %>%
      add_class(c("rstatix_test", "tukey_hsd"))
    return(results)
  }

  tukey_hsd_core (x, formula) %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "tukey_hsd"))
}


tukey_hsd_core <- function(x, formula, ...){
  stats::aov(formula, x) %>%
    tukey_hsd_of_model(...)
}

tukey_hsd_of_model <- function(model, ...){
  comparison <- adj.p.value <- p.adj <-
    term <- group1 <- group2 <- NULL
  magic.text <- "_XX.MAGIC.XX_"
  model %>%
    replace_eventual_minus_symbols_in_factors(by = magic.text) %>%
    TukeyHSD( ...) %>%
    broom::tidy() %>%
    replace_contrast_colname_by_comparison() %>%
    separate(comparison, into= c("group2", "group1"), sep = "-") %>%
    revert_back_eventual_minus_symbols(magic.text) %>%
    rename(p.adj = adj.p.value) %>%
    mutate(p.adj = signif(p.adj, 3)) %>%
    select(term, group1, group2, everything()) %>%
    add_significance("p.adj")
}


# Handling possible minus symbols in factor levels
replace_eventual_minus_symbols_in_factors <- function(res.aov, by = "_XX.MAGIC.XX_"){
  res.aov$model <- res.aov$model %>%
    dplyr::mutate_if(is.factor, fct_replace_minus, by = by)
  res.aov
}
revert_back_eventual_minus_symbols <- function(res.tukey.df, magic.text = "_XX.MAGIC.XX_" ){
  res.tukey.df %>%
    mutate(
      group1 = gsub(magic.text, replacement = "-", .data$group1, fixed = TRUE),
      group2 = gsub(magic.text, replacement = "-", .data$group2, fixed = TRUE)
    )
}

fct_replace_minus <- function(.factor, by = "_XX.MAGIC.XX_"){
  new.levels <- gsub(
    pattern = "-", replacement = by,
    x = levels(.factor), fixed = TRUE
    )
  levels(.factor) <- new.levels
  .factor
}

# in broom v>= 0.7.0; contrast is used instead of comparison
# so this helper function ensures that "comparison" is used as
# column name no matter the version of broom
replace_contrast_colname_by_comparison <- function(data){
  if("contrast" %in% colnames(data)){
    data <- data %>% rename(comparison = .data$contrast)
  }
  data
}

context("test-tukey_hsd")

test_that("tukey_hsd forwards conf.level for ungrouped data (#188)", {
  d <- ToothGrowth; d$dose <- factor(d$dose)
  a95 <- d %>% tukey_hsd(len ~ dose, conf.level = 0.95)
  a99 <- d %>% tukey_hsd(len ~ dose, conf.level = 0.99)
  # a 99% CI must be wider than the 95% CI (lower bound smaller, upper bound larger)
  expect_true(all(a99$conf.low  < a95$conf.low))
  expect_true(all(a99$conf.high > a95$conf.high))
  # the (Tukey-adjusted) p-values do not depend on conf.level
  expect_equal(a95$p.adj, a99$p.adj)
})

test_that("tukey_hsd default conf.level (0.95) is unchanged (no regression, #188)", {
  d <- ToothGrowth; d$dose <- factor(d$dose)
  res   <- d %>% tukey_hsd(len ~ dose)                 # default
  res95 <- d %>% tukey_hsd(len ~ dose, conf.level = 0.95)
  expect_equal(res$conf.low,  res95$conf.low)
  expect_equal(res$conf.high, res95$conf.high)
  expect_equal(res$p.adj,     res95$p.adj)
})

#' @include utilities.R
NULL
#'Exact Binomial Test
#'
#'@description Performs exact binomial test and pairwise comparisons following a
#'  significant exact multinomial test. Wrapper around the R base function
#'  \code{link[stats]{binom.test}()} that returns a data frame as a result.
#'
#'@inheritParams stats::binom.test
#'@param x numeric vector containing the counts.
#'@param p a vector of probabilities of success. The length of p must be the
#'  same as the number of groups specified by x, and its elements must be
#'  greater than 0 and less than 1.
#'@param p.adjust.method method to adjust p values for multiple comparisons.
#'  Used when pairwise comparisons are performed. Allowed values include "holm",
#'  "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
#'  want to adjust the p value (not recommended), use p.adjust.method = "none".
#'@param detailed logical value. Default is FALSE. If TRUE, a detailed result is
#'  shown.
#'@seealso \link{multinom_test}
#'@return return a data frame containing the p-value and its significance. with
#'  some the following columns: \itemize{ \item \code{group, group1, group2}:
#'  the categories or groups being compared. \item \code{statistic}: the number
#'  of successes. \item \code{parameter}: the number of trials. \item \code{p}:
#'  p-value of the test. \item \code{p.adj}: the adjusted p-value. \item
#'  \code{method}: the used statistical test. \item \code{p.signif,
#'  p.adj.signif}: the significance level of p-values and adjusted p-values,
#'  respectively. \item \code{estimate}: the estimated probability of success.
#'  \item \code{alternative}: a character string describing the alternative
#'  hypothesis. \item \code{conf.low,conf.high}: Lower and upper bound on a
#'  confidence interval  for the probability of success.}
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#'
#' @examples
#' # Exact binomial test
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Data: 160 mice with cancer including 95 male and 65 female
#' # Q1: Does cancer affect more males than females?
#' binom_test(x = 95, n = 160)
#' # => yes, there are a significant difference
#'
#'
#' # Q2: compare the observed proportion of males
#' # to an expected proportion (p = 3/5)
#' binom_test(x = 95, n = 160, p = 3/5)
#' # => there are no significant difference
#'
#' # Multinomial test
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Data
#' tulip <- c(red = 81, yellow = 50, white = 27)
#' # Question 1: are the color equally common ?
#' # this is a test of homogeneity
#' res <- multinom_test(tulip)
#' res
#' attr(res, "descriptives")
#'
#' # Pairwise comparisons between groups
#' pairwise_binom_test(tulip, p.adjust.method = "bonferroni")
#'
#'
#' # Question 2: comparing observed to expected proportions
#' # this is a goodness-of-fit test
#' expected.p <- c(red = 0.5, yellow = 0.33, white = 0.17)
#' res <- multinom_test(tulip, expected.p)
#' res
#' attr(res, "descriptives")
#'
#' # Pairwise comparisons against a given probabilities
#' pairwise_binom_test_against_p(tulip, expected.p)

#' @describeIn binom_test performs exact binomial test. Wrapper around the R
#'   base function \code{\link[stats]{binom.test}} that returns a dataframe as a
#'   result.
#' @export
binom_test <- function(x, n, p = 0.5,
                       alternative = "two.sided",
                       conf.level = 0.95, detailed = FALSE){
  args <- as.list(environment()) %>%
    add_item(method = "exact_binom_test")
  if(length(x) == 2) n <- sum(x)
  results <- stats::binom.test(x, n, p, alternative, conf.level) %>%
    tidy() %>%
    rename(p = .data$p.value) %>%
    add_significance("p") %>%
    add_columns(n = n, .before = 1)
  if(!detailed){
    to.keep <- c("n", "estimate", "conf.low", "conf.high", "p", "p.signif")
    results <- results[, to.keep]
  }
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "exact_binom_test"))
}

#' @describeIn binom_test performs pairwise comparisons (binomial test)
#'   following a significant exact multinomial test.
#' @export
pairwise_binom_test <- function(x, p.adjust.method = "holm", alternative = "two.sided",
                                conf.level = 0.95){
  if(is.null(names(x))){
    names(x) <- paste0("grp", 1:length(x))
  }
  compare_pair <- function(levs, x){
    levs <- as.character(levs)
    lev1 <- levs[1]
    lev2 <- levs[2]
    binom_test(
      x[lev1], x[lev1] + x[lev2], p = 0.5,
      alternative = alternative, conf.level = conf.level
      ) %>%
      add_columns(group1 = levs[1], group2 = levs[2], .before = 1)
  }
  args <- as.list(environment()) %>%
    add_item(method = "exact_binom_test")
  comparisons <- names(x) %>%
    .possible_pairs()
  results <- comparisons %>%
    map(compare_pair, x)
  results <- comparisons %>%
    map(compare_pair, x) %>%
    map(keep_only_tbl_df_classes) %>%
    bind_rows() %>%
    adjust_pvalue("p", method = p.adjust.method) %>%
    add_significance("p.adj") %>%
    mutate(p.adj = signif(.data$p.adj, digits = 3)) %>%
    select(-.data$p.signif)
    # select(.data$group1, .data$group2, .data$p, .data$p.adj, .data$p.adj.signif)
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "exact_binom_test"))
}

#' @describeIn binom_test performs pairwise comparisons (binomial test)
#'   following a significant exact multinomial test for given probabilities.
#' @export
pairwise_binom_test_against_p <- function(x, p = rep(1/length(x), length(x)), p.adjust.method = "holm",
                                          alternative = "two.sided", conf.level = 0.95){
  if (sum(p) != 1) {
    stop("sum of probabilities must be 1")
  }
  if (length(x) != length(p)) {
    stop("'x' and 'p' lengths differ")
  }
  groups <- names(x)
  if(is.null(groups)) {
    names(groups) <- paste0("grp", 1:length(x))
  }
  if(inherits(x, "table")){
    x <- as.vector(x)
  }
  names(x) <- groups

  args <- as.list(environment()) %>%
    add_item(method = "exact_binom_test")
  input <- data.frame(x = x, n = sum(x), p = p)
  results <- purrr::pmap(
    input, binom_test, alternative = alternative,
    conf.level = conf.level
    ) %>%
    map(keep_only_tbl_df_classes) %>%
    bind_rows() %>%
    adjust_pvalue("p", method = p.adjust.method) %>%
    add_significance("p.adj") %>%
    mutate(p.adj = signif(.data$p.adj, digits = 3)) %>%
    select(-.data$p.signif) %>%
    # select(.data$p, .data$p.adj, .data$p.adj.signif) %>%
    add_columns(group = groups, observed = x, expected = p*sum(x), .before = 1)
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "exact_binom_test"))
}

context("test-binom_test")

test_that("pairwise_binom_test_against_p works with an unnamed vector (#44)", {
  res <- pairwise_binom_test_against_p(c(20, 30, 50))
  expect_s3_class(res, "rstatix_test")
  expect_equal(nrow(res), 3)
  expect_equal(res$group, c("grp1", "grp2", "grp3"))
  expect_true(all(c("group", "p", "p.adj") %in% colnames(res)))
})

test_that("pairwise_binom_test_against_p preserves names for named/table input (no regression, #44)", {
  named   <- pairwise_binom_test_against_p(c(a = 20, b = 30, c = 50))
  unnamed <- pairwise_binom_test_against_p(c(20, 30, 50))
  tbl     <- pairwise_binom_test_against_p(as.table(c(x = 20, y = 30, z = 50)))
  expect_equal(named$group, c("a", "b", "c"))
  expect_equal(tbl$group,   c("x", "y", "z"))
  # only the group labels differ between named and unnamed; the statistics are identical
  expect_equal(named$p, unnamed$p)
  expect_equal(named$p.adj, unnamed$p.adj)
})

#' @include utilities.R
NULL

#'Kruskal-Wallis Effect Size
#'
#'@description Compute the effect size for Kruskal-Wallis test as the eta
#'  squared based on the H-statistic: \code{eta2[H] = (H - k + 1)/(n - k)};
#'  where \code{H} is the value obtained in the Kruskal-Wallis test; \code{k} is
#'  the number of groups; \code{n} is the total number of observations.
#'
#'
#'  The eta-squared estimate assumes values from 0 to 1 and multiplied by 100%
#'  indicates the percentage of variance in the dependent variable explained by
#'  the independent variable. The interpretation values commonly in published
#'  litterature are: \code{0.01- < 0.06} (small effect), \code{0.06 - < 0.14}
#'  (moderate effect) and \code{>= 0.14} (large effect).
#'
#' Confidence intervals are calculated by bootstap.
#'
#'@inheritParams wilcox_effsize
#'@return return a data frame with some of the following columns: \itemize{
#'  \item \code{.y.}: the y variable used in the test. \item \code{n}: Sample
#'  counts. \item \code{effsize}: estimate of the effect size. \item
#'  \code{magnitude}: magnitude of effect size. \item \code{conf.low,conf.high}:
#'  lower and upper bound of the effect size confidence interval.}
#'
#'@references Maciej Tomczak and Ewa Tomczak. The need to report effect size
#'  estimates revisited. An overview of some recommended measures of effect
#'  size. Trends in Sport Sciences. 2014; 1(21):19-25.
#'
#'  http://imaging.mrc-cbu.cam.ac.uk/statswiki/FAQ/effectSize
#'
#'  http://www.psy.gla.ac.uk/~steve/best/effect.html
#' @examples
#' # Load data
#' #:::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Kruskal-wallis rank sum test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% kruskal_effsize(len ~ dose)
#'
#' # Grouped data
#' df %>%
#'   group_by(supp) %>%
#'   kruskal_effsize(len ~ dose)
#' @export
kruskal_effsize <- function(data, formula, ci = FALSE, conf.level = 0.95,  ci.type = "perc", nboot = 1000){
  args <- as.list(environment()) %>%
    .add_item(method = "kruskal_effsize")
  data %>%
    doo(
      .kruskal_effsize, formula, ci = ci, conf.level = conf.level,
      ci.type = ci.type, nboot = nboot
    ) %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "kruskal_effsize"))
}

.kruskal_effsize <- function(data, formula, ci = FALSE, conf.level = 0.95,  ci.type = "perc", nboot = 1000){
  results <- eta_squared_h(data, formula)
  # Confidence interval of the effect size r
  if (ci == TRUE) {
    stat.func <- function(data, subset) {
      eta_squared_h(data, formula, subset = subset)$effsize
    }
    CI <- get_boot_ci(
      data, stat.func, conf.level = conf.level,
      type = ci.type, nboot = nboot
    )
    results <- results %>%
      add_columns(conf.low = CI[1], conf.high = CI[2], .after = "effsize")
  }
  results %>% mutate(magnitude = get_eta_squared_magnitude(.data$effsize))
}


eta_squared_h <- function(data, formula, subset = NULL, ...){
  if(!is.null(subset)) data <- data[subset, ]
  res.kw <- kruskal_test(data, formula, ...)
  nb.groups <- res.kw$df + 1
  nb.samples <- res.kw$n
  etasq <- (res.kw$statistic - nb.groups + 1) / (nb.samples - nb.groups)
  tibble(
    .y. = get_formula_left_hand_side(formula),
    n = nb.samples,
    effsize = etasq,
    method = "eta2[H]"
    )
}

get_eta_squared_magnitude <- function(d){
  magnitude.levels = c(0.06, 0.14, Inf)
  magnitude = c("small","moderate","large")
  d.index <- findInterval(abs(d), magnitude.levels)+1
  magnitude <- factor(magnitude[d.index], levels = magnitude, ordered = TRUE)
  magnitude
}

context("test-kruskal_effsize")

test_that("kruskal_effsize clamps a degenerate eta-squared to [0,1] and labels it 'small' (#217)", {
  # identical, interleaved group profiles -> tiny H -> formula yields a negative eta-squared
  d <- data.frame(y = c(1, 2, 3, 1, 2, 3, 1, 2, 3),
                  g = factor(rep(c("a", "b", "c"), each = 3)))
  res <- suppressWarnings(kruskal_effsize(d, y ~ g))
  expect_gte(res$effsize, 0)                          # never negative (clamped to [0,1])
  expect_equal(res$effsize, 0)                        # this degenerate case clamps to 0
  expect_equal(as.character(res$magnitude), "small")  # and is NOT reported as 'large'
})

test_that("kruskal_effsize magnitude is unchanged for valid effect sizes (no regression, #217)", {
  # ToothGrowth dose has a strong effect -> magnitude 'large' (unchanged behaviour)
  res <- ToothGrowth %>% kruskal_effsize(len ~ dose)
  expect_true(res$effsize > 0.14)
  expect_equal(as.character(res$magnitude), "large")
})

context("test-kruskal_test")

test_that("kruskal_test reports the complete-case n when the data contain NAs (#224)", {
  set.seed(123)
  df <- data.frame(
    group = factor(rep(c(NA, "A", "B", "C"), each = 10)),
    value = c(rnorm(35), rep(NA, 5))
  )
  res <- kruskal_test(df, value ~ group)
  expect_equal(res$n, 25)                       # complete cases, not nrow = 40
  # an explicit na.action passed through ... is respected
  expect_equal(kruskal_test(df, value ~ group, na.action = stats::na.omit)$n, 25)
  # pre-cleaned data gives the same n
  expect_equal(kruskal_test(na.omit(df), value ~ group)$n, 25)
  # n stays the complete-case count even under na.pass (kruskal.test always uses
  # complete cases regardless of na.action), passed explicitly or set globally
  expect_equal(kruskal_test(df, value ~ group, na.action = stats::na.pass)$n, 25)
  old <- getOption("na.action"); on.exit(options(na.action = old))
  options(na.action = "na.pass")
  expect_equal(kruskal_test(df, value ~ group)$n, 25)
})

test_that("kruskal_test statistic/p are unchanged and match stats::kruskal.test (#224)", {
  set.seed(123)
  df <- data.frame(
    group = factor(rep(c(NA, "A", "B", "C"), each = 10)),
    value = c(rnorm(35), rep(NA, 5))
  )
  res <- kruskal_test(df, value ~ group)
  ref <- suppressWarnings(stats::kruskal.test(value ~ group, data = df))
  expect_equal(unname(res$statistic), unname(ref$statistic))
  expect_equal(res$p, ref$p.value)
})

test_that("kruskal_test n is unchanged for data without NAs (no-regression) (#224)", {
  res <- ToothGrowth %>% kruskal_test(len ~ dose)
  expect_equal(res$n, nrow(ToothGrowth))        # 60
  expect_equal(unname(round(res$statistic, 4)), 40.6689)
})

test_that("kruskal_test works on grouped data (#224)", {
  res <- ToothGrowth %>% dplyr::group_by(supp) %>% kruskal_test(len ~ dose)
  expect_equal(nrow(res), 2L)
  expect_equal(res$n, c(30, 30))
})

test_that("kruskal_effsize uses the complete-case n with NAs (#224)", {
  set.seed(123)
  df <- data.frame(
    group = factor(rep(c(NA, "A", "B", "C"), each = 10)),
    value = c(rnorm(35), rep(NA, 5))
  )
  eff <- kruskal_effsize(df, value ~ group)
  expect_equal(eff$n, 25)
  # eta2[H] uses n in its denominator, so the corrected n also corrects the
  # effect size: recompute it from the (correct) H and complete-case n.
  kt <- kruskal_test(df, value ~ group)
  k <- kt$df + 1
  expected <- max(0, min(1, (kt$statistic - k + 1) / (eff$n - k)))
  expect_equal(eff$effsize, unname(expected), tolerance = 1e-7)
})

#' @include utilities.R
NULL
#'Friedman Rank Sum Test
#'
#'
#'@description Provides a pipe-friendly framework to perform a Friedman rank sum
#'  test, which is the non-parametric alternative to the one-way repeated
#'  measures ANOVA test. Wrapper around the function
#'  \code{\link[stats]{friedman.test}()}. Read more:
#'  \href{https://www.datanovia.com/en/lessons/friedman-test-in-r/}{Friedman
#'  test in R}.
#'@param data a data.frame containing the variables in the formula.
#'@param formula a formula of the form \code{a ~ b | c}, where \code{a}
#'  (numeric) is the dependent variable name; \code{b} is the within-subjects
#'  factor variables; and \code{c} (factor) is the column name containing
#'  individuals/subjects identifier. Should be unique per individual.
#'@param ... other arguments to be passed to the function
#'  \code{\link[stats]{friedman.test}}.
#'
#'@return return a data frame with the following columns: \itemize{ \item
#'  \code{.y.}: the y (dependent) variable used in the test. \item \code{n}:
#'  sample count. \item \code{statistic}: the value of Friedman's chi-squared
#'  statistic, used to compute the p-value. \item \code{p}: p-value. \item
#'  \code{method}: the statistical test used to compare groups.}
#'
#' @examples
#' # Load data
#' #:::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth %>%
#'     filter(supp == "VC") %>%
#'     mutate(id = rep(1:10, 3))
#' head(df)
#'
#' # Friedman rank sum test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% friedman_test(len ~ dose | id)
#'
#'@name friedman_test
#'@export
friedman_test <- function(data, formula, ...){
  args <- c(as.list(environment()), list(...)) %>%
    add_item(method = "friedman_test")
  vars <- get_friedman_vars(formula)
  args <- args %>% add_item(dv = vars$dv, wid = vars$wid, within = args$within)
  if(is_grouped_df(data)){
    results <- data %>% doo(.friedman_test, formula, ...)
  }
  else{
    results <- .friedman_test(data, formula, ...)
  }
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "friedman_test"))
}


.friedman_test <- function(data, formula, ...){
  vars <- get_friedman_vars(formula)
  term <- statistic <- p <- df <- method <- NULL
  sample.size <- data %>% pull(vars$wid) %>% unique() %>% length()
  res <- stats::friedman.test(formula, data = data, ...) %>%
    tidy() %>%
    rename(p = .data$p.value, df = .data$parameter) %>%
    mutate(method = "Friedman test") %>%
    select(.data$statistic, .data$df, .data$p, .data$method)  %>%
    add_columns(.y. = vars$dv, n = sample.size, .before = "statistic")
  res
}


get_friedman_vars <- function(formula){
  outcome <- get_formula_left_hand_side(formula)
  rhs <- get_formula_right_hand_side(formula)
  rhs <- gsub(pattern = " ", replacement = "", rhs)
  rhs <- strsplit(rhs, "|", fixed = TRUE) %>% unlist()
  list(dv = outcome, within = rhs[1],  wid = rhs[2])
}

#' @include utilities.R
NULL
#'McNemar's Chi-squared Test for Count Data
#'@description Performs McNemar chi-squared test to compare paired proportions.
#'
#'  Wrappers around the R base function \code{\link[stats]{mcnemar.test}()}, but
#'  provide pairwise comparisons between multiple groups
#'@inheritParams stats::mcnemar.test
#'@param data a data frame containing the variables in the formula.
#'@param formula a formula of the form \code{a ~ b | c}, where \code{a} is the
#'  outcome variable name; b is the within-subjects factor variables; and c
#'  (factor) is the column name containing individuals/subjects identifier.
#'  Should be unique per individual.
#'@param type type of statistical tests used for pairwise comparisons. Allowed
#'  values are one of \code{c("mcnemar", "exact")}.
#'@param p.adjust.method method to adjust p values for multiple comparisons.
#'  Used when pairwise comparisons are performed. Allowed values include "holm",
#'  "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
#'  want to adjust the p value (not recommended), use p.adjust.method = "none".
#'
#'
#'@return return a data frame with the following columns: \itemize{
#' \item \code{n}: the number of participants.
#' \item \code{statistic}: the value of McNemar's statistic. \item \code{df} the
#'  degrees of freedom of the approximate chi-squared distribution of the test
#'  statistic. \item \code{p}: p-value. \item \code{p.adj}: the adjusted
#'  p-value. \item \code{method}: the used statistical test. \item
#'  \code{p.signif}: the significance level of p-values.}
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#' @examples
#'
#' # Comparing two paired proportions
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Data: frequencies of smokers before and after interventions
#' xtab <- as.table(
#'   rbind(c(25, 6), c(21,10))
#' )
#' dimnames(xtab) <- list(
#'   before = c("non.smoker", "smoker"),
#'   after = c("non.smoker", "smoker")
#' )
#' xtab
#'
#' # Compare the proportion of smokers
#' mcnemar_test(xtab)
#'
#' # Comparing multiple related proportions
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Generate a demo data
#' mydata <- data.frame(
#'   outcome = c(0,1,1,0,0,1,0,1,1,1,1,1,0,0,1,1,0,1,0,1,1,0,0,1,0,1,1,0,0,1),
#'   treatment = gl(3,1,30,labels=LETTERS[1:3]),
#'   participant = gl(10,3,labels=letters[1:10])
#' )
#' mydata$outcome <- factor(
#'   mydata$outcome, levels = c(1, 0),
#'   labels = c("success", "failure")
#'   )
#' # Cross-tabulation
#' xtabs(~outcome + treatment, mydata)
#'
#' # Compare the proportion of success between treatments
#' cochran_qtest(mydata, outcome ~ treatment|participant)
#'
#' # pairwise comparisons between groups
#' pairwise_mcnemar_test(mydata, outcome ~ treatment|participant)
#'
#'@describeIn mcnemar_test performs McNemar's chi-squared test for comparing two
#'  paired proportions
#'@export
mcnemar_test <- function(x, y = NULL, correct = TRUE){
  args <- as.list(environment()) %>%
    add_item(method = "mcnemar_test")
  if(is.data.frame(x)) x <- as.matrix(x)
  if(inherits(x, c("matrix", "table"))) n <- sum(x)
  else n <- length(x)
  results <- stats::mcnemar.test(x, y, correct) %>%
    as_tidy_stat() %>%
    add_significance("p") %>%
    mutate(method = "McNemar test", n = n)
  results[, c("n", "statistic", "df", "p", "p.signif", "method")] %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "mcnemar_test"))
}

#'@describeIn mcnemar_test performs pairwise McNemar's chi-squared test between
#'  multiple groups. Could be used for post-hoc tests following a significant Cochran's Q test.
#'@export
pairwise_mcnemar_test <- function (data, formula, type = c("mcnemar", "exact"), correct = TRUE,
                              p.adjust.method = "bonferroni")
{
  type  <- match.arg(type)
  test.class <- switch (type,
    mcnemar = "mcnemar_test",
    exact = "exact_binom_test"
  )
  data <- data %>% select(!!!syms(all.vars(formula)))
  colnames(data) <- c("outcome", "groups", "participant")
  if(length(unique(data$outcome))> 2){
    stop("Unique possible outcome values should be 2")
  }
  args <- as.list(environment()) %>%
    add_item(method = test.class)

  # helper function to compar pairs
  compare_pair <- function(grps, data, type = "mcnemar"){
    grps <- as.character(grps)
    grps.data <- data[, grps]
    colnames(grps.data) <- c("grp1", "grp2")
    xtab <- stats::xtabs(~grp1+grp2, grps.data)
    if(type == "mcnemar"){
      results <- mcnemar_test(xtab, correct = correct)
    }
    else if(type == "exact"){
      # Get off-diagonal values
      b <- xtab[2, 1]
      c <- xtab[1, 2]
      results <- binom_test(b, (b + c), p = 0.5, detailed = TRUE)
    }
    results %>%
      keep_only_tbl_df_classes() %>%
      select(.data$p, .data$method) %>%
      add_columns(group1 = grps[1], group2 = grps[2], .before = "p")
  }
  # Convert outcome into factor, then spread.
  data <- data %>%
    mutate(outcome = as.factor(.data$outcome)) %>%
    spread(key = "groups", value = "outcome")
  # Pairwise comparisons
  comparisons <- colnames(data)[-1] %>%
    .possible_pairs()
  results <- comparisons %>%
    map(compare_pair, data, type = type) %>%
    bind_rows() %>%
    adjust_pvalue("p", method = p.adjust.method) %>%
    add_significance("p.adj") %>%
    mutate(p.adj = signif(.data$p.adj, digits = 3))
  results [, c("group1", "group2", "p", "p.adj", "p.adj.signif", "method")] %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", test.class))
}

context("test-mcnemar_test")

# Demo data: 3 related treatments, binary outcome (from ?mcnemar_test)
make_mcnemar_data <- function(){
  mydata <- data.frame(
    outcome = c(0,1,1,0,0,1,0,1,1,1,1,1,0,0,1,1,0,1,0,1,1,0,0,1,0,1,1,0,0,1),
    treatment = gl(3, 1, 30, labels = LETTERS[1:3]),
    participant = gl(10, 3, labels = letters[1:10])
  )
  mydata$outcome <- factor(
    mydata$outcome, levels = c(1, 0), labels = c("success", "failure")
  )
  mydata
}

test_that("pairwise_mcnemar_test returns statistic and df for type='mcnemar' (#122)", {
  res <- pairwise_mcnemar_test(make_mcnemar_data(), outcome ~ treatment | participant)
  expect_true(all(c("statistic", "df") %in% colnames(res)))
  expect_equal(
    colnames(res),
    c("group1", "group2", "statistic", "df", "p", "p.adj", "p.adj.signif", "method")
  )
  expect_equal(nrow(res), 3L)                 # 3 pairwise comparisons of 3 groups
  expect_true(all(res$df == 1))               # McNemar chi-squared on a 2x2 table
})

test_that("pairwise_mcnemar_test statistic/df match per-pair mcnemar_test (#122)", {
  mydata <- make_mcnemar_data()
  res <- pairwise_mcnemar_test(mydata, outcome ~ treatment | participant)
  wide <- tidyr::spread(
    dplyr::mutate(mydata, outcome = as.factor(outcome)),
    key = treatment, value = outcome
  )
  manual_pair <- function(g1, g2){
    xt <- stats::xtabs(stats::as.formula(paste0("~", g1, "+", g2)), wide)
    mcnemar_test(xt)
  }
  ab <- manual_pair("A", "B")
  row <- res[res$group1 == "A" & res$group2 == "B", ]
  expect_equal(row$statistic, ab$statistic)
  expect_equal(row$df, ab$df)
  expect_equal(row$p, ab$p)
})

test_that("pairwise_mcnemar_test type='exact' has no statistic/df (#122 no-regression)", {
  res <- pairwise_mcnemar_test(
    make_mcnemar_data(), outcome ~ treatment | participant, type = "exact"
  )
  expect_false(any(c("statistic", "df") %in% colnames(res)))
  expect_equal(
    colnames(res),
    c("group1", "group2", "p", "p.adj", "p.adj.signif", "method")
  )
  expect_equal(unique(res$method), "Exact binomial test")
})

test_that("mcnemar_test (single, 2x2) is unchanged (#122 no-regression)", {
  xtab <- as.table(rbind(c(25, 6), c(21, 10)))
  res <- mcnemar_test(xtab)
  expect_equal(
    colnames(res),
    c("n", "statistic", "df", "p", "p.signif", "method")
  )
  expect_equal(res$n, 62)
  expect_equal(unname(res$df), 1)
  expect_equal(unname(res$p), stats::mcnemar.test(xtab)$p.value)
})

#' @include utilities.R
NULL
#'Compute Frequency Table
#'@description compute frequency table.
#'@param data a data frame
#'@param ... One or more unquoted expressions (or variable names) separated by
#'  commas. Used to specify variables of interest.
#'@param vars optional character vector containing variable names.
#'@param na.rm logical value. If TRUE (default), remove missing values in the
#'  variables used to create the frequency table.
#'@return a data frame
#' @examples
#' data("ToothGrowth")
#' ToothGrowth %>% freq_table(supp, dose)
#'@export
freq_table <- function(data, ..., vars = NULL, na.rm = TRUE){
  if(is.vector(data) | is.factor(data)){
    data <- data.frame(group = data)
    vars <- "group"
  }
  data <- data %>%
    df_select(..., vars = vars)
  vars <- colnames(data)
  if(length(vars) == 0){
    stop("Specify at least one variable")
  }
  if(na.rm){
    data <- data %>%
      filter(stats::complete.cases(data))
  }
  results <- data %>%
    group_by(!!!syms(vars)) %>%
    summarise(n = n()) %>%
    mutate(prop = round(.data$n *100 / sum (.data$n), 1)) %>%
    dplyr::ungroup()
  results
}


spread_table <- function(data, vars){
  last.var <- dplyr::last(vars)
  grouping.vars <- utils::head(vars, -2)
  if(length(vars) >= 2){
    data <- data %>%
      select(-.data$prop) %>%
      group_by(!!!syms(grouping.vars)) %>%
      nest() %>%
      mutate(data = map(.data$data, spread, key = last.var, value = "n"))
  }
  data

}

context("test-freq_table")

tg <- function(){ d <- ToothGrowth; d$dose <- factor(d$dose); d }

test_that("freq_table() works on ungrouped data (no-regression) (#191)", {
  res <- tg() %>% freq_table(supp, dose)
  expect_equal(colnames(res), c("supp", "dose", "n", "prop"))
  expect_equal(nrow(res), 6L)
  expect_true(all(res$n == 10))
  # proportions sum to 100% within each supp (all-but-last grouping)
  sums <- tapply(res$prop, res$supp, sum)
  expect_true(all(abs(sums - 100) < 0.5))
})

test_that("freq_table() handles grouped data, returning a tidy data frame (#191)", {
  res <- tg() %>% group_by(supp) %>% freq_table(dose)
  # tidy data frame (NOT a list), grouping column kept
  expect_s3_class(res, "data.frame")
  expect_false(is.list(res) && !is.data.frame(res))
  expect_true(all(c("supp", "dose", "n", "prop") %in% colnames(res)))
  expect_equal(nrow(res), 6L)
  # equals the equivalent ungrouped call freq_table(supp, dose)
  ref <- tg() %>% freq_table(supp, dose)
  expect_equal(
    as.data.frame(res)[order(res$supp, res$dose), c("supp", "dose", "n", "prop")],
    as.data.frame(ref)[order(ref$supp, ref$dose), c("supp", "dose", "n", "prop")],
    ignore_attr = TRUE
  )
  # proportions sum to ~100% within each group
  sums <- tapply(res$prop, res$supp, sum)
  expect_true(all(abs(sums - 100) < 0.5))
})

test_that("freq_table() grouped by two variables works (#191)", {
  d <- tg(); d$cap <- factor(rep(c("a", "b"), 30))
  res <- d %>% group_by(supp, cap) %>% freq_table(dose)
  expect_true(all(c("supp", "cap", "dose", "n", "prop") %in% colnames(res)))
  # proportions sum to ~100% within each (supp, cap) cell
  sums <- tapply(res$prop, interaction(res$supp, res$cap), sum)
  expect_true(all(abs(sums - 100) < 1))
})

#' @include utilities.R
NULL
#'Create a List of Possible Comparisons Between Groups
#'@description Create a list of possible pairwise comparisons between groups. If
#'  a reference group is specified, only comparisons against reference will be
#'  kept.
#'@param data a data frame
#'@param variable the grouping variable name. Can be unquoted.
#'@param ref.group a character string specifying the reference group. Can be
#'  unquoted. If numeric, then it should be quoted. If specified, for a
#'  given grouping variable, each of the group levels will be compared to the
#'  reference group (i.e. control group).
#'
#'  If \code{ref.group = "all"}, pairwise comparisons are performed between each
#'  grouping variable levels against all (i.e. basemean).
#'@return a list of all possible pairwise comparisons.
#'@examples
#' # All possible pairwise comparisons
#' ToothGrowth %>%
#'   get_comparisons("dose")
#'
#' # Comparisons against reference groups
#' ToothGrowth %>%
#'   get_comparisons("dose", ref.group = "0.5")
#'
#' # Comparisons against  all (basemean)
#' ToothGrowth %>%
#'   get_comparisons("dose", ref.group = "all")
#'@export
get_comparisons <- function(data, variable, ref.group = NULL){
  group <- rlang::enquo(variable) %>% rlang::as_name()
  ref.group <- rlang::enquo(ref.group)
  if(rlang::quo_is_null(ref.group)) ref.group <- NULL
  else ref.group <- rlang::as_name(ref.group)
  group.levels <- data %>%
    .as_factor(group) %>%
    get_levels(group)
  asserttat_ref_group_iscorrect(group.levels, ref.group)
  comparisons <- c(ref.group, group.levels) %>%
    unique() %>%
    .possible_pairs(ref.group = ref.group) %>%
    map(as.character)
  comparisons
}

asserttat_ref_group_iscorrect <- function(.levels, .ref){
  if(!is.null(.ref)){
    .diff <- setdiff(.ref, c("all", ".all.", .levels))
    if(!.is_empty(.diff))
      stop("ref.group is incorrect")
  }
}

#' @include utilities.R utilities_two_sample_test.R
#' @importFrom stats qbinom
#' @importFrom stats pbinom
NULL
#'Sign Test
#'
#'@description Performs one-sample and two-sample sign tests. Read more:
#'  \href{https://www.datanovia.com/en/lessons/sign-test-in-r/}{Sign Test in R}.
#'@inheritParams t_test
#'@param data a data.frame containing the variables in the formula.
#'@param formula a formula of the form \code{x ~ group} where \code{x} is a
#'  numeric variable giving the data values and \code{group} is a factor with
#'  one or multiple levels giving the corresponding groups. For example,
#'  \code{formula = TP53 ~ treatment}.
#'@param mu a single number representing the value of the population median
#'  specified by the null hypothesis.
#'@param ref.group a character string specifying the reference group. If
#'  specified, for a given grouping variable, each of the group levels will be
#'  compared to the reference group (i.e. control group).
#'@param ... other arguments passed to the function \code{sign_test()}
#'
#'@return return a data frame with some the following columns: \itemize{ \item
#'  \code{.y.}: the y variable used in the test. \item \code{group1,group2}: the
#'  compared groups in the pairwise tests. \item \code{n,n1,n2}: Sample counts.
#'  \item \code{statistic}: Test statistic used to compute the p-value. That is
#'  the S-statistic (the number of positive differences between the data and the
#'  hypothesized median), with names attribute \code{"S"}. \item \code{df,
#'  parameter}: degrees of freedom. Here, the total number of valid differences.
#'  \item \code{p}: p-value.  \item \code{method}: the statistical test used to
#'  compare groups. \item \code{p.signif, p.adj.signif}: the significance level
#'  of p-values and adjusted p-values, respectively. \item \code{estimate}:
#'  estimate of the effect size. It corresponds to the median of the
#'  differences. \item \code{alternative}: a character string describing the
#'  alternative hypothesis. \item \code{conf.low,conf.high}: Lower and upper
#'  bound on a confidence interval of the estimate. }
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#'@note This function is a reimplementation of the function \code{SignTest()}
#'  from the \code{DescTools} package.
#' @examples
#' # Load data
#' #:::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # One-sample test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% sign_test(len ~ 1, mu = 0)
#'
#'
#' # Two-samples paired test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% sign_test(len ~ supp)
#'
#'
#' # Compare supp levels after grouping the data by "dose"
#' #::::::::::::::::::::::::::::::::::::::::
#' df %>%
#'   group_by(dose) %>%
#'   sign_test(data =., len ~ supp) %>%
#'   adjust_pvalue(method = "bonferroni") %>%
#'   add_significance("p.adj")
#'
#' # pairwise comparisons
#' #::::::::::::::::::::::::::::::::::::::::
#' # As dose contains more than two levels ==>
#' # pairwise test is automatically performed.
#' df %>% sign_test(len ~ dose)
#'
#' # Comparison against reference group
#' #::::::::::::::::::::::::::::::::::::::::
#' # each level is compared to the ref group
#' df %>% sign_test(len ~ dose, ref.group = "0.5")
#'
#'
#'@describeIn sign_test Sign test
#'@export
sign_test <- function(data, formula, comparisons = NULL, ref.group = NULL,
                      p.adjust.method = "holm", alternative = "two.sided",
                      mu = 0, conf.level = 0.95, detailed = FALSE){

  args <- as.list(environment()) %>%
    .add_item(method = "sign_test")

  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  number.of.groups <- guess_number_of_groups(data, group)

  if(!is.null(ref.group)){
    if(ref.group %in% c("all", ".all."))
      stop("ref.group can't be 'all'.")
  }
  # Case of one sample test
  if(number.of.groups == 1){
    res <- one_sample_sign_test(
      data = data, formula = formula,
      alternative = alternative, mu = mu,
      conf.level = conf.level, detailed = detailed
    )
  }
  # Case of two independents or paired groups
  else if (number.of.groups == 2) {
    res <- two_sample_sign_test(
      data = data, formula = formula,
      alternative = alternative,
      conf.level = conf.level, ref.group = ref.group,
      detailed = detailed
    )
  }
  # Pairwise comparisons
  else if(number.of.groups > 2){
      res <- pairwise_sign_test(
        data = data, formula = formula,
        comparisons = comparisons, ref.group = ref.group,
        p.adjust.method = p.adjust.method,
        alternative = alternative, conf.level = conf.level,
        detailed = detailed
      )
  }
  res %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "sign_test"))
}



one_sample_sign_test <- function(data, formula, mu = 0, ...){
  two_sample_test(data, formula, method = "sign.test", mu = mu, ...)
}
two_sample_sign_test <- function(data, formula,   ...)
{
  two_sample_test(data, formula, method = "sign.test",  ...)
}


#'@describeIn sign_test performs pairwise two sample Wilcoxon test.
#'@export
pairwise_sign_test <- function(
  data, formula, comparisons = NULL, ref.group = NULL,
  p.adjust.method = "holm", detailed = FALSE, ...)
{
  args <- as.list(environment()) %>%
    .add_item(method = "sign_test")

  res <- pairwise_two_sample_test(
    data, formula, method = "sign.test",
    comparisons = comparisons, ref.group = ref.group,
    p.adjust.method = p.adjust.method, detailed = detailed, ...
  )
  res %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "sign_test"))
}


sign.test <- function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
                             mu = 0, conf.level = 0.95, ...) {

  alternative <- match.arg(alternative)
  check_two_samples_test_args(x = x, y = y, mu = mu,  conf.level = conf.level)

  if (!is.null(y)) {
    if (length(x) != length(y))
      stop("'x' and 'y' must have the same length")
    DNAME <- paste(deparse(substitute(x)), "and", deparse(substitute(y)))
    OK <- complete.cases(x, y)
    x <- x[OK]
    y <- y[OK]
    METHOD <- "Paired-samples Sign-Test"
    x <- (x - y)
  }
  else {
    DNAME <- deparse(substitute(x))
    x <- x[is.finite(x)]
    METHOD <- "One-sample Sign-Test"
  }

  d <- (x - mu)
  n.valid <- sum(d > 0) + sum(d < 0)
  if(n.valid > 0) {
    RVAL <- stats::binom.test(x=sum(d > 0), n=n.valid, p=0.5, alternative = alternative, conf.level = conf.level )
  } else {
    RVAL <- stats::binom.test(x=1, n=1)
  }

  RVAL$method <- METHOD
  RVAL$data.name <- DNAME
  names(mu) <- if (!is.null(y)) "median difference" else "median"

  names(RVAL$statistic) <- "S"
  RVAL$estimate <- median(d + mu, na.rm=TRUE)
  names(RVAL$parameter) <- "number of differences"
  mci <- get_median_ci(d + mu, conf.level=conf.level, alternative=alternative)
  RVAL$conf.int <- mci
  attr(RVAL$conf.int, "conf.level") = round(attr(mci,"conf.level"), 3)

  names(RVAL$estimate) <- "median of the differences"
  RVAL$null.value <- mu
  class(RVAL) <- "htest"
  return(RVAL)
}



get_median_ci <- function( x, conf.level = 0.95,
                            alternative = c("two.sided", "less", "greater")){
  # http://www.stat.umn.edu/geyer/old03/5102/notes/rank.pdf
  # http://de.scribd.com/doc/75941305/Confidence-Interval-for-Median-Based-on-Sign-Test
  x <- stats::na.omit(x)
  n <- length(x)
  switch( match.arg(alternative)
          , "two.sided" = {
            k <- qbinom(p = (1 - conf.level) / 2, size=n, prob=0.5, lower.tail=TRUE)
            ci <- sort(x)[c(k, n - k + 1)]
            attr(ci, "conf.level") <- 1 - 2 * pbinom(k-1, size=n, prob=0.5)
          }
          , "greater" = {
            k <- qbinom(p = (1 - conf.level), size=n, prob=0.5, lower.tail=TRUE)
            ci <- c(sort(x)[k], Inf)
            attr(ci, "conf.level") <- 1 - pbinom(k-1, size=n, prob=0.5)
          }
          , "less" = {
            k <- qbinom(p = conf.level, size=n, prob=0.5, lower.tail=TRUE)
            ci <- c(-Inf, sort(x)[k])
            attr(ci, "conf.level") <- pbinom(k, size=n, prob=0.5)
          }
  )
  return(ci)
}

#' @include utilities.R
NULL
#'Autocompute P-value Positions For Plotting Significance
#'@description Compute p-value x and y positions for plotting significance
#'  levels. Many examples are provided at : \itemize{ \item
#'  \href{https://www.datanovia.com/en/blog/how-to-add-p-values-onto-a-grouped-ggplot-using-the-ggpubr-r-package/}{How
#'   to Add P-Values onto a Grouped GGPLOT using the GGPUBR R Package} \item
#'  \href{https://www.datanovia.com/en/blog/ggpubr-how-to-add-adjusted-p-values-to-a-multi-panel-ggplot/}{How
#'   to Add Adjusted P-values to a Multi-Panel GGPlot} \item
#'  \href{https://www.datanovia.com/en/blog/ggpubr-how-to-add-p-values-generated-elsewhere-to-a-ggplot/}{How
#'   to Add P-Values Generated Elsewhere to a GGPLOT} }
#'@inheritParams t_test
#'@param ref.group a character string specifying the reference group. If
#'  specified, for a given grouping variable, each of the group levels will be
#'  compared to the reference group (i.e. control group).
#'@param fun summary statistics functions used to compute automatically suitable
#'  y positions of p-value labels and brackets. Possible values include:
#'  \code{"max", "mean", "mean_sd", "mean_se", "mean_ci", "median",
#'  "median_iqr", "median_mad"}.
#'
#'  For example, if \code{fun = "max"}, the y positions are guessed as follow:
#'  \itemize{ \item 1. Compute the maximum of each group (groups.maximum) \item
#'  2. Use the highest groups maximum as the first bracket y position \item 3.
#'  Add successively a step increase for remaining bracket y positions. }
#'
#'  When the main plot is a boxplot, you need the option \code{fun = "max"}, to
#'  have the p-value bracket displayed at the maximum point of the group.
#'
#'  In some situations the main plot is a line plot or a barplot showing the
#'  \code{mean+/-error bars} of the groups, where error can be SE (standard
#'  error), SD (standard deviation) or CI (confidence interval). In this case,
#'  to correctly compute the bracket y position you need the option \code{fun =
#'  "mean_se"}, etc.
#'@param step.increase numeric vector with the increase in fraction of total
#'  height for every additional comparison to minimize overlap.
#'@param y.trans a function for transforming y axis scale. Value can be
#'  \code{log2}, \code{log10} and \code{sqrt}. Can be also any custom function
#'  that can take a numeric vector as input and returns a numeric vector,
#'  example: \code{y.trans = function(x){log2(x+1)}}
#'@param test an object of class \code{rstatix_test} as returned by
#'  \code{\link{t_test}()}, \code{\link{wilcox_test}()},
#'  \code{\link{sign_test}()}, \code{\link{tukey_hsd}()},
#'  \code{\link{dunn_test}()}.
#'@param x variable on x axis.
#'@param group group variable (legend variable).
#'@param dodge dodge width for grouped ggplot/test. Default is 0.8. Used only
#'  when \code{x} specified.
#'@param stack logical. If TRUE, computes y position for a stacked plot. Useful
#'  when dealing with stacked bar plots.
#'@param scales Should scales be fixed (\code{"fixed"}, the default), free
#'  (\code{"free"}), or free in one dimension (\code{"free_y"})?. For the y
#'  position, \code{"free"} or \code{"free_y"} compute the step increase per plot
#'  panel (otherwise a global step increase is used). For the x position, when
#'  \code{scales = "free"} the x positions (\code{xmin}/\code{xmax}) are computed
#'  \strong{per facet}, so that brackets align correctly when a faceted plot
#'  (\code{facet_*(scales = "free")}) shows a different set of x-axis levels in
#'  each panel (\code{"free_y"} keeps the global x positions).
#'@section Added columns: \code{add_y_position()} adds \code{y.position};
#'  \code{add_x_position()} adds \code{x}, \code{xmin}, \code{xmax}; and
#'  \code{add_xy_position()} adds all of these. A column named \code{groups} (the
#'  pair of compared groups, used internally to position dodged grouped
#'  comparisons) is also added. \strong{\code{groups} is a reserved internal
#'  column}: do not rely on it and avoid using \code{groups} as one of your own
#'  column names in the test object, since it is overwritten here. (It may be
#'  renamed to a dotted, less collision-prone name in a future major version.)
#' @examples
#' # Data preparation
#' #::::::::::::::::::::::::::::::::::::
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#' df$group <- factor(rep(c(1, 2), 30))
#' head(df)
#'
#' # Stat tests
#' #::::::::::::::::::::::::::::::::::::
#' stat.test <- df %>%
#'   t_test(len ~ dose)
#' stat.test
#'
#' # Add the test into box plots
#' #::::::::::::::::::::::::::::::::::::
#' stat.test <- stat.test %>%
#'   add_y_position()
#' \donttest{
#'  if(require("ggpubr")){
#'    ggboxplot(df, x = "dose", y = "len") +
#'      stat_pvalue_manual(stat.test, label = "p.adj.signif", tip.length = 0.01)
#'   }
#' }

#'@describeIn get_pvalue_position compute the p-value y positions
#'@export
get_y_position <- function(data, formula, fun = "max", ref.group = NULL, comparisons = NULL,
                           step.increase = 0.12, y.trans = NULL, stack = FALSE,
                           scales = c("fixed", "free", "free_y")){
  # Estimate step increase
  # 1. Get groups y scale
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  if(.is_empty(group))
    group <- NULL
  if(is_grouped_df(data))
    group <- c(group , dplyr::group_vars(data))
  yscale <- get_y_scale(data, outcome, group, fun, stack = stack)
  # 2. Step increase
  # If fixed scales, then a global step increase is computed,
  # otherwise step.increase is estimated by panel in get_y_position_core().
  scales <- match.arg(scales)
  if(scales == "fixed"){
    step.increase <- step.increase*(yscale$max - yscale$min)
  }
  get_y_position_core(
    data = data, formula = formula, fun = fun, ref.group = ref.group,
    comparisons = comparisons, step.increase = step.increase, y.trans = y.trans,
    stack = stack, scales = scales
    )
}


get_y_position_core <- function(data, formula, fun = "max", ref.group = NULL, comparisons = NULL,
                            step.increase = 0.12, y.trans = NULL, stack = FALSE, scales = "fixed"){
  if(is_grouped_df(data)){
    results <- data %>%
      doo(
        get_y_position_core, formula = formula, fun = fun,
        ref.group = ref.group, comparisons = comparisons,
        step.increase = step.increase, y.trans = y.trans,
        stack = stack, scales = scales
        )
    return(results)
  }

  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  # Possible comparisons between groups
  if(is.null(comparisons)){
    if(.is_empty(group)){
      comparisons <- list(c("1", "null model"))
    }
    else{
      comparisons <- data %>%
        get_comparisons(!!group, !!ref.group)
    }
  }
  ncomparisons <- length(comparisons)
  group1 <- comparisons %>% get_group(1)
  group2 <- comparisons %>% get_group(2)

  # Estimate y axis scale
  yscale <- get_y_scale(data, y = outcome, group = group, fun = fun, stack = stack)
  if(is.null(step.increase)) step.increase <- yscale$max/20
  else if(scales %in% c("free", "free_y")){
    step.increase <- step.increase*(yscale$max - yscale$min)
  }
  ystart <- yscale$max + step.increase
  # The seq() below spreads ncomparisons brackets over [ystart, yend], so the
  # gap between consecutive brackets is (yend - ystart)/(ncomparisons - 1).
  # Using (ncomparisons - 1) here makes that gap exactly step.increase (#201).
  yend <- ystart + (step.increase*(ncomparisons-1))

  if(is.null(ref.group)) ref.group <- ""
  if(ref.group %in% c("all", ".all.")){
    y.position <- yscale$y + step.increase
  }
  else{
    y.position <- seq(
      from = ystart, to = yend,
      length.out = ncomparisons
    )
  }
  if(!is.null(y.trans)) y.position <- y.trans(y.position)
  results <- tibble(group1, group2, y.position) %>%
    mutate(groups = combine_this(group1, group2))
  results
}



#' @describeIn get_pvalue_position add p-value y positions to an object of class \code{rstatix_test}
#' @export
add_y_position <- function(test, fun = "max", step.increase = 0.12,
                           data = NULL, formula = NULL,  ref.group = NULL, comparisons = NULL,
                           y.trans = NULL, stack = FALSE, scales = c("fixed", "free", "free_y"))
  {
  scales <- match.arg(scales)
  asserttat_group_columns_exists(test)
  .attributes <- get_test_attributes(test)
  args  <- get_test_arguments(test)
  test <- keep_only_tbl_df_classes(test)
  if(!is.null(args)){
    if(missing(data)) data <- args$data
    if(missing(formula)) formula <- args$formula
    if(missing(ref.group)) ref.group <- args$ref.group
    if(missing(comparisons)) comparisons <- args$comparisons
  }
  if(is.null(data) | is.null(formula)){
    stop("data and formula arguments should be specified.")
  }
  # Repeated-measures pairwise tests (e.g. friedman_conover_test) carry a
  # 'outcome ~ within | subject' formula. For bracket positioning only the
  # 'outcome ~ within' part is relevant, so drop the '| subject' block to let the
  # grouping (x) variable be parsed correctly. This is a no-op for the usual
  # 'outcome ~ group' formulas (which contain no '|'), so existing behavior is
  # unchanged.
  formula <- drop_formula_block_term(formula)
  # When no explicit comparisons were used, derive them from the test's own
  # group1/group2 rows so that y positions are computed for exactly the
  # comparisons present (e.g. after filtering the test) instead of for the full
  # comparison set, which would otherwise produce uneven bracket spacing (#197).
  # Restricted to the standard ungrouped pairwise case; grouped tests,
  # ref.group = "all" and one-sample tests keep their existing behavior.
  if(is.null(comparisons) && !is_grouped_df(data) && nrow(test) > 0 &&
     all(c("group1", "group2") %in% colnames(test))){
    is.special <- any(test$group1 %in% c("all", ".all.")) ||
      any(test$group2 %in% c("null model"))
    if(!is.special){
      comparisons <- purrr::map2(
        as.character(test$group1), as.character(test$group2), c
      )
      # name like get_comparisons() (V1, V2, ...) so the derived `groups`
      # list-column is identical to the unfiltered/default path
      names(comparisons) <- paste0("V", seq_along(comparisons))
    }
  }
  positions <- get_y_position(
    data = data, formula = formula, fun = fun,
    ref.group = ref.group, comparisons = comparisons,
    step.increase = step.increase, y.trans = y.trans,
    stack = stack, scales = scales
  )
  if(nrow(test) == nrow(positions)){
    test$y.position <- positions$y.position
    test$groups <- positions$groups
  }
  else{
    # this occurs when tests are grouped by two variables (for ex),
    # but y positions are grouped by one variable
    # merging positions and test data frame
    if("y.position" %in% colnames(test)){
      test <- test %>% select(-any_of("y.position"))
    }
    if("groups" %in% colnames(test)){
      test <- test %>% select(-any_of("groups"))
    }
    common.columns <- intersect(colnames(test), colnames(positions))
    test <- test %>% dplyr::left_join(positions, by = common.columns)
  }

  test %>%
    set_test_attributes(.attributes)
}

# Compute y scale depending on fun
get_y_scale <- function(data, y, group, fun = "max", stack = FALSE){
  if(!.is_empty(group)){
    desc.stat <- data %>%
      group_by(!!!syms(group)) %>%
      get_summary_stats(!!y, type = fun)
  }
  else {
    desc.stat <- data %>%
      get_summary_stats(!!y, type = fun)
  }
  # Add error bars positions if any
  fun.splitted <- unlist(strsplit(fun, "_", fixed = TRUE))
  .center <- fun.splitted[1]
  .error <- ifelse(length(fun.splitted) == 2, fun.splitted[2], 0)
  .center <- desc.stat %>% pull(tidyselect::all_of(.center))
  if(.error != 0) .error <- desc.stat %>% pull(tidyselect::all_of(.error))
  if(stack){
    .center <- rep(sum(.center), length(.center))
  }
  y <- .center + .error
  ymax <- max(y, na.rm = TRUE)
  ymin <- min(y, na.rm = TRUE)
  list(y = y, max = ymax, min = ymin)
}


# Return group1 and group2 values from possible pairs
get_group <- function(possible.pairs, index){
  possible.pairs %>%
    map( function(x){as.character(x[index])}) %>%
    unlist()
}
# Combine vectors: c(a, b) & c(c, d) --> list(c(a, c), c(b, d))
# ... two or more character vectors
combine_this <- function(...){
  params <- list(...)
  purrr::pmap(params, c)
}


#' @describeIn get_pvalue_position compute and add p-value x positions.
#' @export
add_x_position <- function(test, x = NULL, group = NULL, dodge = 0.8,
                           scales = c("fixed", "free", "free_y")){
  scales <- match.arg(scales)
  # Checking
  asserttat_group_columns_exists(test)
  .attributes <- get_test_attributes(test)
  if(any(test$group1 %in% c("all", ".all."))) {
    # case when ref.group = "all"
    test$group1 <- test$group2
  }
  groups <- c(as.character(test$group1), as.character(test$group2)) %>%
    unique() %>%
    setdiff(c("all", ".all."))   # case when ref.group = "all"

  is_rstatix_test <- inherits(test, "rstatix_test")
  is.null.model <- ("null model" %in% test$group2) & all(test$group1 %in% 1)
  is.grouped.by.legend <- test %>% is_stat_test_grouped_by(group)
  is.grouped.by.x <- test %>% is_stat_test_grouped_by(x)
  is.basic <- !is.grouped.by.x & !is.grouped.by.legend

  # Data preparation
  if(is_rstatix_test) {
    data <- attr(test, "args")$data
    if(is.basic & is.null(x))
      x <- get_formula_right_hand_side(.attributes$args$formula)
    else if(is.grouped.by.x & is.null(group))
      group <- get_formula_right_hand_side(.attributes$args$formula)
  }
  else if(is.basic){
    data <- data.frame(x = groups, stringsAsFactors = FALSE)
    x <- "x"
  }
  else{
    if(is.grouped.by.x)
      data <- expand.grid(x = unique(test[[x]]), group = groups)
    else if(is.grouped.by.legend)
      data <- expand.grid(x = groups, group = test[[group]])
    colnames(data) <- c(x, group)
  }
  if(is.null.model) {
    data$group <- rep(1, nrow(data))
    group <- "group"
  }

  # Add xmin and x max
  if(is.basic){
    x_coords <- as_numeric_group(data[[x]])
    xmin_id <- as.character(test$group1)
    xmax_id <- as.character(test$group2)
  }
  else{
    x_coords <- get_grouped_x_position(data, x = x, group = group, dodge = dodge)
    if(is.grouped.by.legend){
      # Add x position to stat test when the test is grouped by the legend variable
      # Case when you group by legend and pairwise compare between x-axis groups
      xmin_id <- paste(test$group1, test[[group]], sep = "_")
      xmax_id <- paste(test$group2, test[[group]], sep = "_")
    }
    else if(is.grouped.by.x){
      # Add x position to stat test when the test is grouped by the x variable
      # Case when you pairwise compare legend groups at each x-axis position,
      # so the data is grouped by x position
      xmin_id <- paste(test[[x]], test$group1, sep = "_")
      xmax_id <- paste(test[[x]], test$group2, sep = "_")
      test$x <- unname(as_numeric_group(test[[x]]))
    }
  }
  test$xmin <- unname(x_coords[xmin_id])
  test$xmax <- unname(x_coords[xmax_id])
  if(is.null.model) test$xmax <- test$xmin
  # Free x scales in facets (#203): for a faceted test (grouped by one or more
  # variables) plotted with facet_*(scales = "free"), each facet shows only its
  # own x levels at positions 1..m, but x_coords above are global. Remap
  # xmin/xmax to each facet's positions so the brackets line up within the panel.
  # The panel's x levels are taken from the facet's actual DATA (in factor order),
  # not from the comparison endpoints, so a level that is plotted but not part of
  # any (retained) comparison still occupies its position. Only the basic
  # (non-dodged) case is affected; scales = "fixed" (default) / "free_y" leave the
  # global positions unchanged.
  facet.vars <- if(is.basic && scales == "free") .get_facet_vars(test) else character(0)
  facet.data <- attr(test, "args")$data
  if(length(facet.vars) > 0 && !is.null(facet.data) && !is.null(x) &&
     all(facet.vars %in% colnames(facet.data)) && x %in% colnames(facet.data)){
    x.levels <- levels(as.factor(facet.data[[x]]))   # global factor order
    test.fid <- do.call(paste, c(as.list(test[facet.vars]), sep = "\r"))
    data.fid <- do.call(paste, c(as.list(facet.data[facet.vars]), sep = "\r"))
    for(f in unique(test.fid)){
      present <- x.levels[x.levels %in% as.character(facet.data[[x]][data.fid == f])]
      rows <- test.fid == f
      test$xmin[rows] <- match(as.character(test$group1[rows]), present)
      test$xmax[rows] <- match(as.character(test$group2[rows]), present)
    }
  }
  test %>% set_test_attributes(.attributes)
}

# Leading grouping/facet variables of a (grouped) test result: the columns that
# appear before the first comparison/outcome column (.y., term or group1).
.get_facet_vars <- function(test){
  stop.cols <- c(".y.", "term", "group1")
  idx <- match(stop.cols, colnames(test))
  idx <- idx[!is.na(idx)]
  if(length(idx) == 0) return(character(0))
  first <- min(idx)
  if(first > 1) colnames(test)[seq_len(first - 1)] else character(0)
}


# Compute grouped x positions or coordinates
# data is a dataframe containing the x and the group columns
get_grouped_x_position<- function(data, x, group, dodge = 0.8){
  data <- data.frame(x = data[[x]], group = data[[group]]) %>%
    dplyr::distinct(.data$x, .data$group) %>%
    dplyr::arrange(.data$x, .data$group)
  data$x.position <- as_numeric_group(data$x)
  # Add group.ranks and ngroups at x position
  data <- data %>%
    rstatix::df_nest_by(vars = "x") %>%
    mutate(
      data = map(.data$data, function(data){data$group.ranks = 1:nrow(data); data}),
      n = unlist(map(.data$data, nrow))
    ) %>%
    tidyr::unnest(cols = "data")
  # Compute x coords
  d <- data
  x_coords <-  (((dodge - dodge*d$n) / (2*d$n)) + ((d$group.ranks - 1) * (dodge / d$n))) + d$x.position
  names(x_coords) <- paste(d$x, d$group, sep = "_")
  x_coords
}

# Check if a stat test is grouped by a given variable
is_stat_test_grouped_by <- function(test, x = NULL){
  answer <- FALSE
  if(!is.null(x)){
    if(x %in% colnames(test)){
      answer <- TRUE
    }
  }
  answer
}

# Return a numeric named vector
# c("a", "b", "a") -> c(a = 1, b = 2, a = 1)
as_numeric_group <- function(x){
  grp <- x %>% as.factor() %>% as.numeric()
  names(grp) <- x
  grp
}




#' @describeIn get_pvalue_position compute and add both x and y positions.
#' @export
add_xy_position <- function(test, x = NULL,  group = NULL, dodge = 0.8, stack = FALSE,
                            fun = "max", step.increase = 0.12,
                            scales = c("fixed", "free", "free_y"), ...){
  if(missing(dodge)){
    if(stack) dodge <- 0
  }
  test %>%
    add_y_position(
      fun = fun, step.increase = step.increase,
      stack = stack, scales = scales, ...
      ) %>%
    add_x_position(x = x, group = group, dodge = dodge, scales = scales)
}


# Helper functions
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
asserttat_group_columns_exists <- function(data){
  groups.exist <- all(c("group1", "group2") %in% colnames(data))
  if(!groups.exist){
    stop("data should contain group1 and group2 columns")
  }
}

.is_grouped_test <- function(test, x = NULL){
  answer <- FALSE
  if(!is.null(x)){
    if(x %in% colnames(test)){
      answer <- TRUE
    }
  }
  answer
}

.contains_selected_comparisons <- function(test){
  answer <- FALSE
  if(is_rstatix_test(test)){
    comparisons <- attr(test, "args")$comparisons
    answer <- !is.null(comparisons)
  }
  answer
}


# To be removed
add_x_position0 <- function(test, x = NULL, dodge = 0.8){

  asserttat_group_columns_exists(test)
  .attributes <- get_test_attributes(test)
  group1 <- set_diff(test$group1, c("all", ".all."), keep.dup = TRUE)
  group2 <- set_diff(test$group2, c("all", ".all."), keep.dup = TRUE)
  groups <- unique(c(group1, group2))
  if(.contains_selected_comparisons(test)){
    # get all data groups
    group.var <- get_formula_right_hand_side(.attributes$args$formula)
    groups <- .attributes$args$data %>%
      convert_as_factor(vars = group.var) %>%
      pull(tidyselect::all_of(group.var)) %>%
      levels()
  }
  group1.coords <- group1.ranks <-  match(group1, groups)
  group2.coords <- group2.ranks <- match(group2, groups)
  n <- length(groups)

  if(.is_grouped_test(test, x)){
    test <- test %>%
      .as_factor(x) %>%
      mutate(x = as.numeric(!!sym(x))) # x levels order
    xpos <- test$x
    group1.coords <- (((dodge - dodge*n) / (2*n)) + ((group1.ranks - 1) * (dodge / n))) + xpos
    group2.coords <- (((dodge - dodge*n) / (2*n)) + ((group2.ranks - 1) * (dodge / n))) + xpos
  }

  if(.is_empty(group1)) {
    # case when ref.group = "all"
    group1.coords <- group2.coords
  }
  test %>%
    mutate(xmin = group1.coords, xmax = group2.coords) %>%
    set_test_attributes(.attributes)
}

context("test-get_pvalue_position")

test_that("add_y_position spaces brackets by exactly step.increase (#201)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  ys <- rstatix:::get_y_scale(df, "len", "dose", "max")
  step <- 0.12 * (ys$max - ys$min)   # fixed scales: step.increase scaled to data units
  st <- df %>% t_test(len ~ dose) %>% add_y_position(step.increase = 0.12)
  expect_equal(nrow(st), 3L)
  # consecutive brackets are step.increase apart (was step*n/(n-1) before the fix)
  expect_equal(diff(st$y.position), rep(step, 2), tolerance = 1e-6)
  # first bracket sits one step above the data max
  expect_equal(st$y.position[1], ys$max + step, tolerance = 1e-6)
})

test_that("add_y_position is unchanged for a single comparison (#201 no-regression)", {
  ys <- rstatix:::get_y_scale(ToothGrowth, "len", "supp", "max")
  step <- 0.12 * (ys$max - ys$min)
  st <- ToothGrowth %>% t_test(len ~ supp) %>% add_y_position(step.increase = 0.12)
  expect_equal(nrow(st), 1L)
  # seq(length.out = 1) returns `from`, so this value is identical old vs new
  expect_equal(st$y.position, ys$max + step, tolerance = 1e-6)
})

test_that("add_y_position ref.group = 'all' positions are unaffected (#201 no-regression)", {
  st <- ToothGrowth %>%
    t_test(len ~ dose, ref.group = "all") %>%
    add_y_position(step.increase = 0.12)
  expect_equal(nrow(st), 3L)
  expect_false(any(is.na(st$y.position)))
})

context("test-add_x_position")

# Data preparation
data("ToothGrowth")
df <- ToothGrowth
df$dose <- as.factor(df$dose)

test_that("add_x_position works for any data frame with group1 and group2 cols", {
  stat.test <- data.frame(
    stringsAsFactors = FALSE,
    group1 = c("0.5", "0.5", "1"),
    group2 = c("1", "2", "2"),
    p = c(1.27e-07, 4.4e-14, 1.91e-05)
  ) %>%
    add_x_position()
  expect_equal(stat.test$xmin, c(1, 1, 2))
  expect_equal(stat.test$xmax, c(2, 3, 3))
})


test_that("add_x_position works for rstatix in a basic ggplot setting", {
  stat.test <- df %>%
    t_test(len ~ supp) %>%
    add_x_position()
  expect_equal(stat.test$xmin, 1)
  expect_equal(stat.test$xmax, 2)
})

test_that("add_x_position works for rstatix in a basic ggplot facet setting", {
  stat.test <- df %>%
    group_by(dose) %>%
    t_test(len ~ supp) %>%
    add_x_position(x = "supp")
  expect_equal(stat.test$xmin, c(1, 1, 1))
  expect_equal(stat.test$xmax, c(2, 2, 2))
})

test_that("add_x_position works for comparison against reference groups", {
  stat.test <- df %>%
    t_test(len ~ dose, ref.group = "0.5") %>%
    add_x_position(x = "dose")
  expect_equal(stat.test$xmin, c(1, 1))
  expect_equal(stat.test$xmax, c(2, 3))
})

test_that("add_x_position works for comparison against all (basemean)", {
  stat.test <- df %>%
    t_test(len ~ dose, ref.group = "all") %>%
    add_x_position(x = "dose")
  expect_equal(stat.test$xmin, c(1, 2, 3))
  expect_equal(stat.test$xmax, c(1,2, 3))
})

test_that("add_x_position works for comparison against null (one-sample test)", {
  stat.test <- df %>%
    group_by(dose) %>%
    t_test(len ~ 1) %>%
    add_x_position(x = "dose")
  expect_equal(stat.test$x, c(1, 2, 3))
})


test_that("add_x_position works for specified comparisons of interest", {
  stat.test <- df %>%
    t_test(len ~ dose, comparisons = list(c("0.5", "1"), c("0.5", "2"))) %>%
    add_x_position(x = "dose")
  expect_equal(stat.test$xmin, c(1, 1))
  expect_equal(stat.test$xmax, c(2,3))
})

test_that("add_x_position works for grouped plots: grouping by x-var and performing test between legend groups", {
  stat.test <- df %>%
    group_by(dose) %>%
    t_test(len ~ supp) %>%
    add_x_position(x = "dose")
  expect_equal(stat.test$x, c(1, 2, 3))
  expect_equal(stat.test$xmin, c(0.8, 1.8, 2.8))
  expect_equal(stat.test$xmax, c(1.2, 2.2, 3.2))
})



test_that("add_x_position works for grouped plots: grouping by legend-var and performing test between x-group", {
  stat.test <- df %>%
    group_by(supp) %>%
    t_test(len ~ dose) %>%
    add_x_position(x = "dose")
  expect_equal(stat.test$xmin, c(1, 1, 2, 1, 1, 2))
  expect_equal(stat.test$xmax, c(2, 3, 3, 2, 3, 3))
})


test_that("Grouped pairwise tests: grouping by x-var and performing test between legend groups", {
  stat.test <- df %>%
    group_by(supp) %>%
    t_test(len ~ dose) %>%
    add_x_position(x = "supp", dodge = 0.8)
  expect_equal(stat.test$x, c(1, 1, 1, 2, 2, 2))
  expect_equal(round(stat.test$xmin, 2), c(0.73, 0.73, 1, 1.73, 1.73, 2))
  expect_equal(round(stat.test$xmax, 2), c(1, 1.27, 1.27, 2, 2.27, 2.27))
})

test_that("Grouped pairwise tests: grouping by x-var and performing test between legend groups using ref.group", {
  stat.test <- df %>%
    group_by(supp) %>%
    t_test(len ~ dose, ref.group = "0.5") %>%
    add_x_position(x = "supp", dodge = 0.8)
  expect_equal(stat.test$x, c(1, 1, 2, 2))
  expect_equal(round(stat.test$xmin, 2), c(0.73, 0.73, 1.73, 1.73))
  expect_equal(round(stat.test$xmax, 2), c(1, 1.27, 2, 2.27))
})


test_that("Grouped plots: test that add_x_position works with different number of groups at each x pos.", {
  # https://github.com/kassambara/ggpubr/issues/326
  demo_data <- data.frame(
    stringsAsFactors = FALSE,
    Study = c("A","A","A","A","A","A",
              "A","A","A","A","B","B","B","B","B","B","B","B",
              "B","B","C","C","C","C","C","C","C","C","C",
              "C","C","C","C","C","C","D","D","D","D","D",
              "D","D","D","D","D","D","D","D","D","D"),
    Studytype = c("X","X","X","X","X","X",
                  "X","X","X","X","X","X","X","X","X","X","X","X",
                  "X","X","Y","Y","Y","Y","Y","Y","Y","Y","Y",
                  "Y","Y","Y","Y","Y","Y","Y","Y","Y","Y","Y",
                  "Y","Y","Y","Y","Y","Y","Y","Y","Y","Y"),
    Values = c(4469L,4797L,5101L,5397L,
               4542L,2780L,4326L,3396L,3657L,3199L,9221L,10176L,
               9277L,10500L,9707L,7406L,7756L,7601L,7586L,7353L,
               1811L,1485L,3003L,1629L,2495L,4207L,4265L,3629L,
               4157L,3495L,2075L,2112L,2973L,3086L,2943L,5664L,6690L,
               3538L,5741L,7880L,5848L,6390L,6569L,6114L,6520L,
               7389L,6843L,7611L,6621L,7340L),
    Group = as.factor(c("CTR",
                        "CTR","CTR","CTR","CTR","Dis1","Dis1","Dis1",
                        "Dis1","Dis1","CTR","CTR","CTR","CTR",
                        "CTR","Dis1","Dis1","Dis1","Dis1","Dis1",
                        "CTR","CTR","CTR","CTR","CTR","Dis2","Dis2",
                        "Dis2","Dis2","Dis2","Dis3","Dis3",
                        "Dis3","Dis3","Dis3","CTR","CTR","CTR","CTR",
                        "CTR","Dis2","Dis2","Dis2","Dis2","Dis2",
                        "Dis3","Dis3","Dis3","Dis3","Dis3"))
  )

  stat.test <- demo_data %>%
    group_by(Study) %>%
    wilcox_test(Values ~ Group, ref.group = "CTR") %>%
    add_significance("p")
  stat.test <- stat.test %>%
    add_xy_position(x = "Study", dodge = 0.8)

  #bxp <- ggpubr::ggboxplot(demo_data, x = "Study", y = "Values", fill = "Group") +
  #ggpubr::stat_pvalue_manual(stat.test, label = "p")

  stat.test$x <- round(stat.test$x, 2)
  stat.test$xmin <- round(stat.test$xmin, 2)
  stat.test$xmax <- round(stat.test$xmax, 2)
  expect_equal(stat.test$x, c(1, 2, 3, 3, 4, 4))
  expect_equal(stat.test$xmin, c(0.8, 1.8, 2.73, 2.73, 3.73, 3.73))
  expect_equal(stat.test$xmax, c(1.2, 2.2, 3, 3.27, 4, 4.27))
})

test_that("add_xy_position keeps even bracket spacing after filtering the test (#197)", {
  set.seed(1)
  d <- tibble::tibble(
    A = rnorm(10, 1), B = rnorm(10, 1), c = rnorm(10, 3),
    D = rnorm(10, 5), E = rnorm(10, 1)
  ) %>%
    tidyr::pivot_longer(dplyr::everything(), names_to = "group", values_to = "value")
  tk <- d %>% tukey_hsd(value ~ group) %>% dplyr::filter(p.adj < 0.05)
  filt <- tk %>% add_xy_position()
  # consecutive brackets must be evenly spaced (was uneven before the fix)
  expect_equal(length(unique(round(diff(filt$y.position), 6))), 1L)
  # identical to explicitly setting the comparisons (the reported workaround)
  wk <- tk %>% add_xy_position(comparisons = purrr::map2(tk$group1, tk$group2, c))
  expect_equal(filt$y.position, wk$y.position)
})

test_that("add_xy_position y.position is unchanged for unfiltered/edge cases (#197 no-regression)", {
  # unfiltered ungrouped pairwise
  expect_equal(round((df %>% t_test(len ~ dose) %>% add_xy_position())$y.position, 3),
               c(35.388, 36.876, 38.364))
  # ref.group = "all"
  expect_equal(round((df %>% t_test(len ~ dose, ref.group = "all") %>% add_xy_position())$y.position, 3),
               c(22.988, 28.788, 35.388))
  # grouped (fix is skipped; existing behavior preserved)
  expect_equal(nrow(df %>% group_by(supp) %>% t_test(len ~ dose) %>% add_xy_position()), 6L)
  # empty (zero-row) filtered input returns gracefully, no error (#197 edge)
  empty <- df %>% t_test(len ~ dose) %>% dplyr::filter(p < 0) %>% add_xy_position()
  expect_equal(nrow(empty), 0L)
})

#' @include utilities.R t_test.R
NULL
#'Dunn's Test of Multiple Comparisons
#'
#'@description Performs Dunn's test for pairwise multiple comparisons of the
#'  ranked data. The mean rank of the different groups is compared. Used for
#'  post-hoc test following Kruskal-Wallis test.
#'@inheritParams t_test
#'@return return a data frame with some of the following columns: \itemize{
#'  \item \code{.y.}: the y (outcome) variable used in the test. \item
#'  \code{group1,group2}: the compared groups in the pairwise tests. \item
#'  \code{n1,n2}: Sample counts. \item \code{estimate}: mean ranks difference.
#'  \item \code{statistic}: Test statistic (z-value) used to compute the
#'  p-value. \item \code{p}: p-value. \item \code{p.adj}: the adjusted p-value.
#'  \item \code{method}: the statistical test used to compare groups. \item
#'  \code{p.signif, p.adj.signif}: the significance level of p-values and
#'  adjusted p-values, respectively. }
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#'@details DunnTest performs the post hoc pairwise multiple comparisons
#'  procedure appropriate to follow up a Kruskal-Wallis test, which is a
#'  non-parametric analog of the one-way ANOVA. The Wilcoxon rank sum test,
#'  itself a non-parametric analog of the unpaired t-test, is possibly
#'  intuitive, but inappropriate as a post hoc pairwise test, because (1) it
#'  fails to retain the dependent ranking that produced the Kruskal-Wallis test
#'  statistic, and (2) it does not incorporate the pooled variance estimate
#'  implied by the null hypothesis of the Kruskal-Wallis test.
#'
#'@references Dunn, O. J. (1964) Multiple comparisons using rank sums
#'  Technometrics, 6(3):241-252.
#' @examples
#' # Simple test
#' ToothGrowth %>% dunn_test(len ~ dose)
#'
#' # Grouped data
#' ToothGrowth %>%
#'   group_by(supp) %>%
#'   dunn_test(len ~ dose)
#'@export
dunn_test <- function(data, formula, p.adjust.method = "holm", detailed = FALSE){
  args <- as.list(environment()) %>%
    .add_item(method = "dunn_test")
  if(is_grouped_df(data)){
    results <- data %>%
      doo(.dunn_test, formula, p.adjust.method )
  }
  else{
    results <- .dunn_test(data, formula, p.adjust.method)
  }

  if(!detailed){
    results <- results %>% select(-.data$method, -.data$estimate)
  }
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "dunn_test"))
}


.dunn_test <- function(data, formula, p.adjust.method = "holm"){
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  number.of.groups <- guess_number_of_groups(data, group)
  if(number.of.groups == 1){
    stop("all observations are in the same group")
  }

  data <- data %>%
    select(!!!syms(c(outcome, group))) %>%
    get_complete_cases() %>%
    .as_factor(group)

  x <- data %>% pull(!!outcome)
  g <- data %>% pull(!!group)
  group.size <- data %>% get_group_size(group)
  if (!all(is.finite(g)))
    stop("all group levels must be finite")

  x.rank <- rank(x)
  mean.ranks <- tapply(x.rank, g, mean, na.rm=TRUE)
  grp.sizes <- tapply(x, g, length)
  n <- length(x)
  C <- get_ties(x.rank, n)

  compare.meanrank <- function(i, j){
    mean.ranks[i] - mean.ranks[j]
  }
  compare.stats <- function(i,j) {
    dif <- mean.ranks[i] - mean.ranks[j]
    A <- n * (n+1) / 12
    B <- (1 / grp.sizes[i] + 1 / grp.sizes[j])
    zval <- dif / sqrt((A - C) * B)
    zval
  }
  compare.levels <- function(i, j) {
    dif <- abs(mean.ranks[i] - mean.ranks[j])
    A <- n * (n+1) / 12
    B <- (1 / grp.sizes[i] + 1 / grp.sizes[j])
    zval <- dif / sqrt((A - C) * B)
    pval <- 2 * stats::pnorm(abs(zval), lower.tail = FALSE)
    pval
  }
  ESTIMATE <- stats::pairwise.table(
    compare.meanrank, levels(g),
    p.adjust.method = "none"
  ) %>% tidy_squared_matrix("diff")

  PSTAT <- stats::pairwise.table(
    compare.stats, levels(g),
    p.adjust.method = "none"
  ) %>% tidy_squared_matrix("statistic")

  PVAL <- stats::pairwise.table(
    compare.levels, levels(g),
    p.adjust.method = "none"
    ) %>%
    tidy_squared_matrix("p") %>%
    mutate(method = "Dunn Test", .y. = outcome) %>%
    adjust_pvalue(method = p.adjust.method) %>%
    add_significance("p.adj") %>%
    add_column(statistic = PSTAT$statistic, .before = "p") %>%
    add_column(estimate = ESTIMATE$diff, .before = "group1") %>%
    select(.data$.y., .data$group1, .data$group2, .data$estimate, everything())

  n1 <- group.size[PVAL$group1]
  n2 <- group.size[PVAL$group2]
  PVAL %>% add_column(n1 = n1, n2 = n2, .after = "group2")
}



get_ties <- function(x, n) {
  x.sorted <- sort(x)
  pos <- 1
  tiesum <- 0
  while (pos <= n) {
    val <- x.sorted[pos]
    nt <- length(x.sorted[x.sorted == val])
    pos <- pos + nt
    if (nt > 1){
      tiesum <- tiesum + nt^3  - nt
    }
  }
  tiesum / (12 * (n - 1))
}

context("test-dunn_test")

test_that("dunn_test ref.group keeps only comparisons against the reference (#101)", {
  res <- ToothGrowth %>% dunn_test(len ~ dose, ref.group = "0.5")
  expect_equal(nrow(res), 2L)                       # k - 1 = 2 comparisons
  expect_true(all(res$group1 == "0.5"))            # reference is group1
  expect_setequal(res$group2, c("1", "2"))
})

test_that("dunn_test ref.group leaves raw p and statistic unchanged (#101)", {
  full <- ToothGrowth %>% dunn_test(len ~ dose)
  ref  <- ToothGrowth %>% dunn_test(len ~ dose, ref.group = "0.5")
  # in the full result the 0.5 pairs are already oriented as group1 = "0.5"
  full_ref <- full[full$group1 == "0.5", ]
  full_ref <- full_ref[order(full_ref$group2), ]
  ref <- ref[order(ref$group2), ]
  expect_equal(ref$p, full_ref$p)                   # raw p identical
  expect_equal(ref$statistic, full_ref$statistic)  # z identical (same orientation here)
})

test_that("dunn_test ref.group adjusts over k-1, not all pairs (#101)", {
  ref <- ToothGrowth %>% dunn_test(len ~ dose, ref.group = "0.5")
  # adjustment must be computed over the 2 retained comparisons only
  expect_equal(ref$p.adj, p.adjust(ref$p, method = "holm"))
  # and that differs from adjusting over all 3 pairwise comparisons (over-correction)
  full <- ToothGrowth %>% dunn_test(len ~ dose)
  posthoc <- full[full$group1 == "0.5", ]
  expect_false(isTRUE(all.equal(sort(ref$p.adj), sort(posthoc$p.adj))))
})

test_that("dunn_test ref.group re-orients reference as group1 and flips sign (#101)", {
  full <- ToothGrowth %>% dunn_test(len ~ dose, detailed = TRUE)
  ref2 <- ToothGrowth %>% dunn_test(len ~ dose, ref.group = "2", detailed = TRUE)
  expect_true(all(ref2$group1 == "2"))             # reference oriented as group1
  f <- full[full$group1 == "0.5" & full$group2 == "2", ]
  r <- ref2[ref2$group1 == "2" & ref2$group2 == "0.5", ]
  expect_equal(r$statistic, -f$statistic)          # z is direction-dependent -> negated
  expect_equal(r$estimate,  -f$estimate)           # mean-rank diff -> negated
  expect_equal(r$estimate1, f$estimate2)           # mean ranks swap
  expect_equal(r$estimate2, f$estimate1)
  expect_equal(r$p, f$p)                            # two-sided raw p unchanged
})

test_that("dunn_test invalid ref.group gives an informative error (#101)", {
  expect_error(
    ToothGrowth %>% dunn_test(len ~ dose, ref.group = "99"),
    "not a level of the grouping variable"
  )
})

test_that("dunn_test without ref.group is unchanged (#101 no-regression)", {
  res <- ToothGrowth %>% dunn_test(len ~ dose)
  expect_equal(nrow(res), 3L)                       # all k(k-1)/2 = 3 comparisons
  expect_equal(
    colnames(res),
    c(".y.", "group1", "group2", "n1", "n2", "statistic", "p", "p.adj", "p.adj.signif")
  )
  # p.adj computed over all 3 comparisons
  expect_equal(res$p.adj, p.adjust(res$p, method = "holm"))
})

#' @include utilities.R utilities_two_sample_test.R
#' @importFrom stats wilcox.test
NULL
#'Wilcoxon Tests
#'
#'
#'@description Provides a pipe-friendly framework to performs one and two sample
#'  Wilcoxon tests. Read more:
#'  \href{https://www.datanovia.com/en/lessons/wilcoxon-test-in-r/}{Wilcoxon in
#'  R}.
#'@inheritParams stats::wilcox.test
#'@param data a data.frame containing the variables in the formula.
#'@param formula a formula of the form \code{x ~ group} where \code{x} is a
#'  numeric variable giving the data values and \code{group} is a factor with
#'  one or multiple levels giving the corresponding groups. For example,
#'  \code{formula = TP53 ~ cancer_group}.
#'@param paired a logical indicating whether you want a paired test.
#'@param ref.group a character string specifying the reference group. If
#'  specified, for a given grouping variable, each of the group levels will be
#'  compared to the reference group (i.e. control group).
#'
#'  If \code{ref.group = "all"}, pairwise two sample tests are performed for
#'  comparing each grouping variable levels against all (i.e. basemean).
#'@param mu a number specifying an optional parameter used to form the null
#'  hypothesis.
#'@param comparisons A list of length-2 vectors specifying the groups of
#'  interest to be compared. For example to compare groups "A" vs "B" and "B" vs
#'  "C", the argument is as follow: \code{comparisons = list(c("A", "B"), c("B",
#'  "C"))}
#'@param p.adjust.method method to adjust p values for multiple comparisons.
#'  Used when pairwise comparisons are performed. Allowed values include "holm",
#'  "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
#'  want to adjust the p value (not recommended), use p.adjust.method = "none".
#'
#'@param detailed logical value. Default is FALSE. If TRUE, a detailed result is
#'  shown.
#'@param ... other arguments to be passed to the function
#'  \code{\link[stats]{wilcox.test}}.
#'
#'@details - \code{pairwise_wilcox_test()} applies the standard two sample
#'  Wilcoxon test to all possible pairs of groups. This method calls the
#'  \code{\link[stats]{wilcox.test}()}, so extra arguments are accepted.
#'
#'
#'  - If a list of comparisons is specified, the result of the pairwise tests is
#'  filtered to keep only the comparisons of interest.The p-value is adjusted
#'  after filtering.
#'
#'  - For a grouped data, if pairwise test is performed, then the p-values are
#'  adjusted for each group level independently.
#'
#'
#'  - a nonparametric confidence interval and an estimator for the pseudomedian
#'  (one-sample case) or for the difference of the location parameters
#'  \code{x-y} is computed, where x and y are the compared samples or groups.
#'  The column \code{estimate} and the confidence intervals are displayed in the
#'  test result when the option \code{detailed = TRUE} is specified in the
#'  \code{wilcox_test()} and \code{pairwise_wilcox_test()} functions. Read more
#'  about the calculation of the estimate in the details section of the R base
#'  function \code{wilcox.test()} documentation by typing \code{?wilcox.test} in
#'  the R console.
#'
#'@return return a data frame with some of the following columns: \itemize{
#'  \item \code{.y.}: the y variable used in the test. \item
#'  \code{group1,group2}: the compared groups in the pairwise tests. \item
#'  \code{n,n1,n2}: Sample counts. \item \code{statistic}: Test statistic used
#'  to compute the p-value. \item \code{p}: p-value. \item \code{p.adj}: the
#'  adjusted p-value. \item \code{method}: the statistical test used to compare
#'  groups. \item \code{p.signif, p.adj.signif}: the significance level of
#'  p-values and adjusted p-values, respectively. \item \code{estimate}: an
#'  estimate of the location parameter (Only present if argument \code{detailed
#'  = TRUE}). This corresponds to the pseudomedian (for one-sample case) or to
#'  the difference of the location parameter (for two-samples case). \itemize{
#'  \item The pseudomedian of a distribution \code{F} is the median of the
#'  distribution of \code{(u+v)/2}, where \code{u} and {v} are independent, each
#'  with distribution \code{F}. If \code{F} is symmetric, then the pseudomedian
#'  and median coincide. \item Note that in the two-sample case the estimator
#'  for the difference in location parameters does not estimate the difference
#'  in medians (a common misconception) but rather the median of the difference
#'  between a sample from x and a sample from y. } \item \code{conf.low,
#'  conf.high}: a confidence interval for the location parameter. (Only present
#'  if argument conf.int = TRUE.) }
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#' @examples
#' # Load data
#' #:::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # One-sample test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% wilcox_test(len ~ 1, mu = 0)
#'
#'
#' # Two-samples unpaired test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% wilcox_test(len ~ supp)
#'
#' # Two-samples paired test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% wilcox_test (len ~ supp, paired = TRUE)
#'
#' # Compare supp levels after grouping the data by "dose"
#' #::::::::::::::::::::::::::::::::::::::::
#' df %>%
#'   group_by(dose) %>%
#'   wilcox_test(data =., len ~ supp) %>%
#'   adjust_pvalue(method = "bonferroni") %>%
#'   add_significance("p.adj")
#'
#' # pairwise comparisons
#' #::::::::::::::::::::::::::::::::::::::::
#' # As dose contains more than two levels ==>
#' # pairwise test is automatically performed.
#' df %>% wilcox_test(len ~ dose)
#'
#' # Comparison against reference group
#' #::::::::::::::::::::::::::::::::::::::::
#' # each level is compared to the ref group
#' df %>% wilcox_test(len ~ dose, ref.group = "0.5")
#'
#' # Comparison against all
#' #::::::::::::::::::::::::::::::::::::::::
#' df %>% wilcox_test(len ~ dose, ref.group = "all")
#'
#'@describeIn wilcox_test Wilcoxon test
#'@export
wilcox_test <- function(
  data, formula, comparisons = NULL, ref.group = NULL,
  p.adjust.method = "holm",
  paired = FALSE, exact = NULL, alternative = "two.sided",
  mu = 0, conf.level = 0.95, detailed = FALSE
)
{
  env <- as.list(environment())
  args <- env %>%
    add_item(method = "wilcox_test")
  params <- env %>%
    remove_null_items() %>%
    add_item(conf.int = TRUE, method = "wilcox.test")

  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  number.of.groups <- guess_number_of_groups(data, group)
  if(number.of.groups > 2 & !is.null(ref.group)){
    if(ref.group %in% c("all", ".all.")){
      params$data <- create_data_with_all_ref_group(data, outcome, group)
      params$ref.group <- "all"
    }
  }
  test.func <- two_sample_test
  if(number.of.groups > 2) test.func <- pairwise_two_sample_test
  do.call(test.func, params) %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "wilcox_test"))
}



#'@describeIn wilcox_test performs pairwise two sample Wilcoxon test.
#'@export
pairwise_wilcox_test <- function(
  data, formula, comparisons = NULL, ref.group = NULL,
  p.adjust.method = "holm", detailed = FALSE, ...)
  {
  args <- as.list(environment()) %>%
    .add_item(method = "wilcox_test")

  res <- pairwise_two_sample_test(
    data, formula, method = "wilcox.test",
    comparisons = comparisons, ref.group = ref.group,
    p.adjust.method = p.adjust.method, detailed = detailed,
    conf.int = TRUE, ...
  )
  res %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "wilcox_test"))
}

context("test-wilcox-test")

test_that("Checking one-sample test", {
  data("ToothGrowth")
  res <- ToothGrowth %>% wilcox_test(len ~ 1, mu = 0)
  expect_equal(res$group1, "1")
  expect_equal(res$group2, "null model")
  expect_equal(res$n, 60)
  expect_equal(as.numeric(res$statistic), 1830)
  expect_equal(signif(res$p, 3), 1.66e-11)
})


test_that("Checking two-sample unpaired test", {
  data("ToothGrowth")
  res <- ToothGrowth %>% wilcox_test(len ~ supp)
  expect_equal(res$group1, "OJ")
  expect_equal(res$group2, "VC")
  expect_equal(res$n1, 30)
  expect_equal(res$n2, 30)
  expect_equal(as.numeric(res$statistic), 575.5)
  # Accept either 0.0645 (legacy) or 0.0637 (R-devel with exact conditional inference)
  expect_true(signif(res$p, 3) %in% c(0.0645, 0.0637),
              info = paste("Observed p =", signif(res$p, 3)))
})


test_that("Checking two-sample paired test", {
  data("ToothGrowth")
  res <- ToothGrowth %>% wilcox_test(len ~ supp, paired = TRUE)
  expect_equal(res$group1, "OJ")
  expect_equal(res$group2, "VC")
  expect_equal(res$n1, 30)
  expect_equal(res$n2, 30)
  # Compare against base R computed the same way so the test tracks the active
  # R version instead of chasing hard-coded values: wilcox.test's paired V
  # statistic and p-value shifted on R-devel (350/0.00431 on R <= 4.5,
  # 369/0.00381 on devel).
  ref <- suppressWarnings(stats::wilcox.test(
    ToothGrowth$len[ToothGrowth$supp == "OJ"],
    ToothGrowth$len[ToothGrowth$supp == "VC"],
    paired = TRUE
  ))
  expect_equal(as.numeric(res$statistic), as.numeric(ref$statistic))
  expect_equal(res$p, ref$p.value, tolerance = 1e-3)
})

test_that("Checking pairwise comparisons", {
  data("ToothGrowth")
  res <- ToothGrowth %>% wilcox_test(len ~ dose)
  expect_equal(res$group1, c("0.5", "0.5", "1"))
  expect_equal(res$group2, c("1", "2", "2"))
  expect_equal(res$n1, c(20, 20, 20))
  expect_equal(res$n2, c(20, 20, 20))
  expect_equal(as.numeric(res$statistic), c(33.5, 1.5, 61.0))
  # Accept either legacy or R-devel p-values
  legacy_p <- c(7.02e-6, 8.41e-08, 1.77e-04)
  rdevel_p <- c(7.74e-07, 4.35e-11, 7.57e-05)
  observed_p <- signif(res$p, 3)
  expect_true(all(observed_p %in% c(legacy_p, rdevel_p)) || all(abs(observed_p - legacy_p) < 1e-6) || all(abs(observed_p - rdevel_p) < 1e-6),
              info = paste("Observed p =", paste(observed_p, collapse=", ")))
})

test_that("Checking pairwise comparison against ref group", {
  data("ToothGrowth")
  res <- ToothGrowth %>% wilcox_test(len ~ dose, ref.group = "0.5")
  expect_equal(res$group1, c("0.5", "0.5"))
  expect_equal(res$group2, c("1", "2"))
  expect_equal(res$n1, c(20, 20))
  expect_equal(res$n2, c(20, 20))
  expect_equal(as.numeric(res$statistic), c(33.5, 1.5))
  # Accept either legacy or R-devel p-values
  legacy_p <- c(7.02e-6, 8.41e-08)
  rdevel_p <- c(7.74e-07, 4.35e-11)
  observed_p <- signif(res$p, 3)
  expect_true(all(observed_p %in% c(legacy_p, rdevel_p)) || all(abs(observed_p - legacy_p) < 1e-6) || all(abs(observed_p - rdevel_p) < 1e-6),
              info = paste("Observed p =", paste(observed_p, collapse=", ")))
})


test_that("Checking pairwise comparisons against all", {
  data("ToothGrowth")
  res <- ToothGrowth %>% wilcox_test(len ~ dose, ref.group = "all")
  expect_equal(res$group1, c("all", "all", "all"))
  expect_equal(res$group2, c("0.5", "1", "2"))
  expect_equal(res$n1, c(60, 60, 60))
  expect_equal(res$n2, c(20, 20, 20))
  expect_equal(as.numeric(res$statistic), c(965.0, 572.5, 262.5))
  expect_equal(signif(res$p, 3), c(0.0000508, 0.764, 0.000179))
})


test_that("Checking grouped tests", {
  data("ToothGrowth")
  res <- ToothGrowth %>%
    group_by(dose) %>%
    wilcox_test(len ~ supp)
  expect_equal(res$group1, c("OJ", "OJ", "OJ"))
  expect_equal(res$group2, c("VC", "VC", "VC"))
  expect_equal(res$n1, c(10, 10, 10))
  expect_equal(res$n2, c(10, 10, 10))
  expect_equal(as.numeric(res$statistic), c(80.5, 88.5, 49.5))
  # Accept either legacy or R-devel p-values
  legacy_p <- c(0.0232, 0.00403, 1)
  rdevel_p <- c(0.0198, 0.00223, 0.986)
  observed_p <- signif(res$p, 3)
  expect_true(all(abs(observed_p - legacy_p) < 0.005) || all(abs(observed_p - rdevel_p) < 0.005),
              info = paste("Observed p =", paste(observed_p, collapse=", ")))
})


test_that("Empty values are not counting in group n size (104)", {
  # Data without NA
  df <- data.frame(
    g = rep(c("a", "b"), each = 10),
    v = rnorm(20)
  )

  # run Wilcoxon test --> sample sizes are correct
  res <- wilcox_test(df, v ~ g, paired = TRUE)
  expect_equal(c(res$n1, c(res$n1)), c(10, 10))

  # Insert NAs
  df$v[c(1, 12:14)] <- NA

  #repeat Wilcox test --> sample sizes are still the same
  res <- wilcox_test(data = df, v ~ g, paired = TRUE)
  expect_equal(c(res$n1, res$n2), c(9, 7))
})

test_that("wilcox_test detailed method reports the variant (#124)", {
  expect_equal((ToothGrowth %>% wilcox_test(len ~ supp, detailed = TRUE))$method,
               "Wilcoxon rank sum test")
  expect_equal((ToothGrowth %>% wilcox_test(len ~ supp, paired = TRUE, detailed = TRUE))$method,
               "Wilcoxon signed rank test")
})

test_that("wilcox_test does not crash on degenerate/all-tied data (#79, #167)", {
  set.seed(1)
  deg <- data.frame(value = c(0,0,0,0, 0,0,0,0,0), g = rep(c("a","b"), c(4, 5)))
  # default (detailed = FALSE) no longer runs the crash-prone confidence-interval step
  expect_true(is.data.frame(wilcox_test(deg, value ~ g)))
  expect_true(is.data.frame(pairwise_wilcox_test(deg, value ~ g)))
  # grouped data with a fully-constant subgroup no longer errors
  gdf <- data.frame(value = c(rep(0, 6), rnorm(6, 2)),
                    sex = rep(c("M", "F"), 6), grp = rep(c("g1", "g2"), each = 6))
  expect_true(is.data.frame(gdf %>% group_by(grp) %>% wilcox_test(value ~ sex)))
})

test_that("wilcox_test detailed = TRUE still returns the estimate and CI (#79)", {
  res <- ToothGrowth %>% wilcox_test(len ~ supp, detailed = TRUE)
  expect_true(all(c("estimate", "conf.low", "conf.high") %in% colnames(res)))
  expect_false(is.na(res$conf.low))   # normal data -> a real confidence interval
})


# #127: surface a warning when wilcox.test silently reduced the CI confidence level.
# Whether a given data set forces a reduced CI is R-version dependent (newer
# wilcox.test can achieve the exact level where older versions could not), so the
# tests assert the warning fires *iff* the running R's wilcox.test actually
# reduced the level - tracking the active R version rather than a hard-coded value.
ci127_data <- function(){
  data.frame(
    Result = c(0,9,6,8,0,0,0,0,0,0,0,0,1,2,3,3,1,2,1,1,3,3,7,7),
    Timepoint = rep(c("Baseline", "Month 3"), 12)
  )
}

test_that("wilcox_test surfaces a reduced CI confidence level when one occurs (#127)", {
  d <- ci127_data()
  x <- d$Result[d$Timepoint == "Baseline"]
  y <- d$Result[d$Timepoint == "Month 3"]
  raw <- suppressWarnings(stats::wilcox.test(x, y, paired = TRUE, conf.int = TRUE))
  reduced <- isTRUE(attr(raw$conf.int, "conf.level") < 0.95)

  w <- testthat::capture_warnings(
    res <- wilcox_test(d, Result ~ Timepoint, paired = TRUE, detailed = TRUE)
  )
  # the CI warning is present exactly when this R version reduced the CI level
  expect_equal(any(grepl("confidence interval", w)), reduced)
  # returned values are unchanged either way (only a warning is ever added)
  expect_equal(res$p, raw$p.value)
})

test_that("wilcox_test default (non-detailed) call never warns about the CI (#127)", {
  d <- ci127_data()
  w <- testthat::capture_warnings(wilcox_test(d, Result ~ Timepoint, paired = TRUE))
  expect_false(any(grepl("confidence interval", w)))   # no CI requested -> no warning
})

test_that("clean-data wilcox and t_test never warn about the CI (#127)", {
  set.seed(1)
  cd <- data.frame(v = c(rnorm(15), rnorm(15) + 1), g = rep(c("a", "b"), each = 15))
  w1 <- testthat::capture_warnings(wilcox_test(cd, v ~ g, detailed = TRUE))
  expect_false(any(grepl("confidence interval", w1)))   # full conf.level achievable
  d <- ci127_data()
  w2 <- testthat::capture_warnings(t_test(d, Result ~ Timepoint, paired = TRUE, detailed = TRUE))
  expect_false(any(grepl("confidence interval", w2)))   # t.test never reduces
})

context("test-remove_ns")

stat.test <- PlantGrowth %>% wilcox_test(weight ~ group)
test_that("remove_ns works when col = NULL", {
  result <- remove_ns(stat.test, col = NULL)
  p <- round(result$p, 3)
  # Accept either 0.009 (legacy) or 0.011 (R-devel with exact conditional inference)
  expect_true(p %in% c(0.009, 0.011),
              info = paste("Observed p =", p))
})
test_that("remove_ns works when col = NA", {
  result <- remove_ns(stat.test, col = NA)
  p <- round(result$p, 3)
  # Accept either 0.009 (legacy) or 0.011 (R-devel with exact conditional inference)
  expect_true(p %in% c(0.009, 0.011),
              info = paste("Observed p =", p))
})
test_that("remove_ns works when col is logical", {
  result <- remove_ns(stat.test, col = TRUE)
  p.true <- round(result$p, 3)
  result <- remove_ns(stat.test, col = FALSE)
  p.false <- round(result$p, 3)
  # Accept either 0.009 (legacy) or 0.011 (R-devel with exact conditional inference)
  expect_true(p.true %in% c(0.009, 0.011),
              info = paste("Observed p.true =", p.true))
  # Accept both legacy and R-devel vectors
  expect_true(all(p.false %in% c(0.199, 0.063, 0.009, 0.197)),
              info = paste("Observed p.false =", paste(p.false, collapse=", ")))
})
test_that("remove_ns works when col is specified", {
  stat.test2 <- stat.test %>% add_significance("p")
  result.when.p  <- remove_ns(stat.test2, col = "p")
  result.when.p.adj <- remove_ns(stat.test2, col = "p.adj")
  result.when.p.adj.signif <- remove_ns(stat.test2, col = "p.adj.signif")
  result.when.p.signif <- remove_ns(stat.test2, col = "p.signif")
  # Accept either "**" (legacy, p=0.009) or "*" (R-devel, p=0.011)
  expect_true(result.when.p$p.signif %in% c("**", "*"),
              info = paste("Observed p.signif =", result.when.p$p.signif))
  expect_true(result.when.p.adj$p.signif %in% c("**", "*"),
              info = paste("Observed p.signif =", result.when.p.adj$p.signif))
  expect_true(result.when.p.adj.signif$p.signif %in% c("**", "*"),
              info = paste("Observed p.signif =", result.when.p.adj.signif$p.signif))
  expect_true(result.when.p.signif$p.signif %in% c("**", "*"),
              info = paste("Observed p.signif =", result.when.p.signif$p.signif))
})
test_that("remove_ns works when signif.cutoff is specified", {
  stat.test2 <- stat.test %>% add_significance("p")
  result  <- remove_ns(stat.test2, signif.cutoff = 0.01)
  # Accept either 0 (legacy, p=0.009 < 0.01) or 1 (R-devel, p=0.011 > 0.01)
  expect_true(nrow(result) %in% c(0, 1),
              info = paste("Observed nrow =", nrow(result)))
})
test_that("remove_ns works when signif.cutoff and col are specified", {
  stat.test2 <- stat.test %>% add_significance("p")
  result1  <- remove_ns(stat.test2, col = "p.adj", signif.cutoff = 0.01)
  result2  <- remove_ns(stat.test2, col = "p", signif.cutoff = 0.01)
  # Accept either 0 (legacy) or 1 (R-devel)
  expect_true(nrow(result1) %in% c(0, 1),
              info = paste("Observed nrow(result1) =", nrow(result1)))
  # Legacy: p=0.009 < 0.01, so nrow=1 with "**"
  # R-devel: p=0.011 > 0.01, so nrow=0
  expect_true(nrow(result2) %in% c(0, 1),
              info = paste("Observed nrow(result2) =", nrow(result2)))
  if(nrow(result2) == 1) {
    expect_true(result2$p.signif %in% c("**", "*"),
                info = paste("Observed p.signif =", result2$p.signif))
  }
})

#' @include utilities.R
NULL
#'Kruskal-Wallis Test
#'
#'
#'@description Provides a pipe-friendly framework to perform Kruskal-Wallis
#'  rank sum test. Wrapper around the function
#'  \code{\link[stats]{kruskal.test}()}.
#'@param data a data.frame containing the variables in the formula.
#'@param formula a formula of the form \code{x ~ group} where \code{x} is a
#'  numeric variable giving the data values and \code{group} is a factor with
#'  one or multiple levels giving the corresponding groups. For example,
#'  \code{formula = TP53 ~ cancer_group}.
#'@param ... other arguments to be passed to the function
#'  \code{\link[stats]{kruskal.test}}.
#'
#'@return return a data frame with the following columns: \itemize{ \item
#'  \code{.y.}: the y variable used in the test. \item \code{n}: sample count.
#'  \item \code{statistic}: the kruskal-wallis rank sum statistic used to
#'  compute the p-value. \item \code{p}: p-value. \item \code{method}: the
#'  statistical test used to compare groups.}
#' @examples
#' # Load data
#' #:::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Kruskal-wallis rank sum test
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% kruskal_test(len ~ dose)
#'
#' # Grouped data
#' df %>%
#'   group_by(supp) %>%
#'   kruskal_test(len ~ dose)

#'@name kruskal_test
#'@export
kruskal_test <- function(data, formula, ...){
  args <- c(as.list(environment()), list(...)) %>%
    .add_item(method = "kruskal_test")
  if(is_grouped_df(data)){
    results <- data %>% doo(.kruskal_test, formula, ...)
  }
  else{
    results <- .kruskal_test(data, formula, ...)
  }
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "kruskal_test"))
}

.kruskal_test <- function(data, formula, ...)
{
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  term <- statistic <- p <- df <- method <- NULL
  stats::kruskal.test(formula, data = data, ...) %>%
    as_tidy_stat() %>%
    select(statistic, df, p, method) %>%
    add_column(.y. = outcome, n = nrow(data), .before = "statistic")
}

#' @include utilities.R
#' @importFrom stats quantile
#' @importFrom stats IQR
NULL
#'Identify Univariate Outliers Using Boxplot Methods
#'
#'
#'@description Detect outliers using boxplot methods. Boxplots are a popular and
#'  an easy method for identifying outliers. There are two categories of
#'  outlier: (1) outliers and (2) extreme points.
#'
#'  Values above \code{Q3 + 1.5xIQR} or below \code{Q1 - 1.5xIQR} are considered
#'  as outliers. Values above \code{Q3 + 3xIQR} or below \code{Q1 - 3xIQR} are
#'  considered as extreme points (or extreme outliers).
#'
#'  Q1 and Q3 are the first and third quartile, respectively. IQR is the
#'  interquartile range (IQR = Q3 - Q1).
#'
#'  Generally speaking, data points that are labelled outliers in boxplots are
#'  not considered as troublesome as those considered extreme points and might
#'  even be ignored. Note that, any \code{NA} and \code{NaN} are automatically removed
#'  before the quantiles are computed.
#'
#'@return \itemize{ \item \code{identify_outliers()}. Returns the input data
#'  frame with two additional columns: "is.outlier" and "is.extreme", which hold
#'  logical values. \item \code{is_outlier() and is_extreme()}. Returns logical
#'  vectors. }
#'
#'@param data a data frame
#'@param ... One unquoted expressions (or variable name). Used to select a
#'  variable of interest. Alternative to the argument \code{variable}.
#'@param variable variable name for detecting outliers
#'@param x a numeric vector
#'@param coef coefficient specifying how far the outlier should be from the edge
#'  of their box. Possible values are 1.5 (for outlier) and 3 (for extreme
#'  points only). Default is 1.5
#'
#'
#' @examples
#' # Generate a demo data
#' set.seed(123)
#' demo.data <- data.frame(
#'   sample = 1:20,
#'   score = c(rnorm(19, mean = 5, sd = 2), 50),
#'   gender = rep(c("Male", "Female"), each = 10)
#' )
#'
#' # Identify outliers according to the variable score
#' demo.data %>%
#'   identify_outliers(score)
#'
#' # Identify outliers by groups
#' demo.data %>%
#'   group_by(gender) %>%
#'   identify_outliers("score")

#'@describeIn outliers takes a data frame and extract rows suspected as outliers
#'  according to a numeric column. The following columns are added "is.outlier"
#'  and "is.extreme".
#'@export
identify_outliers <- function(data, ..., variable = NULL){

  is.outlier <- NULL
  if(is_grouped_df(data)){
    results <- data %>%
      doo(identify_outliers, ..., variable = variable)
    if(nrow(results) == 0) results <- as.data.frame(results)
    return(results)
  }

  if(!inherits(data, "data.frame"))
    stop("data should be a data frame")
  variable <- data %>% get_selected_vars(..., vars = variable)
  n.vars <- length(variable)
  if(n.vars > 1)
    stop("Specify only one variable")
  values <- data %>% pull(!!variable)
  results <- data %>%
    mutate(
      is.outlier = is_outlier(values),
      is.extreme = is_extreme(values)
      ) %>%
    filter(is.outlier == TRUE)
  if(nrow(results) == 0) results <- as.data.frame(results)
  results
}


#' @describeIn outliers detect outliers in a numeric vector. Returns logical vector.
#' @export
is_outlier <- function(x, coef = 1.5){
  res <- x
  Q1 <- quantile(x, 0.25, na.rm = TRUE)
  Q3 <- quantile(x, 0.75, na.rm = TRUE)
  .IQR <- IQR(x, na.rm = TRUE)
  upper.limit <- Q3 + (coef*.IQR)
  lower.limit <- Q1 - (coef*.IQR)
  outlier <- ifelse(x < lower.limit | x > upper.limit, TRUE, FALSE )
  outlier
}

#' @describeIn outliers detect extreme points in a numeric vector. An alias of
#'   \code{is_outlier()}, where coef = 3. Returns logical vector.
#' @export
is_extreme <- function(x){
  is_outlier(x, coef = 3)
}

#' @include utilities.R
NULL
#' Welch One-Way ANOVA Test
#'
#' @description Tests for equal means in a one-way design (not assuming equal
#'   variance). A wrapper around the base function
#'   \code{\link[stats]{oneway.test}()}. This is is an alternative to the
#'   standard one-way ANOVA in the situation where the homogeneity of variance
#'   assumption is violated.
#' @param data a data frame containing the variables in the formula.
#' @param formula a formula specifying the ANOVA model similar to aov. Can be of
#'   the form y ~ group where y is a numeric variable giving the data values and
#'   group is a factor with one or multiple levels giving the corresponding
#'   groups. For example, formula = TP53 ~ cancer_group.
#' @return return a data frame with the following columns: \itemize{ \item
#'   \code{.y.}: the y variable used in the test. \item \code{n}: sample count.
#'   \item \code{statistic}: the value of the test statistic. \item \code{p}:
#'   p-value. \item \code{method}: the statistical test used to compare groups.}
#' @examples
#' # Load data
#' #:::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#'
#' # Welch one-way ANOVA test (not assuming equal variance)
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% welch_anova_test(len ~ dose)
#'
#' # Grouped data
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>%
#'   group_by(supp) %>%
#'   welch_anova_test(len ~ dose)
#' @name welch_anova_test
#' @export
welch_anova_test <- function(data, formula){
  args <- as.list(environment()) %>%
    .add_item(method = "welch_anova_test")
  data %>%
    doo(oneway_test, formula) %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "welch_anova_test"))
}

oneway_test <- function(data, formula){
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  res <- stats::oneway.test(formula, data = data, var.equal = FALSE)
  tibble(
    .y. = outcome, n = nrow(data),
    statistic = round_value(res$statistic, 2),
    DFn = res$parameter[1],
    DFd = res$parameter[2],
    p = round_value(res$p.value, 3),
    method = "Welch ANOVA"
  )
}

#' @include utilities.R
#' @importFrom stats median
NULL
#' Levene's Test
#'
#' @description Provide a pipe-friendly framework to easily compute Levene's
#'   test for homogeneity of variance across groups.
#'
#'   Wrapper around the function \code{\link[car]{leveneTest}()}, which can
#'   additionally handles a grouped data.
#' @param data a data frame for evaluating the formula or a model
#' @param formula a formula
#' @param center The name of a function to compute the center of each group;
#'   mean gives the original Levene's test; the default, median, provides a more
#'   robust test.
#'
#' @examples
#' # Prepare the data
#' data("ToothGrowth")
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#' # Compute Levene's Test
#' df %>% levene_test(len ~ dose)
#'
#' # Grouped data
#' df %>%
#'   group_by(supp) %>%
#'   levene_test(len ~ dose)
#'
#' @export
levene_test <- function(data, formula, center = median){
  if(is_grouped_df(data)){
    results <- data %>%
      doo(~levene_test(., formula = formula, center = center))
    return(results)
  }
  else if(is_lm(data)){
    results <- car::leveneTest(data, center = center)
  }
  else{
    results <- car::leveneTest(formula, data, center = center)
  }
  results <- broom::tidy(results) %>%
    rename(df1 = .data$df)
  results <- results %>%
    add_column(df2 = results$df1[2], .after = "df1") %>%
    filter(.data$term != "") %>%
    select(.data$df1, .data$df2, .data$statistic, .data$p.value) %>%
    rename(p = .data$p.value)
  results
}

context("test-levene_test")

test_that("Levene test output is correctly formatted", {
  # Prepare the data
  data("ToothGrowth")
  df <- ToothGrowth
  df$dose <- as.factor(df$dose)
  # Compute Levene's Test
  results <- df %>% levene_test(len ~ dose)
  expect_equal(results$df1, 2)
  expect_equal(results$df2, 57)
  expect_equal(round(results$statistic, 3), 0.646)
  expect_equal(round(results$p, 3), 0.528)
})

#' @include utilities.R
NULL
#'Chi-squared Test for Count Data
#'@description Performs chi-squared tests, including goodness-of-fit,
#'  homogeneity and independence tests.
#'@inheritParams  stats::chisq.test
#'@param res.chisq an object of class \code{chisq_test}.
#'@param p.adjust.method method to adjust p values for multiple comparisons.
#'  Used when pairwise comparisons are performed. Allowed values include "holm",
#'  "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
#'  want to adjust the p value (not recommended), use p.adjust.method = "none".
#'@param ... other arguments passed to the function \code{{chisq_test}()}.
#'
#'@return return a data frame with some the following columns: \itemize{ \item
#'  \code{n}: the number of participants. \item \code{group, group1, group2}:
#'  the categories or groups being compared. \item \code{statistic}: the value
#'  of Pearson's chi-squared test statistic. \item \code{df}: the degrees of
#'  freedom of the approximate chi-squared distribution of the test statistic.
#'  NA if the p-value is computed by Monte Carlo simulation. \item \code{p}:
#'  p-value. \item \code{p.adj}: the adjusted p-value. \item \code{method}: the
#'  used statistical test. \item \code{p.signif, p.adj.signif}: the significance
#'  level of p-values and adjusted p-values, respectively. \item
#'  \code{observed}: observed counts. \item
#'  \code{expected}: the expected counts under the null hypothesis.
#'  }
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#'
#' @examples
#' # Chi-square goodness of fit test
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' tulip <- c(red = 81, yellow = 50, white = 27)
#' # Q1: Are the colors equally common?
#' chisq_test(tulip)
#' pairwise_chisq_gof_test(tulip)
#' # Q2: comparing observed to expected proportions
#' chisq_test(tulip, p = c(1/2, 1/3, 1/6))
#' pairwise_chisq_test_against_p(tulip, p = c(0.5, 0.33, 0.17))
#'
#' # Homogeneity of proportions between groups
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Data: Titanic
#' xtab <- as.table(rbind(
#'   c(203, 118, 178, 212),
#'   c(122, 167, 528, 673)
#' ))
#' dimnames(xtab) <- list(
#'   Survived = c("Yes", "No"),
#'   Class = c("1st", "2nd", "3rd", "Crew")
#' )
#' xtab
#' # Chi-square test
#' chisq_test(xtab)
#' # Compare the proportion of survived between groups
#' pairwise_prop_test(xtab)


#' @describeIn chisq_test performs chi-square tests including goodness-of-fit,
#'   homogeneity and independence tests.
#' @export
chisq_test <- function(x, y = NULL, correct = TRUE,
                       p = rep(1/length(x), length(x)), rescale.p = FALSE,
                       simulate.p.value = FALSE, B = 2000){
  args <- as.list(environment()) %>%
    add_item(method = "chisq_test")
  if(is.data.frame(x)) x <- as.matrix(x)
  if(inherits(x, c("matrix", "table"))) n <- sum(x)
  else n <- length(x)
  res.chisq <- stats::chisq.test(
    x, y,  correct = correct, p = p, rescale.p = rescale.p,
    simulate.p.value = simulate.p.value, B = B
    )
  as_tidy_stat(res.chisq, stat.method = "Chi-square test") %>%
    add_significance("p") %>%
    add_columns(n = n, .before = 1) %>%
    set_attrs(args = args, test = res.chisq) %>%
    add_class(c("rstatix_test", "chisq_test"))
}

#' @describeIn chisq_test perform pairwise comparisons between groups following a global
#'   chi-square goodness of fit test.
#' @export
pairwise_chisq_gof_test <- function(x, p.adjust.method = "holm", ...){
  if(is.null(names(x))){
    names(x) <- paste0("grp", 1:length(x))
  }
  compare_pair <- function(levs, x, ...){
    levs <- as.character(levs)
    suppressWarnings(chisq_test(x[levs],  ...)) %>%
      add_columns(group1 = levs[1], group2 = levs[2], .before = "statistic")
  }
  args <- as.list(environment()) %>%
    add_item(method = "chisq_test")
  comparisons <- names(x) %>%
    .possible_pairs()
  results <- comparisons %>%
    map(compare_pair, x, ...) %>%
    map(keep_only_tbl_df_classes) %>%
    bind_rows() %>%
    adjust_pvalue("p", method = p.adjust.method) %>%
    add_significance("p.adj") %>%
    mutate(p.adj = signif(.data$p.adj, digits = 3)) %>%
    select(-.data$p.signif, -.data$method)
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "chisq_test"))
}

#' @describeIn chisq_test perform pairwise comparisons after a global
#'   chi-squared test for given probabilities. For each group, the observed and
#'   the expected proportions are shown. Each group is compared to the sum of
#'   all others.
#' @export
pairwise_chisq_test_against_p <- function(x, p = rep(1/length(x), length(x)), p.adjust.method = "holm", ...){
  args <- as.list(environment()) %>%
    add_item(method = "chisq_test")
  if (sum(p) != 1) {
    stop(
      "Make sure that the `p` argument is correctly specified.",
      "sum of probabilities must be 1."
      )
  }
  if(is.null(names(x))){
    names(x) <- paste0("grp", 1:length(x))
  }
  results <- list()
  for (i in 1:length(x)) {
    res.chisq <- suppressWarnings(chisq_test(c(x[i], sum(x) - x[i]), p = c(p[i], 1 - p[i]), ...))
    res.desc <- chisq_descriptives(res.chisq)
    res.chisq <- res.chisq %>%
      add_columns(observed = res.desc$observed[1], expected = res.desc$expected[1], .before = 1)
    results[[i]] <- res.chisq
  }
  results <- results %>%
    map(keep_only_tbl_df_classes) %>%
    bind_rows() %>%
    add_columns(group = names(x), .before = 1) %>%
    adjust_pvalue("p", method = p.adjust.method) %>%
    add_significance("p.adj") %>%
    mutate(p.adj = signif(.data$p.adj, digits = 3)) %>%
    select(-.data$p.signif, -.data$method)
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "chisq_test"))
}


#' @describeIn chisq_test returns the descriptive statistics of the chi-square
#'   test. These include, observed and expected frequencies, proportions,
#'   residuals and standardized residuals.
#' @export
chisq_descriptives <- function(res.chisq){
  res <- attr(res.chisq, "test") %>% augment()
  colnames(res) <- gsub(pattern = "^\\.", replacement = "", colnames(res))
  res
}

#' @describeIn chisq_test returns the expected counts from the chi-square test result.
#' @export
expected_freq <- function(res.chisq){
  attr(res.chisq, "test")$expected
}

#' @describeIn chisq_test returns the observed counts from the chi-square test result.
#' @export
observed_freq <- function(res.chisq){
  attr(res.chisq, "test")$observed
}

#' @describeIn chisq_test returns the Pearson residuals, \code{(observed - expected) / sqrt(expected)}.
#' @export
pearson_residuals <- function(res.chisq){
  attr(res.chisq, "test")$residuals
}

#' @describeIn chisq_test returns the standardized residuals
#' @export
std_residuals <- function(res.chisq){
  attr(res.chisq, "test")$stdres
}

context("test-chisq_test")

make_df <- function(){
  set.seed(1)
  data.frame(
    eyes = sample(c("blue", "brown"), 200, replace = TRUE),
    hair = sample(c("dark", "fair"), 200, replace = TRUE),
    stringsAsFactors = FALSE
  )
}

test_that("chisq_test data-frame interface: positional columns (#43)", {
  df <- make_df()
  res <- df %>% chisq_test(eyes, hair)
  ref <- stats::chisq.test(table(df$eyes, df$hair))
  expect_equal(unname(res$statistic), unname(ref$statistic))
  expect_equal(unname(res$df), unname(ref$parameter))
  expect_equal(res$p, ref$p.value)
  expect_equal(res$n, nrow(df))
  expect_equal(
    colnames(res),
    c("n", "statistic", "p", "df", "method", "p.signif")
  )
})

test_that("chisq_test vars= form equals positional form (#43)", {
  df <- make_df()
  pos  <- df %>% chisq_test(eyes, hair)
  vars <- df %>% chisq_test(vars = c("eyes", "hair"))
  expect_equal(as.data.frame(pos)[1:6], as.data.frame(vars)[1:6])
})

test_that("chisq_test vars= keeps correct= settable (#43)", {
  df <- make_df()
  res <- df %>% chisq_test(vars = c("eyes", "hair"), correct = FALSE)
  ref <- stats::chisq.test(table(df$eyes, df$hair), correct = FALSE)
  expect_equal(unname(res$statistic), unname(ref$statistic))
})

test_that("chisq_test accepts string column names positionally (#43)", {
  df <- make_df()
  bare <- df %>% chisq_test(eyes, hair)
  strg <- df %>% chisq_test("eyes", "hair")
  expect_equal(as.data.frame(bare)[1:6], as.data.frame(strg)[1:6])
})

test_that("chisq_test data-frame interface needs exactly two columns (#43)", {
  df <- make_df()
  expect_error(df %>% chisq_test(eyes), "exactly two columns")
})

test_that("chisq_test honors group_by, one test per group (#43)", {
  df <- make_df()
  df$site <- rep(c("A", "B"), each = 100)
  res <- df %>% dplyr::group_by(site) %>% chisq_test(eyes, hair)
  expect_true("site" %in% colnames(res))
  expect_equal(nrow(res), 2L)                       # one row per group, not pooled
  expect_setequal(res$site, c("A", "B"))
  # each group's statistic matches the per-subset table
  for(s in c("A", "B")){
    sub <- df[df$site == s, ]
    ref <- stats::chisq.test(table(sub$eyes, sub$hair))
    expect_equal(unname(res$statistic[res$site == s]), unname(ref$statistic))
    expect_equal(res$n[res$site == s], nrow(sub))
  }
})

test_that("chisq_test(df, v1, v2, correct=FALSE) works (column in p slot) (#43)", {
  df <- make_df()
  res <- df %>% chisq_test(eyes, hair, correct = FALSE)
  ref <- stats::chisq.test(table(df$eyes, df$hair), correct = FALSE)
  expect_equal(unname(res$statistic), unname(ref$statistic))
  expect_equal(res$n, nrow(df))
})

test_that("chisq_descriptives works on the data-frame interface (#43)", {
  df <- make_df()
  res <- df %>% chisq_test(eyes, hair)
  desc <- chisq_descriptives(res)
  expect_equal(sum(desc$observed), nrow(df))
  expect_true(is.matrix(expected_freq(res)))
})

test_that("descriptive accessors error clearly on a grouped result (#43)", {
  df <- make_df()
  df$site <- rep(c("A", "B"), each = 100)
  res <- df %>% dplyr::group_by(site) %>% chisq_test(eyes, hair)
  expect_error(chisq_descriptives(res), "single \\(ungrouped\\)")
  expect_error(expected_freq(res), "single \\(ungrouped\\)")
  expect_error(std_residuals(res), "single \\(ungrouped\\)")
})

# ---- No-regression: existing table / vector interfaces unchanged ----

test_that("chisq_test goodness-of-fit (vector) is unchanged (#43 no-regression)", {
  tulip <- c(red = 81, yellow = 50, white = 27)
  res <- chisq_test(tulip)
  ref <- stats::chisq.test(tulip)
  expect_equal(unname(res$statistic), unname(ref$statistic))
  expect_equal(res$p, ref$p.value)
  expect_equal(res$n, length(tulip))   # existing behavior: n = number of categories
})

test_that("chisq_test on a table/matrix is unchanged (#43 no-regression)", {
  xtab <- as.table(rbind(c(203, 118, 178, 212), c(122, 167, 528, 673)))
  res <- chisq_test(xtab)
  ref <- stats::chisq.test(xtab)
  expect_equal(unname(res$statistic), unname(ref$statistic))
  expect_equal(unname(res$df), unname(ref$parameter))
  expect_equal(res$n, sum(xtab))
})

test_that("chisq_test on a data-frame-as-table is unchanged (#43 no-regression)", {
  # a data frame that IS a contingency table (no column refs) -> treated as table
  df_tab <- as.data.frame.matrix(matrix(c(12, 5, 7, 20), nrow = 2))
  res <- chisq_test(df_tab)
  ref <- stats::chisq.test(as.matrix(df_tab))
  expect_equal(unname(res$statistic), unname(ref$statistic))
  expect_equal(res$n, sum(df_tab))
})

#' @include utilities.R as_cor_mat.R
NULL
#'Compute Correlation Matrix with P-values
#'@description Compute correlation matrix with p-values. Numeric columns in the
#'  data are detected and automatically selected for the analysis. You can also
#'  specify variables of interest to be used in the correlation analysis.
#'@inheritParams cor_test
#'@param x an object of class \code{cor_mat}
#'@param vars a character vector containing the variable names of interest.
#'@param ... One or more unquoted expressions (or variable names) separated by
#'  commas. Used to select a variable of interest.
#'@return a data frame
#'@seealso \code{\link{cor_test}()}, \code{\link{cor_reorder}()},
#'  \code{\link{cor_gather}()}, \code{\link{cor_select}()},
#'  \code{\link{cor_as_symbols}()}, \code{\link{pull_triangle}()},
#'  \code{\link{replace_triangle}()}
#' @examples
#' # Data preparation
#' #:::::::::::::::::::::::::::::::::::::::::::
#' mydata <- mtcars %>%
#'   select(mpg, disp, hp, drat, wt, qsec)
#' head(mydata, 3)
#'
#' # Compute correlation matrix
#' #::::::::::::::::::::::::::::::::::::::::::
#' # Correlation matrix between all variables
#' cor.mat <- mydata %>% cor_mat()
#' cor.mat
#'
#' # Specify some variables of interest
#' mydata %>% cor_mat(mpg, hp, wt)
#'
#' # Or remove some variables in the data
#' # before the analysis
#' mydata %>% cor_mat(-mpg, -hp)
#'
#' # Significance levels
#' #::::::::::::::::::::::::::::::::::::::::::
#' cor.mat %>% cor_get_pval()
#'
#'
#' # Visualize
#' #::::::::::::::::::::::::::::::::::::::::::
#' # Insignificant correlations are marked by crosses
#' cor.mat %>%
#'   cor_reorder() %>%
#'   pull_lower_triangle() %>%
#'   cor_plot(label = TRUE)
#'
#' # Gather/collapse correlation matrix into long format
#' #::::::::::::::::::::::::::::::::::::::::::
#' cor.mat %>% cor_gather()
#'
#'
#'@describeIn cor_mat compute correlation matrix with p-values. Returns a data
#'  frame containing the matrix of the correlation coefficients. The output has
#'  an attribute named "pvalue", which contains the matrix of the correlation
#'  test p-values.
#'@export
cor_mat <- function(data, ..., vars = NULL, method = "pearson",
                    alternative = "two.sided", conf.level = 0.95){


  vars <- data %>% get_selected_vars(..., vars = vars)
  n.vars <- length(vars)
  if(n.vars > 1 & n.vars <= 2){
      stop("At least, 3 variables are required for a correlation matrix. ",
           "Use the function cor_test() for 2 or less variables.", call. = FALSE)
  }

  cor_test(
    data, vars = vars, method = method,
    alternative = alternative, conf.level = conf.level
    ) %>%
    as_cor_mat()
}


#' @describeIn cor_mat compute the correlation matrix but returns only the p-values of the tests.
#' @export
cor_pmat <- function(data, ..., vars = NULL, method = "pearson",
                     alternative = "two.sided", conf.level = 0.95){

  cor_mat(
    data = data, ..., vars = vars, method = method,
    alternative = alternative, conf.level = conf.level
  ) %>%
    cor_get_pval()
}


#' @describeIn cor_mat extract a correlation matrix p-values from an object of
#'   class \code{cor_mat()}.
#' @export
cor_get_pval <- function(x){
  res <- x %>% attr("pvalue")
  if(is.null(res))
    warning("Can't find p-value attributes.", call.= FALSE)
  res
}

context("test-cor_test")

test_that("cor_test reports df for Pearson (#107)", {
  res <- mtcars %>% cor_test(mpg, wt)
  expect_true("df" %in% colnames(res))
  # the df column carries the name "df" (consistent with t_test's df column), so compare values
  expect_equal(unname(res$df), as.integer(nrow(mtcars) - 2))                 # n - 2
  expect_equal(unname(res$df), unname(as.integer(cor.test(mtcars$mpg, mtcars$wt)$parameter)))
  # df sits next to statistic
  expect_equal(colnames(res),
               c("var1", "var2", "cor", "statistic", "df", "p",
                 "conf.low", "conf.high", "method"))
})

test_that("cor_test has no df column for Spearman/Kendall (#107 no-regression)", {
  expect_false("df" %in% colnames(mtcars %>% cor_test(mpg, wt, method = "spearman")))
  expect_false("df" %in% colnames(mtcars %>% cor_test(mpg, wt, method = "kendall")))
})

test_that("grouped Pearson cor_test reports df per group (#107)", {
  g <- iris %>% group_by(Species) %>% cor_test(Sepal.Width, Sepal.Length)
  expect_true("df" %in% colnames(g))
  expect_true(all(g$df == 48L))   # 50 per species - 2
})

test_that("cor_mat / cor_pmat are unaffected by the df column (#107 no-regression)", {
  cm <- mtcars %>% cor_mat(mpg, disp, hp)
  expect_false("df" %in% colnames(cm))
  expect_equal(colnames(cm), c("rowname", "mpg", "disp", "hp"))
  expect_true(is.data.frame(mtcars %>% cor_pmat(mpg, disp, hp)))
})

test_that("cor_test with external character vectors does not warn (deprecation) (#202)", {
  # force lifecycle deprecations to always warn (not once-per-session)
  rlang::local_options(lifecycle_verbosity = "warning")
  matrix_vars  <- c("mpg", "disp")
  matrix_vars2 <- c("hp", "wt")
  w <- capture_warnings(
    res <- cor_test(mtcars, vars = matrix_vars, vars2 = matrix_vars2)
  )
  expect_false(any(grepl("external vector", w)))   # the tidyselect deprecation is gone
  expect_equal(nrow(res), 4L)                       # 2 vars x 2 vars2
  expect_equal(res$var1, c("mpg", "mpg", "disp", "disp"))
  expect_equal(res$var2, c("hp", "wt", "hp", "wt"))

  # inline character vectors behave the same
  w2 <- capture_warnings(
    res2 <- cor_test(mtcars, vars = c("mpg", "disp"), vars2 = c("hp", "wt"))
  )
  expect_false(any(grepl("external vector", w2)))
  expect_equal(res2[, c("var1", "var2")], res[, c("var1", "var2")])
})

test_that("cor_test selection is unchanged for bare names and tidyselect helpers (#202)", {
  # bare names
  bare <- cor_test(mtcars, mpg, disp)
  expect_equal(nrow(bare), 1L)
  expect_equal(c(bare$var1, bare$var2), c("mpg", "disp"))
  # external character vector selects exactly the same variables as bare names
  v <- cor_test(mtcars, vars = c("mpg", "disp"))
  expect_equal(sort(unique(c(v$var1, v$var2))), sort(c("disp", "mpg")))
  # tidyselect helper still works
  helper <- cor_test(mtcars, dplyr::starts_with("d"))
  expect_setequal(unique(c(helper$var1, helper$var2)), c("disp", "drat"))
})

test_that("get_quo_vars: a bare column name is never shadowed by an env object (#202)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  df$grp  <- as.character(df$supp)
  dose <- "supp"   # an env object that shadows the column name
  # bare symbol must resolve to the COLUMN, not the env object
  expect_equal(rstatix:::get_quo_vars(df, rlang::quo(dose)), "dose")
  # bare character column resolves to its name (not its values)
  expect_equal(rstatix:::get_quo_vars(df, rlang::quo(grp)), "grp")
  # external character vector is still selected by name (the #202 path)
  v <- c("len", "dose")
  expect_equal(rstatix:::get_quo_vars(df, rlang::quo(v)), c("len", "dose"))
})

test_that("anova_test bare grouping var is not shadowed by an env object (#202)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  dose <- "supp"   # shadowing object in scope
  res <- anova_test(df, dv = len, between = dose)
  expect_true("dose" %in% res$Effect)
  expect_false("supp" %in% res$Effect)
})

test_that("cor_mat with external character vectors does not warn (#202)", {
  rlang::local_options(lifecycle_verbosity = "warning")
  keep <- c("mpg", "disp", "hp")
  w <- capture_warnings(res <- cor_mat(mtcars, vars = keep))
  expect_false(any(grepl("external vector", w)))
  expect_equal(res$rowname, keep)
})

#' @include utilities.R
NULL
#' Compute Mode
#'
#' @description Compute the mode in a given vector. Mode is the most frequent
#'   value.
#'
#' @param x a vector. Can be numeric, factor or character vector.
#'
#' @examples
#'
#' # Mode of numeric vector
#' x <- c(1:5, 6, 6, 7:10)
#' get_mode(x)
#'
#' # Bimodal
#' x <- c(1:5, 6, 6, 7, 8, 9, 9, 10)
#' get_mode(x)
#'
#' # No mode
#' x <- c(1, 2, 3, 4, 5)
#' get_mode(x)
#'
#' # Nominal vector
#' fruits <-  c(rep("orange", 10), rep("apple", 5), rep("lemon", 2))
#' get_mode(fruits)
#' @export
get_mode <- function(x){
  .x <- factor(x)
  .table <- table(.x)
  .max <- max(.table)
  if(all(.table == .max)){
    .mode <- NA
  }
  else{
   .mode <- names(.table)[.table == .max]
  }
  if(is.numeric(x)){
    .mode <- as.numeric(.mode)
  }
  .mode
}

#' @include utilities.R
NULL
#' Effect Size for ANOVA
#' @description Compute eta-squared and partial eta-squared for all terms in an
#'   ANOVA model.
#' @param model an object of class aov or anova.
#' @return a numeric vector with the effect size statistics
#' @describeIn eta_squared compute eta squared
#' @examples
#' # Data preparation
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#'
#' # Compute ANOVA
#' res.aov <- aov(len ~ supp*dose, data = df)
#' summary(res.aov)
#'
#' # Effect size
#' eta_squared(res.aov)
#' partial_eta_squared(res.aov)

#' @export
eta_squared <- function(model){
  model %>%
    aov_stat_summary() %>%
    aov_stat_core("eta")
}

#' @describeIn eta_squared compute partial eta squared.
#' @export
partial_eta_squared <- function(model){
  model %>%
    aov_stat_summary() %>%
    aov_stat_core("peta")
}


aov_stat_summary <- function (model)
{
  if (!inherits(model, c("aov", "anova")))
    model <- stats::anova(model)
  aov.sum <- broom::tidy(model)
  if (!tibble::has_name(aov.sum, "meansq"))
    aov.sum <- tibble::add_column(aov.sum, meansq = aov.sum$sumsq/aov.sum$df,
                                  .after = "sumsq")
  aov.sum
}

aov_stat_core <- function(aov.sum, type){
    meansq.resid <- aov.sum[["meansq"]][nrow(aov.sum)]
    ss.total <- sum(aov.sum[["sumsq"]])
    ss.resid <- aov.sum[["sumsq"]][nrow(aov.sum)]
    n_terms <- nrow(aov.sum) - 1
    if (type == "omega") {
      aovstat <- purrr::map_dbl(1:n_terms, function(x) {
        ss.term <- aov.sum[["sumsq"]][x]
        df.term <- aov.sum[["df"]][x]
        (ss.term - df.term * meansq.resid)/(ss.total + meansq.resid)
      })
    }
    else if (type == "eta") {
      aovstat <- purrr::map_dbl(1:n_terms, ~aov.sum[["sumsq"]][.x]/sum(aov.sum[["sumsq"]]))
    }
    else if (type %in% c("cohens.f", "peta")) {
      aovstat <- purrr::map_dbl(1:n_terms, ~aov.sum[["sumsq"]][.x]/(aov.sum[["sumsq"]][.x] +
                                                                      ss.resid))
    }
    if (type == "cohens.f")
      aovstat <- sqrt(aovstat/(1 - aovstat))
    names(aovstat) <- aov.sum[["term"]][1:n_terms]
    aovstat
  }

#' @include utilities.R
#' @importFrom stats as.formula
#' @importFrom stats cor.test
#'
NULL
#'Correlation Test
#'
#'
#'@description Provides a pipe-friendly framework to perform correlation test
#'  between paired samples, using Pearson, Kendall or Spearman method. Wrapper
#'  around the function \code{\link[stats]{cor.test}()}.
#'
#'  Can also performs multiple pairwise correlation analyses between more than
#'  two variables or between two different vectors of variables. Using this
#'  function, you can also compute, for example, the correlation between one
#'  variable vs many.
#'
#'
#'@inheritParams stats::cor.test
#'@inheritParams stats::cor
#'@param data a data.frame containing the variables.
#'@param vars optional character vector containing variable names for
#'  correlation analysis. Ignored when dot vars are specified. \itemize{ \item
#'  If \code{vars} is NULL, multiple pairwise correlation tests is performed
#'  between all variables in the data. \item If \code{vars} contain only one
#'  variable, a pairwise correlation analysis is performed between the specified
#'  variable vs either all the remaining numeric variables in the data or
#'  variables in \code{vars2} (if specified). \item If \code{vars} contain two
#'  or more variables: i) if \code{vars2} is not specified, a pairwise
#'  correlation analysis is performed between all possible combinations of
#'  variables. ii) if \code{vars2} is specified, each element in \code{vars} is
#'  tested against all elements in \code{vars2}}. Accept unquoted
#'  variable names: \code{c(var1, var2)}.
#'@param vars2 optional character vector. If specified, each element in
#'  \code{vars} is tested against all elements in \code{vars2}. Accept unquoted
#'  variable names: \code{c(var1, var2)}.
#'@param ... One or more unquoted expressions (or variable names) separated by
#'  commas. Used to select a variable of interest. Alternative to the argument
#'  \code{vars}.
#'
#'@return return a data frame with the following columns: \itemize{ \item
#'  \code{var1, var2}: the variables used in the correlation test. \item
#'  \code{cor}: the correlation coefficient. \item \code{statistic}: Test
#'  statistic used to compute the p-value. \item \code{p}: p-value. \item
#'  \code{conf.low,conf.high}: Lower and upper bounds on a confidence interval.
#'  \item \code{method}: the method used to compute the statistic.}
#'@seealso \code{\link{cor_mat}()}, \code{\link{as_cor_mat}()}
#' @examples
#'
#' # Correlation between the specified variable vs
#' # the remaining numeric variables in the data
#' #:::::::::::::::::::::::::::::::::::::::::
#' mtcars %>% cor_test(mpg)
#'
#' # Correlation test between two variables
#' #:::::::::::::::::::::::::::::::::::::::::
#' mtcars %>% cor_test(wt, mpg)
#'
#' # Pairwise correlation between multiple variables
#' #:::::::::::::::::::::::::::::::::::::::::
#' mtcars %>% cor_test(wt, mpg, disp)
#'
#' # Grouped data
#' #:::::::::::::::::::::::::::::::::::::::::
#' iris %>%
#'   group_by(Species) %>%
#'  cor_test(Sepal.Width, Sepal.Length)
#'
#' # Multiple correlation test
#' #:::::::::::::::::::::::::::::::::::::::::
#' # Correlation between one variable vs many
#' mtcars %>% cor_test(
#'   vars = "mpg",
#'   vars2 = c("disp", "hp", "drat")
#'  )
#'
#' # Correlation between two vectors of variables
#' # Each element in vars is tested against all elements in vars2
#' mtcars %>% cor_test(
#'   vars = c("mpg", "wt"),
#'   vars2 = c("disp", "hp", "drat")
#'  )
#'
#'
#'@describeIn cor_test correlation test between two or more variables.
#'@export

cor_test <- function(
  data, ..., vars = NULL, vars2 =  NULL, alternative = "two.sided",
  method = "pearson", conf.level = 0.95,
  use = "pairwise.complete.obs"
)
{
  . <- NULL
  # Accept unquoted variables
  .args <- rlang::enquos(vars = vars, vars2 = vars2) %>%
    get_quo_vars_list(data, .)
  vars <- .args$vars
  vars2 <- .args$vars2

  vars <- data %>% get_selected_vars(..., vars = vars)
  n.vars <- length(vars)

  # Select only numeric columns
  data.numeric <- data %>%
    select_numeric_columns()

  if(is.null(vars2)){

    if(is.null(vars)){
      # Pairwise correlation test between all vars in data
      vars <- vars2 <- colnames(data.numeric)
    }
    else if(n.vars == 1){
      # Correlation between the specified variable vs
      # all numeric vars in the data
      vars2 <- colnames( data.numeric) %>% setdiff(vars)
    }
    else if(n.vars == 2){
      # Correlation test between two variables
      vars2 <- vars[2]
      vars <- vars[1]
    }
    # Multiple pairwise correlation between multiple variables
    else if(n.vars >2){
      vars2 <- vars
    }
  }
  else if(is.null(vars)){
    stop("You should specify the argument vars in addition to vars2")
  }

  # Multiple correlation tests between two vectors of variables.
  expand.grid(y = vars2, x = vars,  stringsAsFactors = FALSE) %>%
    as.list() %>%
    purrr::pmap_dfr(
      cor_test_xy, data = data, alternative = alternative,
      method = method, conf.level = conf.level, use = use
      ) %>%
    add_class("cor_test")
}



#:::::::::::::::::::::::::::::::::::::::::::::::::::
# Helper functions
#:::::::::::::::::::::::::::::::::::::::::::::::::::

# Correlation test between two variables x and y
#++++++++++++++++++++++++++++++++++++++++++++++++++++
cor_test_xy <- function(
  data, x, y, method = "pearson",
  use = "pairwise.complete.obs", ...
)
{
  if(is_grouped_df(data)){
    results <- data %>%
      doo(cor_test_xy, x, y, method = method, use = use, ...)
    return(results)
  }
  # Correlation test, supress the warning when method = "spearman" or "kendall".
  x.value <- data %>% pull(x)
  y.value <- data %>% pull(y)
  suppressWarnings(cor.test(x.value, y.value, method = method, use = use, ...)) %>%
    as_tidy_cor() %>%
    add_column(var1 = x, var2 = y, .before = "cor")
}



# Multiple correlation tests between two vectors of variables.
#++++++++++++++++++++++++++++++++++++++++++++++++++++
# x,y character vectors containing variable names to be used in the
# correlation analysis.
mcor_test <- function(data, x, y, ...){
  expand.grid(y = y, x = x,  stringsAsFactors = FALSE) %>%
    as.list() %>%
    purrr::pmap_dfr(cor_test_xy, data = data, ...) %>%
    add_class("cor_test")
}


# Tidy output for correlation test
as_tidy_cor <- function(x){

  estimate <- cor <- statistic <- p <-
    conf.low <- conf.high <- method <- NULL
  res <- x %>%
    as_tidy_stat() %>%
    rename(cor = estimate) %>%
    mutate(cor = signif(cor, 2))
  if(res$method == "Pearson"){
    res %>% select(cor, statistic, p, conf.low, conf.high, method)
  }
  else {
    res %>% select(cor, statistic, p, method)
  }

}

context("test-cor_test-weights")

# #47: cor_test() wraps stats::cor.test(), which has no `weights` argument. A
# `weights =` used to be silently mis-handled (treated as an extra variable, or
# dropped) and returned the unweighted result. It must now error clearly.

test_that("cor_test() errors when weights are supplied (#47)", {
  set.seed(1)
  df <- data.frame(x = rnorm(20), y = rnorm(20), wt = runif(20))
  expect_error(df %>% cor_test(x, y, weights = wt), "weighted correlation")
  # also with the vars/vars2 interface
  expect_error(df %>% cor_test(vars = "x", vars2 = "y", weights = wt),
               "weighted correlation")
})

test_that("a bare column literally named 'weights' is unaffected (#47)", {
  set.seed(1)
  df <- data.frame(weights = rnorm(20), x = rnorm(20))
  res <- df %>% cor_test(weights, x)
  expect_equal(nrow(res), 1L)
  expect_true(all(c("weights", "x") %in% c(res$var1, res$var2)))
})

test_that("ordinary cor_test() calls are unchanged (no-regression) (#47)", {
  set.seed(1)
  df <- data.frame(x = rnorm(20), y = rnorm(20))
  res <- df %>% cor_test(x, y)
  expect_equal(nrow(res), 1L)
  expect_equal(res$var1, "x"); expect_equal(res$var2, "y")
  # multi-variable mode still returns the full matrix of pairs
  expect_equal(nrow(mtcars %>% cor_test(wt, mpg, disp)), 9L)
})

#' @include utilities.R t_test.R
NULL
#'Games Howell Post-hoc Tests
#'
#'@description Performs Games-Howell test, which is used to compare all possible
#'  combinations of group differences when the assumption of homogeneity of
#'  variances is violated. This post hoc test provides confidence intervals for
#'  the differences between group means and shows whether the differences are
#'  statistically significant.
#'
#'  The test is based on Welch’s degrees of freedom correction and uses Tukey’s
#'  studentized range distribution for computing the p-values. The test compares
#'  the difference between each pair of means with appropriate adjustment for
#'  the multiple testing. So there is no need to apply additional p-value
#'  corrections.
#'
#'@inheritParams t_test
#'@return return a data frame with some of the following columns: \itemize{
#'  \item \code{.y.}: the y (outcome) variable used in the test. \item
#'  \code{group1,group2}: the compared groups in the pairwise tests. \item
#'  \code{n1,n2}: Sample counts. \item \code{estimate, conf.low, conf.high}:
#'  mean difference and its confidence intervals. \item \code{statistic}: Test
#'  statistic (t-value) used to compute the p-value. \item \code{df}: degrees of
#'  freedom calculated using Welch’s correction. \item \code{p.adj}: adjusted p-value using Tukey's method. \item
#'  \code{method}: the statistical test used to compare groups. \item
#'  \code{p.adj.signif}: the significance level of p-values. }
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#'@details The Games-Howell method is an improved version of the Tukey-Kramer
#'  method and is applicable in cases where the equivalence of variance
#'  assumption is violated. It is a t-test using Welch’s degree of freedom. This
#'  method uses a strategy for controlling the type I error for the entire
#'  comparison and is known to maintain the preset significance level even when
#'  the size of the sample is different. However, the smaller the number of
#'  samples in each group, the it is more tolerant the type I error control.
#'  Thus, this method can be applied when the number of samples is six or more.
#'
#'@references \itemize{ \item Aaron Schlege,
#'  https://rpubs.com/aaronsc32/games-howell-test. \item Sangseok Lee, Dong Kyu
#'  Lee. What is the proper way to apply the multiple comparison test?. Korean J
#'  Anesthesiol. 2018;71(5):353-360. }
#'
#'
#' @examples
#' # Simple test
#' ToothGrowth %>% games_howell_test(len ~ dose)
#'
#' # Grouped data
#' ToothGrowth %>%
#'   group_by(supp) %>%
#'   games_howell_test(len ~ dose)
#'
#'@rdname games_howell_test
#'@export
games_howell_test <- function(data, formula, conf.level = 0.95, detailed = FALSE){
  args <- as.list(environment()) %>%
    .add_item(p.adjust.method = "Tukey", method = "games_howell_test")
  results <- data %>%
    doo(.games_howell_test, formula, conf.level = conf.level)
  if(!detailed){
    results <- results %>%
      select(
        -.data$se, -.data$method, -.data$statistic,
        -.data$df, -.data$n1, -.data$n2
      )
  }
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "games_howell_test"))
}

.games_howell_test <- function(data, formula, conf.level = 0.95){
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  number.of.groups <- guess_number_of_groups(data, group)
  if(number.of.groups == 1){
    stop("all observations are in the same group")
  }

  data <- data %>%
    select(!!!syms(c(outcome, group))) %>%
    get_complete_cases() %>%
    .as_factor(group)

  x <- data %>% pull(!!outcome)
  g <- data %>% pull(!!group)
  if (!all(is.finite(g)))
    stop("all group levels must be finite")

  # Statistics for games howell tests
  grp.sizes <- tapply(x, g, length)
  nb.groups <- length(grp.sizes)
  grp.means <- tapply(x, g, mean)
  grp.vars <- tapply(x, g, stats::var)
  # Helper functions
  get_mean_diff <- function(i, j){
    grp.means[i] - grp.means[j]
  }
  get_weltch_sd <- function(i, j){
    sqrt((grp.vars[i]/grp.sizes[i]) + (grp.vars[j]/grp.sizes[j]))
  }
  get_degree_of_freedom <- function(i, j){
    A <- ((grp.vars[i]/grp.sizes[i]) + (grp.vars[j]/grp.sizes[j]))^2
    B <- ((grp.vars[i]/grp.sizes[i])^2)/(grp.sizes[i] - 1)
    C <- ((grp.vars[j]/grp.sizes[j])^2)/(grp.sizes[j] - 1)
    A/(B+C)
  }

  mean.diff <- stats::pairwise.table(
    get_mean_diff, levels(g),
    p.adjust.method = "none"
  ) %>% tidy_squared_matrix()

  weltch.sd <- stats::pairwise.table(
    get_weltch_sd, levels(g),
    p.adjust.method = "none"
  ) %>% tidy_squared_matrix()

  df <- stats::pairwise.table(
    get_degree_of_freedom, levels(g),
    p.adjust.method = "none"
  ) %>% tidy_squared_matrix()

  t <- abs(mean.diff$value)/weltch.sd$value
  p <- stats::ptukey(t*sqrt(2), nb.groups, df$value, lower.tail = FALSE)
  se <- weltch.sd$value*sqrt(0.5)

  q <- stats::qtukey(p = conf.level, nb.groups, df = df$value)
  conf.high <- mean.diff$value + q*se
  conf.low <- mean.diff$value - q*se

  n1 <- grp.sizes[mean.diff$group1]
  n2 <- grp.sizes[mean.diff$group2]

  results <- mean.diff %>%
    rename(estimate = .data$value) %>%
    mutate(
      conf.low = conf.low, conf.high = conf.high,
      se = se, statistic = t, df = df$value, p.adj = p_round(p, digits = 3)
    ) %>%
    add_column(n1 = n1, n2 = n2, .after = "group2") %>%
    add_column(.y. = outcome, .before = "group1") %>%
    add_significance("p.adj") %>%
    mutate(method = "Games-Howell")
  results
}

context("test-games_howell_test")

# Exact data from issue #183 (6 genotypes; Geno2/4/6 have zero variance)
make_183_data <- function(){
  structure(list(
    Genotype = structure(
      c(1L,1L,1L,1L,2L,2L,2L,2L,3L,3L,3L,3L,4L,4L,4L,4L,5L,5L,5L,5L,6L,6L,6L,6L),
      levels = c("Geno1","Geno2","Geno3","Geno4","Geno5","Geno6"), class = "factor"),
    Tillering = c(22L,23L,23L,23L,22L,22L,22L,22L,23L,22L,23L,23L,
                  23L,23L,23L,23L,21L,21L,20L,21L,22L,22L,22L,22L)),
    row.names = c(NA,-24L), class = "data.frame")
}

test_that("games_howell_test does not crash on zero-variance groups (#183)", {
  df <- make_183_data()
  expect_warning(
    res <- games_howell_test(df, Tillering ~ Genotype, detailed = TRUE),
    "zero or undefined variance"
  )
  expect_equal(nrow(res), 15L)                  # all 6*5/2 comparisons present
})

test_that("games_howell_test returns NA only for both-zero-variance pairs (#183)", {
  df <- make_183_data()
  res <- suppressWarnings(games_howell_test(df, Tillering ~ Genotype, detailed = TRUE))
  zero.var <- c("Geno2", "Geno4", "Geno6")
  both_zero <- res$group1 %in% zero.var & res$group2 %in% zero.var
  # undefined pairs -> NA statistic/df/se/p.adj/conf
  expect_true(all(is.na(res$statistic[both_zero])))
  expect_true(all(is.na(res$df[both_zero])))
  expect_true(all(is.na(res$se[both_zero])))
  expect_true(all(is.na(res$p.adj[both_zero])))
  expect_true(all(is.na(res$conf.low[both_zero])))
  # every other comparison is computable (finite)
  expect_true(all(is.finite(res$statistic[!both_zero])))
  expect_true(all(is.finite(res$p.adj[!both_zero])))
})

test_that("games_howell_test handles a single-observation (n=1) group without crashing (#183)", {
  # groups A and B each have n = 1 (var = NA). A df-only alignment still crashed
  # here (`se must be size 10 or 1, ...`) for mixed group sizes; aligning the
  # Welch sd too fixes it.
  df <- data.frame(
    val = c(5,  9,  1,2,3,  6,7,8,  4,5,6),
    grp = rep(c("A","B","C","D","E"), times = c(1,1,3,3,3))
  )
  expect_warning(
    res <- games_howell_test(df, val ~ grp, detailed = TRUE),
    "undefined variance"
  )
  expect_equal(nrow(res), 10L)                       # 5*4/2 comparisons, none dropped
  involves_n1 <- res$group1 %in% c("A","B") | res$group2 %in% c("A","B")
  expect_true(all(is.na(res$statistic[involves_n1])))
  expect_true(all(is.na(res$se[involves_n1])))
  expect_true(all(is.finite(res$statistic[!involves_n1])))   # C/D/E pairs compute
})

test_that("games_howell_test with a single zero-variance group works (#183)", {
  # only group A is constant -> no pair has TWO zero-variance groups
  df <- data.frame(
    val = c(5,5,5,5,  1,2,3,4,  6,7,8,10),
    grp = rep(c("A","B","C"), each = 4)
  )
  res <- games_howell_test(df, val ~ grp, detailed = TRUE)
  expect_equal(nrow(res), 3L)
  expect_true(all(is.finite(res$p.adj)))       # no undefined pair
})

test_that("games_howell_test on valid data is unchanged (#183 no-regression)", {
  res <- ToothGrowth %>% games_howell_test(len ~ dose, detailed = TRUE)
  expect_equal(nrow(res), 3L)
  expect_equal(
    colnames(res),
    c(".y.", "group1", "group2", "n1", "n2", "estimate",
      "conf.low", "conf.high", "se", "statistic", "df", "p.adj", "p.adj.signif", "method")
  )
  expect_true(all(is.finite(res$p.adj)))
})

#' @include utilities.R
NULL
#' Convert a Correlation Test Data Frame into a Correlation Matrix
#'
#' @description Convert a correlation test data frame, returned by the
#'   \code{\link{cor_test}()}, into a correlation matrix format.
#'
#' @param x an object of class \code{cor_test}.
#' @return  Returns a data frame containing the matrix of the correlation
#'   coefficients. The output has an attribute named "pvalue", which contains
#'   the matrix of the correlation test p-values.
#' @seealso \code{\link{cor_mat}()}, \code{\link{cor_test}()}
#' @examples
#' # Pairwise correlation tests between variables
#' #:::::::::::::::::::::::::::::::::::::::::::::::
#' res.cor.test <- mtcars %>%
#'   select(mpg, disp, hp, drat, wt, qsec) %>%
#'   cor_test()
#' res.cor.test
#'
#' # Convert the correlation test into a correlation matrix
#' #:::::::::::::::::::::::::::::::::::::::::::::::
#' res.cor.test %>% as_cor_mat()
#'
#' @export
as_cor_mat <- function(x){

  if(!inherits(x, "cor_test")){
    stop("x should be an object of class cor_test")
  }
  x <- keep_only_tbl_df_classes(x)

  p.mat <- x %>%
    cor_spread(value = "p") %>%
    add_class("pvalue")

  cor.mat <- x %>%
    cor_spread(value = "cor") %>%
    add_class("cor_mat") %>%
    set_attrs(pvalue = p.mat)

  cor.mat
}

#' @include utilities.R friedman_test.R
NULL
#'Nemenyi Post-Hoc Test for Friedman Rank Sums
#'
#'@description Performs the Nemenyi (Wilcoxon-Nemenyi-McDonald-Thompson) all-pairs
#'  post-hoc test for a two-way balanced complete block design, following a
#'  significant Friedman rank sum test. The treatment rank sums are compared
#'  pairwise and the test statistic is referred to the studentized range
#'  distribution, which already accounts for the multiplicity of the all-pairs
#'  comparisons (so, as for \code{\link{tukey_hsd}()}, there is no separate
#'  p-value adjustment step). It should only be used as a post-hoc procedure when
#'  the Friedman test is itself significant.
#'
#'  The Nemenyi test is the rank-based, repeated-measures analogue of Tukey's
#'  HSD. Unlike \code{\link{friedman_conover_test}()} (the Durbin-Conover test),
#'  it does not borrow the residual rank variance and is therefore more
#'  conservative.
#'
#'@inheritParams friedman_conover_test
#'@param detailed logical value. If TRUE, returns the rank-sum estimate and the
#'  test method in the output.
#'@return return a data frame with some of the following columns: \itemize{
#'  \item \code{.y.}: the y (outcome) variable used in the test. \item
#'  \code{group1,group2}: the compared treatments in the pairwise tests. \item
#'  \code{n1,n2}: the number of blocks (subjects) contributing to each treatment.
#'  \item \code{estimate}: the rank-sum difference. \item \code{estimate1,
#'  estimate2}: the rank sums of the two treatments, respectively. \item
#'  \code{statistic}: the studentized-range test statistic. \item \code{p.adj}:
#'  the p-value (already adjusted for multiple comparisons via the studentized
#'  range distribution). \item \code{method}: the statistical test used to
#'  compare groups. \item \code{p.adj.signif}: the significance level of the
#'  adjusted p-values. }
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#'@details For a balanced complete block design with \eqn{b} blocks and \eqn{k}
#'  treatments, the observations within each block are ranked. Let \eqn{R_j} be
#'  the sum of the within-block ranks for treatment \eqn{j}. The pairwise
#'  statistic for treatments \eqn{i} and \eqn{j} is \deqn{q_{ij} =
#'  \frac{|R_i - R_j|}{\sqrt{b\,k\,(k+1)/12}}} and the p-value is obtained from
#'  the studentized range distribution with \eqn{k} groups and infinite degrees
#'  of freedom.
#'
#'@references Nemenyi, P. (1963) Distribution-free Multiple Comparisons. PhD
#'  Thesis, Princeton University.
#'
#'  Hollander, M., Wolfe, D. A. (1973) Nonparametric Statistical Methods. Wiley.
#' @seealso \code{\link{friedman_test}}, \code{\link{friedman_conover_test}},
#'   \code{\link{friedman_effsize}}
#' @examples
#' # A balanced complete block design: 3 treatments measured on 6 subjects
#' df <- data.frame(
#'   id        = factor(rep(1:6, 3)),
#'   treatment = factor(rep(c("A", "B", "C"), each = 6)),
#'   score     = c(4, 6, 3, 5, 4, 5,    7, 8, 6, 7, 9, 6,    6, 9, 7, 8, 8, 9)
#' )
#'
#' # Omnibus Friedman test
#' df %>% friedman_test(score ~ treatment | id)
#'
#' # Nemenyi all-pairs post-hoc
#' df %>% friedman_nemenyi_test(score ~ treatment | id)
#'@name friedman_nemenyi_test
#'@export
friedman_nemenyi_test <- function(data, formula, detailed = FALSE){
  args <- as.list(environment()) %>%
    .add_item(method = "friedman_nemenyi_test")
  if(is_grouped_df(data)){
    results <- data %>%
      doo(.friedman_nemenyi_test, formula)
  }
  else{
    results <- .friedman_nemenyi_test(data, formula)
  }
  if(!detailed){
    results <- results %>%
      select(-any_of(c("method", "estimate", "estimate1", "estimate2")))
  }
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "friedman_nemenyi_test"))
}


.friedman_nemenyi_test <- function(data, formula){
  vars <- get_friedman_vars(formula)
  outcome <- vars$dv; treatment <- vars$within; block <- vars$wid

  data <- data %>%
    select(!!!syms(c(outcome, treatment, block))) %>%
    .as_factor(treatment)
  treatments <- levels(data %>% pull(!!treatment))
  k <- length(treatments)
  if(k < 2){
    stop("the treatment (within-subjects) factor must have at least two levels")
  }

  block.values <- data %>% pull(!!block)
  treat.values <- factor(data %>% pull(!!treatment), levels = treatments)
  outcome.values <- data %>% pull(!!outcome)
  if(any(is.na(outcome.values))){
    stop(
      "friedman_nemenyi_test() does not allow missing values in the outcome: ",
      "an unreplicated complete block design must have one non-missing ",
      "observation per subject and treatment.",
      call. = FALSE
    )
  }
  counts <- table(block.values, treat.values)
  if(any(counts != 1)){
    stop(
      "friedman_nemenyi_test() requires a balanced complete block design: ",
      "each subject (block) must have exactly one observation per treatment ",
      "(no missing or replicated cells).",
      call. = FALSE
    )
  }
  m <- tapply(outcome.values, list(block.values, treat.values), function(z) z[1])
  m <- m[, treatments, drop = FALSE]
  b <- nrow(m)
  if(b < 2){
    stop(
      "friedman_nemenyi_test() requires at least two blocks (subjects); a ",
      "single block does not provide a meaningful post-hoc comparison.",
      call. = FALSE
    )
  }

  R <- t(apply(m, 1, rank))                 # within-block ranks (ties: average)
  rank.sums <- colSums(R)
  names(rank.sums) <- treatments
  # studentized-range scale: depends only on the design (b, k), so the statistic
  # is always defined (no zero-variance degenerate case).
  denom <- sqrt(b * k * (k + 1) / 12)

  compare.estimate <- function(i, j){
    rank.sums[i] - rank.sums[j]
  }
  compare.stats <- function(i, j){
    (rank.sums[i] - rank.sums[j]) / denom
  }
  compare.levels <- function(i, j){
    # studentized range already accounts for all-pairs multiplicity
    stats::ptukey(abs(compare.stats(i, j)), nmeans = k, df = Inf, lower.tail = FALSE)
  }
  ESTIMATE <- stats::pairwise.table(
    compare.estimate, treatments, p.adjust.method = "none"
  ) %>% tidy_squared_matrix("diff")
  PSTAT <- stats::pairwise.table(
    compare.stats, treatments, p.adjust.method = "none"
  ) %>% tidy_squared_matrix("statistic")

  PVAL <- stats::pairwise.table(
    compare.levels, treatments, p.adjust.method = "none"
  ) %>%
    tidy_squared_matrix("p.adj") %>%
    mutate(method = "Nemenyi test", .y. = outcome) %>%
    add_column(statistic = PSTAT$statistic, .before = "p.adj") %>%
    add_column(estimate = ESTIMATE$diff, .before = "group1") %>%
    select(all_of(c(".y.", "group1", "group2", "estimate")), everything()) %>%
    add_significance("p.adj")

  rank.sums1 <- rank.sums[PVAL$group1]
  rank.sums2 <- rank.sums[PVAL$group2]
  PVAL %>%
    add_column(n1 = b, n2 = b, .after = "group2") %>%
    add_column(estimate1 = rank.sums1, estimate2 = rank.sums2, .after = "estimate") %>%
    select(all_of(c(".y.", "group1", "group2", "n1", "n2", "estimate", "estimate1", "estimate2", "statistic", "p.adj", "p.adj.signif", "method")))
}

#' @include utilities.R utilities_two_sample_test.R
#' @importFrom stats sd
#' @importFrom stats var
NULL

#'Compute Cohen's d Measure of Effect Size
#'
#'@description Compute the effect size for t-test. T-test conventional effect
#'  sizes, proposed by Cohen, are: 0.2 (small effect), 0.5 (moderate effect) and
#'  0.8 (large effect).
#'
#'  Cohen's \code{d} is calculated as the difference between means or mean minus
#'  \code{mu} divided by the estimated standardized deviation.
#'
#'  For independent samples t-test, there are two possibilities implemented. If
#'  the t-test did not make a homogeneity of variance assumption, (the Welch
#'  test), the variance term will mirror the Welch test, otherwise a pooled
#'  estimate is used.
#'
#'  If a paired samples t-test was requested, then effect size desired is based
#'  on the standard deviation of the differences.
#'
#'  It can also returns confidence intervals by bootstap.
#'
#'@inheritParams wilcox_effsize
#'@param data a data.frame containing the variables in the formula.
#'@param formula a formula of the form \code{x ~ group} where \code{x} is a
#'  numeric variable giving the data values and \code{group} is a factor with
#'  one or multiple levels giving the corresponding groups. For example,
#'  \code{formula = TP53 ~ cancer_group}.
#'@param paired a logical indicating whether you want a paired test.
#'@param mu theoretical mean, use for one-sample t-test. Default is 0.
#'@param var.equal a logical variable indicating whether to treat the two
#'  variances as being equal. If TRUE then the pooled variance is used to
#'  estimate the variance otherwise the Welch (or Satterthwaite) approximation
#'  to the degrees of freedom is used. Used only for unpaired or independent samples test.
#'@param hedges.correction logical indicating whether apply the Hedges
#'  correction by multiplying the usual value of Cohen's d by
#'  \code{(N-3)/(N-2.25)} (for unpaired t-test) and by \code{(n1-2)/(n1-1.25)} for paired t-test;
#'  where \code{N} is the total size of the two groups being compared (N = n1 +
#'  n2).
#'@details Quantification of the effect size magnitude is performed using the
#'  thresholds defined in Cohen (1992). The magnitude is assessed using the
#'  thresholds provided in (Cohen 1992), i.e. \code{|d| < 0.2} "negligible",
#'  \code{|d| < 0.5} "small", \code{|d| < 0.8} "medium", otherwise "large".
#'@references \itemize{ \item Cohen, J. (1988). Statistical power analysis for
#'  the behavioral sciences (2nd ed.). New York:Academic Press. \item Cohen, J.
#'  (1992). A power primer. Psychological Bulletin, 112, 155-159. \item Hedges,
#'  Larry & Olkin, Ingram. (1985). Statistical Methods in Meta-Analysis.
#'  10.2307/1164953. \item Navarro, Daniel. 2015. Learning Statistics with R: A
#'  Tutorial for Psychology Students and Other Beginners (Version 0.5). }
#'@return return a data frame with some of the following columns: \itemize{
#'  \item \code{.y.}: the y variable used in the test. \item
#'  \code{group1,group2}: the compared groups in the pairwise tests. \item
#'  \code{n,n1,n2}: Sample counts. \item \code{effsize}: estimate of the effect
#'  size (\code{d} value). \item \code{magnitude}: magnitude of effect size.
#'  \item \code{conf.low,conf.high}: lower and upper bound of the effect size
#'  confidence interval.}
#' @examples
#' # One-sample t test effect size
#' ToothGrowth %>% cohens_d(len ~ 1, mu = 0)
#'
#' # Two indepedent samples t-test effect size
#' ToothGrowth %>% cohens_d(len ~ supp, var.equal = TRUE)
#'
#' # Paired samples effect size
#' df <- data.frame(
#'   id = 1:5,
#'   pre  = c(110, 122, 101, 120, 140),
#'   post = c(150, 160, 110, 140, 155)
#' )
#' df <- df %>% gather(key = "treatment", value = "value", -id)
#' head(df)
#'
#' df %>% cohens_d(value ~ treatment, paired = TRUE)
#'@export
cohens_d <- function(data, formula, comparisons = NULL, ref.group = NULL, paired = FALSE, mu = 0,
                     var.equal = FALSE, hedges.correction = FALSE,
                     ci = FALSE, conf.level = 0.95,  ci.type = "perc", nboot = 1000){
  env <- as.list(environment())
  args <- env %>% .add_item(method = "cohens_d")
  params <- env %>%
    remove_null_items() %>%
    add_item(method = "cohens.d", detailed = FALSE)

  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  number.of.groups <- guess_number_of_groups(data, group)
  if(number.of.groups > 2 & !is.null(ref.group)){
    if(ref.group %in% c("all", ".all.")){
      params$data <- create_data_with_all_ref_group(data, outcome, group)
      params$ref.group <- "all"
    }
  }
  test.func <- two_sample_test
  if(number.of.groups > 2) test.func <- pairwise_two_sample_test
  res <- do.call(test.func, params) %>%
    select(.data$.y., .data$group1, .data$group2, .data$estimate, everything()) %>%
    rename(effsize = .data$estimate) %>%
    mutate(magnitude = get_cohens_magnitude(.data$effsize)) %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "cohens_d"))
  res
}



# Cohens d core function -------------------------------
cohens.d <- function(x, y = NULL, mu = 0, paired = FALSE, var.equal = FALSE,
                     hedges.correction = FALSE,
                     ci = FALSE, conf.level = 0.95,  ci.type = "perc", nboot = 1000, ...){
  check_two_samples_test_args(
    x = x, y = y, mu = mu, paired = paired,
    conf.level = conf.level
  )

  if (!is.null(y)) {
    DNAME <- paste(deparse(substitute(x)), "and", deparse(substitute(y)))
    if (paired) {
      OK <- complete.cases(x, y)
      x <- x[OK] - y[OK]
      y <- NULL
      mu <- 0
      METHOD <- "Paired T-test"
    }
    else {
      x <- x[is.finite(x)]
      y <- y[is.finite(y)]
      METHOD <- "Independent T-test"
    }
  }
  else {
    DNAME <- deparse(substitute(x))
    METHOD <- "One-sample T-test"
    x <- x[is.finite(x)]
  }

  if(is.null(y)){
   formula <- x ~ 1
   y <- rep(mu, length(x))
  }
  else{
    group <- rep(c("grp1", "grp2"), times = c(length(x), length(y))) %>%
      factor()
    x <- c(x, y)
    y <- group
    formula <- x ~ y
  }
  data <- data.frame(x, y)
  results <- get_cohens_d(
    data, formula, paired = paired, var.equal = var.equal,
    mu = mu, hedges.correction = hedges.correction
    )
  # Confidence interval of the effect size r
  if (ci == TRUE) {
    stat.func <- function(data, subset) {
      get_cohens_d(
        data, formula = formula, subset = subset,
        paired = paired, var.equal = var.equal,
        mu = mu, hedges.correction = hedges.correction
      )$d
    }
    CI <- get_boot_ci(
      data, stat.func, conf.level = conf.level,
      type = ci.type, nboot = nboot
    )
    results <- results %>% mutate(conf.low = CI[1], conf.high = CI[2])
  }
  RVAL <- list(statistic = NA, p.value = NA, null.value = mu, method = METHOD,
               data.name = DNAME, estimate = results$d)
  if (ci) {
    attr(CI, "conf.level") <- conf.level
    RVAL <- c(RVAL, list(conf.int = CI))
  }
  names(RVAL$estimate) <- "Cohen's d"
  class(RVAL) <- "htest"
  RVAL
}

# Helper to compute cohens d -----------------------------------
get_cohens_d <- function(data, formula, subset = NULL, paired = FALSE, mu = 0, var.equal = FALSE,
                         hedges.correction = FALSE){
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  if(!is.null(subset)) data <- data[subset, ]
  if(.is_empty(group))
    number.of.groups <- 1  # Null model
  else
    number.of.groups <- data %>%
    pull(group) %>% unique() %>% length()

  unpaired.two.samples <- paired == FALSE & number.of.groups == 2

  if(number.of.groups == 1){
    x <- data %>% pull(outcome)
    d <- one_sample_d(x, mu)
  }
  else if(number.of.groups == 2){
    groups <- data %>% pull(group)
    data <- data %>% split(groups)
    x <- data[[1]] %>% pull(outcome)
    y <- data[[2]] %>% pull(outcome)
    if(paired){
      d <- paired_sample_d(x, y)
    }
    else{
      d <- two_independent_sample_d(x, y, var.equal)
    }
  }
  else{
    stop("The grouping factors contain more than 2 levels.")
  }
  # Hedge's correction
  if(hedges.correction){
    if(paired){
      n <- length(x)
      d <- d*(n - 2)/(n - 1.25)
    }
    else if (unpaired.two.samples){
      n <- length(x) + length(y)
      d <- d * (n - 3)/(n - 2.25)
    }
    else{
      stop(
        "Hedge's Correction for One Sample Test is not supported.\n",
        "Please use `hedges.correction = FALSE` (default) for one sample test.",
        call. = FALSE
        )
    }
  }

  tibble(
    d,
    magnitude = get_cohens_magnitude(d)
  )
}

one_sample_d <- function(x, mu = 0){
  (mean(x) - mu)/sd(x)
}
two_independent_sample_d <- function(x, y, var.equal = TRUE){
  if(var.equal){
    squared.dev <- (c(x - mean(x), y - mean(y)))^2
    n <- length(squared.dev)
    SD <- sqrt(sum(squared.dev)/(n-2))
  }
  else {
    SD <- sqrt((var(x) + var(y))/2)
  }
  mean.diff <- mean(x) - mean(y)
  mean.diff/SD
}
paired_sample_d <- function(x, y){
  mean(x-y)/sd(x-y)
}


get_cohens_magnitude <- function(d){
  magnitude.levels = c(0.2,0.5,0.8)
  magnitude = c("negligible","small","moderate","large")
  d.index <- findInterval(abs(d), magnitude.levels)+1
  magnitude <- factor(magnitude[d.index], levels = magnitude, ordered = TRUE)
  magnitude
}

context("test-cohens_d")

oj <- ToothGrowth$len[ToothGrowth$supp == "OJ"]
vc <- ToothGrowth$len[ToothGrowth$supp == "VC"]

test_that("cohens_d applies mu for an independent two-sample test (#200)", {
  d0 <- ToothGrowth %>% cohens_d(len ~ supp)
  d3 <- ToothGrowth %>% cohens_d(len ~ supp, mu = 3)
  expect_false(isTRUE(all.equal(d0$effsize, d3$effsize)))            # mu now changes the result
  expected <- ((mean(oj) - mean(vc)) - 3) / sqrt((var(oj) + var(vc)) / 2)
  expect_equal(as.numeric(d3$effsize), expected, tolerance = 1e-4)
})

test_that("cohens_d applies mu for a paired two-sample test (#200)", {
  d0 <- ToothGrowth %>% cohens_d(len ~ supp, paired = TRUE)
  d3 <- ToothGrowth %>% cohens_d(len ~ supp, paired = TRUE, mu = 3)
  expect_false(isTRUE(all.equal(d0$effsize, d3$effsize)))
  diffs <- oj - vc
  expect_equal(as.numeric(d3$effsize), (mean(diffs) - 3) / sd(diffs), tolerance = 1e-4)
})

test_that("cohens_d default (mu = 0) is the standard Cohen's d (no regression, #200)", {
  d_ind <- ToothGrowth %>% cohens_d(len ~ supp)
  expect_equal(as.numeric(d_ind$effsize),
               (mean(oj) - mean(vc)) / sqrt((var(oj) + var(vc)) / 2), tolerance = 1e-4)
  diffs <- oj - vc
  d_pr <- ToothGrowth %>% cohens_d(len ~ supp, paired = TRUE)
  expect_equal(as.numeric(d_pr$effsize), mean(diffs) / sd(diffs), tolerance = 1e-4)
  # one-sample behaviour is unchanged (mu already worked there)
  os <- ToothGrowth %>% cohens_d(len ~ 1, mu = 20)
  expect_equal(as.numeric(os$effsize), (mean(ToothGrowth$len) - 20) / sd(ToothGrowth$len), tolerance = 1e-4)
})

#' @include utilities.R
NULL
#' Add P-value Significance Symbols
#' @description Add p-value significance symbols into a data frame.
#' @param data a data frame containing a p-value column.
#' @param p.col column name containing p-values.
#' @param output.col the output column name to hold the adjusted p-values.
#' @param cutpoints numeric vector used for intervals.
#' @param symbols character vector, one shorter than cutpoints, used as
#'   significance symbols.
#'
#' @return a data frame
#'
#' @examples
#' # Perform pairwise comparisons and adjust p-values
#' ToothGrowth %>%
#'  t_test(len ~ dose) %>%
#'  adjust_pvalue() %>%
#'  add_significance("p.adj")
#'
#' @rdname add_significance
#' @export
add_significance <- function(
  data, p.col = NULL, output.col = NULL,
  cutpoints = c(0, 0.0001, 0.001, 0.01, 0.05,  1),
  symbols = c("****", "***", "**", "*",  "ns")
)
{
  .attributes <- get_test_attributes(data)
  if(is.null(p.col))
    p.col <- data %>% p_detect("p.adj")
  if(is.null(p.col))
    p.col <- data %>% p_detect("p")
  if(is.null(p.col))
    return(data)
  else if(!(p.col %in% colnames(data)))
    stop("The column ", p.col, " does not exist in the data")
  if(is.null(output.col))
    output.col <- paste0(p.col, ".signif")
  .p.signif <- data %>% pull(!!p.col) %>%
    stats::symnum(cutpoints = cutpoints, symbols = symbols) %>%
    as.character()
  data %>%
    keep_only_tbl_df_classes() %>%
    mutate(!!output.col := .p.signif) %>%
    set_test_attributes(.attributes)
}

#' @include utilities.R
NULL
#'Test for Trend in Proportions
#'@description Performs chi-squared test for trend in proportion. This test is
#'  also known as Cochran-Armitage trend test.
#'
#'  Wrappers around the R base function \code{\link[stats]{prop.trend.test}()} but
#'  returns a data frame for easy data visualization.
#'@param xtab a cross-tabulation (or contingency table) with two columns and
#'  multiple rows (rx2 design). The columns give the counts of successes and
#'  failures respectively.
#' @param score group score. If \code{NULL}, the default is group number.
#'
#'@return return a data frame with some the following columns: \itemize{ \item
#'  \code{n}: the number of participants.  \item \code{statistic}: the value of
#'  Chi-squared trend test statistic. \item \code{df}: the degrees of
#'  freedom.
#'  \item \code{p}: p-value. \item
#'  \code{method}: the used statistical test. \item \code{p.signif}: the significance level of p-values and adjusted p-values,
#'  respectively.}
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#' @examples
#' # Proportion of renal stone (calculi) across age
#' #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Data
#' xtab <- as.table(rbind(
#'   c(384, 536, 335),
#'   c(951, 869, 438)
#' ))
#' dimnames(xtab) <- list(
#'   stone = c("yes", "no"),
#'   age = c("30-39", "40-49", "50-59")
#' )
#' xtab
#' # Compare the proportion of survived between groups
#' prop_trend_test(xtab)

#' @export
prop_trend_test <- function(xtab, score = NULL){
  args <- as.list(environment()) %>%
    add_item(method = "chisq_trend_test")
  if(is.data.frame(xtab)) xtab <- as.matrix(xtab)

  if(inherits(xtab, c("matrix", "table"))){
    if(ncol(xtab) > 2 & nrow(xtab) == 2) xtab <- t(xtab)
  }
  total <- sum(xtab)
  events <- xtab[, 1]
  trials <- rowSums(xtab)
  if(is.null(score)) score <- seq_along(events)

  results <- stats::prop.trend.test(events, trials, score) %>%
    as_tidy_stat() %>%
    add_significance("p") %>%
    mutate(method = "Chi-square trend test") %>%
    add_columns(n = total, .before = 1) %>%
    select(.data$n, .data$statistic, .data$p, .data$p.signif, everything())

  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "chisq_trend_test"))
}

#' @include utilities.R
NULL
#' Subset Correlation Matrix
#' @name cor_select
#' @param x a correlation matrix. Particularly, an object of class \code{cor_mat}.
#' @param vars a character vector containing the variable names of interest.
#' @param ... One or more unquoted expressions (or variable names) separated by
#'  commas. Used to select variables of interest.
#'@return a data frame
#'@seealso \code{\link{cor_mat}()}, \code{\link{pull_triangle}()}, \code{\link{replace_triangle}()}
#' @examples
#' # Compute correlation matrix
#' #::::::::::::::::::::::::::::::::::::::::::
#' cor.mat <- mtcars %>%
#'   select(mpg, disp, hp, drat, wt, qsec) %>%
#'   cor_mat()
#'
#' # Subsetting correlation matrix
#' #::::::::::::::::::::::::::::::::::::::::::
#'
#' # Select some variables of interest
#' cor.mat %>%
#'   cor_select(mpg, drat, wt)
#'
#' # Remove variables
#' cor.mat %>%
#'   cor_select(-mpg, -wt)
#'
#' @export
cor_select <- function(x, ..., vars = NULL){

  vars <- x %>% get_selected_vars(..., vars = vars) %>%
    setdiff("rowname")

  if(!.is_empty(vars)){
    # Filter the correlation matrix
    vars <- unique(vars)
    x <- x %>% subset_matrix(vars)
    # Filter p-values
    pvalue <- x %>% attr("pvalue")
    if(!is.null(pvalue)){
      pvalue <- pvalue %>% subset_matrix(vars)
      x <- x %>% set_attrs(pvalue = pvalue) %>%
        add_class("cor_mat")
    }
  }

  x
}

#' @include utilities.R friedman_test.R
NULL

#' Friedman Test Effect Size (Kendall's W Value)
#'
#'@description Compute the effect size estimate (referred to as \code{w}) for
#'  Friedman test: \code{W = X2/N(K-1)}; where \code{W} is the Kendall's W
#'  value; \code{X2} is the Friedman test statistic value; \code{N} is the sample
#'  size. \code{k} is the number of measurements per subject.
#'
#'  The Kendall’s W coefficient assumes the value from 0 (indicating no
#'  relationship) to 1 (indicating a perfect relationship).
#'
#'  Kendalls uses the Cohen’s interpretation guidelines of \code{0.1 - < 0.3} (small
#'  effect), \code{0.3 - < 0.5} (moderate effect) and \code{>= 0.5} (large
#'  effect)
#'
#'  Confidence intervals are calculated by bootstap.
#'
#'@inheritParams friedman_test
#'@inheritParams wilcox_effsize
#'@param ... other arguments passed to the function \code{\link[stats]{friedman.test}()}
#'@return return a data frame with some of the following columns: \itemize{
#'  \item \code{.y.}: the y variable used in the test. \item \code{n}: Sample
#'  counts. \item \code{effsize}: estimate of the effect size. \item
#'  \code{magnitude}: magnitude of effect size. \item \code{conf.low,conf.high}:
#'  lower and upper bound of the effect size confidence interval.}
#'
#'@references Maciej Tomczak and Ewa Tomczak. The need to report effect size
#'  estimates revisited. An overview of some recommended measures of effect
#'  size. Trends in Sport Sciences. 2014; 1(21):19-25.
#'
#' @examples
#' # Load data
#' #:::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth %>%
#'     filter(supp == "VC") %>%
#'     mutate(id = rep(1:10, 3))
#' head(df)
#'
#' # Friedman test effect size
#' #:::::::::::::::::::::::::::::::::::::::::
#' df %>% friedman_effsize(len ~ dose | id)

#' @export
friedman_effsize <- function(data, formula, ci = FALSE, conf.level = 0.95,  ci.type = "perc", nboot = 1000, ...){
  args <- as.list(environment()) %>%
    .add_item(method = "friedman_effsize")
  if(is_grouped_df(data)){
    results <- data %>%
      doo(
        .friedman_effsize, formula, ci = ci, conf.level = conf.level,
        ci.type = ci.type, nboot = nboot, ...
      )
  }
  else{
    results <- .friedman_effsize(
      data, formula, ci = ci, conf.level = conf.level,
      ci.type = ci.type, nboot = nboot, ...
    )
  }
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "friedman_effsize"))
}


.friedman_effsize <- function(data, formula, ci = FALSE, conf.level = 0.95,  ci.type = "perc", nboot = 1000, ...){
  results <- kendall_w(data, formula, ...)
  # Confidence interval of the effect size
  if (ci == TRUE) {
    # Paired data, bootstrap should be performed on wide data
    vars <- get_friedman_vars(formula)
    data.wide <- data %>%
      select(!!!syms(c(vars$wid, vars$dv, vars$within))) %>%
      spread(key = vars$within, value = vars$dv)
    # Boot function
    stat.func <- function(data.wide, subset) {
      if(!is.null(subset)) data.wide <- data.wide[subset, ]
      data.long <- data.wide %>%
        mutate(!!vars$wid := 1:nrow(data.wide)) %>%
        gather(key = !!vars$within, value = !!vars$dv, - !!vars$wid)
      kendall_w(data.long, formula)$effsize
    }
    CI <- get_boot_ci(
      data.wide, stat.func, conf.level = conf.level,
      type = ci.type, nboot = nboot
    )
    results <- results %>%
      add_columns(conf.low = CI[1], conf.high = CI[2], .after = "effsize")
  }
  results %>%
    mutate(magnitude = get_kendall_w_magnitude(.data$effsize))
}


kendall_w <- function(data, formula, subset = NULL, ...){
  if(!is.null(subset)) data <- data[subset, ]
  res.f <- friedman_test(data, formula, ...)
  x2 <- res.f$statistic
  nb.samples <- res.f$n
  k <- nrow(data)/nb.samples # number of measurements per sample
  w <- x2 / (nb.samples * (k-1))
  tibble(
    .y. = get_formula_left_hand_side(formula),
    n = nb.samples,
    effsize = w,
    method = "Kendall W"
    )
}


get_kendall_w_magnitude <- function(d){
  magnitude.levels = c(0.3, 0.5, Inf)
  magnitude = c("small","moderate","large")
  d.index <- findInterval(abs(d), magnitude.levels)+1
  magnitude <- factor(magnitude[d.index], levels = magnitude, ordered = TRUE)
  magnitude
}

#' @include utilities.R dunn_test.R
NULL
#'Conover's All-Pairs Rank Comparison Test
#'
#'@description Performs Conover's test (also known as the Conover-Iman test) for
#'  pairwise multiple comparisons of the ranked data, following a significant
#'  Kruskal-Wallis test. It is closely related to \code{\link{dunn_test}()}, but
#'  uses the pooled within-group rank variance and refers the test statistic to a
#'  \emph{t}-distribution (with \eqn{N - k} degrees of freedom) instead of the
#'  standard normal distribution. The Conover-Iman test is generally more
#'  powerful than Dunn's test, but should only be used as a post-hoc procedure
#'  when the Kruskal-Wallis test is itself significant (Conover, 1999).
#'
#'  If a reference group is specified (via \code{ref.group}), then each of the
#'  remaining group levels is compared only to the reference (control) group, and
#'  the p-value adjustment for multiple comparisons is computed over only these
#'  \code{k - 1} comparisons (instead of all \code{k(k - 1)/2} pairwise
#'  comparisons), exactly as for \code{\link{dunn_test}()}.
#'
#'@inheritParams dunn_test
#'@param ref.group a character string specifying the reference group. If
#'  specified, for a given grouping variable, each of the group levels will be
#'  compared to the reference (control) group, and the p-value adjustment is
#'  computed over only these comparisons. Note that, like \code{dunn_test()},
#'  \code{conover_test()} does not support \code{ref.group = "all"}.
#'@return return a data frame with some of the following columns: \itemize{
#'  \item \code{.y.}: the y (outcome) variable used in the test. \item
#'  \code{group1,group2}: the compared groups in the pairwise tests. \item
#'  \code{n1,n2}: Sample counts. \item \code{estimate}: mean ranks difference.
#'  \item \code{estimate1, estimate2}: show the mean rank values of the two
#'  groups, respectively. \item \code{statistic}: Test statistic (t-value) used
#'  to compute the p-value. \item \code{df}: degrees of freedom (\eqn{N - k},
#'  the same for every comparison). \item \code{p}: p-value. \item \code{p.adj}:
#'  the adjusted p-value. \item \code{method}: the statistical test used to
#'  compare groups. \item \code{p.adj.signif}: the significance level of the
#'  adjusted p-values. }
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments.
#'@details The Conover-Iman pairwise statistic for comparing groups \eqn{i} and
#'  \eqn{j} is \deqn{t_{ij} = \frac{\bar{R}_i - \bar{R}_j}{\sqrt{S^2 \,
#'  \frac{N - 1 - H}{N - k} \left(\frac{1}{n_i} + \frac{1}{n_j}\right)}}} where
#'  \eqn{\bar{R}} are the mean ranks, \eqn{H} is the (tie-corrected)
#'  Kruskal-Wallis statistic, \eqn{N} is the total sample size, \eqn{k} is the
#'  number of groups, and \eqn{S^2} is the variance of the ranks
#'  (\eqn{S^2 = N(N+1)/12} when there are no ties; otherwise
#'  \eqn{S^2 = \frac{1}{N - 1}\left(\sum r^2 - \frac{N(N+1)^2}{4}\right)}). The
#'  statistic is referred to a \emph{t}-distribution with \eqn{N - k} degrees of
#'  freedom.
#'
#'@references Conover, W. J. (1999) Practical Nonparametric Statistics, 3rd
#'  edition. Wiley.
#'
#'  Conover, W. J. and Iman, R. L. (1979) On multiple-comparisons procedures.
#'  Technical Report LA-7677-MS, Los Alamos Scientific Laboratory.
#' @seealso \code{\link{dunn_test}}, \code{\link{kruskal_test}}
#' @examples
#' # Simple test
#' ToothGrowth %>% conover_test(len ~ dose)
#'
#' # Comparison against a reference (control) group
#' # each group is compared to the reference; the p-value
#' # adjustment corrects for only these k - 1 comparisons
#' ToothGrowth %>% conover_test(len ~ dose, ref.group = "0.5")
#'
#' # Grouped data
#' ToothGrowth %>%
#'   group_by(supp) %>%
#'   conover_test(len ~ dose)
#'@export
conover_test <- function(data, formula, p.adjust.method = "holm", ref.group = NULL, detailed = FALSE){
  if(!is.null(ref.group)) ref.group <- as.character(ref.group)
  args <- as.list(environment()) %>%
    .add_item(method = "conover_test")
  if(is_grouped_df(data)){
    results <- data %>%
      doo(.conover_test, formula, p.adjust.method, ref.group = ref.group)
  }
  else{
    results <- .conover_test(data, formula, p.adjust.method, ref.group = ref.group)
  }

  if(!detailed){
    results <- results %>%
      select(-any_of(c("method", "estimate", "estimate1", "estimate2")))
  }
  results %>%
    set_attrs(args = args) %>%
    add_class(c("rstatix_test", "conover_test"))
}


.conover_test <- function(data, formula, p.adjust.method = "holm", ref.group = NULL){
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  number.of.groups <- guess_number_of_groups(data, group)
  if(number.of.groups == 1){
    stop("all observations are in the same group")
  }

  data <- data %>%
    select(!!!syms(c(outcome, group))) %>%
    get_complete_cases() %>%
    .as_factor(group)

  x <- data %>% pull(!!outcome)
  g <- data %>% pull(!!group)
  group.size <- data %>% get_group_size(group)
  if (!all(is.finite(g)))
    stop("all group levels must be finite")
  if(!is.null(ref.group)){
    ref.group <- as.character(ref.group)
    if(!(ref.group %in% levels(g))){
      stop(
        "Specified reference group ('", ref.group, "') is not a level of the ",
        "grouping variable. Valid levels are: ",
        paste(levels(g), collapse = ", "), ".",
        call. = FALSE
      )
    }
  }

  x.rank <- rank(x)
  mean.ranks <- tapply(x.rank, g, mean, na.rm = TRUE)
  grp.sizes <- tapply(x, g, length)
  n <- length(x)
  k <- nlevels(g)
  df.value <- n - k

  # Tie-corrected Kruskal-Wallis statistic H and the pooled rank variance S^2,
  # following Conover (1999). S^2 reduces to N(N+1)/12 when there are no ties.
  rank.sums <- tapply(x.rank, g, sum)
  H.uncorrected <- (12 / (n * (n + 1))) * sum(rank.sums^2 / grp.sizes) - 3 * (n + 1)
  ties <- table(x.rank)
  tie.correction <- 1 - sum(ties^3 - ties) / (n^3 - n)
  H <- H.uncorrected / tie.correction
  S2 <- (1 / (n - 1)) * (sum(x.rank^2) - n * (n + 1)^2 / 4)
  pooled.scale <- S2 * ((n - 1 - H) / (n - k))

  # The Conover-Iman statistic is undefined when there is no residual degree of
  # freedom (N - k < 1, e.g. one observation per group) or no rank variability
  # (all values tied, or every group internally constant), in which case the
  # pooled scale is zero or non-finite and the t-ratio would be NaN/Inf. Fail
  # with a clear message rather than a cryptic downstream error.
  if(df.value < 1 || !is.finite(pooled.scale) || pooled.scale <= 0){
    stop(
      "Conover test is undefined for these data: there is no residual variability ",
      "in the ranks (e.g. each group has a single observation, or all values are ",
      "tied). The Kruskal-Wallis test and its post-hoc require N - k >= 1 and some ",
      "within-group rank variation.",
      call. = FALSE
    )
  }

  compare.meanrank <- function(i, j){
    mean.ranks[i] - mean.ranks[j]
  }
  compare.stats <- function(i, j){
    dif <- mean.ranks[i] - mean.ranks[j]
    B <- (1 / grp.sizes[i] + 1 / grp.sizes[j])
    dif / sqrt(pooled.scale * B)
  }
  compare.levels <- function(i, j){
    tval <- compare.stats(i, j)
    2 * stats::pt(abs(tval), df = df.value, lower.tail = FALSE)
  }
  ESTIMATE <- stats::pairwise.table(
    compare.meanrank, levels(g),
    p.adjust.method = "none"
  ) %>% tidy_squared_matrix("diff")

  PSTAT <- stats::pairwise.table(
    compare.stats, levels(g),
    p.adjust.method = "none"
  ) %>% tidy_squared_matrix("statistic")

  # Assemble the full per-pair table (estimate, statistic, raw p) BEFORE adjusting
  # p-values, so that, when a reference group is specified, the multiple-comparison
  # adjustment is applied to only the retained (k - 1) comparisons (cf. dunn_test).
  PVAL <- stats::pairwise.table(
    compare.levels, levels(g),
    p.adjust.method = "none"
  ) %>%
    tidy_squared_matrix("p") %>%
    mutate(method = "Conover test", .y. = outcome, df = df.value) %>%
    add_column(statistic = PSTAT$statistic, .before = "p") %>%
    add_column(estimate = ESTIMATE$diff, .before = "group1") %>%
    select(all_of(c(".y.", "group1", "group2", "estimate")), everything())

  # Reference group: keep only comparisons against the reference and orient it as
  # group1 (consistent with t_test()/wilcox_test()/dunn_test()). The per-pair
  # t-statistic and the two-sided raw p are unchanged; only the set of comparisons
  # (and thus the p-value adjustment) differs.
  if(!is.null(ref.group)){
    PVAL <- PVAL %>%
      filter(.data$group1 == ref.group | .data$group2 == ref.group)
    to.flip <- PVAL$group2 == ref.group
    if(any(to.flip)){
      g1 <- PVAL$group1; g2 <- PVAL$group2
      PVAL$group1[to.flip] <- g2[to.flip]
      PVAL$group2[to.flip] <- g1[to.flip]
      # estimate (mean-rank difference) and statistic (t) are direction-dependent
      PVAL$estimate[to.flip]  <- -PVAL$estimate[to.flip]
      PVAL$statistic[to.flip] <- -PVAL$statistic[to.flip]
    }
  }

  PVAL <- PVAL %>%
    adjust_pvalue(method = p.adjust.method) %>%
    add_significance("p.adj")

  n1 <- group.size[PVAL$group1]
  n2 <- group.size[PVAL$group2]
  mean.ranks1 <- mean.ranks[PVAL$group1]
  mean.ranks2 <- mean.ranks[PVAL$group2]
  PVAL %>%
    add_column(n1 = n1, n2 = n2, .after = "group2") %>%
    add_column(estimate1 = mean.ranks1, estimate2 = mean.ranks2, .after = "estimate") %>%
    select(all_of(c(".y.", "group1", "group2", "n1", "n2", "estimate", "estimate1", "estimate2", "statistic", "df", "p", "p.adj", "p.adj.signif", "method")))
}

#' Programming with rstatix (tidy evaluation)
#'
#' @description
#' How to use \code{rstatix} functions programmatically — i.e. when the variable
#' names are held in character strings or passed into your own wrapper functions,
#' as is common in package development, loops, and Shiny apps.
#'
#' \code{rstatix} has two kinds of interfaces, and each supports a standard way of
#' "programming over variables":
#'
#' \itemize{
#'   \item \strong{Selection interface} (functions that select columns through
#'   \code{...} or \code{vars=}, e.g. \code{\link{cor_test}()},
#'   \code{\link{get_summary_stats}()}, \code{\link{cor_mat}()},
#'   \code{\link{freq_table}()}). These support full tidy-evaluation: bare names,
#'   the injection operators \code{!!}/\code{!!!}, the embracing operator
#'   \code{\{\{ \}\}} inside your own functions, a character vector via
#'   \code{vars=}, and tidyselect helpers such as \code{all_of()}/\code{any_of()}.
#'
#'   \item \strong{Formula interface} (tests that take a \code{formula}, e.g.
#'   \code{\link{t_test}()}, \code{\link{wilcox_test}()}, \code{\link{kruskal_test}()},
#'   \code{\link{anova_test}()}, \code{\link{friedman_test}()}). A formula is an
#'   ordinary R object, so build it from strings with \code{\link[stats]{reformulate}()}
#'   or \code{stats::as.formula(paste(...))} and pass it in.
#' }
#'
#' @details
#' Injecting a string directly into a raw formula (e.g. \code{t_test(df, y ~ \{\{var\}\})})
#' is \strong{not} supported: a formula is captured as a syntax tree, not a quosure,
#' so the embracing/injection operators do not apply there. Build the formula
#' instead with \code{reformulate(rhs, lhs)} — see the examples.
#'
#' In examples below, helpers that \code{rstatix} does not re-export are namespaced
#' (\code{rlang::sym}, \code{dplyr::all_of}, \code{dplyr::across}); attach
#' \code{rlang}/\code{dplyr} and you can drop the prefixes.
#'
#' @examples
#' # Selection interface -----------------------------------------------------
#'
#' # A variable name held in a string, injected with !!
#' x <- "mpg"; y <- "wt"
#' mtcars %>% cor_test(!!rlang::sym(x), !!rlang::sym(y))
#'
#' # Several names at once, spliced with !!!
#' vars <- c("mpg", "disp", "hp")
#' mtcars %>% cor_test(!!!rlang::syms(vars))
#'
#' # A character vector via `vars =` (no rlang needed)
#' mtcars %>% cor_test(vars = c("mpg", "disp"))
#'
#' # tidyselect helpers
#' iris %>% get_summary_stats(dplyr::all_of(c("Sepal.Length", "Sepal.Width")))
#'
#' # Your own wrapper function: embrace the argument with {{ }}
#' my_summary <- function(data, var) {
#'   data %>% get_summary_stats({{ var }}, type = "mean_sd")
#' }
#' my_summary(iris, Sepal.Length)
#'
#' # Formula interface -------------------------------------------------------
#'
#' # Build the formula from strings with reformulate(rhs, lhs)
#' outcome <- "len"; group <- "supp"
#' ToothGrowth %>% t_test(reformulate(group, outcome))
#'
#' # The same inside a wrapper function
#' my_test <- function(data, outcome, group) {
#'   data %>% t_test(reformulate(group, outcome))
#' }
#' my_test(ToothGrowth, "len", "supp")
#'
#' # Programmatic grouping + a built formula (a common end-to-end pattern)
#' gv <- "supp"
#' ToothGrowth %>%
#'   group_by(dplyr::across(dplyr::all_of(gv))) %>%
#'   t_test(reformulate("dose", "len"))
#'
#' @seealso \code{\link{cor_test}()}, \code{\link{get_summary_stats}()},
#'   \code{\link{t_test}()}.
#' @name rstatix-programming
NULL

#' @include utilities.R
NULL

#' Remove Non-Significant from Statistical Tests
#' @description Filter out non-significant (NS) p-values from a statistical
#'   test. Can detect automatically p-value columns
#' @param stat.test statistical test results returned by \code{rstatix}
#'   functions or any data frame containing a p-value column.
#' @param col (optional) character specifying the column containing the p-value
#'   or the significance information, to be used for the filtering step.
#'   Possible values include: \code{"p"}, \code{"p.adj"}, \code{"p.signif"},
#'   \code{"p.adj.signif"}. If missing, the function will automatically look for
#'   p.adj.signif, p.adj, p.signif, p in this order.
#' @param signif.cutoff the significance cutoff; default is 0.05. Significance
#'   is declared at \code{p-value <= signif.cutoff}
#' @return a data frame
#' @examples
#' # Statistical test
#' stat.test <- PlantGrowth %>% wilcox_test(weight ~ group)
#' # Remove ns: automatic detection of p-value columns
#' stat.test %>% remove_ns()
#' # Remove ns by the column p
#' stat.test %>% remove_ns(col ="p")
#' @export
remove_ns <- function(stat.test, col = NULL, signif.cutoff = 0.05){

  if(is.null(col)) col <- "any"
  else if(is.na(col)) col <- "any"
  else if(is.logical(col) ){
    if(is.na(col)) col <- "any"
    else if(col == TRUE) col <- "any"
    else if(col == FALSE) return(stat.test)
  }
  if(col == "any"){
    p.adj <- p_adj_names()
    p.adj.signif <- paste0(p.adj, ".signif")
    p <- p_names()
    p.signif <- paste0(p, ".signif")
    possible.cols <- c(p.adj.signif, p.adj, p.signif, p)
    if(!missing(signif.cutoff)) {
      # numeric columns are checked first
      possible.cols <- c(p.adj, p, p.adj.signif, p.signif)
    }
    col <- intersect(possible.cols, colnames(stat.test))
    if(length(col) > 1) col <- col[1]
    else if(length(col) == 0) {
      warning("Specify a column for filtering out ns.",
              "Can't found any automatically", call. = FALSE)
    }
  }

  if(col %in% colnames(stat.test)){
    .value <- stat.test[[col]]
    if(is.numeric(.value)) stat.test <- filter(stat.test, .value <= signif.cutoff)
    else if(is.character(.value)) stat.test <- filter(stat.test, !(.value %in% c("ns", "NS")))
  }
  else{
    stop("Can't find the column `", col, ", in the data", call. = FALSE)
  }
  stat.test
}

# Comparing means
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

# Global function to compare means
compare_mean <- function(  data, formula, method = "t.test", paired = FALSE,
                           comparisons = NULL, ref.group = NULL,
                           p.adjust.method = "holm", detailed = FALSE, ...)
{

  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  number.of.groups <- guess_number_of_groups(data, group)

  if(method %in% c("anova", "kruskal.test") & number.of.groups <= 2)
    stop("The number of groups <= 2; you should use t.test or wilcox.test")

  # Case of one sample test
  if(number.of.groups <= 2){
    res <- two_sample_test(data, formula, method = method, paired = paired, ...)
  }
  # Pairwise comparisons
  else if(number.of.groups > 2){

    if(method == "anova"){
      res <- anova_test(data, formula, ...) %>%
        select(all_of(c("Effect", "F", "p"))) %>%
        set_colnames(c("term", "statistic", "p")) %>%
        add_column(method = "Anova", .after = "p") %>%
        add_column(.y. = outcome, .before = "term") %>%
        as_tibble()
    }
    else if(method == "kruskal.test")
      res <- kruskal_test(data, formula, ...)

    else if(is.null(ref.group))
      res <- pairwise_two_sample_test(
        data, formula, method = method, paired = paired,
        comparisons = comparisons, ref.group = ref.group,
        p.adjust.method = p.adjust.method, detailed = detailed, ...
      )

    else if(ref.group %in% c("all", ".all."))
      res <- two_sample_test_one_vs_all (
        data, formula, method = method,
        p.adjust.method = p.adjust.method, detailed = detailed, ...
      )
    else
      res <- pairwise_two_sample_test(
        data, formula, method = method, paired = paired,
        comparisons = comparisons, ref.group = ref.group,
        p.adjust.method = p.adjust.method, detailed = detailed, ...
      )
  }
  if(!detailed) res <- remove_details(res, method = method)

  res

}



# Performs one or two samples mean comparisons
two_sample_test <- function(data, formula, method = "t.test", ref.group = NULL, id = NULL, error.as.na = FALSE, detailed = FALSE, ...) {

  if (is_grouped_df(data)) {
    res <- data %>%
      doo(two_sample_test, formula, method = method,
          ref.group = ref.group, id = id, error.as.na = error.as.na, detailed = detailed, ...)
    return(res)
  }
  test.function <- method
  test.args <- list()
  grp1 <- grp2 <- NULL
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  # One sample mean comparison =========================
  if (.is_empty(group)) {
    grp1 <- "1"
    grp2 <- "null model"
    outcome.values <- data %>% pull(!!outcome)
    n <- sum(!is.na(outcome.values))
    test.args <- list(x = outcome.values, ...)
  }
  # Two sample mean comparisons ========================
  else {
    # Convert group into factor if this is not already the case
    data <- data %>% .as_factor(group, ref.group = ref.group)
    outcome.values <- data %>% pull(!!outcome)
    group.values <- data %>% pull(!!group)
    group.levels <- data %>% get_levels(group)
    grp1 <- group.levels[1]
    grp2 <- group.levels[2]
    x <- outcome.values[group.values == grp1]
    y <- outcome.values[group.values == grp2]
    n1 <- sum(!is.na(x))
    n2 <- sum(!is.na(y))
    # Paired test with an explicit subject identifier (#136, #175, #192): align
    # the two groups by `id` so that x[i] and y[i] are the same subject, keeping
    # only subjects measured in BOTH groups (complete pairs, i.e. per-comparison
    # pairwise deletion). Only fires for a paired test with id supplied; the
    # default id = NULL path is unchanged (groups taken in row order as before).
    if(!is.null(id) && isTRUE(list(...)$paired)){
      x <- y <- NULL
      paired.data <- align_paired_by_id(data, outcome, group, id, grp1, grp2)
      x <- paired.data$x
      y <- paired.data$y
      n1 <- n2 <- nrow(paired.data)
    }
    test.args <- list(x = x, y = y, ...)
  }

  statistic <- p <- NULL
  # error.as.na (#208, #158): a comparison can fail because a group has too few
  # observations or the data are essentially constant. By default this stops with
  # an error (unchanged). When error.as.na = TRUE, catch that error, warn (naming
  # the comparison), and return an NA result row so the remaining comparisons /
  # groups are still computed.
  res.raw <- tryCatch(
    suppressWarnings(do.call(test.function, test.args)),
    error = function(e){
      # Only convert genuine "cannot be computed" failures on numeric data (too
      # few observations, essentially constant data) into an NA row. Structural
      # problems such as a non-numeric outcome must still surface as an error,
      # so we re-raise when the inputs are not numeric.
      inputs.numeric <- is.numeric(test.args$x) &&
        (is.null(test.args$y) || is.numeric(test.args$y))
      if(isTRUE(error.as.na) && inputs.numeric){
        comparison <- if(isTRUE(grp2 == "null model")) paste("one-sample:", grp1)
                      else paste(grp1, "vs", grp2)
        warning(
          "Could not compute the comparison (", comparison, "): ",
          conditionMessage(e), ". Returning NA for this comparison.",
          call. = FALSE
        )
        return(NULL)
      }
      stop(e)
    }
  )
  if(is.null(res.raw)){
    # Degenerate comparison turned into an NA row (error.as.na = TRUE). Match the
    # method's normal schema: t.test has a `df` column, wilcox.test does not, so
    # only add `df` for tests that report it (avoids injecting a phantom df column
    # into wilcox_test output).
    res <- tibble(statistic = NA_real_, p = NA_real_)
    if(identical(method, "t.test")){
      res <- tibble(statistic = NA_real_, df = NA_real_, p = NA_real_)
    }
    res <- res %>%
      add_columns(
        .y. = outcome, group1 = grp1, group2 = grp2,
        .before = "statistic"
      )
  }
  else {
  # #127: wilcox.test silently lowers the confidence interval's confidence level
  # when the requested one cannot be achieved (ties / zeroes), which can make the
  # CI contradict the p-value (e.g. p > 0.05 while the CI excludes 0). Surface
  # this clearly. Locale-independent: read the achieved level off the conf.int.
  ci <- res.raw$conf.int
  if(!is.null(ci)){
    achieved  <- attr(ci, "conf.level")
    requested <- test.args$conf.level
    if(is.null(requested)) requested <- 0.95
    if(!is.null(achieved) && isTRUE(achieved < requested)){
      warning(
        "The requested ", round(requested*100), "% confidence interval could ",
        "not be computed (likely due to ties or zeroes); the reported interval ",
        "is at ", round(achieved*100), "% confidence and may be inconsistent ",
        "with the p-value. Interpret the confidence interval with caution.",
        call. = FALSE
      )
    }
  }
  res <- res.raw %>%
    as_tidy_stat() %>%
    add_columns(
      .y. = outcome, group1 = grp1, group2 = grp2,
      .before = "statistic"
    )
  }
  # Add n columns
  if(grp2 == "null model"){
    res <- res %>% add_columns(n = n, .before = "statistic")
  }
  else{
    res <- res %>% add_columns(n1 = n1, n2 = n2, .before = "statistic")
  }
  if(!detailed) res <- remove_details(res, method = method)
  res
}

# Pairwise mean comparisons
pairwise_two_sample_test <- function(data, formula, method = "t.test",
                               comparisons = NULL, ref.group = NULL,
                               p.adjust.method = "holm", id = NULL, error.as.na = FALSE, detailed = FALSE, ...) {
  if (is_grouped_df(data)) {
    res <- data %>%
      doo(
        pairwise_two_sample_test, formula, method,
        comparisons, ref.group, p.adjust.method,
        id = id, error.as.na = error.as.na, detailed = detailed, ...
        )
    return(res)
  }
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  data <- data %>% .as_factor(group, ref.group = ref.group)
  group.levels <- data %>% get_levels(group)
  # All possible pairwise comparisons
  # if ref.group specified, only comparisons against reference will be kept
  if (is.null(comparisons)) {
    comparisons <- group.levels %>% .possible_pairs(ref.group = ref.group)
  }
  res <- compare_pairs(data, formula, comparisons, method, id = id, error.as.na = error.as.na, detailed = detailed, ...) %>%
    adjust_pvalue(method = p.adjust.method) %>%
    add_significance()
 if(!detailed) res <- remove_details(res, method = method)
 res
}

# One vs all mean comparisons -----------------------------------
two_sample_test_one_vs_all <- function(data, formula, method = "t.test", p.adjust.method = "holm", detailed = FALSE, ...) {

  if (is_grouped_df(data)) {
    results <- data %>%
      doo(two_sample_test_one_vs_all, formula, method, p.adjust.method,
          detailed = detailed, ...)
    return(results)
  }
  outcome <- get_formula_left_hand_side(formula)
  group <- get_formula_right_hand_side(formula)
  new.data <- create_data_with_all_ref_group(data, outcome, group)
  pairwise_two_sample_test(
    data = new.data, formula = formula,
    method = method, ref.group = "all",
    p.adjust.method = p.adjust.method,
    detailed = detailed, ...
  )
}
# Create new data set containing the "all" group level
create_data_with_all_ref_group <- function(data, outcome, group){
  grouping.vars <- grouping.vars.data <- NULL
  if(is_grouped_df(data)){
    grouping.vars <- dplyr::group_vars(data)
    data <- dplyr::ungroup(data)
    grouping.vars.data <- data %>% select(!!!syms(grouping.vars))
  }
  data <- data %>% .as_factor(group)
  outcome.values <- data %>% pull(!!outcome)
  group.values <- data %>% pull(!!group)
  group.levels <- group.values %>% levels()
  all.data <- tibble(
    outcome = outcome.values,
    group = "all"
  )
  source.data <- tibble(
    outcome = outcome.values,
    group = as.character(group.values)
  )
  new.data <- all.data %>%
    bind_rows(source.data) %>%
    mutate(group = factor(group, levels = c("all", group.levels)))
  colnames(new.data) <- c(outcome, group)
  if(!is.null(grouping.vars)){
    # repeat grouping.vars.data for "all" group
    new.data <- dplyr::bind_rows(grouping.vars.data, grouping.vars.data) %>%
      dplyr::bind_cols(new.data) %>%
      group_by(!!!syms(grouping.vars))
  }
  new.data
}


# Align two groups of a paired test by a subject identifier, keeping only
# subjects present (and non-missing) in BOTH groups (complete pairs). Returns a
# data frame with columns x (grp1 values) and y (grp2 values), row-matched by id.
align_paired_by_id <- function(data, outcome, group, id, grp1, grp2){
  if(!(id %in% colnames(data))){
    stop("The id column '", id, "' was not found in the data.", call. = FALSE)
  }
  outcome.values <- data %>% pull(!!sym(outcome))
  group.values <- as.character(data %>% pull(!!sym(group)))
  id.values <- data %>% pull(!!sym(id))
  keep1 <- group.values == grp1
  keep2 <- group.values == grp2
  # Drop rows with a missing id up front: an unidentified subject (NA id) cannot
  # be matched, and would otherwise be cartesian-joined (dplyr matches NA keys by
  # default), inflating the pair count.
  d1 <- tibble(.id = id.values[keep1], x = outcome.values[keep1]) %>%
    filter(!is.na(.data$.id))
  d2 <- tibble(.id = id.values[keep2], y = outcome.values[keep2]) %>%
    filter(!is.na(.data$.id))
  # A proper paired design has at most one observation per subject and group.
  if(anyDuplicated(d1$.id) > 0 || anyDuplicated(d2$.id) > 0){
    stop(
      "Each id ('", id, "') must be unique within a group for a paired test, ",
      "but duplicated ids were found. Check the data or aggregate replicates ",
      "before testing.", call. = FALSE
    )
  }
  dplyr::inner_join(d1, d2, by = ".id", na_matches = "never") %>%
    filter(!is.na(.data$x), !is.na(.data$y)) %>%
    dplyr::arrange(.data$.id)
}

# compare_pair(ToothGrowth, len ~ dose, c("0.5", "1"))
compare_pair <- function(data, formula, pair, method = "t.test", id = NULL, error.as.na = FALSE, ...){
  group <- get_formula_right_hand_side(formula)
  data %>%
    filter(!!sym(group) %in% pair) %>%
    droplevels() %>%
    two_sample_test(formula, method = method, id = id, error.as.na = error.as.na, ...)
}
# compare_pairs(ToothGrowth, len ~ dose, list(c("0.5", "1"), c("1", "2")))
compare_pairs <- function(data, formula, pairs, method = "t.test", id = NULL, error.as.na = FALSE, ...){
  .f <- function(pair, data, formula, method, ...){
    compare_pair(data, formula, pair, method, id = id, error.as.na = error.as.na, ...)
  }
  pairs %>%
    map(.f, data, formula, method, ...) %>%
    bind_rows()
}



# Remove details from statistical test results
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
remove_details <- function(res, method){
  if(method == "anova"){
    # Remove details from ANOVA summary: such as intercept row, Sum Sq columns
    aov.table <- res$ANOVA
    aov.table = aov.table[, names(aov.table) %in% c('Effect','DFn','DFd','F','p','p<.05', 'ges', 'pes', 'conf.low', 'conf.high')]
    intercept.row <- grepl("Intercept", aov.table$Effect)
    res$ANOVA<- aov.table[!intercept.row, ]
  }
  else if(method %in% c("t.test", "wilcox.test", "kruskal.test", "sign.test", "ks.test") ){
    columns.to.keep <- intersect(
      c(".y.", "group1", "group2", "n", "n1", "n2",  "statistic",
        "df", "p", "p.signif", "p.adj", "p.adj.signif"),
      colnames(res)
    )
    res <- res[, columns.to.keep]
  }
  else if(method %in% c("coin.wilcox.test", "cohens.d")){
    columns.to.remove <- c("p", "p.adj", "p.adj.signif", "p.signif",
                           "statistic", "method", "alternative", "df")
    columns.to.keep  <- setdiff(colnames(res), columns.to.remove)
    res <- res %>% select(!!!syms(columns.to.keep))
  }
  else if(method %in% c("prop.test")){
    columns.to.keep <- intersect(
      c("n",  "group",  "statistic", "df", "p", "p.signif", "p.adj", "p.adj.signif"),
      colnames(res)
    )
    res <- res[, columns.to.keep]
  }
  else{
    columns.to.remove <- c("n1", "n2", "n", "method", "alternative", "statistic", "df")
    columns.to.keep  <- setdiff(colnames(res), columns.to.remove)
    res <- res %>% select(!!!syms(columns.to.keep))
  }
  res
}

# Two samples tests--------------------------------------
# Check two samples test args
check_two_samples_test_args <- function(x, y = NULL, mu = 0, paired = FALSE, conf.level = 0.5){
  if (!missing(mu) & ((length(mu) > 1L) || !is.finite(mu)))
    stop("'mu' must be a single number")
  if (!((length(conf.level) == 1L) & is.finite(conf.level) &
        (conf.level > 0) & (conf.level < 1)))
    stop("'conf.level' must be a single number between 0 and 1")
  if (!is.numeric(x))
    stop("'x' must be numeric")
  if (!is.null(y)) {
    if (!is.numeric(y))
      stop("'y' must be numeric")
    if (paired) {
      if (length(x) != length(y))
        stop("'x' and 'y' must have the same length")
    }
  }
  else {
    if (paired)
      stop("'y' is missing for paired test")
  }
  if (length(x) < 1L)
    stop("not enough (finite) 'x' observations")
}

#' @include utilities.R
NULL
#'Pairwise Comparisons of Estimated Marginal Means
#'
#'
#'@description Performs pairwise comparisons between groups using the estimated
#'  marginal means. Pipe-friendly wrapper arround the functions \code{emmans() +
#'  contrast()} from the \code{emmeans} package, which need to be installed
#'  before using this function. This function is useful for performing post-hoc
#'  analyses following ANOVA/ANCOVA tests.
#'@inheritParams t_test
#'@param model a fitted-model object such as the result of a call to
#'  \code{lm()}, \code{stats::aov()} (including a within-subject \code{Error()}
#'  term) or \code{nlme::lme()}, on which the estimated marginal means are
#'  computed. Supplying \code{model} lets you (i) average over factors that are
#'  in the model but not in \code{formula} (e.g. for a factorial design, fit
#'  \code{lm(y ~ a * b)} and compare \code{a} with \code{formula = y ~ a},
#'  averaging over \code{b}), and (ii) run pairwise comparisons for
#'  repeated-measures / mixed designs (pass a within-subject model). When
#'  \code{model = NULL} (default), a simple \code{lm()} is fitted from
#'  \code{formula} (plus the grouping and covariate variables). See examples.
#'@param covariate (optional) covariate names (for ANCOVA)
#'@return return a data frame with some the following columns: \itemize{ \item
#'  \code{.y.}: the y variable used in the test. \item \code{group1,group2}: the
#'  compared groups in the pairwise tests. \item \code{statistic}: Test
#'  statistic (t.ratio) used to compute the p-value. \item \code{df}: degrees of
#'  freedom. \item \code{p}: p-value. \item \code{p.adj}: the adjusted p-value.
#'  \item \code{method}: the statistical test used to compare groups. \item
#'  \code{p.signif, p.adj.signif}: the significance level of p-values and
#'  adjusted p-values, respectively. \item \code{estimate}: estimate of the
#'  effect size, that is the difference between the two emmeans (estimated
#'  marginal means). \item \code{conf.low,conf.high}: Lower and upper bound on a
#'  confidence interval of the estimate. }
#'
#'  The \strong{returned object has an attribute called args}, which is a list
#'  holding the test arguments. It has also an attribute named "emmeans", a data
#'  frame containing the groups emmeans.
#'@examples
#' # Data preparation
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#'
#'# Pairwise comparisons
#' res <- df %>%
#'  group_by(supp) %>%
#'  emmeans_test(len ~ dose, p.adjust.method = "bonferroni")
#'res
#'
#' # Display estimated marginal means
#' attr(res, "emmeans")
#'
#' # Show details
#' df %>%
#'  group_by(supp) %>%
#'  emmeans_test(len ~ dose, p.adjust.method = "bonferroni", detailed = TRUE)
#'
#' # Marginal means averaged over another factor (e.g. a 2x3 design).
#' # Fit the full model and pass it with `model =` so that the estimated
#' # marginal means for `dose` are averaged over `supp` (instead of fitting
#' # `len ~ dose` alone, which would ignore `supp`):
#' model <- lm(len ~ supp * dose, data = df)
#' df %>% emmeans_test(len ~ dose, model = model)
#'
#' # Repeated-measures / mixed designs: pass a fitted within-subject model
#' # (e.g. stats::aov() with an Error() term, or nlme::lme()) with `model =`:
#' \donttest{
#' set.seed(123)
#' d <- data.frame(
#'   id    = factor(rep(1:10, 3)),
#'   time  = factor(rep(c("t1", "t2", "t3"), each = 10)),
#'   score = rnorm(30)
#' )
#' rm_model <- stats::aov(score ~ time + Error(id / time), data = d)
#' d %>% emmeans_test(score ~ time, model = rm_model)
#' }
#'@export
emmeans_test <- function(data, formula, covariate = NULL, ref.group = NULL,
                         comparisons = NULL, p.adjust.method = "bonferroni",
                         conf.level = 0.95, model = NULL, detailed = FALSE){
  . <- NULL
  covariate <- rlang::enquos(covariate = covariate) %>%
    get_quo_vars_list(data, .) %>% unlist()
  args <- as.list(environment()) %>%
    .add_item(method = "emmeans_test")

  required_package("emmeans")
  outcome <- get_formula_left_hand_side(formula)
  rhs <- group <- get_formula_right_hand_side(formula)
  grouping.vars <- NULL

  if(is_grouped_df(data)){
    grouping.vars <- dplyr::group_vars(data)
    rhs <- c(grouping.vars, rhs) %>%
      paste(collapse = "*")
    data <- dplyr::ungroup(data)
  }
  # Reference grid for emmeans is built over the comparison variable(s) only
  # (group + grouping vars). The covariate goes into the model but NOT the grid:
  # emmeans averages over it (ANCOVA). Keeping the covariate in the grid breaks
  # for a 2-value numeric covariate, which emmeans keeps as a factor by default
  # (cov.keep = "2"), producing "Nonconforming number of contrast coefficients"
  # (#206, #86).
  emmeans.rhs <- rhs
  if(!is.null(covariate)){
    covariate <- paste(covariate, collapse = "+")
    rhs <- paste(covariate, rhs, sep = "+")
  }

  data <- data %>% .as_factor(group, ref.group = ref.group)
  group.levels <- data %>% get_levels(group)

  # Build linear model
  formula <- stats::as.formula(paste(outcome, rhs, sep = " ~ "))
  if(is.null(model))
    model <- stats::lm(formula, data)

  # Fit emmeans
  # Possible pairwise comparisons: if ref.group specified,
  # only comparisons against reference will be kept
  if (is.null(comparisons)) {
    comparisons <- get_comparisons(data, variable = !!group, ref.group = !!ref.group)
  }
  method <- get_emmeans_contrasts(data, group, comparisons)
  formula.emmeans <- stats::as.formula(paste0("~", emmeans.rhs))
  res.emmeans <- emmeans::emmeans(model, formula.emmeans)
  comparisons <- pairwise_emmeans_test(
    res.emmeans, grouping.vars, method = method,
    p.adjust.method = p.adjust.method,
    conf.level = conf.level
    )

  res.emmeans <- res.emmeans %>%
    tibble::as_tibble() %>%
    dplyr::arrange(!!!syms(grouping.vars)) %>%
    dplyr::rename(se = "SE", conf.low = "lower.CL", conf.high = "upper.CL") %>%
    mutate(method = "Emmeans test")

  if(!detailed){
    to.remove <- c("estimate", "estimate1", "estimate2", "se", "conf.low", "conf.high", "method", "null.value")
    to.keep <- setdiff(colnames(comparisons), to.remove)
    comparisons <- comparisons[, to.keep]
  }
  comparisons %>%
    add_column(.y. = outcome, .before = "group1") %>%
    set_attrs(args = args, emmeans = res.emmeans) %>%
    add_class(c("rstatix_test", "emmeans_test"))
}

#' @export
#' @param emmeans.test an object of class \code{emmeans_test}.
#' @describeIn emmeans_test returns the estimated marginal means from an object of class \code{emmeans_test}
get_emmeans <- function(emmeans.test){
  if(!inherits(emmeans.test, "emmeans_test")){
    stop("An object of class 'emmeans_test' required.")
  }
  attr(emmeans.test, "emmeans")
}



pairwise_emmeans_test <- function(res.emmeans, grouping.vars = NULL, method = "pairwise",
                                  p.adjust.method = "bonferroni", conf.level = 0.95){
  # Comparisons without adjusting the pvalue
  # reverse the order of subtraction for consistency with pairwise_t_test
  comparisons <- emmeans::contrast(
    res.emmeans, by = grouping.vars, method = method,
    adjust = "none"
  )
  comparisons <- tidy(comparisons, conf.int = TRUE, conf.level = conf.level)
  comparisons <- comparisons %>%
    tidyr::separate(col = "contrast", into = c("group1", "group2"), sep = "-") %>%
    dplyr::rename(se = "std.error", p = "p.value") %>%
    dplyr::select(!!!syms(grouping.vars), everything())

    # Adjust the pvalue. We don't want to use adjust_pvalue here, because
    # emmeans support method = "tukey", but this is not the case for adjust_pvalue
    p.adjusted <- emmeans::contrast(
      res.emmeans, by = grouping.vars, method = method,
      adjust = p.adjust.method
    )  %>%
      as.data.frame() %>%
      pull("p.value")
    comparisons  <- comparisons %>%
      mutate(p.adj = p.adjusted) %>%
      add_significance("p.adj")

  # Homogenize variable classes between res.emmeans and comparisons
  # because: tidy() is converting factor to character -> so restoring back factors
    res.emmeans.tbl <- tibble::as_tibble(res.emmeans)
    variables <- intersect(colnames(res.emmeans.tbl), colnames(comparisons))
    for(variable in variables){
      if(is.factor(res.emmeans.tbl[[variable]])){
        comparisons[[variable]] <- factor(
          comparisons[[variable]],
          levels = levels(res.emmeans.tbl[[variable]])
          )
      }
    }
    comparisons <- base::droplevels(comparisons)

  comparisons %>%
    dplyr::arrange(!!!syms(grouping.vars))
}

# Returns a list of contrasts for specific comparisons
# data: data frame,
# group: grouping columns,
# comparisons a list of comparisons
get_emmeans_contrasts <- function(data, group, comparisons){
  get_dummy_code <- function(level, group.levels){
    dummy.code <- rep(0, length(group.levels))
    lev.pos <- which(group.levels == level)
    dummy.code[lev.pos] <- 1
    dummy.code
  }
  make_emmeans_contrast <- function(groups, contrasts.list ){
    group1 <- groups[1]
    group2 <- groups[2]
    contrasts.list[[group1]]-contrasts.list[[group2]]
  }
  make_comparison_name <- function(groups){
    paste(groups[1], groups[2], sep = "-")
  }
  group.levels <- get_levels(data, group)
  contrasts.list <- group.levels %>%
    map(get_dummy_code, group.levels)
  names(contrasts.list) <- group.levels

  comparison.contrasts <- comparisons %>%
    map(make_emmeans_contrast, contrasts.list)
  comparison.names <- comparisons %>% map(make_comparison_name)
  names(comparison.contrasts) <- comparison.names
  comparison.contrasts
}

test_that("emmeans_test works", {
  # Data preparation
  df <- ToothGrowth
  df$dose <- as.factor(df$dose)
  # Pairwise comparisons
  comparisons <- df %>%
    group_by(supp) %>%
    emmeans_test(len ~ dose, p.adjust.method = "bonferroni") %>%
    as.data.frame(stringsAsFactors = FALSE)
  # raw emmeans output
  res_emmeans <- attr(comparisons, "emmeans") %>%
    as.data.frame(stringsAsFactors = FALSE)

  attributes(comparisons) <- list(
    names = colnames(comparisons),
    row.names = row.names(comparisons),
    class = "data.frame"
  )
  attributes(res_emmeans) <- list(
    names = colnames(res_emmeans),
    row.names = row.names(res_emmeans),
    class = "data.frame"
  )

  # Expected values
  expected_comparisons <- tibble::tribble(
     ~supp,  ~term,  ~.y., ~group1, ~group2, ~df,        ~statistic,                   ~p,               ~p.adj, ~p.adj.signif,
      "OJ", "dose", "len",   "0.5",     "1",  54, -5.83122150109434, 3.17564054631384e-07, 9.52692163894153e-07,        "****",
      "OJ", "dose", "len",   "0.5",     "2",  54,  -7.9001659830032, 1.42971201237994e-10, 4.28913603713983e-10,        "****",
      "OJ", "dose", "len",     "1",     "2",  54, -2.06894448190887,   0.0433521450968846,    0.130056435290654,          "ns",
      "VC", "dose", "len",   "0.5",     "1",  54, -5.41250654642231, 1.46293144787886e-06, 4.38879434363658e-06,        "****",
      "VC", "dose", "len",   "0.5",     "2",  54, -11.1821523188884, 1.13067681037436e-15, 3.39203043112308e-15,        "****",
      "VC", "dose", "len",     "1",     "2",  54,  -5.7696457724661, 3.98114048489776e-07, 1.19434214546933e-06,        "****"
     ) %>%
    dplyr::mutate(supp = factor(supp, levels =  c("OJ", "VC"))) %>%
    data.frame(stringsAsFactors = FALSE,  row.names = as.character(1:6))

  expected_emmeans <- tibble::tribble(
     ~supp, ~dose, ~emmean,              ~se, ~df,        ~conf.low,       ~conf.high,        ~method,
      "OJ", "0.5",   13.23, 1.14835308804166,  54, 10.9276906782585, 15.5323093217415, "Emmeans test",
      "OJ",   "1",    22.7, 1.14835308804166,  54, 20.3976906782585, 25.0023093217415, "Emmeans test",
      "OJ",   "2",   26.06, 1.14835308804166,  54, 23.7576906782585, 28.3623093217415, "Emmeans test",
      "VC", "0.5",    7.98, 1.14835308804166,  54, 5.67769067825848, 10.2823093217415, "Emmeans test",
      "VC",   "1",   16.77, 1.14835308804166,  54, 14.4676906782585, 19.0723093217415, "Emmeans test",
      "VC",   "2",   26.14, 1.14835308804166,  54, 23.8376906782585, 28.4423093217415, "Emmeans test"
     ) %>%
    dplyr::mutate(
      supp = factor(supp, levels =  c("OJ", "VC")),
      dose = factor(dose, levels = c("0.5", "1", "2"))
      ) %>%
    data.frame(stringsAsFactors = FALSE, row.names = as.character(1:6))

  # Make sure that he class of grouping variable is preserved
  expect_equal(class(comparisons$supp), "factor")
  expect_equal(comparisons, expected_comparisons, tolerance = 1e-4)
  expect_equal(res_emmeans, expected_emmeans, tolerance = 1e-4)
})

test_that("emmeans_test works with a binary (0/1) numeric covariate (#206, #86)", {
  skip_if_not_installed("emmeans")
  df <- mtcars
  df$am <- factor(df$am)
  # df$vs is numeric with only 2 distinct values (0/1); this previously errored
  # with 'Nonconforming number of contrast coefficients'
  res <- emmeans_test(df, wt ~ am, covariate = vs)
  expect_s3_class(res, "rstatix_test")
  expect_equal(nrow(res), 1L)            # am has 2 levels -> a single comparison
  expect_true(all(c("group1", "group2", "p", "p.adj") %in% colnames(res)))
  # multiple covariates (incl. a binary one) also work
  res2 <- emmeans_test(df, wt ~ am, covariate = c(gear, vs))
  expect_equal(nrow(res2), 1L)
})

test_that("emmeans_test averages the covariate (correct ANCOVA), not gridding it (#206)", {
  skip_if_not_installed("emmeans")
  df <- mtcars
  df$am <- factor(df$am)
  res <- emmeans_test(df, wt ~ am, covariate = gear)
  # equals the ANCOVA emmeans over the group with the covariate held at its mean
  m   <- stats::lm(wt ~ gear + am, data = df)
  ref <- as.data.frame(emmeans::contrast(emmeans::emmeans(m, ~ am),
                                         method = "pairwise", adjust = "none"))
  expect_equal(abs(res$statistic), abs(ref$t.ratio), tolerance = 1e-6)
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/p_value.R
\name{p_round}
\alias{p_round}
\alias{p_format}
\alias{p_mark_significant}
\alias{p_detect}
\alias{p_names}
\alias{p_adj_names}
\title{Rounding and Formatting p-values}
\usage{
p_round(x, ..., digits = 3)

p_format(
  x,
  ...,
  new.col = FALSE,
  digits = 2,
  accuracy = 1e-04,
  decimal.mark = ".",
  leading.zero = TRUE,
  trailing.zero = FALSE,
  add.p = FALSE,
  space = FALSE
)

p_mark_significant(
  x,
  ...,
  new.col = FALSE,
  cutpoints = c(0, 1e-04, 0.001, 0.01, 0.05, 1),
  symbols = c("****", "***", "**", "*", "")
)

p_detect(data, type = c("all", "p", "p.adj"))

p_names()

p_adj_names()
}
\arguments{
\item{x}{a numeric vector of p-values or a data frame containing a p value
column. If data frame, the p-value column(s) will be automatically detected.
Known p-value column names can be obtained using the functions
\code{p_names()} and \code{p_adj_names()}}

\item{...}{column names to manipulate in the case where \code{x} is a data
frame. P value columns are automatically detected if not specified.}

\item{digits}{the number of significant digits to be used.}

\item{new.col}{logical, used only when \code{x} is a data frame. If TRUE, add
a new column to hold the results. The new column name is created by adding,
to the p column, the suffix "format" (for \code{p_format()}), "signif" (for
\code{p_mak_significant()}).}

\item{accuracy}{number to round to, that is the threshold value above wich the
function will replace the pvalue by "<0.0xxx".}

\item{decimal.mark}{the character to be used to indicate the numeric decimal
point.}

\item{leading.zero}{logical. If FALSE, remove the leading zero.}

\item{trailing.zero}{logical. If FALSE (default), remove the training extra
zero.}

\item{add.p}{logical value. If TRUE, add "p=" before the value.}

\item{space}{logical. If TRUE (default) use space as separator between
different elements and symbols.}

\item{cutpoints}{numeric vector used for intervals}

\item{symbols}{character vector, one shorter than cutpoints, used as
significance symbols.}

\item{data}{a data frame}

\item{type}{the type of p-value to detect. Can be one of \code{c("all", "p", "p.adj")}.}
}
\value{
a vector or a data frame containing the rounded/formatted p-values.
}
\description{
Round and format p-values. Can also mark significant p-values with stars.
}
\section{Functions}{
\itemize{
\item \code{p_round}: round p-values

\item \code{p_format}: format p-values. Add a symbol "<" for small p-values.

\item \code{p_mark_significant}: mark p-values with significance levels

\item \code{p_detect}: detects and returns p-value column names in a data frame.

\item \code{p_names}: returns known p-value column names

\item \code{p_adj_names}: returns known adjust p-value column names
}}

\examples{

# Round and format a vector of p-values
# ::::::::::::::::::::::::::::::::::::::::::::
# Format
p <- c(0.5678, 0.127, 0.045, 0.011, 0.009, 0.00002, NA)
p_format(p)

# Specify the accuracy
p_format(p, accuracy = 0.01)

# Add p and remove the leading zero
p_format(p, add.p = TRUE, leading.zero = FALSE)

# Remove space before and after "=" or "<".
p_format(p, add.p = TRUE, leading.zero = FALSE, space = FALSE)

# Mark significant p-values
# ::::::::::::::::::::::::::::::::::::::::::::
p_mark_significant(p)

# Round, the mark significant
p \%>\% p_round(digits = 2) \%>\% p_mark_significant()

# Format, then mark significant
p \%>\% p_format(digits = 2) \%>\% p_mark_significant()

# Perform stat test, format p and mark significant
# ::::::::::::::::::::::::::::::::::::::::::::
ToothGrowth \%>\%
  group_by(dose) \%>\%
  t_test(len ~ supp) \%>\%
  p_format(digits = 2, leading.zero = FALSE) \%>\%
  p_mark_significant()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dunnett_test.R
\name{dunnett_test}
\alias{dunnett_test}
\title{Dunnett's Many-to-One Comparisons Test}
\usage{
dunnett_test(
  data,
  formula,
  ref.group = NULL,
  conf.level = 0.95,
  detailed = FALSE
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{ref.group}{a character string specifying the reference (control) group.
Each remaining group level is compared against this group. If \code{NULL}
(default), the first level of the grouping variable is used as the control.}

\item{conf.level}{confidence level of the (simultaneous) confidence intervals.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}
}
\value{
a data frame with some of the following columns: \itemize{ \item
 \code{.y.}: the outcome variable used in the test. \item
 \code{group1,group2}: the compared groups; \code{group1} is the control
 (reference) and \code{group2} is the treatment, consistent with the
 \code{ref.group} convention of \code{t_test()}/\code{wilcox_test()}/
 \code{dunn_test()}/\code{emmeans_test()}. \item \code{n1,n2}: sample sizes of
 the control and treatment groups. \item \code{estimate}: the estimated mean
 difference \code{group1 - group2} (control minus treatment). \item
 \code{conf.low,conf.high}: simultaneous confidence interval for the
 difference. \item \code{statistic}: the t-statistic. \item \code{df}:
 degrees of freedom. \item \code{p.adj}: the Dunnett-adjusted p-value. \item
 \code{method}: the statistical test used. \item \code{p.adj.signif}: the
 significance level of the adjusted p-value. } The estimate, confidence
 interval, se and method columns are returned only when \code{detailed =
 TRUE}.

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs Dunnett's test for comparing each of several treatment
 groups against a single control (reference) group. Unlike all-pairwise
 post-hoc tests, Dunnett's procedure controls the family-wise error rate over
 only the \code{k - 1} treatment-vs-control comparisons, using the exact
 multivariate-t distribution (which accounts for the correlation between the
 comparisons that share the control group).

 This is a pipe-friendly wrapper around \code{emmeans::emmeans()} +
 \code{emmeans::contrast()} (with \code{adjust = "mvt"}), so the
 \code{emmeans} package must be installed. The results match
 \code{DescTools::DunnettTest()} and \code{multcomp::glht(..., mcp(... =
 "Dunnett"))}.
}
\examples{
# Compare each dose to the control dose ("0.5")
ToothGrowth \%>\% dunnett_test(len ~ dose)

# Detailed output (estimate + simultaneous confidence interval)
ToothGrowth \%>\% dunnett_test(len ~ dose, detailed = TRUE)

# Grouped data
ToothGrowth \%>\%
  group_by(supp) \%>\%
  dunnett_test(len ~ dose)
}
\references{
Dunnett, C. W. (1955) A multiple comparison procedure for comparing
 several treatments with a control. Journal of the American Statistical
 Association, 50, 1096-1121.
}
\seealso{
\code{\link{tukey_hsd}()}, \code{\link{games_howell_test}()},
 \code{\link{emmeans_test}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doo.R
\name{doo}
\alias{doo}
\title{Alternative to dplyr::do for Doing Anything}
\usage{
doo(data, .f, ..., result = ".results.")
}
\arguments{
\item{data}{a (grouped) data frame}

\item{.f}{A function, formula, or atomic vector. For example
\code{~t.test(len ~ supp, data = .)}.}

\item{...}{Additional arguments passed on to .f}

\item{result}{the column name to hold the results. Default is ".results.".}
}
\value{
a data frame
}
\description{
Provides a flexible alternative to the \code{dplyr:do()} function.
 Technically it uses \code{nest() + mutate() + map()} to apply arbitrary
 computation to a grouped data frame.

 The output is a data frame. If the applied function returns a data frame,
 then the output will be automatically unnested. Otherwise, the output includes the grouping
 variables and a column named ".results." (by default), which is a "list-columns"
 containing the results for group combinations.
}
\examples{
# Custom function
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
stat_test <- function(data, formula){
  t.test(formula, data) \%>\%
    tidy()
}
# Example 1: pipe-friendly stat_test().
# Two possibilities of usage are available
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Use this
ToothGrowth \%>\%
  group_by(dose) \%>\%
  doo(~stat_test(data =., len ~ supp))

# Or this
ToothGrowth \%>\%
  group_by(dose) \%>\%
  doo(stat_test, len ~ supp)

# Example 2: R base function t.test() (not pipe friendly)
# One possibility of usage
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
comparisons <- ToothGrowth \%>\%
   group_by(dose) \%>\%
   doo(~t.test(len ~ supp, data =.))
comparisons
comparisons$.results.

# Example 3: R base function combined with tidy()
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
ToothGrowth \%>\%
   group_by(dose) \%>\%
   doo(~t.test(len ~ supp, data =.) \%>\% tidy())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-pipe.R
\name{\%>\%}
\alias{\%>\%}
\title{Pipe operator}
\usage{
lhs \%>\% rhs
}
\description{
See \code{magrittr::\link[magrittr:pipe]{\%>\%}} for details.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anova_summary.R
\name{anova_summary}
\alias{anova_summary}
\title{Create Nice Summary Tables of ANOVA Results}
\usage{
anova_summary(object, effect.size = "ges", detailed = FALSE, observed = NULL)
}
\arguments{
\item{object}{an object of returned by either \code{\link[car]{Anova}()}, or
\code{\link[stats]{aov}()}.}

\item{effect.size}{the effect size to compute and to show in the ANOVA
results. Allowed values can be either "ges" (generalized eta squared) or
"pes" (partial eta squared) or both. Default is "ges".}

\item{detailed}{If TRUE, returns extra information (sums of squares columns,
intercept row, etc.) in the ANOVA table.}

\item{observed}{Variables that are observed (i.e, measured) as compared to
experimentally manipulated. The default effect size reported (generalized
eta-squared) requires correct specification of the observed variables.}
}
\value{
return an object of class \code{anova_test} a data frame containing
 the ANOVA table for independent measures ANOVA. However, for repeated/mixed
 measures ANOVA, it is a list containing the following components are
 returned:

 \itemize{ \item \strong{ANOVA}: a data frame containing ANOVA results \item
 \strong{Mauchly's Test for Sphericity}: If any within-Ss variables with more
 than 2 levels are present, a data frame containing the results of Mauchly's
 test for Sphericity. Only reported for effects that have more than 2 levels
 because sphericity necessarily holds for effects with only 2 levels. \item
 \strong{Sphericity Corrections}: If any within-Ss variables are present, a
 data frame containing the Greenhouse-Geisser and Huynh-Feldt epsilon values,
 and corresponding corrected p-values. }

 The \strong{returned object might have an attribute} called \code{args} if
 you compute ANOVA using the function \code{\link{anova_test}()}. The attribute \code{args} is a
 list holding the arguments used to fit the ANOVA model, including: data, dv,
 within, between, type, model, etc.


 The following abbreviations are used in the different results tables:

 \itemize{ \item DFn	Degrees of Freedom in the numerator (i.e. DF effect).
 \item DFd	Degrees of Freedom in the denominator (i.e., DF error). \item
 SSn	Sum of Squares in the numerator (i.e., SS effect). \item SSd	Sum of
 Squares in the denominator (i.e.,SS error). \item F	F-value. \item p	p-value
 (probability of the data given the null hypothesis). \item p<.05	Highlights
 p-values less than the traditional alpha level of .05. \item ges	Generalized
 Eta-Squared measure of effect size. \item GGe	Greenhouse-Geisser epsilon.
 \item p[GGe]	p-value after correction using Greenhouse-Geisser epsilon.
 \item p[GGe]<.05	Highlights p-values (after correction using
 Greenhouse-Geisser epsilon) less than the traditional alpha level of .05.
 \item HFe	Huynh-Feldt epsilon. \item p[HFe]	p-value after correction using
 Huynh-Feldt epsilon. \item p[HFe]<.05	Highlights p-values (after correction
 using Huynh-Feldt epsilon) less than the traditional alpha level of .05.
 \item W	Mauchly's W statistic }
}
\description{
Create beautiful summary tables of ANOVA test results obtained
 from either \code{\link[car]{Anova}()} or \code{\link[stats]{aov}()}.

 The results include ANOVA table, generalized effect size and some assumption
 checks.
}
\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth
df$dose <- as.factor(df$dose)

# Independent measures ANOVA
#:::::::::::::::::::::::::::::::::::::::::
# Compute ANOVA and display the summary
res.anova <- Anova(lm(len ~ dose*supp, data = df))
anova_summary(res.anova)

# Display both SSn and SSd using detailed = TRUE
# Show generalized eta squared using effect.size = "ges"
anova_summary(res.anova, detailed = TRUE, effect.size = "ges")

# Show partial eta squared using effect.size = "pes"
anova_summary(res.anova, detailed = TRUE, effect.size = "pes")

# Repeated measures designs using car::Anova()
#:::::::::::::::::::::::::::::::::::::::::
# Prepare the data
df$id <- as.factor(rep(1:10, 6)) # Add individuals ids
head(df)

# Easily perform repeated measures ANOVA using the car package
design <- factorial_design(df, dv = len, wid = id, within = c(supp, dose))
res.anova <- Anova(design$model, idata = design$idata, idesign = design$idesign, type = 3)
anova_summary(res.anova)

# Repeated measures designs using stats::Aov()
#:::::::::::::::::::::::::::::::::::::::::
res.anova <- aov(len ~ dose*supp + Error(id/(supp*dose)), data = df)
anova_summary(res.anova)
}
\seealso{
\code{\link{anova_test}()}, \code{\link{factorial_design}()}
}
\author{
Alboukadel Kassambara, \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor_reorder.R
\name{cor_reorder}
\alias{cor_reorder}
\title{Reorder Correlation Matrix}
\usage{
cor_reorder(x)
}
\arguments{
\item{x}{a correlation matrix. Particularly, an object of class \code{cor_mat}.}
}
\value{
a data frame
}
\description{
reorder correlation matrix, according to the coefficients,
  using the hierarchical clustering method.
}
\examples{
# Compute correlation matrix
#::::::::::::::::::::::::::::::::::::::::::
cor.mat <- mtcars \%>\%
  select(mpg, disp, hp, drat, wt, qsec) \%>\%
  cor_mat()

# Reorder by correlation and get p-values
#::::::::::::::::::::::::::::::::::::::::::
# Reorder
cor.mat \%>\%
  cor_reorder()
# Get p-values of the reordered cor_mat
cor.mat \%>\%
  cor_reorder() \%>\%
  cor_get_pval()

}
\seealso{
\code{\link{cor_mat}()}, \code{\link{cor_gather}()}, \code{\link{cor_spread}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/df.R
\name{df_unite}
\alias{df_unite}
\alias{df_unite_factors}
\title{Unite Multiple Columns into One}
\usage{
df_unite(data, col, ..., vars = NULL, sep = "_", remove = TRUE, na.rm = FALSE)

df_unite_factors(
  data,
  col,
  ...,
  vars = NULL,
  sep = "_",
  remove = TRUE,
  na.rm = FALSE
)
}
\arguments{
\item{data}{a data frame}

\item{col}{the name of the new column as a string or a symbol.}

\item{...}{a selection of columns. One or more unquoted expressions (or variable names) separated by
commas.}

\item{vars}{a character vector containing the column names of interest.}

\item{sep}{Separator to use between values.}

\item{remove}{If \code{TRUE}, remove input columns from output data frame.}

\item{na.rm}{If \code{TRUE}, missing values will be remove prior to uniting
each value.}
}
\description{
Paste together multiple columns into one. Wrapper arround
  \code{\link[tidyr]{unite}()} that supports standard and non standard
  evaluation.
}
\section{Functions}{
\itemize{
\item \code{df_unite}: Unite multiple columns into one.

\item \code{df_unite_factors}: Unite factor columns. First, order factors levels then
merge them into one column. The output column is a factor.
}}

\examples{
# Non standard evaluation
head(ToothGrowth) \%>\%
 df_unite(col = "dose_supp", dose, supp)

# Standard evaluation
head(ToothGrowth) \%>\%
 df_unite(col = "dose_supp", vars = c("dose", "supp"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/box_m.R
\name{box_m}
\alias{box_m}
\title{Box's M-test for Homogeneity of Covariance Matrices}
\usage{
box_m(data, group)
}
\arguments{
\item{data}{a numeric data.frame or matrix containing n observations of p
variables; it is expected that n > p.}

\item{group}{a vector of length n containing the class of each
observation; it is usually a factor.}
}
\value{
A data frame containing the following components:
\itemize{
\item{statistic }{an approximated value of the chi-square distribution.}
\item{parameter }{the degrees of freedom related of the test statistic in this case that it follows a Chi-square distribution.}
\item{p.value }{the p-value of the test.}
\item{method }{the character string "Box's M-test for Homogeneity of Covariance Matrices".}
}
}
\description{
Performs the Box's M-test for homogeneity of covariance matrices
  obtained from multivariate normal data according to one grouping variable.
  The test is based on the chi-square approximation.
}
\examples{
data(iris)
box_m(iris[, -5], iris[, 5])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-manova.R
\name{Manova}
\alias{Manova}
\title{Manova exported from car package}
\description{
See \code{car::\link[car:Anova]{Manova}} for details.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor_mat.R
\name{cor_mat}
\alias{cor_mat}
\alias{cor_pmat}
\alias{cor_get_pval}
\title{Compute Correlation Matrix with P-values}
\usage{
cor_mat(
  data,
  ...,
  vars = NULL,
  method = "pearson",
  alternative = "two.sided",
  conf.level = 0.95
)

cor_pmat(
  data,
  ...,
  vars = NULL,
  method = "pearson",
  alternative = "two.sided",
  conf.level = 0.95
)

cor_get_pval(x)
}
\arguments{
\item{data}{a data.frame containing the variables.}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used to select a variable of interest.}

\item{vars}{a character vector containing the variable names of interest.}

\item{method}{a character string indicating which correlation
    coefficient is to be  used for the test.  One of \code{"pearson"},
    \code{"kendall"}, or \code{"spearman"}, can be abbreviated.}

\item{alternative}{indicates the alternative hypothesis and must be
    one of \code{"two.sided"}, \code{"greater"} or \code{"less"}.  You
    can specify just the initial letter.  \code{"greater"} corresponds
    to positive association, \code{"less"} to negative association.}

\item{conf.level}{confidence level for the returned confidence
    interval.  Currently only used for the Pearson product moment
    correlation coefficient if there are at least 4 complete pairs of
    observations.}

\item{x}{an object of class \code{cor_mat}}
}
\value{
a data frame
}
\description{
Compute correlation matrix with p-values. Numeric columns in the
 data are detected and automatically selected for the analysis. You can also
 specify variables of interest to be used in the correlation analysis.
}
\section{Functions}{
\itemize{
\item \code{cor_mat}: compute correlation matrix with p-values. Returns a data
frame containing the matrix of the correlation coefficients. The output has
an attribute named "pvalue", which contains the matrix of the correlation
test p-values.

\item \code{cor_pmat}: compute the correlation matrix but returns only the p-values of the tests.

\item \code{cor_get_pval}: extract a correlation matrix p-values from an object of
class \code{cor_mat()}.
}}

\examples{
# Data preparation
#:::::::::::::::::::::::::::::::::::::::::::
mydata <- mtcars \%>\%
  select(mpg, disp, hp, drat, wt, qsec)
head(mydata, 3)

# Compute correlation matrix
#::::::::::::::::::::::::::::::::::::::::::
# Correlation matrix between all variables
cor.mat <- mydata \%>\% cor_mat()
cor.mat

# Specify some variables of interest
mydata \%>\% cor_mat(mpg, hp, wt)

# Or remove some variables in the data
# before the analysis
mydata \%>\% cor_mat(-mpg, -hp)

# Significance levels
#::::::::::::::::::::::::::::::::::::::::::
cor.mat \%>\% cor_get_pval()


# Visualize
#::::::::::::::::::::::::::::::::::::::::::
# Insignificant correlations are marked by crosses
cor.mat \%>\%
  cor_reorder() \%>\%
  pull_lower_triangle() \%>\%
  cor_plot(label = TRUE)

# Gather/collapse correlation matrix into long format
#::::::::::::::::::::::::::::::::::::::::::
cor.mat \%>\% cor_gather()


}
\seealso{
\code{\link{cor_test}()}, \code{\link{cor_reorder}()},
 \code{\link{cor_gather}()}, \code{\link{cor_select}()},
 \code{\link{cor_as_symbols}()}, \code{\link{pull_triangle}()},
 \code{\link{replace_triangle}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mahalanobis_distance.R
\name{mahalanobis_distance}
\alias{mahalanobis_distance}
\title{Compute Mahalanobis Distance and Flag Multivariate Outliers}
\usage{
mahalanobis_distance(data, ...)
}
\arguments{
\item{data}{a data frame. Columns are variables.}

\item{...}{One unquoted expressions (or variable name). Used to select a
variable of interest. Can be also used to ignore a variable that are not
needed for the computation. For example specify \code{-id} to ignore the id
column.}
}
\value{
Returns the input data frame with two additional columns: 1)
 "mahal.dist": Mahalanobis distance values; and 2) "is.outlier": logical
 values specifying whether a given observation is a multivariate outlier
}
\description{
Pipe-friendly wrapper around to the function
 \code{\link[stats]{mahalanobis}()}, which returns the squared
 Mahalanobis distance of all rows in x. Compared to the base function, it
 automatically flags multivariate outliers.

 Mahalanobis distance is a common metric used to identify multivariate
 outliers. The larger the value of Mahalanobis distance, the more unusual the
 data point (i.e., the more likely it is to be a multivariate outlier).

 The distance tells us how far an observation is from the center of the cloud, taking into
 account the shape (covariance) of the cloud as well.

 To detect outliers, the calculated Mahalanobis distance is compared against
 a chi-square (X^2) distribution with degrees of freedom equal to the number
 of dependent (outcome) variables and an alpha level of 0.001.

 The threshold to declare a multivariate outlier is determined using the
 function \code{qchisq(0.999, df) }, where df is the degree of freedom (i.e.,
 the number of dependent variable used in the computation).
}
\examples{

# Compute mahalonobis distance and flag outliers if any
iris \%>\%
  doo(~mahalanobis_distance(.))

# Compute distance by groups and filter outliers
iris \%>\%
 group_by(Species) \%>\%
 doo(~mahalanobis_distance(.)) \%>\%
 filter(is.outlier == TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conover_test.R
\name{conover_test}
\alias{conover_test}
\title{Conover's All-Pairs Rank Comparison Test}
\usage{
conover_test(
  data,
  formula,
  p.adjust.method = "holm",
  ref.group = NULL,
  detailed = FALSE
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{p.adjust.method}{method to adjust p values for multiple comparisons.
Used when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}

\item{ref.group}{a character string specifying the reference group. If
specified, for a given grouping variable, each of the group levels will be
compared to the reference (control) group, and the p-value adjustment is
computed over only these comparisons. Note that, like \code{dunn_test()},
\code{conover_test()} does not support \code{ref.group = "all"}.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}
}
\value{
return a data frame with some of the following columns: \itemize{
 \item \code{.y.}: the y (outcome) variable used in the test. \item
 \code{group1,group2}: the compared groups in the pairwise tests. \item
 \code{n1,n2}: Sample counts. \item \code{estimate}: mean ranks difference.
 \item \code{estimate1, estimate2}: show the mean rank values of the two
 groups, respectively. \item \code{statistic}: Test statistic (t-value) used
 to compute the p-value. \item \code{df}: degrees of freedom (\eqn{N - k},
 the same for every comparison). \item \code{p}: p-value. \item \code{p.adj}:
 the adjusted p-value. \item \code{method}: the statistical test used to
 compare groups. \item \code{p.adj.signif}: the significance level of the
 adjusted p-values. }

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs Conover's test (also known as the Conover-Iman test) for
 pairwise multiple comparisons of the ranked data, following a significant
 Kruskal-Wallis test. It is closely related to \code{\link{dunn_test}()}, but
 uses the pooled within-group rank variance and refers the test statistic to a
 \emph{t}-distribution (with \eqn{N - k} degrees of freedom) instead of the
 standard normal distribution. The Conover-Iman test is generally more
 powerful than Dunn's test, but should only be used as a post-hoc procedure
 when the Kruskal-Wallis test is itself significant (Conover, 1999).

 If a reference group is specified (via \code{ref.group}), then each of the
 remaining group levels is compared only to the reference (control) group, and
 the p-value adjustment for multiple comparisons is computed over only these
 \code{k - 1} comparisons (instead of all \code{k(k - 1)/2} pairwise
 comparisons), exactly as for \code{\link{dunn_test}()}.
}
\details{
The Conover-Iman pairwise statistic for comparing groups \eqn{i} and
 \eqn{j} is \deqn{t_{ij} = \frac{\bar{R}_i - \bar{R}_j}{\sqrt{S^2 \,
 \frac{N - 1 - H}{N - k} \left(\frac{1}{n_i} + \frac{1}{n_j}\right)}}} where
 \eqn{\bar{R}} are the mean ranks, \eqn{H} is the (tie-corrected)
 Kruskal-Wallis statistic, \eqn{N} is the total sample size, \eqn{k} is the
 number of groups, and \eqn{S^2} is the variance of the ranks
 (\eqn{S^2 = N(N+1)/12} when there are no ties; otherwise
 \eqn{S^2 = \frac{1}{N - 1}\left(\sum r^2 - \frac{N(N+1)^2}{4}\right)}). The
 statistic is referred to a \emph{t}-distribution with \eqn{N - k} degrees of
 freedom.
}
\examples{
# Simple test
ToothGrowth \%>\% conover_test(len ~ dose)

# Comparison against a reference (control) group
# each group is compared to the reference; the p-value
# adjustment corrects for only these k - 1 comparisons
ToothGrowth \%>\% conover_test(len ~ dose, ref.group = "0.5")

# Grouped data
ToothGrowth \%>\%
  group_by(supp) \%>\%
  conover_test(len ~ dose)
}
\references{
Conover, W. J. (1999) Practical Nonparametric Statistics, 3rd
 edition. Wiley.

 Conover, W. J. and Iman, R. L. (1979) On multiple-comparisons procedures.
 Technical Report LA-7677-MS, Los Alamos Scientific Laboratory.
}
\seealso{
\code{\link{dunn_test}}, \code{\link{kruskal_test}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shapiro_test.R
\name{shapiro_test}
\alias{shapiro_test}
\alias{mshapiro_test}
\title{Shapiro-Wilk Normality Test}
\usage{
shapiro_test(data, ..., vars = NULL)

mshapiro_test(data)
}
\arguments{
\item{data}{a data frame. Columns are variables.}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used to select a variable of interest.}

\item{vars}{optional character vector containing variable names. Ignored when
dot vars are specified.}
}
\value{
a data frame containing the value of the Shapiro-Wilk statistic and
  the corresponding p.value.
}
\description{
Provides a pipe-friendly framework to performs Shapiro-Wilk test
  of normality. Support grouped data and multiple variables for multivariate
  normality tests. Wrapper around the R base function
  \code{\link[stats]{shapiro.test}()}. Can handle grouped data. Read more:
  \href{https://www.datanovia.com/en/lessons/normality-test-in-r/}{Normality
  Test in R}.
}
\section{Functions}{
\itemize{
\item \code{shapiro_test}: univariate Shapiro-Wilk normality test

\item \code{mshapiro_test}: multivariate Shapiro-Wilk normality test. This is a
modified copy of the \code{mshapiro.test()} function of the package
mvnormtest, for internal convenience.
}}

\examples{

# Shapiro Wilk normality test for one variable
iris \%>\% shapiro_test(Sepal.Length)

# Shapiro Wilk normality test for two variables
iris \%>\% shapiro_test(Sepal.Length, Petal.Width)

# Multivariate normality test
mshapiro_test(iris[, 1:3])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cochran_qtest.R
\name{cochran_qtest}
\alias{cochran_qtest}
\title{Cochran's Q Test}
\usage{
cochran_qtest(data, formula)
}
\arguments{
\item{data}{a data frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{a ~ b | c}, where \code{a} is the
outcome variable name; b is the within-subjects factor variables; and c
(factor) is the column name containing individuals/subjects identifier.
Should be unique per individual.}
}
\description{
Performs the Cochran's Q test for unreplicated randomized block
 design experiments with a binary response variable and paired data. This
 test is analogue to the \code{\link{friedman.test}()} with 0,1 coded
 response. It's an extension of the McNemar Chi-squared test for comparing
 more than two paired proportions.
}
\examples{
# Generate a demo data
mydata <- data.frame(
  outcome = c(0,1,1,0,0,1,0,1,1,1,1,1,0,0,1,1,0,1,0,1,1,0,0,1,0,1,1,0,0,1),
  treatment = gl(3,1,30,labels=LETTERS[1:3]),
  participant = gl(10,3,labels=letters[1:10])
)
mydata$outcome <- factor(
  mydata$outcome, levels = c(1, 0),
  labels = c("success", "failure")
  )
# Cross-tabulation
xtabs(~outcome + treatment, mydata)

# Compare the proportion of success between treatments
cochran_qtest(mydata, outcome ~ treatment|participant)

# pairwise comparisons between groups
pairwise_mcnemar_test(mydata, outcome ~ treatment|participant)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/df.R
\name{df_group_by}
\alias{df_group_by}
\title{Group a Data Frame by One or more Variables}
\usage{
df_group_by(data, ..., vars = NULL)
}
\arguments{
\item{data}{a data frame}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used to select a variable of interest.}

\item{vars}{a character vector containing the variable names of interest.}
}
\description{
Group a data frame by one or more variables. Supports standard
  and non standard evaluation.
}
\examples{

# Non standard evaluation
by_dose <- head(ToothGrowth) \%>\%
   df_group_by(dose)
by_dose

# Standard evaluation
head(ToothGrowth) \%>\%
   df_group_by(vars = c("dose", "supp"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor_test.R
\name{cor_test}
\alias{cor_test}
\title{Correlation Test}
\usage{
cor_test(
  data,
  ...,
  vars = NULL,
  vars2 = NULL,
  alternative = "two.sided",
  method = "pearson",
  conf.level = 0.95,
  use = "pairwise.complete.obs"
)
}
\arguments{
\item{data}{a data.frame containing the variables.}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used to select a variable of interest. Alternative to the argument
\code{vars}.}

\item{vars}{optional character vector containing variable names for
correlation analysis. Ignored when dot vars are specified. \itemize{ \item
If \code{vars} is NULL, multiple pairwise correlation tests is performed
between all variables in the data. \item If \code{vars} contain only one
variable, a pairwise correlation analysis is performed between the specified
variable vs either all the remaining numeric variables in the data or
variables in \code{vars2} (if specified). \item If \code{vars} contain two
or more variables: i) if \code{vars2} is not specified, a pairwise
correlation analysis is performed between all possible combinations of
variables. ii) if \code{vars2} is specified, each element in \code{vars} is
tested against all elements in \code{vars2}}. Accept unquoted
variable names: \code{c(var1, var2)}.}

\item{vars2}{optional character vector. If specified, each element in
\code{vars} is tested against all elements in \code{vars2}. Accept unquoted
variable names: \code{c(var1, var2)}.}

\item{alternative}{indicates the alternative hypothesis and must be
    one of \code{"two.sided"}, \code{"greater"} or \code{"less"}.  You
    can specify just the initial letter.  \code{"greater"} corresponds
    to positive association, \code{"less"} to negative association.}

\item{method}{a character string indicating which correlation
    coefficient is to be  used for the test.  One of \code{"pearson"},
    \code{"kendall"}, or \code{"spearman"}, can be abbreviated.}

\item{conf.level}{confidence level for the returned confidence
    interval.  Currently only used for the Pearson product moment
    correlation coefficient if there are at least 4 complete pairs of
    observations.}

\item{use}{an optional character string giving a
    method for computing covariances in the presence
    of missing values.  This must be (an abbreviation of) one of the strings
    \code{"everything"}, \code{"all.obs"}, \code{"complete.obs"},
    \code{"na.or.complete"}, or \code{"pairwise.complete.obs"}.}
}
\value{
return a data frame with the following columns: \itemize{ \item
 \code{var1, var2}: the variables used in the correlation test. \item
 \code{cor}: the correlation coefficient. \item \code{statistic}: Test
 statistic used to compute the p-value. \item \code{p}: p-value. \item
 \code{conf.low,conf.high}: Lower and upper bounds on a confidence interval.
 \item \code{method}: the method used to compute the statistic.}
}
\description{
Provides a pipe-friendly framework to perform correlation test
 between paired samples, using Pearson, Kendall or Spearman method. Wrapper
 around the function \code{\link[stats]{cor.test}()}.

 Can also performs multiple pairwise correlation analyses between more than
 two variables or between two different vectors of variables. Using this
 function, you can also compute, for example, the correlation between one
 variable vs many.
}
\section{Functions}{
\itemize{
\item \code{cor_test}: correlation test between two or more variables.
}}

\examples{

# Correlation between the specified variable vs
# the remaining numeric variables in the data
#:::::::::::::::::::::::::::::::::::::::::
mtcars \%>\% cor_test(mpg)

# Correlation test between two variables
#:::::::::::::::::::::::::::::::::::::::::
mtcars \%>\% cor_test(wt, mpg)

# Pairwise correlation between multiple variables
#:::::::::::::::::::::::::::::::::::::::::
mtcars \%>\% cor_test(wt, mpg, disp)

# Grouped data
#:::::::::::::::::::::::::::::::::::::::::
iris \%>\%
  group_by(Species) \%>\%
 cor_test(Sepal.Width, Sepal.Length)

# Multiple correlation test
#:::::::::::::::::::::::::::::::::::::::::
# Correlation between one variable vs many
mtcars \%>\% cor_test(
  vars = "mpg",
  vars2 = c("disp", "hp", "drat")
 )

# Correlation between two vectors of variables
# Each element in vars is tested against all elements in vars2
mtcars \%>\% cor_test(
  vars = c("mpg", "wt"),
  vars2 = c("disp", "hp", "drat")
 )


}
\seealso{
\code{\link{cor_mat}()}, \code{\link{as_cor_mat}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor_select.R
\name{cor_select}
\alias{cor_select}
\title{Subset Correlation Matrix}
\usage{
cor_select(x, ..., vars = NULL)
}
\arguments{
\item{x}{a correlation matrix. Particularly, an object of class \code{cor_mat}.}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used to select variables of interest.}

\item{vars}{a character vector containing the variable names of interest.}
}
\value{
a data frame
}
\description{
Subset Correlation Matrix
}
\examples{
# Compute correlation matrix
#::::::::::::::::::::::::::::::::::::::::::
cor.mat <- mtcars \%>\%
  select(mpg, disp, hp, drat, wt, qsec) \%>\%
  cor_mat()

# Subsetting correlation matrix
#::::::::::::::::::::::::::::::::::::::::::

# Select some variables of interest
cor.mat \%>\%
  cor_select(mpg, drat, wt)

# Remove variables
cor.mat \%>\%
  cor_select(-mpg, -wt)

}
\seealso{
\code{\link{cor_mat}()}, \code{\link{pull_triangle}()}, \code{\link{replace_triangle}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/factors.R
\name{convert_as_factor}
\alias{convert_as_factor}
\alias{set_ref_level}
\alias{reorder_levels}
\title{Factors}
\usage{
convert_as_factor(data, ..., vars = NULL, make.valid.levels = FALSE)

set_ref_level(data, name, ref)

reorder_levels(data, name, order)
}
\arguments{
\item{data}{a data frame}

\item{...}{one unquoted expressions (or variable name) specifying the name of
the variables you want to convert into factor. Alternative to the argument
\code{vars}.}

\item{vars}{a character vector specifying the variables to convert into
factor.}

\item{make.valid.levels}{logical. Default is FALSE. If TRUE, converts the
variable to factor and add a leading character (x) if starting with a digit.}

\item{name}{a factor variable name. Can be unquoted. For example, use
\code{group} or \code{"group"}.}

\item{ref}{the reference level.}

\item{order}{a character vector specifying the order of the factor levels}
}
\description{
Provides pipe-friendly functions to convert simultaneously
 multiple variables into a factor variable.

 Helper functions are also available to set the reference level and the
 levels order.
}
\section{Functions}{
\itemize{
\item \code{convert_as_factor}: Convert one or multiple variables into factor.

\item \code{set_ref_level}: Change a factor reference level or group.

\item \code{reorder_levels}: Change the order of a factor levels
}}

\examples{
# Create a demo data
df <- tibble(
  group = c("a", "a", "b", "b", "c", "c"),
  time = c("t1", "t2", "t1", "t2", "t1", "t2"),
  value = c(5, 6, 1, 3, 4, 5)
)
df
# Convert group and time into factor variable
result <- df \%>\% convert_as_factor(group, time)
result
# Show group levels
levels(result$group)

# Set c as the reference level (the first one)
result <- result \%>\%
  set_ref_level("group", ref = "c")
levels(result$group)

# Set the order of levels
result <- result \%>\%
  reorder_levels("group", order = c("b", "c", "a"))
levels(result$group)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cohens_d.R
\name{cohens_d}
\alias{cohens_d}
\title{Compute Cohen's d Measure of Effect Size}
\usage{
cohens_d(
  data,
  formula,
  comparisons = NULL,
  ref.group = NULL,
  paired = FALSE,
  mu = 0,
  var.equal = FALSE,
  hedges.correction = FALSE,
  ci = FALSE,
  conf.level = 0.95,
  ci.type = "perc",
  nboot = 1000
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{comparisons}{A list of length-2 vectors specifying the groups of
interest to be compared. For example to compare groups "A" vs "B" and "B" vs
"C", the argument is as follow: \code{comparisons = list(c("A", "B"), c("B",
"C"))}}

\item{ref.group}{a character string specifying the reference group. If
 specified, for a given grouping variable, each of the group levels will be
 compared to the reference group (i.e. control group).

 If \code{ref.group = "all"}, pairwise two sample tests are performed for
 comparing each grouping variable levels against all (i.e. basemean).}

\item{paired}{a logical indicating whether you want a paired test.}

\item{mu}{theoretical mean, use for one-sample t-test. Default is 0.}

\item{var.equal}{a logical variable indicating whether to treat the two
variances as being equal. If TRUE then the pooled variance is used to
estimate the variance otherwise the Welch (or Satterthwaite) approximation
to the degrees of freedom is used. Used only for unpaired or independent samples test.}

\item{hedges.correction}{logical indicating whether apply the Hedges
correction by multiplying the usual value of Cohen's d by
\code{(N-3)/(N-2.25)} (for unpaired t-test) and by \code{(n1-2)/(n1-1.25)} for paired t-test;
where \code{N} is the total size of the two groups being compared (N = n1 +
n2).}

\item{ci}{If TRUE, returns confidence intervals by bootstrap. May be slow.}

\item{conf.level}{The level for the confidence interval.}

\item{ci.type}{The type of confidence interval to use. Can be any of "norm",
"basic", "perc", or "bca". Passed to \code{boot::boot.ci}.}

\item{nboot}{The number of replications to use for bootstrap.}
}
\value{
return a data frame with some of the following columns: \itemize{
 \item \code{.y.}: the y variable used in the test. \item
 \code{group1,group2}: the compared groups in the pairwise tests. \item
 \code{n,n1,n2}: Sample counts. \item \code{effsize}: estimate of the effect
 size (\code{d} value). \item \code{magnitude}: magnitude of effect size.
 \item \code{conf.low,conf.high}: lower and upper bound of the effect size
 confidence interval.}
}
\description{
Compute the effect size for t-test. T-test conventional effect
 sizes, proposed by Cohen, are: 0.2 (small effect), 0.5 (moderate effect) and
 0.8 (large effect).

 Cohen's \code{d} is calculated as the difference between means or mean minus
 \code{mu} divided by the estimated standardized deviation.

 For independent samples t-test, there are two possibilities implemented. If
 the t-test did not make a homogeneity of variance assumption, (the Welch
 test), the variance term will mirror the Welch test, otherwise a pooled
 estimate is used.

 If a paired samples t-test was requested, then effect size desired is based
 on the standard deviation of the differences.

 It can also returns confidence intervals by bootstap.
}
\details{
Quantification of the effect size magnitude is performed using the
 thresholds defined in Cohen (1992). The magnitude is assessed using the
 thresholds provided in (Cohen 1992), i.e. \code{|d| < 0.2} "negligible",
 \code{|d| < 0.5} "small", \code{|d| < 0.8} "medium", otherwise "large".
}
\examples{
# One-sample t test effect size
ToothGrowth \%>\% cohens_d(len ~ 1, mu = 0)

# Two indepedent samples t-test effect size
ToothGrowth \%>\% cohens_d(len ~ supp, var.equal = TRUE)

# Paired samples effect size
df <- data.frame(
  id = 1:5,
  pre  = c(110, 122, 101, 120, 140),
  post = c(150, 160, 110, 140, 155)
)
df <- df \%>\% gather(key = "treatment", value = "value", -id)
head(df)

df \%>\% cohens_d(value ~ treatment, paired = TRUE)
}
\references{
\itemize{ \item Cohen, J. (1988). Statistical power analysis for
 the behavioral sciences (2nd ed.). New York:Academic Press. \item Cohen, J.
 (1992). A power primer. Psychological Bulletin, 112, 155-159. \item Hedges,
 Larry & Olkin, Ingram. (1985). Statistical Methods in Meta-Analysis.
 10.2307/1164953. \item Navarro, Daniel. 2015. Learning Statistics with R: A
 Tutorial for Psychology Students and Other Beginners (Version 0.5). }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_summary_stats.R
\name{get_summary_stats}
\alias{get_summary_stats}
\title{Compute Summary Statistics}
\usage{
get_summary_stats(
  data,
  ...,
  type = c("full", "common", "robust", "five_number", "mean_sd", "mean_se", "mean_ci",
    "median_iqr", "median_mad", "quantile", "mean", "median", "min", "max"),
  show = NULL,
  probs = seq(0, 1, 0.25),
  digits = 3
)
}
\arguments{
\item{data}{a data frame}

\item{...}{(optional) One or more unquoted expressions (or variable names)
separated by commas. Used to select a variable of interest. If no variable
is specified, then the summary statistics of all numeric variables in the
data frame is computed.}

\item{type}{type of summary statistics. Possible values include: \code{"full",
"common", "robust",  "five_number", "mean_sd", "mean_se", "mean_ci",
"median_iqr", "median_mad", "quantile", "mean", "median",  "min", "max"}}

\item{show}{a character vector specifying the summary statistics you want to
show. Example: \code{show = c("n", "mean", "sd")}. This is used to filter
the output after computation. It can additionally include \code{"skewness"}
and/or \code{"kurtosis"} (e.g. \code{show = c("mean", "sd", "skewness",
"kurtosis")}); these two are computed on demand and are not part of any
default \code{type}.}

\item{probs}{numeric vector of probabilities with values in [0,1]. Used only when type = "quantile".}

\item{digits}{integer indicating the number of decimal places to round the
summary statistics to. Default is 3. Increase it when summarizing very small
values that would otherwise round to 0.}
}
\value{
A data frame containing descriptive statistics, such as: \itemize{
 \item \strong{n}: the number of individuals \item \strong{min}: minimum
 \item \strong{max}: maximum \item \strong{median}: median \item
 \strong{mean}: mean \item \strong{q1, q3}: the first and the third quartile,
 respectively. \item \strong{iqr}: interquartile range \item \strong{mad}:
 median absolute deviation (see ?MAD) \item \strong{sd}: standard deviation
 of the mean \item \strong{se}: standard error of the mean \item \strong{ci}: 95 percent confidence interval of the mean }

 When requested through \code{show}, the output can also contain: \itemize{
 \item \strong{skewness}: bias-corrected sample skewness \item
 \strong{kurtosis}: bias-corrected sample excess kurtosis (0 for a normal
 distribution). } Both use the type-2 (bias-corrected) estimator, matching
 \code{e1071} with \code{type = 2}:
 skewness \eqn{= g_1\sqrt{n(n-1)}/(n-2)} and kurtosis \eqn{= [(n+1)g_2 + 6]
 (n-1)/[(n-2)(n-3)]}, where \eqn{g_1 = m_3/m_2^{1.5}} and \eqn{g_2 =
 m_4/m_2^2 - 3}. Skewness is \code{NA} for n < 3 and kurtosis for n < 4.
}
\description{
Compute summary statistics for one or multiple numeric variables.
}
\examples{
# Full summary statistics
data("ToothGrowth")
ToothGrowth \%>\% get_summary_stats(len)

# Summary statistics of grouped data
# Show only common summary
ToothGrowth \%>\%
  group_by(dose, supp) \%>\%
  get_summary_stats(len, type = "common")

# Robust summary statistics
ToothGrowth \%>\% get_summary_stats(len, type = "robust")

# Five number summary statistics
ToothGrowth \%>\% get_summary_stats(len, type = "five_number")

# Compute only mean and sd
ToothGrowth \%>\% get_summary_stats(len, type = "mean_sd")

# Compute full summary statistics but show only mean, sd, median, iqr
ToothGrowth \%>\%
    get_summary_stats(len, show = c("mean", "sd", "median", "iqr"))

# Include skewness and kurtosis (computed on demand via show)
ToothGrowth \%>\%
    get_summary_stats(len, show = c("mean", "sd", "skewness", "kurtosis"))

}
\seealso{
\code{\link{rstatix-programming}} for selecting columns by names held
 in strings (\code{!!}, \code{\{\{ \}\}}, \code{vars=}, \code{all_of()}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/factorial_design.R
\name{factorial_design}
\alias{factorial_design}
\title{Build Factorial Designs for ANOVA}
\usage{
factorial_design(data, dv, wid, between, within, covariate)
}
\arguments{
\item{data}{a data frame containing the variables}

\item{dv}{(numeric) dependent variable name.}

\item{wid}{(factor) column name containing individuals/subjects identifier.
Should be unique per individual.}

\item{between}{(optional) between-subject factor variables.}

\item{within}{(optional) within-subjects factor variables}

\item{covariate}{(optional) covariate names (for ANCOVA)}
}
\value{
a list with the following components: \itemize{ \item \strong{the
 specified arguments}: \code{dv, wid, between, within} \item \strong{data}:
 the original data (long format) or independent ANOVA. The wide format is
 returned for repeated measures ANOVA. \item \strong{idata}: an optional data
 frame giving the levels of factors defining the intra-subject model for
 multivariate repeated-measures data. \item \strong{idesign}: a one-sided
 model formula using the “data” in idata and specifying the intra-subject
 design. \item \strong{repeated}: logical. Value is TRUE when the data is a
 repeated design. \item \strong{lm_formula}: the formula used to build the
 \code{lm} model. \item \strong{lm_data}: the data used to build the \code{lm}
 model. Can be either in a long format (i.e., the original data for
 independent measures ANOVA) or in a wide format (case of repeated measures ANOVA). \item \strong{model}: the \code{lm} model }
}
\description{
Provides helper functions to build factorial design for easily
 computing ANOVA using the \code{\link[car]{Anova}()} function. This might be
 very useful for repeated measures ANOVA, which is hard to set up with the
 \code{car} package.
}
\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth
head(df)

# Repeated measures designs
#:::::::::::::::::::::::::::::::::::::::::
# Prepare the data
df$id <- rep(1:10, 6) # Add individuals id
head(df)
# Build factorial designs
design <- factorial_design(df, dv = len, wid = id, within = c(supp, dose))
design
# Easily perform repeated measures ANOVA using the car package
res.anova <- Anova(design$model, idata = design$idata, idesign = design$idesign, type = 3)
summary(res.anova, multivariate = FALSE)

# Independent measures designs
#:::::::::::::::::::::::::::::::::::::::::
# Build factorial designs
df$id <- 1:nrow(df)
design <- factorial_design(df, dv = len, wid = id, between = c(supp, dose))
design
# Perform ANOVA
Anova(design$model, type = 3)

}
\seealso{
\code{\link{anova_test}()}, \code{\link{anova_summary}()}
}
\author{
Alboukadel Kassambara, \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ks_test.R
\name{ks_test}
\alias{ks_test}
\title{Two-Sample Kolmogorov-Smirnov Test}
\usage{
ks_test(
  data,
  formula,
  comparisons = NULL,
  ref.group = NULL,
  p.adjust.method = "holm",
  alternative = "two.sided",
  exact = NULL,
  detailed = FALSE
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with two
or more levels giving the corresponding groups.}

\item{comparisons}{A list of length-2 vectors specifying the groups of interest
to be compared. For example to compare groups "A" vs "B" and "B" vs "C", the
argument is as follow: \code{comparisons = list(c("A", "B"), c("B", "C"))}.}

\item{ref.group}{a character string specifying the reference group. If
specified, for a given grouping variable, each of the group levels will be
compared to the reference group (i.e. control group). If \code{ref.group =
"all"}, pairwise two sample tests are performed for comparing each grouping
variable level against all (i.e. basemean).}

\item{p.adjust.method}{method to adjust p values for multiple comparisons. Used
when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}

\item{alternative}{indicates the alternative hypothesis and must be
    one of \code{"two.sided"} (default), \code{"less"}, or
    \code{"greater"}.  You can specify just the initial letter of the
    value, but the argument name must be given in full.
    See \sQuote{Details} for the meanings of the possible values.}

\item{exact}{\code{NULL} or a logical indicating whether an exact
    p-value should be computed.  See \sQuote{Details} for the meaning of
    \code{NULL}.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}
}
\value{
return a data frame with some of the following columns: \itemize{ \item
 \code{.y.}: the y variable used in the test. \item \code{group1,group2}: the
 compared groups in the pairwise tests. \item \code{n1,n2}: sample counts.
 \item \code{statistic}: the value of the test statistic \code{D} (the maximum
 difference between the two empirical cumulative distribution functions). \item
 \code{p}: p-value. \item \code{p.adj}: the adjusted p-value. \item
 \code{method}: the statistical test used to compare groups. \item
 \code{p.signif, p.adj.signif}: the significance level of p-values and adjusted
 p-values, respectively. \item \code{alternative}: the alternative hypothesis.}

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Provides a pipe-friendly framework to perform the two-sample
 Kolmogorov-Smirnov test, comparing the (empirical) distributions of a numeric
 variable between two groups. Wrapper around the R base function
 \code{\link[stats]{ks.test}()}.

 When the grouping factor contains more than two levels, pairwise
 Kolmogorov-Smirnov tests are automatically performed, with p-value
 adjustment.
}
\examples{
# Two-samples test
#:::::::::::::::::::::::::::::::::::::::::
ToothGrowth \%>\% ks_test(len ~ supp)

# Pairwise comparisons (more than two groups)
#:::::::::::::::::::::::::::::::::::::::::
ToothGrowth \%>\% ks_test(len ~ dose)

# Comparison against a reference group
#:::::::::::::::::::::::::::::::::::::::::
ToothGrowth \%>\% ks_test(len ~ dose, ref.group = "0.5")
}
\seealso{
\code{\link{wilcox_test}()}, \code{\link{t_test}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/df.R
\name{df_arrange}
\alias{df_arrange}
\title{Arrange Rows by Column Values}
\usage{
df_arrange(data, ..., vars = NULL, .by_group = FALSE)
}
\arguments{
\item{data}{a data frame}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used to select a variable of interest. Use
\code{\link[dplyr]{desc}()} to sort a variable in descending order.}

\item{vars}{a character vector containing the variable names of interest.}

\item{.by_group}{If TRUE, will sort first by grouping variable. Applies to
grouped data frames only.}
}
\value{
a data frame
}
\description{
Order the rows of a data frame by values of specified columns.
  Wrapper arround the \code{\link[dplyr]{arrange}()} function. Supports
  standard and non standard evaluation.
}
\examples{
df <- head(ToothGrowth)
df

# Select column using standard evaluation
df \%>\% df_arrange(vars = c("dose", "len"))

# Select column using non-standard evaluation
df \%>\% df_arrange(dose, desc(len))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/counts_to_cases.R
\name{counts_to_cases}
\alias{counts_to_cases}
\title{Convert a Table of Counts into a Data Frame of cases}
\usage{
counts_to_cases(x, count.col = "Freq")
}
\arguments{
\item{x}{a contingency table or a data frame}

\item{count.col}{the name of the column containing the counts. Default is "Freq".}
}
\value{
a data frame of cases
}
\description{
converts a contingency table or a data frame of counts into a
  data frame of individual observations.
}
\examples{
# Create a cross-tabulation demo data
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
xtab <- as.table(
  rbind(c(20, 5), c(16,9))
)
dimnames(xtab) <- list(
  before = c("non.smoker", "smoker"),
  after = c("non.smoker", "smoker")
)
xtab

# Convert into a data frame of cases
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
df <- counts_to_cases(xtab)
head(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_pvalue_position.R
\name{get_y_position}
\alias{get_y_position}
\alias{add_y_position}
\alias{add_x_position}
\alias{add_xy_position}
\title{Autocompute P-value Positions For Plotting Significance}
\usage{
get_y_position(
  data,
  formula,
  fun = "max",
  ref.group = NULL,
  comparisons = NULL,
  step.increase = 0.12,
  y.trans = NULL,
  stack = FALSE
)

add_y_position(
  test,
  fun = "max",
  step.increase = 0.12,
  data = NULL,
  formula = NULL,
  ref.group = NULL,
  comparisons = NULL,
  y.trans = NULL,
  stack = FALSE
)

add_x_position(test, x = NULL, dodge = 0.8)

add_xy_position(
  test,
  x = NULL,
  dodge = 0.8,
  stack = FALSE,
  fun = "max",
  step.increase = 0.12,
  ...
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{fun}{summary statistics functions used to compute automatically suitable
 y positions of p-value labels and brackets. Possible values include:
 \code{"max", "mean", "mean_sd", "mean_se", "mean_ci", "median",
 "median_iqr", "median_mad"}.

 For example, if \code{fun = "max"}, the y positions are guessed as follow:
 \itemize{ \item 1. Compute the maximum of each group (groups.maximum) \item
 2. Use the highest groups maximum as the first bracket y position \item 3.
 Add successively a step increase for remaining bracket y positions. }

 When the main plot is a boxplot, you need the option \code{fun = "max"}, to
 have the p-value bracket displayed at the maximum point of the group.

 In some situations the main plot is a line plot or a barplot showing the
 \code{mean+/-error bars} of the groups, where error can be SE (standard
 error), SD (standard deviation) or CI (confidence interval). In this case,
 to correctly compute the bracket y position you need the option \code{fun =
 "mean_se"}, etc.}

\item{ref.group}{a character string specifying the reference group. If
specified, for a given grouping variable, each of the group levels will be
compared to the reference group (i.e. control group).}

\item{comparisons}{A list of length-2 vectors specifying the groups of
interest to be compared. For example to compare groups "A" vs "B" and "B" vs
"C", the argument is as follow: \code{comparisons = list(c("A", "B"), c("B",
"C"))}}

\item{step.increase}{numeric vector with the increase in fraction of total
height for every additional comparison to minimize overlap.}

\item{y.trans}{a function for transforming y axis scale. Value can be
\code{log2}, \code{log10} and \code{sqrt}. Can be also any custom function
that can take a numeric vector as input and returns a numeric vector,
example: \code{y.trans = function(x){log2(x+1)}}}

\item{stack}{logical. If TRUE, computes y position for a stacked plot. Useful
when dealing with stacked bar plots.}

\item{test}{an object of class \code{rstatix_test} as returned by
\code{\link{t_test}()}, \code{\link{wilcox_test}()},
\code{\link{sign_test}()}, \code{\link{tukey_hsd}()},
\code{\link{dunn_test}()}.}

\item{x}{variable on x axis.}

\item{dodge}{dodge width for grouped ggplot/test. Default is 0.8. Used only
when \code{x} specified.}

\item{...}{other arguments to be passed to the function
\code{\link[stats]{t.test}}.}
}
\description{
Compute p-value x and y positions for plotting significance
 levels. Many examples are provided at :
 \itemize{
 \item \href{https://www.datanovia.com/en/blog/how-to-add-p-values-onto-a-grouped-ggplot-using-the-ggpubr-r-package/}{How to Add P-Values onto a Grouped GGPLOT using the GGPUBR R Package}
 \item \href{https://www.datanovia.com/en/blog/ggpubr-how-to-add-adjusted-p-values-to-a-multi-panel-ggplot/}{How to Add Adjusted P-values to a Multi-Panel GGPlot}
 \item \href{https://www.datanovia.com/en/blog/ggpubr-how-to-add-p-values-generated-elsewhere-to-a-ggplot/}{How to Add P-Values Generated Elsewhere to a GGPLOT}
 }
}
\section{Functions}{
\itemize{
\item \code{get_y_position}: compute the p-value y positions

\item \code{add_y_position}: add p-value y positions to an object of class \code{rstatix_test}

\item \code{add_x_position}: compute and add p-value x positions.

\item \code{add_xy_position}: compute and add both x and y positions.
}}

\examples{
# Data preparation
#::::::::::::::::::::::::::::::::::::
df <- ToothGrowth
df$dose <- as.factor(df$dose)
df$group <- factor(rep(c(1, 2), 30))
head(df)

# Stat tests
#::::::::::::::::::::::::::::::::::::
stat.test <- df \%>\%
  t_test(len ~ dose)
stat.test

# Add the test into box plots
#::::::::::::::::::::::::::::::::::::
stat.test <- stat.test \%>\%
  add_y_position()
\donttest{
 if(require("ggpubr")){
   ggboxplot(df, x = "dose", y = "len") +
     stat_pvalue_manual(stat.test, label = "p.adj.signif", tip.length = 0.01)
  }
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cramer_v.R
\name{cramer_v}
\alias{cramer_v}
\title{Compute Cramer's V}
\usage{
cramer_v(x, y = NULL, correct = TRUE, ...)
}
\arguments{
\item{x}{a numeric vector or matrix. \code{x} and \code{y} can also
    both be factors.}

\item{y}{a numeric vector; ignored if \code{x} is a matrix.  If
    \code{x} is a factor, \code{y} should be a factor of the same length.}

\item{correct}{a logical indicating whether to apply continuity
    correction when computing the test statistic for 2 by 2 tables: one
    half is subtracted from all \eqn{|O - E|} differences; however, the
    correction will not be bigger than the differences themselves.  No correction
    is done if \code{simulate.p.value = TRUE}.}

\item{...}{other arguments passed to the function
\code{\link[stats]{chisq.test}()}.}
}
\description{
Compute Cramer's V, which measures the strength of the
 association between categorical variables.
}
\examples{

# Data preparation
df <- as.table(rbind(c(762, 327, 468), c(484, 239, 477)))
dimnames(df) <- list(
  gender = c("F", "M"),
  party = c("Democrat","Independent", "Republican")
)
df
# Compute cramer's V
cramer_v(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wilcox_effsize.R
\name{wilcox_effsize}
\alias{wilcox_effsize}
\title{Wilcoxon Effect Size}
\usage{
wilcox_effsize(
  data,
  formula,
  comparisons = NULL,
  ref.group = NULL,
  paired = FALSE,
  alternative = "two.sided",
  mu = 0,
  ci = FALSE,
  conf.level = 0.95,
  ci.type = "perc",
  nboot = 1000,
  ...
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{comparisons}{A list of length-2 vectors specifying the groups of
interest to be compared. For example to compare groups "A" vs "B" and "B" vs
"C", the argument is as follow: \code{comparisons = list(c("A", "B"), c("B",
"C"))}}

\item{ref.group}{a character string specifying the reference group. If
 specified, for a given grouping variable, each of the group levels will be
 compared to the reference group (i.e. control group).

 If \code{ref.group = "all"}, pairwise two sample tests are performed for
 comparing each grouping variable levels against all (i.e. basemean).}

\item{paired}{a logical indicating whether you want a paired test.}

\item{alternative}{a character string specifying the alternative
    hypothesis, must be one of \code{"two.sided"} (default),
    \code{"greater"} or \code{"less"}.  You can specify just the initial
    letter.}

\item{mu}{a number specifying an optional parameter used to form the null
hypothesis.}

\item{ci}{If TRUE, returns confidence intervals by bootstrap. May be slow.}

\item{conf.level}{The level for the confidence interval.}

\item{ci.type}{The type of confidence interval to use. Can be any of "norm",
"basic", "perc", or "bca". Passed to \code{boot::boot.ci}.}

\item{nboot}{The number of replications to use for bootstrap.}

\item{...}{Additional arguments passed to the functions
\code{coin::wilcoxsign_test()} (case of one- or paired-samples test) or
\code{coin::wilcox_test()} (case of independent two-samples test).}
}
\value{
return a data frame with some of the following columns: \itemize{
 \item \code{.y.}: the y variable used in the test. \item
 \code{group1,group2}: the compared groups in the pairwise tests. \item
 \code{n,n1,n2}: Sample counts. \item \code{effsize}: estimate of the effect
 size (\code{r} value). \item \code{magnitude}: magnitude of effect size.
 \item \code{conf.low,conf.high}: lower and upper bound of the effect size
 confidence interval.}
}
\description{
Compute Wilcoxon effect size (\code{r}) for: \itemize{ \item
 one-sample test (Wilcoxon one-sample signed-rank test); \item paired
 two-samples test (Wilcoxon two-sample paired signed-rank test) and \item
 independent two-samples test ( Mann-Whitney, two-sample rank-sum test). }

 It can also returns confidence intervals by bootstap.

 The effect size \code{r} is calculated as \code{Z} statistic divided by
 square root of the sample size (N) (\eqn{Z/\sqrt{N}}). The \code{Z} value is
 extracted from either \code{coin::wilcoxsign_test()} (case of one- or
 paired-samples test) or \code{coin::wilcox_test()} (case of independent
 two-samples test).

 Note that \code{N} corresponds to total sample size for independent samples
 test and to total number of pairs for paired samples test.

 The \code{r} value varies from 0 to close to 1. The interpretation values
 for r commonly in published litterature and on the internet are: \code{0.10
 - < 0.3} (small effect), \code{0.30 - < 0.5} (moderate effect) and \code{>=
 0.5} (large effect).
}
\examples{
if(require("coin")){

# One-sample Wilcoxon test effect size
ToothGrowth \%>\% wilcox_effsize(len ~ 1, mu = 0)

# Independent two-samples wilcoxon effect size
ToothGrowth \%>\% wilcox_effsize(len ~ supp)


# Paired-samples wilcoxon effect size
ToothGrowth \%>\% wilcox_effsize(len ~ supp, paired = TRUE)

# Pairwise comparisons
ToothGrowth \%>\% wilcox_effsize(len ~ dose)

# Grouped data
ToothGrowth \%>\%
  group_by(supp) \%>\%
  wilcox_effsize(len ~ dose)

}
}
\references{
Maciej Tomczak and Ewa Tomczak. The need to report effect size
 estimates revisited. An overview of some recommended measures of effect
 size. Trends in Sport Sciences. 2014; 1(21):19-25.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fligner_test.R
\name{fligner_test}
\alias{fligner_test}
\title{Fligner-Killeen Test}
\usage{
fligner_test(data, formula, ...)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{...}{other arguments to be passed to the function
\code{\link[stats]{fligner.test}}.}
}
\value{
return a data frame with the following columns: \itemize{ \item
 \code{.y.}: the y variable used in the test. \item \code{n}: sample count.
 \item \code{statistic}: the Fligner-Killeen test statistic (a chi-squared
 statistic) used to compute the p-value. \item \code{df}: the degrees of
 freedom. \item \code{p}: p-value. \item \code{method}: the statistical test
 used to compare groups.}
}
\description{
Provides a pipe-friendly framework to perform the Fligner-Killeen
 test, a non-parametric (rank-based) test of the homogeneity of group
 variances. It is robust against departures from normality and is a useful
 alternative to \code{\link{levene_test}()}. Wrapper around the function
 \code{\link[stats]{fligner.test}()}.
}
\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth

# Fligner-Killeen test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% fligner_test(len ~ dose)

# Grouped data
df \%>\%
  group_by(supp) \%>\%
  fligner_test(len ~ dose)
}
\seealso{
\code{\link{levene_test}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/df.R
\name{df_nest_by}
\alias{df_nest_by}
\title{Nest a Tibble By Groups}
\usage{
df_nest_by(data, ..., vars = NULL)
}
\arguments{
\item{data}{a data frame}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used as grouping variables.}

\item{vars}{a character vector containing the grouping variables of interest.}
}
\value{
A tbl with one row per unique combination of the grouping variables.
  The first columns are the grouping variables, followed by a list column of
  tibbles with matching rows of the remaining columns.
}
\description{
Nest a tibble data frame using grouping specification. Supports standard and non standard evaluation.
}
\examples{

# Non standard evaluation
ToothGrowth \%>\%
 df_nest_by(dose, supp)

# Standard evaluation
ToothGrowth \%>\%
 df_nest_by(vars = c("dose", "supp"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/friedman_effsize.R
\name{friedman_effsize}
\alias{friedman_effsize}
\title{Friedman Test Effect Size (Kendall's W Value)}
\usage{
friedman_effsize(
  data,
  formula,
  ci = FALSE,
  conf.level = 0.95,
  ci.type = "perc",
  nboot = 1000,
  ...
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{a ~ b | c}, where \code{a}
(numeric) is the dependent variable name; \code{b} is the within-subjects
factor variables; and \code{c} (factor) is the column name containing
individuals/subjects identifier. Should be unique per individual.}

\item{ci}{If TRUE, returns confidence intervals by bootstrap. May be slow.}

\item{conf.level}{The level for the confidence interval.}

\item{ci.type}{The type of confidence interval to use. Can be any of "norm",
"basic", "perc", or "bca". Passed to \code{boot::boot.ci}.}

\item{nboot}{The number of replications to use for bootstrap.}

\item{...}{other arguments passed to the function \code{\link[stats]{friedman.test}()}}
}
\value{
return a data frame with some of the following columns: \itemize{
 \item \code{.y.}: the y variable used in the test. \item \code{n}: Sample
 counts. \item \code{effsize}: estimate of the effect size. \item
 \code{magnitude}: magnitude of effect size. \item \code{conf.low,conf.high}:
 lower and upper bound of the effect size confidence interval.}
}
\description{
Compute the effect size estimate (referred to as \code{w}) for
 Friedman test: \code{W = X2/N(K-1)}; where \code{W} is the Kendall's W
 value; \code{X2} is the Friedman test statistic value; \code{N} is the sample
 size. \code{k} is the number of measurements per subject.

 The Kendall’s W coefficient assumes the value from 0 (indicating no
 relationship) to 1 (indicating a perfect relationship).

 Kendalls uses the Cohen’s interpretation guidelines of \code{0.1 - < 0.3} (small
 effect), \code{0.3 - < 0.5} (moderate effect) and \code{>= 0.5} (large
 effect)

 Confidence intervals are calculated by bootstap.
}
\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth \%>\%
    filter(supp == "VC") \%>\%
    mutate(id = rep(1:10, 3))
head(df)

# Friedman test effect size
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% friedman_effsize(len ~ dose | id)
}
\references{
Maciej Tomczak and Ewa Tomczak. The need to report effect size
 estimates revisited. An overview of some recommended measures of effect
 size. Trends in Sport Sciences. 2014; 1(21):19-25.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outliers.R
\name{identify_outliers}
\alias{identify_outliers}
\alias{is_outlier}
\alias{is_extreme}
\title{Identify Univariate Outliers Using Boxplot Methods}
\usage{
identify_outliers(data, ..., variable = NULL)

is_outlier(x, coef = 1.5)

is_extreme(x)
}
\arguments{
\item{data}{a data frame}

\item{...}{One unquoted expressions (or variable name). Used to select a
variable of interest. Alternative to the argument \code{variable}.}

\item{variable}{variable name for detecting outliers}

\item{x}{a numeric vector}

\item{coef}{coefficient specifying how far the outlier should be from the edge
of their box. Possible values are 1.5 (for outlier) and 3 (for extreme
points only). Default is 1.5}
}
\value{
\itemize{ \item \code{identify_outliers()}. Returns the input data
 frame with two additional columns: "is.outlier" and "is.extreme", which hold
 logical values. \item \code{is_outlier() and is_extreme()}. Returns logical
 vectors. }
}
\description{
Detect outliers using boxplot methods. Boxplots are a popular and
 an easy method for identifying outliers. There are two categories of
 outlier: (1) outliers and (2) extreme points.

 Values above \code{Q3 + 1.5xIQR} or below \code{Q1 - 1.5xIQR} are considered
 as outliers. Values above \code{Q3 + 3xIQR} or below \code{Q1 - 3xIQR} are
 considered as extreme points (or extreme outliers).

 Q1 and Q3 are the first and third quartile, respectively. IQR is the
 interquartile range (IQR = Q3 - Q1).

 Generally speaking, data points that are labelled outliers in boxplots are
 not considered as troublesome as those considered extreme points and might
 even be ignored. Note that, any \code{NA} and \code{NaN} are automatically removed
 before the quantiles are computed.
}
\section{Functions}{
\itemize{
\item \code{identify_outliers}: takes a data frame and extract rows suspected as outliers
according to a numeric column. The following columns are added "is.outlier"
and "is.extreme".

\item \code{is_outlier}: detect outliers in a numeric vector. Returns logical vector.

\item \code{is_extreme}: detect extreme points in a numeric vector. An alias of
\code{is_outlier()}, where coef = 3. Returns logical vector.
}}

\examples{
# Generate a demo data
set.seed(123)
demo.data <- data.frame(
  sample = 1:20,
  score = c(rnorm(19, mean = 5, sd = 2), 50),
  gender = rep(c("Male", "Female"), each = 10)
)

# Identify outliers according to the variable score
demo.data \%>\%
  identify_outliers(score)

# Identify outliers by groups
demo.data \%>\%
  group_by(gender) \%>\%
  identify_outliers("score")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor_as_symbols.R
\name{cor_as_symbols}
\alias{cor_as_symbols}
\title{Replace Correlation Coefficients by Symbols}
\usage{
cor_as_symbols(
  x,
  cutpoints = c(0, 0.25, 0.5, 0.75, 1),
  symbols = c(" ", ".", "+", "*")
)
}
\arguments{
\item{x}{a correlation matrix. Particularly, an object of class \code{cor_mat}.}

\item{cutpoints}{numeric vector used for intervals. Default values are
\code{c(0, 0.25, 0.5, 0.75, 1)}.}

\item{symbols}{character vector, one shorter than cutpoints, used as
correlation coefficient symbols. Default values are \code{c(" ", ".",  "+",
"*")}.}
}
\description{
Take a correlation matrix and replace the correlation coefficients by symbols according to the
  level of the correlation.
}
\examples{
# Compute correlation matrix
#::::::::::::::::::::::::::::::::::::::::::
cor.mat <- mtcars \%>\%
  select(mpg, disp, hp, drat, wt, qsec) \%>\%
  cor_mat()

# Replace correlation coefficient by symbols
#::::::::::::::::::::::::::::::::::::::::::
cor.mat \%>\%
  cor_as_symbols() \%>\%
  pull_lower_triangle()

}
\seealso{
\code{\link{cor_mat}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anova_test.R
\name{anova_test}
\alias{anova_test}
\alias{get_anova_table}
\alias{print.anova_test}
\alias{plot.anova_test}
\title{Anova Test}
\usage{
anova_test(
  data,
  formula,
  dv,
  wid,
  between,
  within,
  covariate,
  type = NULL,
  effect.size = "ges",
  error = NULL,
  white.adjust = FALSE,
  observed = NULL,
  detailed = FALSE,
  ci = NULL
)

get_anova_table(x, correction = c("auto", "GG", "HF", "none"))

\method{print}{anova_test}(x, ...)

\method{plot}{anova_test}(x, ...)
}
\arguments{
\item{data}{a data.frame or a model to be analyzed.}

\item{formula}{a formula specifying the ANOVA model similar to
 \link[stats]{aov}. Can be of the form \code{y ~ group} where \code{y} is a
 numeric variable giving the data values and \code{group} is a factor with
 one or multiple levels giving the corresponding groups. For example,
 \code{formula = TP53 ~ cancer_group}.

 Examples of supported formula include: \itemize{ \item Between-Ss ANOVA
 (independent measures ANOVA): \code{y ~ b1*b2} \item Within-Ss ANOVA
 (repeated measures ANOVA): \code{y ~ w1*w2 + Error(id/(w1*w2))} \item Mixed
 ANOVA: \code{y ~ b1*b2*w1 + Error(id/w1)} }

 If the formula doesn't contain any within vars, a linear model is directly
 fitted and  passed to the ANOVA function. For repeated designs, the ANOVA
 variables are parsed from the formula.}

\item{dv}{(numeric) dependent variable name.}

\item{wid}{(factor) column name containing individuals/subjects identifier.
Should be unique per individual.}

\item{between}{(optional) between-subject factor variables.}

\item{within}{(optional) within-subjects factor variables}

\item{covariate}{(optional) covariate names (for ANCOVA)}

\item{type}{the type of sums of squares for ANOVA. Allowed values are either
 1, 2 or 3. \code{type = 2} is the recommended default because it yields
 identical ANOVA results to type = 1 when data are balanced, while also
 producing various assumption tests where appropriate. When the data are
 unbalanced, \code{type = 3} is the choice used by popular commercial
 softwares including SPSS.

 \strong{Default when \code{type} is not specified:} the chosen default
 depends on the interface (see Note), so for unbalanced designs the two
 interfaces can differ. To get reproducible, interface-independent results,
 \strong{set \code{type} explicitly} (e.g. \code{type = 3} for an unbalanced
 factorial design with interactions).}

\item{effect.size}{the effect size to compute and to show in the ANOVA
results. Allowed values can be either "ges" (generalized eta squared) or
"pes" (partial eta squared) or both. Default is "ges".}

\item{error}{(optional) for a linear model, an lm model object from which the
overall error sum of squares and degrees of freedom are to be calculated.
Read more in \code{\link[car]{Anova}()} documentation.}

\item{white.adjust}{Default is FALSE. If TRUE, heteroscedasticity correction
is applied to the coefficient of covariance matrix. Used only for
independent measures ANOVA.}

\item{observed}{Variables that are observed (i.e, measured) as compared to
experimentally manipulated. The default effect size reported (generalized
eta-squared) requires correct specification of the observed variables.}

\item{detailed}{If TRUE, returns extra information (sums of squares columns,
intercept row, etc.) in the ANOVA table.}

\item{ci}{confidence level for a confidence interval on the effect size. If a
number between 0 and 1 (e.g. \code{0.95}), two columns \code{conf.low} and
\code{conf.high} are added giving the confidence interval for \strong{partial
eta squared} (\code{effect.size} must include \code{"pes"}). The interval is
computed from the noncentral F distribution (Steiger, 2004) in base R. No
interval is provided for generalized eta squared (\code{"ges"}), which has no
standard closed-form interval. Default is \code{NULL} (no interval; output
unchanged).}

\item{x}{an object of class \code{anova_test}}

\item{correction}{character. Used only in repeated measures ANOVA test to
specify which correction of the degrees of freedom should be reported for
the within-subject factors. Possible values are: \itemize{ \item{"GG"}:
applies Greenhouse-Geisser correction to all within-subjects factors even if
the assumption of sphericity is met (i.e., Mauchly's test is not
significant, p > 0.05). \item{"HF"}: applies Hyunh-Feldt correction to all
within-subjects factors even if the assumption of sphericity is met,
\item{"none"}: returns the ANOVA table without any correction and
\item{"auto"}: apply automatically GG correction to only within-subjects
factors violating the sphericity assumption (i.e., Mauchly's test p-value is
significant, p <= 0.05). }}

\item{...}{additional arguments}
}
\value{
return an object of class \code{anova_test} a data frame containing
 the ANOVA table for independent measures ANOVA.

 However, for repeated/mixed measures ANOVA, a list containing the following
 components are returned: ANOVA table, Mauchly's Test for Sphericity,
 Sphericity Corrections. These table are described more in the documentation
 of the function \code{\link{anova_summary}()}.

 The \strong{returned object has an attribute} called \code{args}, which is a
 list holding the arguments used to fit the ANOVA model, including: data, dv,
 within, between, type, model, etc.
}
\description{
Provides a pipe-friendly framework to perform different types of
 ANOVA tests, including: \itemize{ \item
 \strong{\href{https://www.datanovia.com/en/lessons/anova-in-r/}{Independent
 measures ANOVA}}: between-Subjects designs, \item
 \strong{\href{https://www.datanovia.com/en/lessons/repeated-measures-anova-in-r/}{Repeated
  measures ANOVA}}: within-Subjects designs \item
 \strong{\href{https://www.datanovia.com/en/lessons/mixed-anova-in-r/}{Mixed
 ANOVA}}: Mixed within within- and between-Subjects designs, also known as
 split-plot ANOVA and \item
 \strong{\href{https://www.datanovia.com/en/lessons/ancova-in-r/}{ANCOVA:
 Analysis of Covariance}}. }

 The function is an easy to use wrapper around \code{\link[car]{Anova}()} and
 \code{\link[stats]{aov}()}. It makes ANOVA computation handy in R and It's
 highly flexible: can support model and formula as input. Variables can be
 also specified as character vector using the arguments \code{dv, wid,
 between, within, covariate}.

 The results include ANOVA table, generalized effect size and some assumption
 checks.
}
\details{
\strong{Contrasts}. By default, R uses treatment contrasts
 (\code{contr.treatment}), where each factor level is compared to the first
 level used as baseline; the current setting can be checked with
 \code{options('contrasts')}.

 How \code{anova_test()} handles contrasts depends on the interface you use:
 \itemize{ \item When you use the \strong{formula interface} (or the \code{dv}
 + \code{between}/\code{within} arguments), \code{anova_test()} fits the model
 internally with \code{options(contrasts = c('contr.sum', 'contr.poly'))},
 restoring your global option afterwards. This gives orthogonal contrasts,
 where every level is compared to the overall mean, and type-III results that
 match the most commonly used commercial softwares, like SPSS. \item When you
 instead pass a \strong{pre-fitted model} (\code{lm()} or \code{aov()}),
 \code{anova_test()} does not change its contrasts: the model keeps whatever
 contrasts were in effect when it was fitted (R's default
 \code{contr.treatment} unless you set otherwise). Fitting with the default
 treatment contrasts and then requesting \code{type = 3} can therefore give
 different results from the formula interface. }

 To reproduce the formula-interface (SPSS) result from a pre-fitted model, or
 to obtain the same result with \code{car::Anova()} directly, set
 \code{options(contrasts = c('contr.sum', 'contr.poly'))} \emph{before}
 fitting the model and use \code{type = 3}.
}
\section{Functions}{
\itemize{
\item \code{anova_test()}: perform anova test

\item \code{get_anova_table()}: extract anova table from an object of class
\code{anova_test}. When within-subject factors are present, either
sphericity corrected or uncorrected degrees of freedom can be reported.

}}
\note{
\strong{Default sums-of-squares type differs between the two interfaces
 for unbalanced designs.} When \code{type} is not supplied:
 \itemize{
 \item the \strong{formula} interface (\code{anova_test(data, y ~ a*b)}) uses
 \strong{type II} for between-subjects designs, regardless of balance;
 \item the \strong{\code{dv=}/\code{between=}/\code{within=}} interface uses
 type II for \emph{balanced} between-subjects designs but switches to
 \strong{type III} for \emph{unbalanced} between-subjects designs with more
 than one factor (both interfaces use type III for repeated-measures designs).
 }
 For a \strong{balanced} design the SS types coincide, so the two interfaces
 agree. For an \strong{unbalanced} factorial design they can give different
 main-effect F/p values unless you \strong{pass \code{type} explicitly} — which
 is recommended for reproducibility. Example:
 \preformatted{
 d <- mtcars \%>\% dplyr::mutate(cyl = factor(cyl), am = factor(am))
 # differ (unbalanced): formula -> type II, dv/between -> type III
 d \%>\% anova_test(mpg ~ cyl * am)
 d \%>\% anova_test(dv = mpg, between = c(cyl, am))
 # agree once type is explicit:
 d \%>\% anova_test(mpg ~ cyl * am, type = 3)
 d \%>\% anova_test(dv = mpg, between = c(cyl, am), type = 3)
 }
}
\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth

# One-way ANOVA test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% anova_test(len ~ dose)

# Grouped One-way ANOVA test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\%
  group_by(supp) \%>\%
  anova_test(len ~ dose)

# Two-way ANOVA test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% anova_test(len ~ supp*dose)

# Two-way repeated measures ANOVA
#:::::::::::::::::::::::::::::::::::::::::
df$id <- rep(1:10, 6) # Add individuals id
# Use formula
\donttest{
df \%>\% anova_test(len ~ supp*dose + Error(id/(supp*dose)))
}


# or use character vector
df \%>\% anova_test(dv = len, wid = id, within = c(supp, dose))

# Extract ANOVA table and apply correction
#:::::::::::::::::::::::::::::::::::::::::
res.aov <- df \%>\% anova_test(dv = len, wid = id, within = c(supp, dose))
get_anova_table(res.aov, correction = "GG")


# Use model as arguments
#:::::::::::::::::::::::::::::::::::::::::
.my.model <- lm(yield ~ block + N*P*K, npk)
anova_test(.my.model)


}
\seealso{
\code{\link{anova_summary}()}, \code{\link{factorial_design}()}
}
\author{
Alboukadel Kassambara, \email{alboukadel.kassambara@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emmeans_test.R
\name{emmeans_test}
\alias{emmeans_test}
\alias{get_emmeans}
\title{Pairwise Comparisons of Estimated Marginal Means}
\usage{
emmeans_test(
  data,
  formula,
  covariate = NULL,
  ref.group = NULL,
  comparisons = NULL,
  p.adjust.method = "bonferroni",
  conf.level = 0.95,
  model = NULL,
  detailed = FALSE
)

get_emmeans(emmeans.test)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{covariate}{(optional) covariate names (for ANCOVA)}

\item{ref.group}{a character string specifying the reference group. If
 specified, for a given grouping variable, each of the group levels will be
 compared to the reference group (i.e. control group).

 If \code{ref.group = "all"}, pairwise two sample tests are performed for
 comparing each grouping variable levels against all (i.e. basemean).}

\item{comparisons}{A list of length-2 vectors specifying the groups of
interest to be compared. For example to compare groups "A" vs "B" and "B" vs
"C", the argument is as follow: \code{comparisons = list(c("A", "B"), c("B",
"C"))}}

\item{p.adjust.method}{method to adjust p values for multiple comparisons.
Used when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}

\item{conf.level}{confidence level of the interval.}

\item{model}{a fitted-model objects such as the result of a call to
\code{lm()}, from which the overall degrees of
freedom are to be calculated.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}

\item{emmeans.test}{an object of class \code{emmeans_test}.}
}
\value{
return a data frame with some the following columns: \itemize{ \item
 \code{.y.}: the y variable used in the test. \item \code{group1,group2}: the
 compared groups in the pairwise tests. \item \code{statistic}: Test
 statistic (t.ratio) used to compute the p-value. \item \code{df}: degrees of
 freedom. \item \code{p}: p-value. \item \code{p.adj}: the adjusted p-value.
 \item \code{method}: the statistical test used to compare groups. \item
 \code{p.signif, p.adj.signif}: the significance level of p-values and
 adjusted p-values, respectively. \item \code{estimate}: estimate of the
 effect size, that is the difference between the two emmeans (estimated
 marginal means). \item \code{conf.low,conf.high}: Lower and upper bound on a
 confidence interval of the estimate. }

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments. It has also an attribute named "emmeans", a data
 frame containing the groups emmeans.
}
\description{
Performs pairwise comparisons between groups using the estimated
 marginal means. Pipe-friendly wrapper arround the functions \code{emmans() +
 contrast()} from the \code{emmeans} package, which need to be installed
 before using this function. This function is useful for performing post-hoc
 analyses following ANOVA/ANCOVA tests.
}
\section{Functions}{
\itemize{
\item \code{get_emmeans}: returns the estimated marginal means from an object of class \code{emmeans_test}
}}

\examples{
# Data preparation
df <- ToothGrowth
df$dose <- as.factor(df$dose)

# Pairwise comparisons
res <- df \%>\%
 group_by(supp) \%>\%
 emmeans_test(len ~ dose, p.adjust.method = "bonferroni")
res

# Display estimated marginal means
attr(res, "emmeans")

# Show details
df \%>\%
 group_by(supp) \%>\%
 emmeans_test(len ~ dose, p.adjust.method = "bonferroni", detailed = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_test.R
\name{prop_test}
\alias{prop_test}
\alias{pairwise_prop_test}
\alias{row_wise_prop_test}
\title{Proportion Test}
\usage{
prop_test(
  x,
  n,
  p = NULL,
  alternative = c("two.sided", "less", "greater"),
  correct = TRUE,
  conf.level = 0.95,
  detailed = FALSE
)

pairwise_prop_test(xtab, p.adjust.method = "holm", ...)

row_wise_prop_test(xtab, p.adjust.method = "holm", detailed = FALSE, ...)
}
\arguments{
\item{x}{a vector of counts of successes, a one-dimensional table with
    two entries, or a two-dimensional table (or matrix) with 2 columns,
    giving the counts of successes and failures, respectively.}

\item{n}{a vector of counts of trials; ignored if \code{x} is a
    matrix or a table.}

\item{p}{a vector of probabilities of success.  The length of
    \code{p} must be the same as the number of groups specified by
    \code{x}, and its elements must be greater than 0 and less than 1.}

\item{alternative}{a character string specifying the alternative
    hypothesis, must be one of \code{"two.sided"} (default),
    \code{"greater"} or \code{"less"}.  You can specify just the initial
    letter.  Only used for testing the null that a single proportion
    equals a given value, or that two proportions are equal; ignored
    otherwise.}

\item{correct}{a logical indicating whether Yates' continuity
    correction should be applied where possible.}

\item{conf.level}{confidence level of the returned confidence
    interval.  Must be a single number between 0 and 1.  Only used
    when testing the null that a single proportion equals a given
    value, or that two proportions are equal; ignored otherwise.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}

\item{xtab}{a cross-tabulation (or contingency table) with two columns and
multiple rows (rx2 design). The columns give the counts of successes and
failures respectively.}

\item{p.adjust.method}{method to adjust p values for multiple comparisons.
Used when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}

\item{...}{Other arguments passed to the function \code{prop_test()}.}
}
\value{
return a data frame with some the following columns: \itemize{
\item \code{n}: the number of participants.
\item \code{group}: the categories in the row-wise proportion tests. \item
 \code{statistic}: the value of Pearson's chi-squared test statistic. \item
 \code{df}: the degrees of freedom of the approximate chi-squared
 distribution of the test statistic. \item \code{p}: p-value. \item
 \code{p.adj}: the adjusted p-value. \item \code{method}: the used
 statistical test. \item \code{p.signif, p.adj.signif}: the significance
 level of p-values and adjusted p-values, respectively. \item
 \code{estimate}: a vector with the sample proportions x/n. \item
 \code{estimate1, estimate2}: the proportion in each of the two populations.
 \item \code{alternative}: a character string describing the alternative
 hypothesis. \item \code{conf.low,conf.high}: Lower and upper bound on a
 confidence interval. a confidence interval for the true proportion if there
 is one group, or for the difference in proportions if there are 2 groups and
 p is not given, or NULL otherwise. In the cases where it is not NULL, the
 returned confidence interval has an asymptotic confidence level as specified
 by conf.level, and is appropriate to the specified alternative hypothesis.}

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs proportion tests to either evaluate the homogeneity of
 proportions (probabilities of success) in several groups or to test that the
 proportions are equal to certain given values.

 Wrappers around the R base function \code{\link[stats]{prop.test}()} but have
 the advantage of performing pairwise and row-wise z-test of two proportions,
 the post-hoc tests following a significant chi-square test of homogeneity
 for 2xc and rx2 contingency tables.
}
\section{Functions}{
\itemize{
\item \code{prop_test}: performs one-sample and two-samples z-test of
proportions. Wrapper around the function \code{\link[stats]{prop.test}()}.

\item \code{pairwise_prop_test}: pairwise comparisons between proportions, a post-hoc
tests following a significant chi-square test of homogeneity for 2xc
design. Wrapper around \code{\link[stats]{pairwise.prop.test}()}

\item \code{row_wise_prop_test}: performs row-wise z-test of two proportions, a post-hoc tests following a significant chi-square test
of homogeneity for rx2 contingency table. The z-test of two proportions is calculated for each category (row).
}}

\examples{
# Comparing an observed proportion to an expected proportion
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
prop_test(x = 95, n = 160, p = 0.5, detailed = TRUE)

# Comparing two proportions
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Data: frequencies of smokers between two groups
xtab <- as.table(rbind(c(490, 10), c(400, 100)))
dimnames(xtab) <- list(
  group = c("grp1", "grp2"),
  smoker = c("yes", "no")
)
xtab
# compare the proportion of smokers
prop_test(xtab, detailed = TRUE)

# Homogeneity of proportions between groups
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# H0: the proportion of smokers is similar in the four groups
# Ha:  this proportion is different in at least one of the populations.
#
# Data preparation
grp.size <- c( 106, 113, 156, 102 )
smokers  <- c( 50, 100, 139, 80 )
no.smokers <- grp.size - smokers
xtab <- as.table(rbind(
  smokers,
  no.smokers
))
dimnames(xtab) <- list(
  Smokers = c("Yes", "No"),
  Groups = c("grp1", "grp2", "grp3", "grp4")
)
xtab

# Compare the proportions of smokers between groups
prop_test(xtab, detailed = TRUE)

# Pairwise comparison between groups
pairwise_prop_test(xtab)


# Pairwise proportion tests
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Data: Titanic
xtab <- as.table(rbind(
  c(122, 167, 528, 673),
  c(203, 118, 178, 212)
))
dimnames(xtab) <- list(
  Survived = c("No", "Yes"),
  Class = c("1st", "2nd", "3rd", "Crew")
)
xtab
# Compare the proportion of survived between groups
pairwise_prop_test(xtab)

# Row-wise proportion tests
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Data: Titanic
xtab <- as.table(rbind(
  c(180, 145), c(179, 106),
  c(510, 196), c(862, 23)
))
dimnames(xtab) <- list(
  Class = c("1st", "2nd", "3rd", "Crew"),
  Gender = c("Male", "Female")
)
xtab
# Compare the proportion of males and females in each category
row_wise_prop_test(xtab)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freq_table.R
\name{freq_table}
\alias{freq_table}
\title{Compute Frequency Table}
\usage{
freq_table(data, ..., vars = NULL, na.rm = TRUE)
}
\arguments{
\item{data}{a data frame}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used to specify variables of interest.}

\item{vars}{optional character vector containing variable names.}

\item{na.rm}{logical value. If TRUE (default), remove missing values in the
variables used to create the frequency table.}
}
\value{
a data frame
}
\description{
compute frequency table.
}
\examples{
data("ToothGrowth")
ToothGrowth \%>\% freq_table(supp, dose)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dunn_test.R
\name{dunn_test}
\alias{dunn_test}
\title{Dunn's Test of Multiple Comparisons}
\usage{
dunn_test(data, formula, p.adjust.method = "holm", detailed = FALSE)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{p.adjust.method}{method to adjust p values for multiple comparisons.
Used when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}
}
\value{
return a data frame with some of the following columns: \itemize{
 \item \code{.y.}: the y (outcome) variable used in the test. \item
 \code{group1,group2}: the compared groups in the pairwise tests. \item
 \code{n1,n2}: Sample counts. \item \code{estimate}: mean ranks difference.
 \item \code{statistic}: Test statistic (z-value) used to compute the
 p-value. \item \code{p}: p-value. \item \code{p.adj}: the adjusted p-value.
 \item \code{method}: the statistical test used to compare groups. \item
 \code{p.signif, p.adj.signif}: the significance level of p-values and
 adjusted p-values, respectively. }

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs Dunn's test for pairwise multiple comparisons of the
 ranked data. The mean rank of the different groups is compared. Used for
 post-hoc test following Kruskal-Wallis test.
}
\details{
DunnTest performs the post hoc pairwise multiple comparisons
 procedure appropriate to follow up a Kruskal-Wallis test, which is a
 non-parametric analog of the one-way ANOVA. The Wilcoxon rank sum test,
 itself a non-parametric analog of the unpaired t-test, is possibly
 intuitive, but inappropriate as a post hoc pairwise test, because (1) it
 fails to retain the dependent ranking that produced the Kruskal-Wallis test
 statistic, and (2) it does not incorporate the pooled variance estimate
 implied by the null hypothesis of the Kruskal-Wallis test.
}
\examples{
# Simple test
ToothGrowth \%>\% dunn_test(len ~ dose)

# Grouped data
ToothGrowth \%>\%
  group_by(supp) \%>\%
  dunn_test(len ~ dose)
}
\references{
Dunn, O. J. (1964) Multiple comparisons using rank sums
 Technometrics, 6(3):241-252.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eta_squared.R
\name{eta_squared}
\alias{eta_squared}
\alias{partial_eta_squared}
\title{Effect Size for ANOVA}
\usage{
eta_squared(model)

partial_eta_squared(model)
}
\arguments{
\item{model}{an object of class aov or anova.}
}
\value{
a numeric vector with the effect size statistics
}
\description{
Compute eta-squared and partial eta-squared for all terms in an
  ANOVA model.
}
\section{Functions}{
\itemize{
\item \code{eta_squared}: compute eta squared

\item \code{partial_eta_squared}: compute partial eta squared.
}}

\examples{
# Data preparation
df <- ToothGrowth
df$dose <- as.factor(df$dose)

# Compute ANOVA
res.aov <- aov(len ~ supp*dose, data = df)
summary(res.aov)

# Effect size
eta_squared(res.aov)
partial_eta_squared(res.aov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjust_pvalue.R
\name{adjust_pvalue}
\alias{adjust_pvalue}
\title{Adjust P-values for Multiple Comparisons}
\usage{
adjust_pvalue(data, p.col = NULL, output.col = NULL, method = "holm")
}
\arguments{
\item{data}{a data frame containing a p-value column}

\item{p.col}{column name containing p-values}

\item{output.col}{the output column name to hold the adjusted p-values}

\item{method}{method for adjusting p values (see
\code{\link[stats]{p.adjust}}). Allowed values include "holm", "hochberg",
"hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't want to
adjust the p value (not recommended), use p.adjust.method = "none".}
}
\value{
a data frame
}
\description{
A pipe-friendly function to add an adjusted p-value column into
  a data frame. Supports grouped data.
}
\examples{
# Perform pairwise comparisons and adjust p-values
ToothGrowth \%>\%
 t_test(len ~ dose) \%>\%
 adjust_pvalue()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/welch_anova_test.R
\name{welch_anova_test}
\alias{welch_anova_test}
\title{Welch One-Way ANOVA Test}
\usage{
welch_anova_test(data, formula)
}
\arguments{
\item{data}{a data frame containing the variables in the formula.}

\item{formula}{a formula specifying the ANOVA model similar to aov. Can be of
the form y ~ group where y is a numeric variable giving the data values and
group is a factor with one or multiple levels giving the corresponding
groups. For example, formula = TP53 ~ cancer_group.}
}
\value{
return a data frame with the following columns: \itemize{ \item
  \code{.y.}: the y variable used in the test. \item \code{n}: sample count.
  \item \code{statistic}: the value of the test statistic. \item \code{p}:
  p-value. \item \code{method}: the statistical test used to compare groups.}
}
\description{
Tests for equal means in a one-way design (not assuming equal
  variance). A wrapper around the base function
  \code{\link[stats]{oneway.test}()}. This is is an alternative to the
  standard one-way ANOVA in the situation where the homogeneity of variance
  assumption is violated.
}
\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth
df$dose <- as.factor(df$dose)

# Welch one-way ANOVA test (not assuming equal variance)
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% welch_anova_test(len ~ dose)

# Grouped data
#:::::::::::::::::::::::::::::::::::::::::
df \%>\%
  group_by(supp) \%>\%
  welch_anova_test(len ~ dose)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binom_test.R
\name{binom_test}
\alias{binom_test}
\alias{pairwise_binom_test}
\alias{pairwise_binom_test_against_p}
\title{Exact Binomial Test}
\usage{
binom_test(
  x,
  n,
  p = 0.5,
  alternative = "two.sided",
  conf.level = 0.95,
  detailed = FALSE
)

pairwise_binom_test(
  x,
  p.adjust.method = "holm",
  alternative = "two.sided",
  conf.level = 0.95
)

pairwise_binom_test_against_p(
  x,
  p = rep(1/length(x), length(x)),
  p.adjust.method = "holm",
  alternative = "two.sided",
  conf.level = 0.95
)
}
\arguments{
\item{x}{numeric vector containing the counts.}

\item{n}{number of trials; ignored if \code{x} has length 2.}

\item{p}{a vector of probabilities of success. The length of p must be the
same as the number of groups specified by x, and its elements must be
greater than 0 and less than 1.}

\item{alternative}{indicates the alternative hypothesis and must be
    one of \code{"two.sided"}, \code{"greater"} or \code{"less"}.
    You can specify just the initial letter.}

\item{conf.level}{confidence level for the returned confidence
    interval.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}

\item{p.adjust.method}{method to adjust p values for multiple comparisons.
Used when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}
}
\value{
return a data frame containing the p-value and its significance. with
 some the following columns: \itemize{ \item \code{group, group1, group2}:
 the categories or groups being compared. \item \code{statistic}: the number
 of successes. \item \code{parameter}: the number of trials. \item \code{p}:
 p-value of the test. \item \code{p.adj}: the adjusted p-value. \item
 \code{method}: the used statistical test. \item \code{p.signif,
 p.adj.signif}: the significance level of p-values and adjusted p-values,
 respectively. \item \code{estimate}: the estimated probability of success.
 \item \code{alternative}: a character string describing the alternative
 hypothesis. \item \code{conf.low,conf.high}: Lower and upper bound on a
 confidence interval  for the probability of success.}

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs exact binomial test and pairwise comparisons following a
 significant exact multinomial test. Wrapper around the R base function
 \code{link[stats]{binom.test}()} that returns a data frame as a result.
}
\section{Functions}{
\itemize{
\item \code{binom_test}: performs exact binomial test. Wrapper around the R
base function \code{\link[stats]{binom.test}} that returns a dataframe as a
result.

\item \code{pairwise_binom_test}: performs pairwise comparisons (binomial test)
following a significant exact multinomial test.

\item \code{pairwise_binom_test_against_p}: performs pairwise comparisons (binomial test)
following a significant exact multinomial test for given probabilities.
}}

\examples{
# Exact binomial test
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Data: 160 mice with cancer including 95 male and 65 female
# Q1: Does cancer affect more males than females?
binom_test(x = 95, n = 160)
# => yes, there are a significant difference


# Q2: compare the observed proportion of males
# to an expected proportion (p = 3/5)
binom_test(x = 95, n = 160, p = 3/5)
# => there are no significant difference

# Multinomial test
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Data
tulip <- c(red = 81, yellow = 50, white = 27)
# Question 1: are the color equally common ?
# this is a test of homogeneity
res <- multinom_test(tulip)
res
attr(res, "descriptives")

# Pairwise comparisons between groups
pairwise_binom_test(tulip, p.adjust.method = "bonferroni")


# Question 2: comparing observed to expected proportions
# this is a goodness-of-fit test
expected.p <- c(red = 0.5, yellow = 0.33, white = 0.17)
res <- multinom_test(tulip, expected.p)
res
attr(res, "descriptives")

# Pairwise comparisons against a given probabilities
pairwise_binom_test_against_p(tulip, expected.p)
}
\seealso{
\link{multinom_test}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_n_by.R
\name{sample_n_by}
\alias{sample_n_by}
\title{Sample n Rows By Group From a Table}
\usage{
sample_n_by(data, ..., size = 1, replace = FALSE)
}
\arguments{
\item{data}{a data frame}

\item{...}{Variables to group by}

\item{size}{the number of rows to select}

\item{replace}{with or without replacement?}
}
\description{
sample n rows by group from a table using the \code{\link[dplyr]{sample_n}()} function.
}
\examples{
ToothGrowth \%>\% sample_n_by(dose, supp, size = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multinom_test.R
\name{multinom_test}
\alias{multinom_test}
\title{Exact Multinomial Test}
\usage{
multinom_test(x, p = rep(1/length(x), length(x)), detailed = FALSE)
}
\arguments{
\item{x}{numeric vector containing the counts.}

\item{p}{a vector of probabilities of success. The length of p must be the
same as the number of groups specified by x, and its elements must be
greater than 0 and less than 1.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}
}
\value{
return a data frame containing the p-value and its significance.

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs an exact multinomial test. Alternative to the chi-square test of goodness-of-fit-test when the sample
 size is small.
}
\examples{
# Data
tulip <- c(red = 81, yellow = 50, white = 27)

# Question 1: are the color equally common ?
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# this is a test of homogeneity
res <- multinom_test(tulip)
res

attr(res, "descriptives")

# Pairwise comparisons between groups
pairwise_binom_test(tulip, p.adjust.method = "bonferroni")


# Question 2: comparing observed to expected proportions
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# this is a goodness-of-fit test
expected.p <- c(red = 0.5, yellow = 0.33, white = 0.17)
res <- multinom_test(tulip, expected.p)
res
attr(res, "descriptives")

# Pairwise comparisons against a given probabilities
pairwise_binom_test_against_p(tulip, expected.p)
}
\seealso{
\link{binom_test}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_mode.R
\name{get_mode}
\alias{get_mode}
\title{Compute Mode}
\usage{
get_mode(x)
}
\arguments{
\item{x}{a vector. Can be numeric, factor or character vector.}
}
\description{
Compute the mode in a given vector. Mode is the most frequent
  value.
}
\examples{

# Mode of numeric vector
x <- c(1:5, 6, 6, 7:10)
get_mode(x)

# Bimodal
x <- c(1:5, 6, 6, 7, 8, 9, 9, 10)
get_mode(x)

# No mode
x <- c(1, 2, 3, 4, 5)
get_mode(x)

# Nominal vector
fruits <-  c(rep("orange", 10), rep("apple", 5), rep("lemon", 2))
get_mode(fruits)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/df.R
\name{df_split_by}
\alias{df_split_by}
\title{Split a Data Frame into Subset}
\usage{
df_split_by(
  data,
  ...,
  vars = NULL,
  label_col = "label",
  labeller = df_label_both,
  sep = c(", ", ":")
)
}
\arguments{
\item{data}{a data frame}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used as grouping variables.}

\item{vars}{a character vector containing the grouping variables of interest.}

\item{label_col}{column to hold the label of the data subsets. Default column
name is "label".}

\item{labeller}{A function that takes a data frame, the grouping variables,
label_col and label_sep arguments, and add labels into the data frame.
Example of possible values are: \code{\link{df_label_both}()} and
\code{\link{df_label_value}()}.}

\item{sep}{String separating labelling variables and values. Should be of
length 2 in the function \code{df_label_both()}. 1) One sep is used to
separate groups, for example ','; 2) The other sep between group name and
levels; for example ':'.}
}
\value{
A tbl with one row per unique combination of the grouping variables.
  The first columns are the grouping variables, followed by a list column of
  tibbles with matching rows of the remaining columns, and a column named
  label, containing labels.
}
\description{
Split a data frame by groups into subsets or data panel. Very
  similar to the function \code{\link{df_nest_by}()}. The only difference is
  that, it adds label to each data subset. Labels are the combination of the
  grouping variable levels. The column holding labels are named "label".
}
\examples{

# Split a data frame
# :::::::::::::::::::::::::::::::::::::::::::::::::
# Create a grouped data
res <- ToothGrowth \%>\%
  df_split_by(dose, supp)
res

# Show subsets
res$data

# Add panel/subset labels
res <- ToothGrowth \%>\%
  df_split_by(dose, supp)
res
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/df.R
\name{df_select}
\alias{df_select}
\title{Select Columns in a Data Frame}
\usage{
df_select(data, ..., vars = NULL)
}
\arguments{
\item{data}{a data frame}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used to select a variable of interest.}

\item{vars}{a character vector containing the variable names of interest.}
}
\value{
a data frame
}
\description{
A wrapper around the \code{\link[dplyr]{select}()} function for
  selection data frame columns. Supports standard and non standard
  evaluations. Usefull to easily program with \code{dplyr}
}
\examples{
df <- head(ToothGrowth)
df

# Select column using standard evaluation
df \%>\% df_select(vars = c("dose", "len"))

# Select column using non-standard evaluation
df \%>\% df_select(dose, len)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_ns.R
\name{remove_ns}
\alias{remove_ns}
\title{Remove Non-Significant from Statistical Tests}
\usage{
remove_ns(stat.test, col = NULL, signif.cutoff = 0.05)
}
\arguments{
\item{stat.test}{statistical test results returned by \code{rstatix}
functions or any data frame containing a p-value column.}

\item{col}{(optional) character specifying the column containing the p-value
or the significance information, to be used for the filtering step.
Possible values include: \code{"p"}, \code{"p.adj"}, \code{"p.signif"},
\code{"p.adj.signif"}. If missing, the function will automatically look for
p.adj.signif, p.adj, p.signif, p in this order.}

\item{signif.cutoff}{the significance cutoff; default is 0.05. Significance
is declared at \code{p-value <= signif.cutoff}}
}
\value{
a data frame
}
\description{
Filter out non-significant (NS) p-values from a statistical
  test. Can detect automatically p-value columns
}
\examples{
# Statistical test
stat.test <- PlantGrowth \%>\% wilcox_test(weight ~ group)
# Remove ns: automatic detection of p-value columns
stat.test \%>\% remove_ns()
# Remove ns by the column p
stat.test \%>\% remove_ns(col ="p")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_trend_test.R
\name{prop_trend_test}
\alias{prop_trend_test}
\title{Test for Trend in Proportions}
\usage{
prop_trend_test(xtab, score = NULL)
}
\arguments{
\item{xtab}{a cross-tabulation (or contingency table) with two columns and
multiple rows (rx2 design). The columns give the counts of successes and
failures respectively.}

\item{score}{group score. If \code{NULL}, the default is group number.}
}
\value{
return a data frame with some the following columns: \itemize{ \item
 \code{n}: the number of participants.  \item \code{statistic}: the value of
 Chi-squared trend test statistic. \item \code{df}: the degrees of
 freedom.
 \item \code{p}: p-value. \item
 \code{method}: the used statistical test. \item \code{p.signif}: the significance level of p-values and adjusted p-values,
 respectively.}

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs chi-squared test for trend in proportion. This test is
 also known as Cochran-Armitage trend test.

 Wrappers around the R base function \code{\link[stats]{prop.trend.test}()} but
 returns a data frame for easy data visualization.
}
\examples{
# Proportion of renal stone (calculi) across age
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Data
xtab <- as.table(rbind(
  c(384, 536, 335),
  c(951, 869, 438)
))
dimnames(xtab) <- list(
  stone = c("yes", "no"),
  age = c("30-39", "40-49", "50-59")
)
xtab
# Compare the proportion of survived between groups
prop_trend_test(xtab)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_comparisons.R
\name{get_comparisons}
\alias{get_comparisons}
\title{Create a List of Possible Comparisons Between Groups}
\usage{
get_comparisons(data, variable, ref.group = NULL)
}
\arguments{
\item{data}{a data frame}

\item{variable}{the grouping variable name. Can be unquoted.}

\item{ref.group}{a character string specifying the reference group. Can be
 unquoted. If numeric, then it should be quoted. If specified, for a
 given grouping variable, each of the group levels will be compared to the
 reference group (i.e. control group).

 If \code{ref.group = "all"}, pairwise comparisons are performed between each
 grouping variable levels against all (i.e. basemean).}
}
\value{
a list of all possible pairwise comparisons.
}
\description{
Create a list of possible pairwise comparisons between groups. If
 a reference group is specified, only comparisons against reference will be
 kept.
}
\examples{
# All possible pairwise comparisons
ToothGrowth \%>\%
  get_comparisons("dose")

# Comparisons against reference groups
ToothGrowth \%>\%
  get_comparisons("dose", ref.group = "0.5")

# Comparisons against  all (basemean)
ToothGrowth \%>\%
  get_comparisons("dose", ref.group = "all")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sign_test.R
\name{sign_test}
\alias{sign_test}
\alias{pairwise_sign_test}
\title{Sign Test}
\usage{
sign_test(
  data,
  formula,
  comparisons = NULL,
  ref.group = NULL,
  p.adjust.method = "holm",
  alternative = "two.sided",
  mu = 0,
  conf.level = 0.95,
  detailed = FALSE
)

pairwise_sign_test(
  data,
  formula,
  comparisons = NULL,
  ref.group = NULL,
  p.adjust.method = "holm",
  detailed = FALSE,
  ...
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ treatment}.}

\item{comparisons}{A list of length-2 vectors specifying the groups of
interest to be compared. For example to compare groups "A" vs "B" and "B" vs
"C", the argument is as follow: \code{comparisons = list(c("A", "B"), c("B",
"C"))}}

\item{ref.group}{a character string specifying the reference group. If
specified, for a given grouping variable, each of the group levels will be
compared to the reference group (i.e. control group).}

\item{p.adjust.method}{method to adjust p values for multiple comparisons.
Used when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}

\item{alternative}{a character string specifying the alternative
    hypothesis, must be one of \code{"two.sided"} (default),
    \code{"greater"} or \code{"less"}.  You can specify just the initial
    letter.}

\item{mu}{a single number representing the value of the population median
specified by the null hypothesis.}

\item{conf.level}{confidence level of the interval.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}

\item{...}{other arguments passed to the function \code{sign_test()}}
}
\value{
return a data frame with some the following columns: \itemize{ \item
 \code{.y.}: the y variable used in the test. \item \code{group1,group2}: the
 compared groups in the pairwise tests. \item \code{n,n1,n2}: Sample counts.
 \item \code{statistic}: Test statistic used to compute the p-value. That is
 the S-statistic (the number of positive differences between the data and the
 hypothesized median), with names attribute \code{"S"}. \item \code{df,
 parameter}: degrees of freedom. Here, the total number of valid differences.
 \item \code{p}: p-value.  \item \code{method}: the statistical test used to
 compare groups. \item \code{p.signif, p.adj.signif}: the significance level
 of p-values and adjusted p-values, respectively. \item \code{estimate}:
 estimate of the effect size. It corresponds to the median of the
 differences. \item \code{alternative}: a character string describing the
 alternative hypothesis. \item \code{conf.low,conf.high}: Lower and upper
 bound on a confidence interval of the estimate. }

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs one-sample and two-sample sign tests. Read more:
 \href{https://www.datanovia.com/en/lessons/sign-test-in-r/}{Sign Test in R}.
}
\section{Functions}{
\itemize{
\item \code{sign_test}: Sign test

\item \code{pairwise_sign_test}: performs pairwise two sample Wilcoxon test.
}}

\note{
This function is a reimplementation of the function \code{SignTest()}
 from the \code{DescTools} package.
}
\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth

# One-sample test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% sign_test(len ~ 1, mu = 0)


# Two-samples paired test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% sign_test(len ~ supp)


# Compare supp levels after grouping the data by "dose"
#::::::::::::::::::::::::::::::::::::::::
df \%>\%
  group_by(dose) \%>\%
  sign_test(data =., len ~ supp) \%>\%
  adjust_pvalue(method = "bonferroni") \%>\%
  add_significance("p.adj")

# pairwise comparisons
#::::::::::::::::::::::::::::::::::::::::
# As dose contains more than two levels ==>
# pairwise test is automatically performed.
df \%>\% sign_test(len ~ dose)

# Comparison against reference group
#::::::::::::::::::::::::::::::::::::::::
# each level is compared to the ref group
df \%>\% sign_test(len ~ dose, ref.group = "0.5")


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor_mark_significant.R
\name{cor_mark_significant}
\alias{cor_mark_significant}
\title{Add Significance Levels To a Correlation Matrix}
\usage{
cor_mark_significant(
  x,
  cutpoints = c(0, 1e-04, 0.001, 0.01, 0.05, 1),
  symbols = c("****", "***", "**", "*", "")
)
}
\arguments{
\item{x}{an object of class \code{\link{cor_mat}()}.}

\item{cutpoints}{numeric vector used for intervals.}

\item{symbols}{character vector, one shorter than cutpoints, used as
significance symbols.}
}
\value{
a data frame containing the lower triangular part of the correlation
  matrix marked by significance symbols.
}
\description{
Combines correlation coefficients and significance levels in a
  correlation matrix data.
}
\examples{
mtcars \%>\%
  select(mpg, disp, hp, drat, wt, qsec) \%>\%
  cor_mat() \%>\%
  cor_mark_significant()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wilcox_test.R
\name{wilcox_test}
\alias{wilcox_test}
\alias{pairwise_wilcox_test}
\title{Wilcoxon Tests}
\usage{
wilcox_test(
  data,
  formula,
  comparisons = NULL,
  ref.group = NULL,
  p.adjust.method = "holm",
  paired = FALSE,
  exact = NULL,
  alternative = "two.sided",
  mu = 0,
  conf.level = 0.95,
  detailed = FALSE,
  id = NULL,
  error.as.na = FALSE
)

pairwise_wilcox_test(
  data,
  formula,
  comparisons = NULL,
  ref.group = NULL,
  p.adjust.method = "holm",
  detailed = FALSE,
  ...
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{comparisons}{A list of length-2 vectors specifying the groups of
interest to be compared. For example to compare groups "A" vs "B" and "B" vs
"C", the argument is as follow: \code{comparisons = list(c("A", "B"), c("B",
"C"))}}

\item{ref.group}{a character string specifying the reference group. If
 specified, for a given grouping variable, each of the group levels will be
 compared to the reference group (i.e. control group).

 If \code{ref.group = "all"}, pairwise two sample tests are performed for
 comparing each grouping variable levels against all (i.e. basemean).}

\item{p.adjust.method}{method to adjust p values for multiple comparisons.
Used when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}

\item{paired}{a logical indicating whether you want a paired test.}

\item{exact}{a logical indicating whether an exact p-value
    should be computed.}

\item{alternative}{a character string specifying the alternative
    hypothesis, must be one of \code{"two.sided"} (default),
    \code{"greater"} or \code{"less"}.  You can specify just the initial
    letter.}

\item{mu}{a number specifying an optional parameter used to form the null
hypothesis.}

\item{conf.level}{confidence level of the interval.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}

\item{id}{(optional) character string specifying the column that contains the
sample/subject identifier, used only for a \strong{paired} test
(\code{paired = TRUE}). When supplied, observations of the two compared
groups are matched by \code{id} (instead of by row order), and only subjects
present in both groups are used. For more than two groups, the matching is
done independently for each pairwise comparison, so different comparisons can
be based on different numbers of pairs (per-comparison pairwise deletion).
This makes paired tests work when some observations are missing or the groups
have unequal sizes. The default (\code{id = NULL}) keeps the previous
behaviour (groups paired in row order).}

\item{error.as.na}{logical. If \code{TRUE}, a comparison that cannot be
computed (for example a group with fewer than two observations, or data that
are essentially constant) returns an \code{NA} result row with a warning
instead of stopping with an error; the other comparisons (or groups, for a
grouped analysis) are still computed. Default is \code{FALSE} (the comparison
errors as before).}

\item{...}{other arguments to be passed to the function
\code{\link[stats]{wilcox.test}}.}
}
\value{
return a data frame with some of the following columns: \itemize{
 \item \code{.y.}: the y variable used in the test. \item
 \code{group1,group2}: the compared groups in the pairwise tests. \item
 \code{n,n1,n2}: Sample counts. \item \code{statistic}: Test statistic used
 to compute the p-value. \item \code{p}: p-value. \item \code{p.adj}: the
 adjusted p-value. \item \code{method}: the statistical test used to compare
 groups. \item \code{p.signif, p.adj.signif}: the significance level of
 p-values and adjusted p-values, respectively. \item \code{estimate}: an
 estimate of the location parameter (Only present if argument \code{detailed
 = TRUE}). This corresponds to the pseudomedian (for one-sample case) or to
 the difference of the location parameter (for two-samples case). \itemize{
 \item The pseudomedian of a distribution \code{F} is the median of the
 distribution of \code{(u+v)/2}, where \code{u} and \code{v} are independent, each
 with distribution \code{F}. If \code{F} is symmetric, then the pseudomedian
 and median coincide. \item Note that in the two-sample case the estimator
 for the difference in location parameters does not estimate the difference
 in medians (a common misconception) but rather the median of the difference
 between a sample from x and a sample from y. } \item \code{conf.low,
 conf.high}: a confidence interval for the location parameter. (Only present
 if argument conf.int = TRUE.) }

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Provides a pipe-friendly framework to performs one and two sample
 Wilcoxon tests. Read more:
 \href{https://www.datanovia.com/en/lessons/wilcoxon-test-in-r/}{Wilcoxon in
 R}.
}
\details{
- \code{pairwise_wilcox_test()} applies the standard two sample
 Wilcoxon test to all possible pairs of groups. This method calls the
 \code{\link[stats]{wilcox.test}()}, so extra arguments are accepted.


 - If a list of comparisons is specified, the result of the pairwise tests is
 filtered to keep only the comparisons of interest.The p-value is adjusted
 after filtering.

 - For a grouped data, if pairwise test is performed, then the p-values are
 adjusted for each group level independently.


 - a nonparametric confidence interval and an estimator for the pseudomedian
 (one-sample case) or for the difference of the location parameters
 \code{x-y} is computed, where x and y are the compared samples or groups.
 The column \code{estimate} and the confidence intervals are displayed in the
 test result when the option \code{detailed = TRUE} is specified in the
 \code{wilcox_test()} and \code{pairwise_wilcox_test()} functions. Read more
 about the calculation of the estimate in the details section of the R base
 function \code{wilcox.test()} documentation by typing \code{?wilcox.test} in
 the R console.
}
\section{Functions}{
\itemize{
\item \code{wilcox_test()}: Wilcoxon test

\item \code{pairwise_wilcox_test()}: performs pairwise two sample Wilcoxon test.

}}
\note{
When a \code{ref.group} is specified, the reference group is taken as
 \code{group1} and the other group as \code{group2}, and the comparison is
 computed as \code{group1} versus \code{group2} (i.e. \code{ref.group} versus
 the other group), following the \code{\link[stats]{wilcox.test}} convention.
 With \code{detailed = TRUE}, the \code{estimate} is the Hodges-Lehmann
 location shift of \code{group1} relative to \code{group2}, so a positive
 \code{estimate} means the reference group is shifted higher; flip its sign
 (\code{mutate(estimate = -estimate)}) if you want a positive sign to mean
 "higher in the non-reference group". (The \code{statistic} is the
 rank-sum/signed-rank \code{W}, which is not a signed difference.)
}
\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth

# One-sample test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% wilcox_test(len ~ 1, mu = 0)


# Two-samples unpaired test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% wilcox_test(len ~ supp)

# Two-samples paired test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% wilcox_test (len ~ supp, paired = TRUE)

# Compare supp levels after grouping the data by "dose"
#::::::::::::::::::::::::::::::::::::::::
df \%>\%
  group_by(dose) \%>\%
  wilcox_test(data =., len ~ supp) \%>\%
  adjust_pvalue(method = "bonferroni") \%>\%
  add_significance("p.adj")

# pairwise comparisons
#::::::::::::::::::::::::::::::::::::::::
# As dose contains more than two levels ==>
# pairwise test is automatically performed.
df \%>\% wilcox_test(len ~ dose)

# Comparison against reference group
#::::::::::::::::::::::::::::::::::::::::
# each level is compared to the ref group
df \%>\% wilcox_test(len ~ dose, ref.group = "0.5")

# Comparison against all
#::::::::::::::::::::::::::::::::::::::::
df \%>\% wilcox_test(len ~ dose, ref.group = "all")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_cor_mat.R
\name{as_cor_mat}
\alias{as_cor_mat}
\title{Convert a Correlation Test Data Frame into a Correlation Matrix}
\usage{
as_cor_mat(x)
}
\arguments{
\item{x}{an object of class \code{cor_test}.}
}
\value{
Returns a data frame containing the matrix of the correlation
  coefficients. The output has an attribute named "pvalue", which contains
  the matrix of the correlation test p-values.
}
\description{
Convert a correlation test data frame, returned by the
  \code{\link{cor_test}()}, into a correlation matrix format.
}
\examples{
# Pairwise correlation tests between variables
#:::::::::::::::::::::::::::::::::::::::::::::::
res.cor.test <- mtcars \%>\%
  select(mpg, disp, hp, drat, wt, qsec) \%>\%
  cor_test()
res.cor.test

# Convert the correlation test into a correlation matrix
#:::::::::::::::::::::::::::::::::::::::::::::::
res.cor.test \%>\% as_cor_mat()

}
\seealso{
\code{\link{cor_mat}()}, \code{\link{cor_test}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor_plot.R
\name{cor_plot}
\alias{cor_plot}
\title{Visualize Correlation Matrix Using Base Plot}
\usage{
cor_plot(
  cor.mat,
  method = "circle",
  type = "full",
  palette = NULL,
  p.mat = NULL,
  significant.level = 0.05,
  insignificant = c("cross", "blank"),
  label = FALSE,
  font.label = list()
)
}
\arguments{
\item{cor.mat}{the correlation matrix to visualize}

\item{method}{Character, the visualization method of correlation matrix to be
  used. Currently, it supports seven methods, named \code{"circle"}
  (default), \code{"square"}, \code{"ellipse"}, \code{"number"},
  \code{"pie"}, \code{"shade"} and \code{"color"}. See examples for details.

  The areas of circles or squares show the absolute value of corresponding
  correlation coefficients. Method \code{"pie"} and \code{"shade"} came from
  Michael Friendly's job (with some adjustment about the shade added on), and
  \code{"ellipse"} came from D.J. Murdoch and E.D. Chow's job, see in section
  References.}

\item{type}{Character, \code{"full"} (default), \code{"upper"} or
\code{"lower"}, display full matrix, lower triangular or upper triangular
matrix.}

\item{palette}{character vector containing the color palette.}

\item{p.mat}{matrix of p-value corresponding to the correlation matrix.}

\item{significant.level}{significant level, if the p-value is bigger than
\code{significant.level}, then the corresponding correlation coefficient is
regarded as insignificant.}

\item{insignificant}{character, specialized insignificant correlation
coefficients, "cross" (default), "blank". If "blank", wipe away the
corresponding glyphs; if "cross", add crosses (X) on corresponding glyphs.}

\item{label}{logical value. If TRUE, shows the correlation coefficient
labels.}

\item{font.label}{a list with one or more of the following elements: size
(e.g., 1), color (e.g., "black") and style (e.g., "bold"). Used to
customize the correlation coefficient labels. For example \code{font.label
= list(size = 1, color = "black", style = "bold")}.}
}
\description{
Provide a tibble-friendly framework to visualize a correlation
  matrix. Wrapper around the R base function
  \code{\link[corrplot]{corrplot}()}. Compared to
  \code{\link[corrplot]{corrplot}()}, it can handle directly the output of the
  functions \code{\link{cor_mat}() (in rstatix)}, \code{rcorr() (in Hmisc)},
  \code{correlate() (in corrr)} and \code{cor() (in stats)}.

  The p-values contained in the outputs of the functions
  \code{\link{cor_mat}()} and \code{rcorr()} are automatically detected and
  used in the visualization.
}
\examples{
# Compute correlation matrix
#::::::::::::::::::::::::::::::::::::::::::
cor.mat <- mtcars \%>\%
  select(mpg, disp, hp, drat, wt, qsec) \%>\%
  cor_mat()

# Visualize correlation matrix
#::::::::::::::::::::::::::::::::::::::::::
# Full correlation matrix,
# insignificant correlations are marked by crosses
cor.mat \%>\% cor_plot()

# Reorder by correlation coefficient
# pull lower triangle and visualize
cor.lower.tri <- cor.mat \%>\%
  cor_reorder() \%>\%
  pull_lower_triangle()
cor.lower.tri \%>\% cor_plot()

# Change visualization methods
#::::::::::::::::::::::::::::::::::::::::::
cor.lower.tri \%>\%
  cor_plot(method = "pie")

cor.lower.tri \%>\%
  cor_plot(method = "color")

cor.lower.tri \%>\%
  cor_plot(method = "number")

# Show the correlation coefficient: label = TRUE
# Blank the insignificant correlation
#::::::::::::::::::::::::::::::::::::::::::
cor.lower.tri \%>\%
  cor_plot(
    method = "color",
    label = TRUE,
    insignificant = "blank"
  )

# Change the color palettes
#::::::::::::::::::::::::::::::::::::::::::

# Using custom color palette
# Require ggpubr: install.packages("ggpubr")
if(require("ggpubr")){
  my.palette <- get_palette(c("red", "white", "blue"), 200)
  cor.lower.tri \%>\%
   cor_plot(palette = my.palette)
}

# Using RcolorBrewer color palette
if(require("ggpubr")){
  my.palette <- get_palette("PuOr", 200)
  cor.lower.tri \%>\%
   cor_plot(palette = my.palette)
}

}
\seealso{
\code{\link{cor_as_symbols}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/games_howell_test.R
\name{games_howell_test}
\alias{games_howell_test}
\title{Games Howell Post-hoc Tests}
\usage{
games_howell_test(data, formula, conf.level = 0.95, detailed = FALSE)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{conf.level}{confidence level of the interval.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}
}
\value{
return a data frame with some of the following columns: \itemize{
 \item \code{.y.}: the y (outcome) variable used in the test. \item
 \code{group1,group2}: the compared groups in the pairwise tests. \item
 \code{n1,n2}: Sample counts. \item \code{estimate, conf.low, conf.high}:
 mean difference and its confidence intervals. \item \code{statistic}: Test
 statistic (t-value) used to compute the p-value. \item \code{df}: degrees of
 freedom calculated using Welch’s correction. \item \code{p.adj}: adjusted p-value using Tukey's method. \item
 \code{method}: the statistical test used to compare groups. \item
 \code{p.adj.signif}: the significance level of p-values. }

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs Games-Howell test, which is used to compare all possible
 combinations of group differences when the assumption of homogeneity of
 variances is violated. This post hoc test provides confidence intervals for
 the differences between group means and shows whether the differences are
 statistically significant.

 The test is based on Welch’s degrees of freedom correction and uses Tukey’s
 studentized range distribution for computing the p-values. The test compares
 the difference between each pair of means with appropriate adjustment for
 the multiple testing. So there is no need to apply additional p-value
 corrections.
}
\details{
The Games-Howell method is an improved version of the Tukey-Kramer
 method and is applicable in cases where the equivalence of variance
 assumption is violated. It is a t-test using Welch’s degree of freedom. This
 method uses a strategy for controlling the type I error for the entire
 comparison and is known to maintain the preset significance level even when
 the size of the sample is different. However, the smaller the number of
 samples in each group, the it is more tolerant the type I error control.
 Thus, this method can be applied when the number of samples is six or more.
}
\examples{
# Simple test
ToothGrowth \%>\% games_howell_test(len ~ dose)

# Grouped data
ToothGrowth \%>\%
  group_by(supp) \%>\%
  games_howell_test(len ~ dose)

}
\references{
\itemize{ \item Aaron Schlege,
 https://rpubs.com/aaronsc32/games-howell-test. \item Sangseok Lee, Dong Kyu
 Lee. What is the proper way to apply the multiple comparison test?. Korean J
 Anesthesiol. 2018;71(5):353-360. }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcnemar_test.R
\name{mcnemar_test}
\alias{mcnemar_test}
\alias{pairwise_mcnemar_test}
\title{McNemar's Chi-squared Test for Count Data}
\usage{
mcnemar_test(x, y = NULL, correct = TRUE)

pairwise_mcnemar_test(
  data,
  formula,
  type = c("mcnemar", "exact"),
  correct = TRUE,
  p.adjust.method = "bonferroni"
)
}
\arguments{
\item{x}{either a two-dimensional contingency table in matrix form,
    or a factor object.}

\item{y}{a factor object; ignored if \code{x} is a matrix.}

\item{correct}{a logical indicating whether to apply continuity
    correction when computing the test statistic.}

\item{data}{a data frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{a ~ b | c}, where \code{a} is the
outcome variable name; b is the within-subjects factor variables; and c
(factor) is the column name containing individuals/subjects identifier.
Should be unique per individual.}

\item{type}{type of statistical tests used for pairwise comparisons. Allowed
values are one of \code{c("mcnemar", "exact")}.}

\item{p.adjust.method}{method to adjust p values for multiple comparisons.
Used when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}
}
\value{
return a data frame with the following columns: \itemize{
\item \code{n}: the number of participants.
\item \code{statistic}: the value of McNemar's statistic. \item \code{df} the
 degrees of freedom of the approximate chi-squared distribution of the test
 statistic. \item \code{p}: p-value. \item \code{p.adj}: the adjusted
 p-value. \item \code{method}: the used statistical test. \item
 \code{p.signif}: the significance level of p-values.}

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs McNemar chi-squared test to compare paired proportions.

 Wrappers around the R base function \code{\link[stats]{mcnemar.test}()}, but
 provide pairwise comparisons between multiple groups
}
\section{Functions}{
\itemize{
\item \code{mcnemar_test}: performs McNemar's chi-squared test for comparing two
paired proportions

\item \code{pairwise_mcnemar_test}: performs pairwise McNemar's chi-squared test between
multiple groups. Could be used for post-hoc tests following a significant Cochran's Q test.
}}

\examples{

# Comparing two paired proportions
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Data: frequencies of smokers before and after interventions
xtab <- as.table(
  rbind(c(25, 6), c(21,10))
)
dimnames(xtab) <- list(
  before = c("non.smoker", "smoker"),
  after = c("non.smoker", "smoker")
)
xtab

# Compare the proportion of smokers
mcnemar_test(xtab)

# Comparing multiple related proportions
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Generate a demo data
mydata <- data.frame(
  outcome = c(0,1,1,0,0,1,0,1,1,1,1,1,0,0,1,1,0,1,0,1,1,0,0,1,0,1,1,0,0,1),
  treatment = gl(3,1,30,labels=LETTERS[1:3]),
  participant = gl(10,3,labels=letters[1:10])
)
mydata$outcome <- factor(
  mydata$outcome, levels = c(1, 0),
  labels = c("success", "failure")
  )
# Cross-tabulation
xtabs(~outcome + treatment, mydata)

# Compare the proportion of success between treatments
cochran_qtest(mydata, outcome ~ treatment|participant)

# pairwise comparisons between groups
pairwise_mcnemar_test(mydata, outcome ~ treatment|participant)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fisher_test.R
\name{fisher_test}
\alias{fisher_test}
\alias{pairwise_fisher_test}
\alias{row_wise_fisher_test}
\title{Fisher's Exact Test for Count Data}
\usage{
fisher_test(
  xtab,
  workspace = 2e+05,
  alternative = "two.sided",
  conf.int = TRUE,
  conf.level = 0.95,
  simulate.p.value = FALSE,
  B = 2000,
  detailed = FALSE,
  ...
)

pairwise_fisher_test(xtab, p.adjust.method = "holm", detailed = FALSE, ...)

row_wise_fisher_test(xtab, p.adjust.method = "holm", detailed = FALSE, ...)
}
\arguments{
\item{xtab}{a contingency table in a matrix form.}

\item{workspace}{an integer specifying the size of the workspace
    used in the network algorithm.  In units of 4 bytes.  Only used for
    non-simulated p-values larger than \eqn{2 \times 2}{2 by 2} tables.
    Since \R version 3.5.0, this also increases the internal stack size
    which allows larger problems to be solved, however sometimes needing
    hours.  In such cases, \code{simulate.p.values=TRUE} may be more
    reasonable.}

\item{alternative}{indicates the alternative hypothesis and must be
    one of \code{"two.sided"}, \code{"greater"} or \code{"less"}.
    You can specify just the initial letter.  Only used in the
    \eqn{2 \times 2}{2 by 2} case.}

\item{conf.int}{logical indicating if a confidence interval for the
    odds ratio in a \eqn{2 \times 2}{2 by 2} table should be
    computed (and returned).}

\item{conf.level}{confidence level for the returned confidence
    interval.  Only used in the \eqn{2 \times 2}{2 by 2} case and if
    \code{conf.int = TRUE}.}

\item{simulate.p.value}{a logical indicating whether to compute
    p-values by Monte Carlo simulation, in larger than \eqn{2 \times
      2}{2 by 2} tables.}

\item{B}{an integer specifying the number of replicates used in the
    Monte Carlo test.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}

\item{...}{Other arguments passed to the function \code{fisher_test()}.}

\item{p.adjust.method}{method to adjust p values for multiple comparisons.
Used when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}
}
\value{
return a data frame with some the following columns: \itemize{ \item
 \code{group}: the categories in the row-wise proportion tests. \item
 \code{p}: p-value. \item \code{p.adj}: the adjusted p-value. \item
 \code{method}: the used statistical test. \item \code{p.signif,
 p.adj.signif}: the significance level of p-values and adjusted p-values,
 respectively. \item \code{estimate}: an estimate of the odds ratio. Only
 present in the 2 by 2 case. \item \code{alternative}: a character string
 describing the alternative hypothesis. \item \code{conf.low,conf.high}: a
 confidence interval for the odds ratio. Only present in the 2 by 2 case and
 if argument conf.int = TRUE.}

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs Fisher's exact test for testing the null of independence
 of rows and columns in a contingency table.

 Wrappers around the R base function \code{\link[stats]{fisher.test}()} but
 have the advantage of performing pairwise and row-wise fisher tests, the
 post-hoc tests following a significant chi-square test of homogeneity for 2xc
 and rx2 contingency tables.
}
\section{Functions}{
\itemize{
\item \code{fisher_test}: performs Fisher's exact test for testing the null of
independence of rows and columns in a contingency table with fixed
marginals. Wrapper around the function \code{\link[stats]{fisher.test}()}.

\item \code{pairwise_fisher_test}: pairwise comparisons between proportions, a post-hoc
tests following a significant Fisher's exact test of homogeneity for 2xc
design.

\item \code{row_wise_fisher_test}: performs row-wise Fisher's exact test of count data, a post-hoc tests following a significant chi-square test
of homogeneity for rx2 contingency table. The test is conducted for each category (row).
}}

\examples{

# Comparing two proportions
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Data: frequencies of smokers between two groups
xtab <- as.table(rbind(c(490, 10), c(400, 100)))
dimnames(xtab) <- list(
  group = c("grp1", "grp2"),
  smoker = c("yes", "no")
)
xtab
# compare the proportion of smokers
fisher_test(xtab, detailed = TRUE)

# Homogeneity of proportions between groups
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# H0: the proportion of smokers is similar in the four groups
# Ha:  this proportion is different in at least one of the populations.
#
# Data preparation
grp.size <- c( 106, 113, 156, 102 )
smokers  <- c( 50, 100, 139, 80 )
no.smokers <- grp.size - smokers
xtab <- as.table(rbind(
  smokers,
  no.smokers
))
dimnames(xtab) <- list(
  Smokers = c("Yes", "No"),
  Groups = c("grp1", "grp2", "grp3", "grp4")
)
xtab

# Compare the proportions of smokers between groups
fisher_test(xtab, detailed = TRUE)

# Pairwise comparison between groups
pairwise_fisher_test(xtab)


# Pairwise proportion tests
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Data: Titanic
xtab <- as.table(rbind(
  c(122, 167, 528, 673),
  c(203, 118, 178, 212)
))
dimnames(xtab) <- list(
  Survived = c("No", "Yes"),
  Class = c("1st", "2nd", "3rd", "Crew")
)
xtab
# Compare the proportion of survived between groups
pairwise_fisher_test(xtab)

# Row-wise proportion tests
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Data: Titanic
xtab <- as.table(rbind(
  c(180, 145), c(179, 106),
  c(510, 196), c(862, 23)
))
dimnames(xtab) <- list(
  Class = c("1st", "2nd", "3rd", "Crew"),
  Gender = c("Male", "Female")
)
xtab
# Compare the proportion of males and females in each category
row_wise_fisher_test(xtab)

# A r x c table  Agresti (2002, p. 57) Job Satisfaction
Job <- matrix(c(1,2,1,0, 3,3,6,1, 10,10,14,9, 6,7,12,11), 4, 4,
              dimnames = list(income = c("< 15k", "15-25k", "25-40k", "> 40k"),
                             satisfaction = c("VeryD", "LittleD", "ModerateS", "VeryS")))
fisher_test(Job)
fisher_test(Job, simulate.p.value = TRUE, B = 1e5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_clean_names.R
\name{make_clean_names}
\alias{make_clean_names}
\title{Make Clean Names}
\usage{
make_clean_names(data)
}
\arguments{
\item{data}{a data frame or vector}
}
\value{
a data frame or a vector depending on the input data
}
\description{
Pipe-friendly function to make syntactically valid names out of
 character vectors.
}
\examples{

# Vector
make_clean_names(c("a and b", "a-and-b"))
make_clean_names(1:10)

# data frame
df <- data.frame(
`a and b` = 1:4,
`c and d` = 5:8,
 check.names = FALSE
)
df
make_clean_names(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/df.R
\name{df_label_both}
\alias{df_label_both}
\alias{df_label_value}
\title{Functions to Label Data Frames by Grouping Variables}
\usage{
df_label_both(data, ..., vars = NULL, label_col = "label", sep = c(", ", ":"))

df_label_value(data, ..., vars = NULL, label_col = "label", sep = ", ")
}
\arguments{
\item{data}{a data frame}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used as grouping variables.}

\item{vars}{a character vector containing the grouping variables of interest.}

\item{label_col}{column to hold the label of the data subsets. Default column
name is "label".}

\item{sep}{String separating labelling variables and values. Should be of
length 2 in the function \code{df_label_both()}. 1) One sep is used to
separate groups, for example ','; 2) The other sep between group name and
levels; for example ':'.}
}
\value{
a modified data frame with a column containing row labels.
}
\description{
Functions to label data frame rows by one or multiple grouping
  variables.
}
\section{Functions}{
\itemize{
\item \code{df_label_both}: Displays both the variable name and the factor value.

\item \code{df_label_value}: Displays only the value of a factor.
}}

\examples{
# Data preparation
df <- head(ToothGrowth)

# Labelling: Non standard evaluation
df \%>\%
  df_label_both(dose, supp)

# Standard evaluation
df \%>\%
  df_label_both(dose, supp)

# Nesting the data then label each subset by groups
ToothGrowth \%>\%
  df_nest_by(dose, supp) \%>\%
  df_label_both(supp, dose)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/replace_triangle.R
\name{replace_triangle}
\alias{replace_triangle}
\alias{replace_upper_triangle}
\alias{replace_lower_triangle}
\title{Replace Lower and Upper Triangular Part of a Matrix}
\usage{
replace_triangle(x, triangle = c("lower", "upper"), by = "", diagonal = FALSE)

replace_upper_triangle(x, by = "", diagonal = FALSE)

replace_lower_triangle(x, by = "", diagonal = FALSE)
}
\arguments{
\item{x}{a (correlation) matrix}

\item{triangle}{the triangle to replace. Allowed values are one of
"upper" and "lower".}

\item{by}{a replacement argument. Appropriate values are either "" or NA. Used to replace
the upper, lower or the diagonal part of the matrix.}

\item{diagonal}{logical. Default is FALSE. If TRUE, the matrix diagonal is
included.}
}
\value{
an object of class \code{cor_mat_tri}, which is a data frame
}
\description{
Replace the lower or the upper triangular part of a
  (correlation) matrix.
}
\section{Functions}{
\itemize{
\item \code{replace_triangle}: replaces the specified triangle by empty or NA.

\item \code{replace_upper_triangle}: replaces the upper triangular part of a matrix.
Returns an object of class \code{lower_tri}.

\item \code{replace_lower_triangle}: replaces the lower triangular part of a matrix.
Returns an object of class \code{lower_tri}
}}

\examples{
# Compute correlation matrix and pull triangles
#::::::::::::::::::::::::::::::::::::::::::
# Correlation matrix
 cor.mat <- mtcars \%>\%
   select(mpg, disp, hp, drat, wt, qsec) \%>\%
   cor_mat()
 cor.mat

 # Replace upper triangle by NA
#::::::::::::::::::::::::::::::::::::::::::
cor.mat \%>\% replace_upper_triangle(by = NA)


# Replace upper triangle by NA and reshape the
# correlation matrix to have unique combinations of variables
#::::::::::::::::::::::::::::::::::::::::::
cor.mat \%>\%
  replace_upper_triangle(by = NA) \%>\%
  cor_gather()
}
\seealso{
\code{\link{pull_triangle}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rstatix-programming.R
\name{rstatix-programming}
\alias{rstatix-programming}
\title{Programming with rstatix (tidy evaluation)}
\description{
How to use \code{rstatix} functions programmatically — i.e. when the variable
names are held in character strings or passed into your own wrapper functions,
as is common in package development, loops, and Shiny apps.

\code{rstatix} has two kinds of interfaces, and each supports a standard way of
"programming over variables":

\itemize{
  \item \strong{Selection interface} (functions that select columns through
  \code{...} or \code{vars=}, e.g. \code{\link{cor_test}()},
  \code{\link{get_summary_stats}()}, \code{\link{cor_mat}()},
  \code{\link{freq_table}()}). These support full tidy-evaluation: bare names,
  the injection operators \code{!!}/\code{!!!}, the embracing operator
  \code{\{\{ \}\}} inside your own functions, a character vector via
  \code{vars=}, and tidyselect helpers such as \code{all_of()}/\code{any_of()}.

  \item \strong{Formula interface} (tests that take a \code{formula}, e.g.
  \code{\link{t_test}()}, \code{\link{wilcox_test}()}, \code{\link{kruskal_test}()},
  \code{\link{anova_test}()}, \code{\link{friedman_test}()}). A formula is an
  ordinary R object, so build it from strings with \code{\link[stats]{reformulate}()}
  or \code{stats::as.formula(paste(...))} and pass it in.
}
}
\details{
Injecting a string directly into a raw formula (e.g. \code{t_test(df, y ~ \{\{var\}\})})
is \strong{not} supported: a formula is captured as a syntax tree, not a quosure,
so the embracing/injection operators do not apply there. Build the formula
instead with \code{reformulate(rhs, lhs)} — see the examples.

In examples below, helpers that \code{rstatix} does not re-export are namespaced
(\code{rlang::sym}, \code{dplyr::all_of}, \code{dplyr::across}); attach
\code{rlang}/\code{dplyr} and you can drop the prefixes.
}
\examples{
# Selection interface -----------------------------------------------------

# A variable name held in a string, injected with !!
x <- "mpg"; y <- "wt"
mtcars \%>\% cor_test(!!rlang::sym(x), !!rlang::sym(y))

# Several names at once, spliced with !!!
vars <- c("mpg", "disp", "hp")
mtcars \%>\% cor_test(!!!rlang::syms(vars))

# A character vector via `vars =` (no rlang needed)
mtcars \%>\% cor_test(vars = c("mpg", "disp"))

# tidyselect helpers
iris \%>\% get_summary_stats(dplyr::all_of(c("Sepal.Length", "Sepal.Width")))

# Your own wrapper function: embrace the argument with {{ }}
my_summary <- function(data, var) {
  data \%>\% get_summary_stats({{ var }}, type = "mean_sd")
}
my_summary(iris, Sepal.Length)

# Formula interface -------------------------------------------------------

# Build the formula from strings with reformulate(rhs, lhs)
outcome <- "len"; group <- "supp"
ToothGrowth \%>\% t_test(reformulate(group, outcome))

# The same inside a wrapper function
my_test <- function(data, outcome, group) {
  data \%>\% t_test(reformulate(group, outcome))
}
my_test(ToothGrowth, "len", "supp")

# Programmatic grouping + a built formula (a common end-to-end pattern)
gv <- "supp"
ToothGrowth \%>\%
  group_by(dplyr::across(dplyr::all_of(gv))) \%>\%
  t_test(reformulate("dose", "len"))

}
\seealso{
\code{\link{cor_test}()}, \code{\link{get_summary_stats}()},
  \code{\link{t_test}()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tukey_hsd.R
\name{tukey_hsd}
\alias{tukey_hsd}
\alias{tukey_hsd.default}
\alias{tukey_hsd.lm}
\alias{tukey_hsd.data.frame}
\title{Tukey Honest Significant Differences}
\usage{
tukey_hsd(x, ...)

\method{tukey_hsd}{default}(x, ...)

\method{tukey_hsd}{lm}(x, ...)

\method{tukey_hsd}{data.frame}(x, formula, ...)
}
\arguments{
\item{x}{an object of class \code{aov}, \code{lm} or \code{data.frame}
containing the variables used in the formula.}

\item{...}{other arguments passed to the function
\code{\link[stats]{TukeyHSD}()}. These include: \itemize{ \item
\strong{which}: A character vector listing terms in the fitted model for
which the intervals should be calculated. Defaults to all the terms. \item
\strong{ordered}: A logical value indicating if the levels of the factor
should be ordered according to increasing average in the sample before
taking differences. If ordered is true then the calculated differences in
the means will all be positive. The significant differences will be those
for which the lwr end point is positive. }}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{data}{a data.frame containing the variables in the formula.}
}
\value{
a tibble data frame containing the results of the different
 comparisons.
}
\description{
Provides a pipe-friendly framework to performs Tukey post-hoc
 tests. Wrapper around the function \code{\link[stats]{TukeyHSD}()}. It is
 essentially a t-test that corrects for multiple testing.

 Can handle different inputs formats: aov, lm, formula.
}
\section{Methods (by class)}{
\itemize{
\item \code{default}: performs tukey post-hoc test from \code{aov()} results.

\item \code{lm}: performs tukey post-hoc test from \code{lm()} model.

\item \code{data.frame}: performs tukey post-hoc tests using data and formula as
inputs. ANOVA will be automatically performed using the function
\code{\link[stats]{aov}()}
}}

\examples{
# Data preparation
df <- ToothGrowth
df$dose <- as.factor(df$dose)
# Tukey HSD from ANOVA results
aov(len ~ dose, data = df) \%>\% tukey_hsd()

# two-way anova with interaction
aov(len ~ dose*supp, data = df) \%>\% tukey_hsd()

# Tukey HSD from lm() results
lm(len ~ dose, data = df) \%>\% tukey_hsd()

# Tukey HSD from data frame and formula
tukey_hsd(df, len ~ dose)

# Tukey HSD using grouped data
df \%>\%
  group_by(supp) \%>\%
  tukey_hsd(len ~ dose)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/friedman_test.R
\name{friedman_test}
\alias{friedman_test}
\title{Friedman Rank Sum Test}
\usage{
friedman_test(data, formula, ...)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{a ~ b | c}, where \code{a}
(numeric) is the dependent variable name; \code{b} is the within-subjects
factor variables; and \code{c} (factor) is the column name containing
individuals/subjects identifier. Should be unique per individual.}

\item{...}{other arguments to be passed to the function
\code{\link[stats]{friedman.test}}.}
}
\value{
return a data frame with the following columns: \itemize{ \item
 \code{.y.}: the y (dependent) variable used in the test. \item \code{n}:
 sample count. \item \code{statistic}: the value of Friedman's chi-squared
 statistic, used to compute the p-value. \item \code{p}: p-value. \item
 \code{method}: the statistical test used to compare groups.}
}
\description{
Provides a pipe-friendly framework to perform a Friedman rank sum
 test, which is the non-parametric alternative to the one-way repeated
 measures ANOVA test. Wrapper around the function
 \code{\link[stats]{friedman.test}()}. Read more:
 \href{https://www.datanovia.com/en/lessons/friedman-test-in-r/}{Friedman
 test in R}.
}
\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth \%>\%
    filter(supp == "VC") \%>\%
    mutate(id = rep(1:10, 3))
head(df)

# Friedman rank sum test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% friedman_test(len ~ dose | id)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chisq_test.R
\name{chisq_test}
\alias{chisq_test}
\alias{pairwise_chisq_gof_test}
\alias{pairwise_chisq_test_against_p}
\alias{chisq_descriptives}
\alias{expected_freq}
\alias{observed_freq}
\alias{pearson_residuals}
\alias{std_residuals}
\title{Chi-squared Test for Count Data}
\usage{
chisq_test(
  x,
  y = NULL,
  correct = TRUE,
  p = rep(1/length(x), length(x)),
  rescale.p = FALSE,
  simulate.p.value = FALSE,
  B = 2000
)

pairwise_chisq_gof_test(x, p.adjust.method = "holm", ...)

pairwise_chisq_test_against_p(
  x,
  p = rep(1/length(x), length(x)),
  p.adjust.method = "holm",
  ...
)

chisq_descriptives(res.chisq)

expected_freq(res.chisq)

observed_freq(res.chisq)

pearson_residuals(res.chisq)

std_residuals(res.chisq)
}
\arguments{
\item{x}{a numeric vector or matrix. \code{x} and \code{y} can also
    both be factors.}

\item{y}{a numeric vector; ignored if \code{x} is a matrix.  If
    \code{x} is a factor, \code{y} should be a factor of the same length.}

\item{correct}{a logical indicating whether to apply continuity
    correction when computing the test statistic for 2 by 2 tables: one
    half is subtracted from all \eqn{|O - E|} differences; however, the
    correction will not be bigger than the differences themselves.  No correction
    is done if \code{simulate.p.value = TRUE}.}

\item{p}{a vector of probabilities of the same length of \code{x}.
    An error is given if any entry of \code{p} is negative.}

\item{rescale.p}{a logical scalar; if TRUE then \code{p} is rescaled
    (if necessary) to sum to 1.  If \code{rescale.p} is FALSE, and
    \code{p} does not sum to 1, an error is given.}

\item{simulate.p.value}{a logical indicating whether to compute
    p-values by Monte Carlo simulation.}

\item{B}{an integer specifying the number of replicates used in the
    Monte Carlo test.}

\item{p.adjust.method}{method to adjust p values for multiple comparisons.
Used when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}

\item{...}{other arguments passed to the function \code{{chisq_test}()}.}

\item{res.chisq}{an object of class \code{chisq_test}.}
}
\value{
return a data frame with some the following columns: \itemize{ \item
 \code{n}: the number of participants. \item \code{group, group1, group2}:
 the categories or groups being compared. \item \code{statistic}: the value
 of Pearson's chi-squared test statistic. \item \code{df}: the degrees of
 freedom of the approximate chi-squared distribution of the test statistic.
 NA if the p-value is computed by Monte Carlo simulation. \item \code{p}:
 p-value. \item \code{p.adj}: the adjusted p-value. \item \code{method}: the
 used statistical test. \item \code{p.signif, p.adj.signif}: the significance
 level of p-values and adjusted p-values, respectively. \item
 \code{observed}: observed counts. \item
 \code{expected}: the expected counts under the null hypothesis.
 }
 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs chi-squared tests, including goodness-of-fit,
 homogeneity and independence tests.
}
\section{Functions}{
\itemize{
\item \code{chisq_test}: performs chi-square tests including goodness-of-fit,
homogeneity and independence tests.

\item \code{pairwise_chisq_gof_test}: perform pairwise comparisons between groups following a global
chi-square goodness of fit test.

\item \code{pairwise_chisq_test_against_p}: perform pairwise comparisons after a global
chi-squared test for given probabilities. For each group, the observed and
the expected proportions are shown. Each group is compared to the sum of
all others.

\item \code{chisq_descriptives}: returns the descriptive statistics of the chi-square
test. These include, observed and expected frequencies, proportions,
residuals and standardized residuals.

\item \code{expected_freq}: returns the expected counts from the chi-square test result.

\item \code{observed_freq}: returns the observed counts from the chi-square test result.

\item \code{pearson_residuals}: returns the Pearson residuals, \code{(observed - expected) / sqrt(expected)}.

\item \code{std_residuals}: returns the standardized residuals
}}

\examples{
# Chi-square goodness of fit test
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
tulip <- c(red = 81, yellow = 50, white = 27)
# Q1: Are the colors equally common?
chisq_test(tulip)
pairwise_chisq_gof_test(tulip)
# Q2: comparing observed to expected proportions
chisq_test(tulip, p = c(1/2, 1/3, 1/6))
pairwise_chisq_test_against_p(tulip, p = c(0.5, 0.33, 0.17))

# Homogeneity of proportions between groups
#\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Data: Titanic
xtab <- as.table(rbind(
  c(203, 118, 178, 212),
  c(122, 167, 528, 673)
))
dimnames(xtab) <- list(
  Survived = c("Yes", "No"),
  Class = c("1st", "2nd", "3rd", "Crew")
)
xtab
# Chi-square test
chisq_test(xtab)
# Compare the proportion of survived between groups
pairwise_prop_test(xtab)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pull_triangle.R
\name{pull_triangle}
\alias{pull_triangle}
\alias{pull_upper_triangle}
\alias{pull_lower_triangle}
\title{Pull Lower and Upper Triangular Part of a Matrix}
\usage{
pull_triangle(x, triangle = c("lower", "upper"), diagonal = FALSE)

pull_upper_triangle(x, diagonal = FALSE)

pull_lower_triangle(x, diagonal = FALSE)
}
\arguments{
\item{x}{a (correlation) matrix}

\item{triangle}{the triangle to pull. Allowed values are one of
"upper" and "lower".}

\item{diagonal}{logical. Default is FALSE. If TRUE, the matrix diagonal is
included.}
}
\value{
an object of class \code{cor_mat_tri}, which is a data frame
}
\description{
Returns the lower or the upper triangular part of a
  (correlation) matrix.
}
\section{Functions}{
\itemize{
\item \code{pull_triangle}: returns either the lower or upper triangular part of a matrix.

\item \code{pull_upper_triangle}: returns an object of class \code{upper_tri}, which
is a data frame containing the upper triangular part of a matrix.

\item \code{pull_lower_triangle}: returns an object of class \code{lower_tri}, which
is a data frame containing the lower triangular part of a matrix.
}}

\examples{

# Data preparation
#::::::::::::::::::::::::::::::::::::::::::
mydata <- mtcars \%>\%
  select(mpg, disp, hp, drat, wt, qsec)
head(mydata, 3)

# Compute correlation matrix and pull triangles
#::::::::::::::::::::::::::::::::::::::::::
# Correlation matrix
cor.mat <- cor_mat(mydata)
cor.mat

# Pull lower triangular part
cor.mat \%>\% pull_lower_triangle()

# Pull upper triangular part
cor.mat \%>\% pull_upper_triangle()


}
\seealso{
\code{\link{replace_triangle}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_significance.R
\name{add_significance}
\alias{add_significance}
\title{Add P-value Significance Symbols}
\usage{
add_significance(
  data,
  p.col = NULL,
  output.col = NULL,
  cutpoints = c(0, 1e-04, 0.001, 0.01, 0.05, 1),
  symbols = c("****", "***", "**", "*", "ns")
)
}
\arguments{
\item{data}{a data frame containing a p-value column.}

\item{p.col}{column name containing p-values.}

\item{output.col}{the output column name to hold the adjusted p-values.}

\item{cutpoints}{numeric vector used for intervals.}

\item{symbols}{character vector, one shorter than cutpoints, used as
significance symbols.}
}
\value{
a data frame
}
\description{
Add p-value significance symbols into a data frame.
}
\examples{
# Perform pairwise comparisons and adjust p-values
ToothGrowth \%>\%
 t_test(len ~ dose) \%>\%
 adjust_pvalue() \%>\%
 add_significance("p.adj")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/df.R
\name{df_get_var_names}
\alias{df_get_var_names}
\title{Get User Specified Variable Names}
\usage{
df_get_var_names(data, ..., vars = NULL)
}
\arguments{
\item{data}{a data frame}

\item{...}{One or more unquoted expressions (or variable names) separated by
commas. Used to select a variable of interest.}

\item{vars}{a character vector containing the variable names of interest.}
}
\value{
a character vector
}
\description{
Returns user specified variable names. Supports standard and non standard evaluation.
}
\examples{

# Non standard evaluation
ToothGrowth \%>\%
 df_get_var_names(dose, len)

# Standard evaluation
ToothGrowth \%>\%
 df_get_var_names(vars = c("len", "dose"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kruskal_test.R
\name{kruskal_test}
\alias{kruskal_test}
\title{Kruskal-Wallis Test}
\usage{
kruskal_test(data, formula, ...)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{...}{other arguments to be passed to the function
\code{\link[stats]{kruskal.test}}.}
}
\value{
return a data frame with the following columns: \itemize{ \item
 \code{.y.}: the y variable used in the test. \item \code{n}: sample count.
 \item \code{statistic}: the kruskal-wallis rank sum statistic used to
 compute the p-value. \item \code{p}: p-value. \item \code{method}: the
 statistical test used to compare groups.}
}
\description{
Provides a pipe-friendly framework to perform Kruskal-Wallis
 rank sum test. Wrapper around the function
 \code{\link[stats]{kruskal.test}()}.
}
\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth

# Kruskal-wallis rank sum test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% kruskal_test(len ~ dose)

# Grouped data
df \%>\%
  group_by(supp) \%>\%
  kruskal_test(len ~ dose)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kruskal_effesize.R
\name{kruskal_effsize}
\alias{kruskal_effsize}
\title{Kruskal-Wallis Effect Size}
\usage{
kruskal_effsize(
  data,
  formula,
  ci = FALSE,
  conf.level = 0.95,
  ci.type = "perc",
  nboot = 1000
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{ci}{If TRUE, returns confidence intervals by bootstrap. May be slow.}

\item{conf.level}{The level for the confidence interval.}

\item{ci.type}{The type of confidence interval to use. Can be any of "norm",
"basic", "perc", or "bca". Passed to \code{boot::boot.ci}.}

\item{nboot}{The number of replications to use for bootstrap.}
}
\value{
return a data frame with some of the following columns: \itemize{
 \item \code{.y.}: the y variable used in the test. \item \code{n}: Sample
 counts. \item \code{effsize}: estimate of the effect size. \item
 \code{magnitude}: magnitude of effect size. \item \code{conf.low,conf.high}:
 lower and upper bound of the effect size confidence interval.}
}
\description{
Compute the effect size for Kruskal-Wallis test as the eta
 squared based on the H-statistic: \code{eta2[H] = (H - k + 1)/(n - k)};
 where \code{H} is the value obtained in the Kruskal-Wallis test; \code{k} is
 the number of groups; \code{n} is the total number of observations.


 The eta-squared estimate assumes values from 0 to 1 and multiplied by 100%
 indicates the percentage of variance in the dependent variable explained by
 the independent variable. The interpretation values commonly in published
 litterature are: \code{0.01- < 0.06} (small effect), \code{0.06 - < 0.14}
 (moderate effect) and \code{>= 0.14} (large effect).

Confidence intervals are calculated by bootstap.
}
\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth

# Kruskal-wallis rank sum test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% kruskal_effsize(len ~ dose)

# Grouped data
df \%>\%
  group_by(supp) \%>\%
  kruskal_effsize(len ~ dose)
}
\references{
Maciej Tomczak and Ewa Tomczak. The need to report effect size
 estimates revisited. An overview of some recommended measures of effect
 size. Trends in Sport Sciences. 2014; 1(21):19-25.

 http://imaging.mrc-cbu.cam.ac.uk/statswiki/FAQ/effectSize

 http://www.psy.gla.ac.uk/~steve/best/effect.html
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_cld.R
\name{add_cld}
\alias{add_cld}
\title{Compact Letter Display of All-Pairwise Comparisons}
\usage{
add_cld(test, p.col = NULL, threshold = 0.05, reversed = FALSE, ...)
}
\arguments{
\item{test}{an all-pairwise comparison result returned by an \code{rstatix}
function (e.g. \code{tukey_hsd()}, \code{dunn_test()}, a pairwise
\code{t_test()}/\code{wilcox_test()}, ...). Must contain the \code{group1}
and \code{group2} columns and a p-value column.}

\item{p.col}{character. The p-value column to threshold. If \code{NULL}
(default), \code{"p.adj"} is used when present, otherwise \code{"p"}.}

\item{threshold}{the significance threshold (default 0.05). Comparisons with a
p-value below \code{threshold} are treated as significant; comparisons with a
missing (\code{NA}) p-value are treated as non-significant.}

\item{reversed}{logical. If \code{TRUE}, reverses the order in which the
letters are assigned (so that, with groups ordered by increasing level, the
later groups receive the earlier letters). Default is \code{FALSE}.}

\item{...}{not used.}
}
\value{
a tibble with one row per group and the following columns: any grouping
 variables (for a grouped test), \code{.y.} (the outcome variable, when
 present), \code{group} (the group level) and \code{cld} (the compact letter
 display). Groups sharing a letter are not significantly different.
}
\description{
Adds the \strong{compact letter display} (CLD) to a pairwise
 comparison result. Groups that do \emph{not} share a letter are
 significantly different. This is a convenient way to annotate plots
 (e.g. one letter per box/bar) after an all-pairwise post-hoc test such as
 \code{\link{tukey_hsd}()}, \code{\link{dunn_test}()},
 \code{\link{games_howell_test}()}, \code{\link{conover_test}()},
 \code{\link{wilcox_test}()} or \code{\link{t_test}()}.

 The letters are computed with the insert-and-absorb algorithm (Piepho, 2004)
 using base R only, so no additional package is required (the results match
 \code{multcompView::multcompLetters()}).
}
\examples{
# Tukey HSD post-hoc, then compact letter display
res <- ToothGrowth \%>\%
  mutate(dose = factor(dose)) \%>\%
  tukey_hsd(len ~ dose)
res \%>\% add_cld()

# Works on rank-based post-hocs too
ToothGrowth \%>\% dunn_test(len ~ dose) \%>\% add_cld()

# Grouped pairwise test -> one CLD per group
ToothGrowth \%>\%
  mutate(dose = factor(dose)) \%>\%
  group_by(supp) \%>\%
  tukey_hsd(len ~ dose) \%>\%
  add_cld()
}
\references{
Piepho, H.-P. (2004) An Algorithm for a Letter-Based Representation
 of All-Pairwise Comparisons. Journal of Computational and Graphical
 Statistics, 13(2), 456-466.
}
\seealso{
\code{\link{tukey_hsd}}, \code{\link{dunn_test}},
 \code{\link{games_howell_test}}, \code{\link{add_significance}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/levene_test.R
\name{levene_test}
\alias{levene_test}
\title{Levene's Test}
\usage{
levene_test(data, formula, center = median)
}
\arguments{
\item{data}{a data frame for evaluating the formula or a model}

\item{formula}{a formula}

\item{center}{The name of a function to compute the center of each group;
mean gives the original Levene's test; the default, median, provides a more
robust test.}
}
\description{
Provide a pipe-friendly framework to easily compute Levene's
  test for homogeneity of variance across groups.

  Wrapper around the function \code{\link[car]{leveneTest}()}, which can
  additionally handles a grouped data.
}
\examples{
# Prepare the data
data("ToothGrowth")
df <- ToothGrowth
df$dose <- as.factor(df$dose)
# Compute Levene's Test
df \%>\% levene_test(len ~ dose)

# Grouped data
df \%>\%
  group_by(supp) \%>\%
  levene_test(len ~ dose)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexports.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{tibble}
\alias{mutate}
\alias{filter}
\alias{group_by}
\alias{select}
\alias{desc}
\alias{drop_na}
\alias{gather}
\alias{spread}
\alias{tidy}
\alias{augment}
\alias{Anova}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{car}{\code{\link[car]{Anova}}}

  \item{dplyr}{\code{\link[dplyr]{desc}}, \code{\link[dplyr]{filter}}, \code{\link[dplyr]{group_by}}, \code{\link[dplyr]{mutate}}, \code{\link[dplyr]{select}}}

  \item{generics}{\code{\link[generics]{augment}}, \code{\link[generics]{tidy}}}

  \item{tibble}{\code{\link[tibble]{tibble}}}

  \item{tidyr}{\code{\link[tidyr]{drop_na}}, \code{\link[tidyr]{gather}}, \code{\link[tidyr]{spread}}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/t_test.R
\name{t_test}
\alias{t_test}
\alias{pairwise_t_test}
\title{T-test}
\usage{
t_test(
  data,
  formula,
  comparisons = NULL,
  ref.group = NULL,
  p.adjust.method = "holm",
  paired = FALSE,
  var.equal = FALSE,
  alternative = "two.sided",
  mu = 0,
  conf.level = 0.95,
  detailed = FALSE
)

pairwise_t_test(
  data,
  formula,
  comparisons = NULL,
  ref.group = NULL,
  p.adjust.method = "holm",
  paired = FALSE,
  pool.sd = !paired,
  detailed = FALSE,
  ...
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
numeric variable giving the data values and \code{group} is a factor with
one or multiple levels giving the corresponding groups. For example,
\code{formula = TP53 ~ cancer_group}.}

\item{comparisons}{A list of length-2 vectors specifying the groups of
interest to be compared. For example to compare groups "A" vs "B" and "B" vs
"C", the argument is as follow: \code{comparisons = list(c("A", "B"), c("B",
"C"))}}

\item{ref.group}{a character string specifying the reference group. If
 specified, for a given grouping variable, each of the group levels will be
 compared to the reference group (i.e. control group).

 If \code{ref.group = "all"}, pairwise two sample tests are performed for
 comparing each grouping variable levels against all (i.e. basemean).}

\item{p.adjust.method}{method to adjust p values for multiple comparisons.
Used when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". If you don't
want to adjust the p value (not recommended), use p.adjust.method = "none".}

\item{paired}{a logical indicating whether you want a paired test.}

\item{var.equal}{a logical variable indicating whether to treat the
    two variances as being equal. If \code{TRUE} then the pooled
    variance is used to estimate the variance otherwise the Welch
    (or Satterthwaite) approximation to the degrees of freedom is used.}

\item{alternative}{a character string specifying the alternative
    hypothesis, must be one of \code{"two.sided"} (default),
    \code{"greater"} or \code{"less"}.  You can specify just the initial
    letter.}

\item{mu}{a number specifying an optional parameter used to form the null hypothesis.}

\item{conf.level}{confidence level of the interval.}

\item{detailed}{logical value. Default is FALSE. If TRUE, a detailed result is
shown.}

\item{pool.sd}{logical value used in the function \code{pairwise_t_test()}.
 Switch to allow/disallow the use of a pooled SD.

 The \code{pool.sd = TRUE} (default) calculates a common SD for all groups
 and uses that for all comparisons (this can be useful if some groups are
 small). This method does not actually call t.test, so extra arguments are
 ignored. Pooling does not generalize to paired tests so pool.sd and paired
 cannot both be TRUE.

 If \code{pool.sd = FALSE} the standard two sample t-test is applied to all
 possible pairs of groups. This method calls the \code{t.test()}, so extra
 arguments, such as \code{var.equal} are accepted.}

\item{...}{other arguments to be passed to the function
\code{\link[stats]{t.test}}.}
}
\value{
return a data frame with some the following columns: \itemize{ \item
 \code{.y.}: the y variable used in the test. \item \code{group1,group2}: the
 compared groups in the pairwise tests. \item \code{n,n1,n2}: Sample counts.
 \item \code{statistic}: Test statistic used to compute the p-value. \item
 \code{df}: degrees of freedom. \item \code{p}: p-value. \item \code{p.adj}:
 the adjusted p-value. \item \code{method}: the statistical test used to
 compare groups. \item \code{p.signif, p.adj.signif}: the significance level
 of p-values and adjusted p-values, respectively. \item \code{estimate}:
 estimate of the effect size. It corresponds to the estimated mean or
 difference in means depending on whether it was a one-sample test or a
 two-sample test. \item \code{estimate1, estimate2}: show the mean values of
 the two groups, respectively, for independent samples t-tests. \item
 \code{alternative}: a character string describing the alternative
 hypothesis. \item \code{conf.low,conf.high}: Lower and upper bound on a
 confidence interval. }

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Provides a pipe-friendly framework to performs one and two sample
 t-tests. Read more: \href{https://www.datanovia.com/en/lessons/t-test-in-r/}{T-test in R}.
}
\details{
- If a list of comparisons is specified, the result of the pairwise tests is
filtered to keep only the comparisons of interest. The p-value is adjusted
after filtering.

- For a grouped data, if pairwise test is performed, then the p-values are
adjusted for each group level independently.
}
\section{Functions}{
\itemize{
\item \code{t_test}: t test

\item \code{pairwise_t_test}: performs pairwise two sample t-test. Wrapper around the R
base function \code{\link[stats]{pairwise.t.test}}.
}}

\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth

# One-sample test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% t_test(len ~ 1, mu = 0)


# Two-samples unpaired test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% t_test(len ~ supp)

# Two-samples paired test
#:::::::::::::::::::::::::::::::::::::::::
df \%>\% t_test (len ~ supp, paired = TRUE)

# Compare supp levels after grouping the data by "dose"
#::::::::::::::::::::::::::::::::::::::::
df \%>\%
  group_by(dose) \%>\%
  t_test(data =., len ~ supp) \%>\%
  adjust_pvalue(method = "bonferroni") \%>\%
  add_significance("p.adj")

# pairwise comparisons
#::::::::::::::::::::::::::::::::::::::::
# As dose contains more than two levels ==>
# pairwise test is automatically performed.
df \%>\% t_test(len ~ dose)

# Comparison against reference group
#::::::::::::::::::::::::::::::::::::::::
# each level is compared to the ref group
df \%>\% t_test(len ~ dose, ref.group = "0.5")

# Comparison against all
#::::::::::::::::::::::::::::::::::::::::
df \%>\% t_test(len ~ dose, ref.group = "all")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor_reshape.R
\name{cor_gather}
\alias{cor_gather}
\alias{cor_spread}
\title{Reshape Correlation Data}
\usage{
cor_gather(data, drop.na = TRUE)

cor_spread(data, value = "cor")
}
\arguments{
\item{data}{a data frame or matrix.}

\item{drop.na}{logical. If TRUE, drop rows containing missing values after gathering the data.}

\item{value}{column name containing the value to spread.}
}
\description{
Reshape correlation analysis results. Key functions: \itemize{
  \item \code{cor_gather()}: takes a correlation matrix and collapses (i.e. melt) it into a paired list
  (long format). \item \code{cor_spread()}: spread a long correlation data format across
  multiple columns. Particularly, it takes the results of \code{\link{cor_test}}
  and transforms it into a correlation matrix. }
}
\section{Functions}{
\itemize{
\item \code{cor_gather}: takes a correlation matrix and collapses (or melt) it into long
format data frame (paired list)

\item \code{cor_spread}: spread a long correlation data frame into wide format
(correlation matrix).
}}

\examples{
# Data preparation
#::::::::::::::::::::::::::::::::::::::::::
mydata <- mtcars \%>\%
  select(mpg, disp, hp, drat, wt, qsec)
head(mydata, 3)

# Reshape a correlation matrix
#::::::::::::::::::::::::::::::::::::::::::
# Compute a correlation matrix
cor.mat <- mydata \%>\% cor_mat()
cor.mat

# Collapse the correlation matrix into long format
# paired list data frame
long.format <- cor.mat \%>\% cor_gather()
long.format

# Spread a correlation data format
#::::::::::::::::::::::::::::::::::::::::::
# Spread the correlation coefficient value
long.format \%>\% cor_spread(value = "cor")
# Spread the p-value
long.format \%>\% cor_spread(value = "p")
}
\seealso{
\code{\link{cor_mat}()}, \code{\link{cor_reorder}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_test_label.R
\name{get_pwc_label}
\alias{get_pwc_label}
\alias{get_test_label}
\alias{create_test_label}
\title{Extract Label Information from Statistical Tests}
\usage{
get_pwc_label(stat.test, type = c("expression", "text"))

get_test_label(
  stat.test,
  description = NULL,
  p.col = "p",
  type = c("expression", "text"),
  correction = c("auto", "GG", "HF", "none"),
  row = NULL,
  detailed = FALSE
)

create_test_label(
  statistic.text,
  statistic,
  p,
  parameter = NA,
  description = NULL,
  n = NA,
  effect.size = NA,
  effect.size.text = NA,
  type = c("expression", "text"),
  detailed = FALSE
)
}
\arguments{
\item{stat.test}{statistical test results returned by \code{rstatix}
functions.}

\item{type}{the label type. Can be one of "text" and "expression". Partial
match allowed. If you want to add the label onto a ggplot, it might be
useful to specify \code{type = "expresion"}.}

\item{description}{the test description used as the prefix of the label.
Examples of description are "ANOVA", "Two Way ANOVA". To remove the default
description, specify \code{description = NULL}. If missing, we'll try to
guess the statistical test default description.}

\item{p.col}{character specifying the column containing the p-value. Default
is \code{"p"}, can be \code{"p.adj"}.}

\item{correction}{character, considered only in the case of ANOVA test. Which sphericity
correction of the degrees of freedom should be reported for the
within-subject factors (repeated measures). The default is set to
\code{"GG"} corresponding to the Greenhouse-Geisser correction. Possible
values are \code{"GG"}, \code{"HF"} (i.e., Hyunh-Feldt correction),
\code{"none"} (i.e., no correction) and \code{"auto"} (apply automatically
GG correction if the sphericity assumption is not for within-subject
design.}

\item{row}{numeric, the row index to be considered. If NULL, the last row is
automatically considered for ANOVA test.}

\item{detailed}{logical value. If TRUE, returns detailed label.}

\item{statistic.text}{character specifying the test statistic. For example
\code{statistic.text = "F"} (for ANOVA test ); \code{statistic.text = "t"}
(for t-test ).}

\item{statistic}{the numeric value of a statistic.}

\item{p}{the p-value of the test.}

\item{parameter}{string containing the degree of freedom (if exists). Default
is \code{NA} to accommodate non-parametric tests. For example
\code{parameter = "1,9"} (for ANOVA test. Two parameters exist: DFn and
DFd); \code{sparameter = "9"} (for t-test ).}

\item{n}{sample count, example: \code{n = 10}.}

\item{effect.size}{the effect size value}

\item{effect.size.text}{a character specifying the relevant effect size. For
example, for \code{Cohens d} statistic, \code{effect.size.text = "d"}. You
can also use plotmath expression as follow \code{quote(italic("d"))}.}
}
\value{
a text label or an expression to pass to a plotting function.
}
\description{
Extracts label information from statistical tests. Useful for
  labelling plots with test outputs.
}
\section{Functions}{
\itemize{
\item \code{get_pwc_label}: Extract label from pairwise comparisons.

\item \code{get_test_label}: Extract labels for statistical tests.

\item \code{create_test_label}: Create labels from user specified test results.
}}

\examples{
# Load data
#:::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth

# One-way ANOVA test
#:::::::::::::::::::::::::::::::::::::::::
anov <- df \%>\% anova_test(len ~ dose)
get_test_label(anov, detailed = TRUE, type = "text")

# Two-way ANOVA test
#:::::::::::::::::::::::::::::::::::::::::
anov <- df \%>\% anova_test(len ~ supp*dose)
get_test_label(anov, detailed = TRUE, type = "text",
   description = "Two Way ANOVA")


# Kruskal-Wallis test
#:::::::::::::::::::::::::::::::::::::::::
kruskal<- df \%>\% kruskal_test(len ~ dose)
get_test_label(kruskal, detailed = TRUE, type = "text")

# Wilcoxon test
#:::::::::::::::::::::::::::::::::::::::::
# Unpaired test
wilcox <- df \%>\% wilcox_test(len ~ supp)
get_test_label(wilcox, detailed = TRUE, type = "text")
# Paired test
wilcox <- df \%>\% wilcox_test(len ~ supp, paired = TRUE)
get_test_label(wilcox, detailed = TRUE, type = "text")

# T test
#:::::::::::::::::::::::::::::::::::::::::
ttest <- df \%>\% t_test(len ~ dose)
get_test_label(ttest, detailed = TRUE, type = "text")


# Pairwise comparisons labels
#:::::::::::::::::::::::::::::::::::::::::
get_pwc_label(ttest, type = "text")


# Create test labels
#:::::::::::::::::::::::::::::::::::::::::
create_test_label(
  statistic.text = "F", statistic = 71.82,
  parameter = "4, 294",
  p = "<0.0001",
  description = "ANOVA",
  type = "text"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/friedman_conover_test.R
\name{friedman_conover_test}
\alias{friedman_conover_test}
\title{Conover's All-Pairs Comparisons Test for Friedman Rank Sums}
\usage{
friedman_conover_test(
  data,
  formula,
  p.adjust.method = "holm",
  ref.group = NULL,
  detailed = FALSE
)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{a ~ b | c}, where \code{a}
(numeric) is the dependent variable name; \code{b} is the within-subjects
factor variable (the treatment); and \code{c} is the column name containing
the individuals/subjects (block) identifier. Should be unique per individual.}

\item{p.adjust.method}{method to adjust p-values for multiple comparisons. Used
when pairwise comparisons are performed. Allowed values include "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none". Default is
"holm".}

\item{ref.group}{a character string specifying the reference treatment. If
specified, each of the treatment levels is compared to the reference
(control), and the p-value adjustment is computed over only these
comparisons.}

\item{detailed}{logical value. If TRUE, returns the rank-sum estimate and the
test method in the output.}
}
\value{
return a data frame with some of the following columns: \itemize{
 \item \code{.y.}: the y (outcome) variable used in the test. \item
 \code{group1,group2}: the compared treatments in the pairwise tests. \item
 \code{n1,n2}: the number of blocks (subjects) contributing to each treatment.
 \item \code{estimate}: the rank-sum difference. \item \code{estimate1,
 estimate2}: the rank sums of the two treatments, respectively. \item
 \code{statistic}: Test statistic (t-value) used to compute the p-value. \item
 \code{df}: degrees of freedom (\eqn{(b - 1)(k - 1)}). \item \code{p}: p-value.
 \item \code{p.adj}: the adjusted p-value. \item \code{method}: the
 statistical test used to compare groups. \item \code{p.adj.signif}: the
 significance level of the adjusted p-values. }

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs Conover's all-pairs comparison test (also known as the
 Durbin-Conover test) for a two-way balanced complete block design, following
 a significant Friedman rank sum test. It is the repeated-measures analogue of
 Conover's test for the Kruskal-Wallis design: the
 within-block ranks are compared pairwise using the pooled rank variance and
 the statistic is referred to a \emph{t}-distribution with \eqn{(b - 1)(k - 1)}
 degrees of freedom (\eqn{b} blocks, \eqn{k} treatments). It should only be
 used as a post-hoc procedure when the Friedman test is itself significant
 (Conover, 1999).

 If a reference group is specified (via \code{ref.group}), then each of the
 remaining treatments is compared only to the reference (control) treatment,
 and the p-value adjustment for multiple comparisons is computed over only
 these \code{k - 1} comparisons (as for \code{\link{dunn_test}()}).
}
\details{
For a balanced complete block design with \eqn{b} blocks and \eqn{k}
 treatments, the observations within each block are ranked. Let \eqn{R_j} be
 the sum of the within-block ranks for treatment \eqn{j} and let \eqn{A =
 \sum r^2} be the sum of the squared within-block ranks. The pairwise
 statistic for treatments \eqn{i} and \eqn{j} is \deqn{t_{ij} =
 \frac{R_i - R_j}{\sqrt{\dfrac{2\,(b\,A - \sum_j R_j^2)}{(b - 1)(k - 1)}}}}
 referred to a \emph{t}-distribution with \eqn{(b - 1)(k - 1)} degrees of
 freedom. This is the Conover (1999) post-hoc, also known as the
 Durbin-Conover test.
}
\examples{
# A balanced complete block design: 3 treatments measured on 6 subjects
df <- data.frame(
  id        = factor(rep(1:6, 3)),
  treatment = factor(rep(c("A", "B", "C"), each = 6)),
  score     = c(4, 6, 3, 5, 4, 5,    7, 8, 6, 7, 9, 6,    6, 9, 7, 8, 8, 9)
)

# Omnibus Friedman test
df \%>\% friedman_test(score ~ treatment | id)

# Conover (Durbin-Conover) all-pairs post-hoc
df \%>\% friedman_conover_test(score ~ treatment | id)

# Comparison against a reference (control) treatment
df \%>\% friedman_conover_test(score ~ treatment | id, ref.group = "A")
}
\references{
Conover, W. J. (1999) Practical Nonparametric Statistics, 3rd
 edition. Wiley.
}
\seealso{
\code{\link{friedman_test}}, \code{\link{friedman_nemenyi_test}},
  \code{\link{friedman_effsize}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/friedman_nemenyi_test.R
\name{friedman_nemenyi_test}
\alias{friedman_nemenyi_test}
\title{Nemenyi Post-Hoc Test for Friedman Rank Sums}
\usage{
friedman_nemenyi_test(data, formula, detailed = FALSE)
}
\arguments{
\item{data}{a data.frame containing the variables in the formula.}

\item{formula}{a formula of the form \code{a ~ b | c}, where \code{a}
(numeric) is the dependent variable name; \code{b} is the within-subjects
factor variable (the treatment); and \code{c} is the column name containing
the individuals/subjects (block) identifier. Should be unique per individual.}

\item{detailed}{logical value. If TRUE, returns the rank-sum estimate and the
test method in the output.}
}
\value{
return a data frame with some of the following columns: \itemize{
 \item \code{.y.}: the y (outcome) variable used in the test. \item
 \code{group1,group2}: the compared treatments in the pairwise tests. \item
 \code{n1,n2}: the number of blocks (subjects) contributing to each treatment.
 \item \code{estimate}: the rank-sum difference. \item \code{estimate1,
 estimate2}: the rank sums of the two treatments, respectively. \item
 \code{statistic}: the studentized-range test statistic. \item \code{p.adj}:
 the p-value (already adjusted for multiple comparisons via the studentized
 range distribution). \item \code{method}: the statistical test used to
 compare groups. \item \code{p.adj.signif}: the significance level of the
 adjusted p-values. }

 The \strong{returned object has an attribute called args}, which is a list
 holding the test arguments.
}
\description{
Performs the Nemenyi (Wilcoxon-Nemenyi-McDonald-Thompson) all-pairs
 post-hoc test for a two-way balanced complete block design, following a
 significant Friedman rank sum test. The treatment rank sums are compared
 pairwise and the test statistic is referred to the studentized range
 distribution, which already accounts for the multiplicity of the all-pairs
 comparisons (so, as for \code{\link{tukey_hsd}()}, there is no separate
 p-value adjustment step). It should only be used as a post-hoc procedure when
 the Friedman test is itself significant.

 The Nemenyi test is the rank-based, repeated-measures analogue of Tukey's
 HSD. Unlike \code{\link{friedman_conover_test}()} (the Durbin-Conover test),
 it does not borrow the residual rank variance and is therefore more
 conservative.
}
\details{
For a balanced complete block design with \eqn{b} blocks and \eqn{k}
 treatments, the observations within each block are ranked. Let \eqn{R_j} be
 the sum of the within-block ranks for treatment \eqn{j}. The pairwise
 statistic for treatments \eqn{i} and \eqn{j} is \deqn{q_{ij} =
 \frac{|R_i - R_j|}{\sqrt{b\,k\,(k+1)/12}}} and the p-value is obtained from
 the studentized range distribution with \eqn{k} groups and infinite degrees
 of freedom.
}
\examples{
# A balanced complete block design: 3 treatments measured on 6 subjects
df <- data.frame(
  id        = factor(rep(1:6, 3)),
  treatment = factor(rep(c("A", "B", "C"), each = 6)),
  score     = c(4, 6, 3, 5, 4, 5,    7, 8, 6, 7, 9, 6,    6, 9, 7, 8, 8, 9)
)

# Omnibus Friedman test
df \%>\% friedman_test(score ~ treatment | id)

# Nemenyi all-pairs post-hoc
df \%>\% friedman_nemenyi_test(score ~ treatment | id)
}
\references{
Nemenyi, P. (1963) Distribution-free Multiple Comparisons. PhD
 Thesis, Princeton University.

 Hollander, M., Wolfe, D. A. (1973) Nonparametric Statistical Methods. Wiley.
}
\seealso{
\code{\link{friedman_test}}, \code{\link{friedman_conover_test}},
  \code{\link{friedman_effsize}}
}
